import { Fetch } from "../HttpClient/FetchImplementation/Fetch";
import { ExceptionLogger } from "../HttpClient/HttpClient";
import { HttpStatusCode, HttpStatusHandlers, IHttpClient } from "../HttpClient/IHttpClient";
import { AuthenticationMode } from "../HttpClient/AuthenticationMode";
import { ICombineByTransfers } from "../../Engine/src/Engine/Core/ModalManager/Modals/UploadSection/Types";
import { AbsoluteModelPath } from "../ModelPath/ModelPath";
import { BaseGetFufResponse, CheckScenarioType, DraftSettings, FufWithAttachmentsResponse, GetUserResponse, ICandyApiClient, ICandyApiDraftClient, IInnerChangeResult, NormalizationProgressTask, PathLoadSpecification, RequestNdsFileOptions } from "./ICandyApiClient";
import { ServerErrorLog } from "./Types/ErrorLog/ServerErrorLog";
import { ServerInnerResponse, ServerJsonResponse } from "./Types/GetInner/ServerInnerResponse";
import { ExcelsToPartialFufsResponse, FileProperties, ServerErrorResponse, UploadFilePropertiesResponse } from "./Types/DraftActions/FileProperties";
import { CreateDraftResponse } from "./Types/CreateDraft/CreateDraftResponse";
import { CreateExplanatoryNoteResponse } from "./Types/ExplanatoryNote/CreateExplanatoryNoteResponse";
import { DraftFullId } from "./Types/DraftFullId";
import { ServerChangeSet } from "./Types/ChangeSet/ServerChangeSet";
import { ServerTask, VerifyAndFixFnsRequirementsTaskInfo } from "./Types/Tasks/Tasks";
import { GetByInnResponse } from "./Types/Focus/FocusTypes";
import { AttachInfo } from "./Types/Attachments/Attacments";
import { DraftPresentationType } from "./Types/Presentations/DraftPresentationType";
import { PresentationContent } from "./Types/Presentations/PresentationContent";
import { ConvertDraftRequest } from "./Types/DraftActions/ConvertDraftRequest";
import { ServerRequisitesResponse } from "./Types/GetInner/ServerRequisitesResponse";
import { CombinatorType, CombineFufResult, FufCombineMethod } from "./Types/DraftActions/CombineFufResult";
import { FeatureFlagsResponse } from "./Types/FeatureFlags/FeatureFlagsResponse";
import { ImportEmployeesRequest } from "./Types/Staff/ImportEmployeesRequest";
import { MassActionParameters } from "./Types/MassActions/MassActionParameters";
import { PrintType } from "./Types/Print/PrintType";
import { PrintResponse } from "./Types/Print/PrintResponse";
import { ServerRawResponse } from "./Types/GetInner/ServerRawResponse";
import { CandyPopdProtocol } from "./Types/ErrorLog/CandyPopdProtocol";
import { InstancesDeletingRule } from "./Types/InstancesDeleting/InstancesDeletingRule";
import { ValueForInstancesSettingRule } from "./Types/MassActions/ValueForInstancesSettingRule";
import { InnerQueryRequest } from "./Types/QueryCollecton/InnerQueryRequest";
import { InnerQueryResponse } from "./Types/QueryCollecton/InnerQueryResponse";
import { ResolveInnerPathRequest } from "./Types/ErrorLog/ResolveInnerPathRequest";
import { ResolvePathResponse } from "./Types/ErrorLog/ResolvePathResponse";
import { InnerHistoryResponse } from "./Types/HistoryEditing/InnerHistoryResponse";
export declare const CandyResourceZones: {
    TestForms: string;
};
export declare enum ServerErrors {
    FufSizeTooLarge = "FufSizeTooLarge",
    BadRequest = "BadRequest",
    UnknownError = "UnknownError"
}
export declare class CandyApiClient implements ICandyApiClient {
    private readonly httpClient;
    constructor(fetchImpl: Fetch, baseUrl: string, resourceZone: undefined | string, authentication: undefined | AuthenticationMode, exceptionLogger?: ExceptionLogger, httpStatusHandlers?: HttpStatusHandlers);
    forDraft(draftFullId: DraftFullId): ICandyApiDraftClient;
    getInfoByInns(draftFullId: DraftFullId, inns: readonly string[]): Promise<GetByInnResponse>;
    getAllGfvs(abortSignal?: AbortSignal): Promise<string[]>;
    forceGetAllGfvs(): Promise<string[]>;
    createDraftFromDrive(ns: string, fufDriveUri: string, abortSignal?: AbortSignal): Promise<CreateDraftResponse>;
    createEmptyDraft(ns: string, gfv: string, abortSignal?: AbortSignal): Promise<CreateDraftResponse>;
    createDraft(ns: string, gfv: string): Promise<CreateDraftResponse>;
    createDraft(ns: string, gfv: undefined, fufContent: Buffer): Promise<CreateDraftResponse>;
    checkAlive(): Promise<void>;
    private getDraftUrl;
    getFeatureFlags(abonId?: string, orgId?: string, userId?: string): Promise<FeatureFlagsResponse>;
}
export declare class CandyApiDraftClient implements ICandyApiDraftClient {
    private readonly httpClient;
    private readonly draftFullId;
    private readonly normalizeDownloadLink;
    constructor(httpClient: IHttpClient, draftFullId: DraftFullId);
    getHrefToInnerEditor(): string;
    getHrefToConvertationLog(): string;
    getInnerFuf(): Promise<string | undefined>;
    getHrefToDownloadInnerFuf(): string;
    getHrefToDownloadClientInfo(): string;
    getHrefToDownloadFormsClientInfo(): string;
    getErrorLog(): Promise<ServerErrorLog>;
    getErrorLogWithCheckScenarios(checkScenarios: CheckScenarioType[]): Promise<ServerErrorLog>;
    getTaskById(taskId: string): Promise<ServerTask | undefined>;
    getFufWithAttachments(): Promise<FufWithAttachmentsResponse | undefined>;
    getPartialFuf(rules: string[]): Promise<BaseGetFufResponse | undefined>;
    getAllTasks(): Promise<ServerTask[]>;
    downloadFuf(): Promise<void>;
    makeDownloadDriveLink(uri: string, fileName?: string): Promise<void>;
    downloadInner(): Promise<void>;
    enqueueTask(taskId: string, taskInfo: VerifyAndFixFnsRequirementsTaskInfo): Promise<void>;
    cancelTask(taskId: string): Promise<void>;
    getExpertErrorLog(): Promise<ServerErrorLog>;
    getHrefToErrorLogPdf(errorLogName: string, propsName: string): string;
    clearDraft(): Promise<void>;
    convertDraftFromPrevGfv({ prevGfvDraftId, gfv }: ConvertDraftRequest): Promise<void>;
    sendChangeWithNormalizationSet(data: ServerChangeSet, currentDataVersion: number, normalizerId?: string, operationName?: string): Promise<IInnerChangeResult>;
    sendChangeSet(data: ServerChangeSet, currentDataVersion: number, operationName?: string): Promise<IInnerChangeResult>;
    sendChangeSetSync(data: ServerChangeSet, currentDataVersion: number, normalizerId?: string): IInnerChangeResult;
    getRequisites(requisiteList: readonly string[]): Promise<ServerRequisitesResponse>;
    getInner(paths: undefined | readonly string[]): Promise<ServerInnerResponse>;
    getPartialInner(rules: PathLoadSpecification[], maxInstanceCount?: number): Promise<ServerInnerResponse>;
    getPartialInnerAsJsonTree(rules: PathLoadSpecification[], maxInstanceCount?: number, skipNormalization?: boolean): Promise<ServerJsonResponse>;
    getRawPartialInnerAsJsonTree(rules: readonly PathLoadSpecification[], maxInstanceCount?: number, skipNormalization?: boolean): Promise<ServerRawResponse>;
    getDataVersion(): Promise<number>;
    startNormalizationTask(): Promise<NormalizationProgressTask>;
    getNormalizationTask(): Promise<NormalizationProgressTask>;
    getExplanatoryNote(): Promise<CreateExplanatoryNoteResponse>;
    getAttachmentById(attachmentId: string): Promise<AttachInfo>;
    getAllAttachments(): Promise<AttachInfo[]>;
    removeAttachments(attachmentIds: readonly string[]): Promise<void>;
    createOrUpdateAttachments(attachments: readonly AttachInfo[]): Promise<void>;
    createDraftAttachment(gfv: string, path: string): Promise<AttachInfo>;
    openLoadedAttachmentAsDraft(id: string): Promise<AttachInfo>;
    setPresentation(presentationType: DraftPresentationType, content: string | unknown): Promise<void>;
    patchPresentation(presentationType: DraftPresentationType, presentationContent: PresentationContent): Promise<void>;
    getFileProperties(driveUri: string, ignoreSizeCheck?: boolean): Promise<FileProperties | undefined>;
    getFileNdsProperties({ driveUri, declarationGfv, requestedGfv, ignoreSizeCheck, }: RequestNdsFileOptions): Promise<FileProperties | undefined>;
    prepareAndGetFileProperties(driveUri: string, gfv: string, ignoreSizeCheck?: boolean): Promise<FileProperties | undefined>;
    uploadAndGetFileProperties(driveUris: string[], gfv: string, abortSignal?: AbortSignal): Promise<UploadFilePropertiesResponse>;
    acceptMultipleTransferType(files: ICombineByTransfers[], abortSignal?: AbortSignal): Promise<ServerErrorResponse | undefined>;
    setFufByDriveUri(driveUri: string): Promise<void>;
    excelsToPartialFufs(gfv: string, pathToTable: string, driveUris: string[]): Promise<ExcelsToPartialFufsResponse | undefined>;
    importXmls(driveUris: string[], combineMethod: FufCombineMethod, combinatorType: CombinatorType, isImportByFuf: boolean, args?: string[]): Promise<CombineFufResult | undefined>;
    setDraftSettings(settings: DraftSettings): Promise<void>;
    getDraftSettings(): Promise<DraftSettings>;
    normalizeByRules(rules: string): Promise<void>;
    private getHrefDownloadDraftPresentation;
    private getDraftUrl;
    private getDraftPresentationUrl;
    setMultipleWithNormalization(rules: ValueForInstancesSettingRule[]): Promise<IInnerChangeResult>;
    normalizeDirtyMasks(): Promise<IInnerChangeResult>;
    searchInstancesByText(text: string, pathsPrefix: string, pathsToSearch: string[], maxInstanceCount?: number): Promise<string[]>;
    queryCollection(request: InnerQueryRequest, abortSignal?: AbortSignal): Promise<InnerQueryResponse>;
    resolveInnerPath(request: ResolveInnerPathRequest, abortSignal?: AbortSignal): Promise<ResolvePathResponse>;
    customNormalize(normalizerId: string | null, pathPrefixes: AbsoluteModelPath[], abortSignal?: AbortSignal): Promise<IInnerChangeResult>;
    importEmployees(request: ImportEmployeesRequest): Promise<void>;
    deleteInstances(deletingRules: InstancesDeletingRule[]): Promise<void>;
    runMassAction(actionId: string, parameters?: MassActionParameters): Promise<void>;
    print(type: PrintType, body?: string[]): Promise<PrintResponse>;
    printExcel(pathToTable: string): Promise<PrintResponse | undefined>;
    checkByPopd(): Promise<CandyPopdProtocol>;
    getHistory(): Promise<InnerHistoryResponse | HttpStatusCode>;
    undoToChange(changeGUID: string): Promise<number>;
    getGFV(): Promise<string>;
    getUser(): Promise<GetUserResponse>;
}
