"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypingError = exports.InvalidArgumentError = exports.InvalidProgramStateError = exports.NotSupportedError = exports.NotImplementedError = exports.ApplicationError = void 0;
class ApplicationError extends Error {
    constructor(message) {
        super(message);
        this.name = this.constructor.name;
    }
}
exports.ApplicationError = ApplicationError;
class NotImplementedError extends ApplicationError {
    constructor() {
        super("Operation not implemented");
    }
}
exports.NotImplementedError = NotImplementedError;
class NotSupportedError extends ApplicationError {
    constructor(message) {
        super(message ?? "Operation not supported");
    }
}
exports.NotSupportedError = NotSupportedError;
class InvalidProgramStateError extends ApplicationError {
    constructor(message) {
        super("Invalid program state" + (message != undefined ? ": " + message : ""));
    }
}
exports.InvalidProgramStateError = InvalidProgramStateError;
class InvalidArgumentError extends ApplicationError {
    constructor(argumentName, message) {
        super(message != undefined ? `${message} (argument: ${argumentName})` : `Invalid argument value: ${argumentName}`);
    }
}
exports.InvalidArgumentError = InvalidArgumentError;
class TypingError extends ApplicationError {
    constructor() {
        super("Invalid typing");
    }
}
exports.TypingError = TypingError;
