import { Fetch } from "./FetchImplementation/Fetch";
import { HttpStatusCode, HttpStatusHandler, HttpStatusHandlers, IHttpClient, IHttpClientRequestWrapper, IHttpClientResponse, IHttpClientResponseSync, QueryParamsObject } from "./IHttpClient";
import { HttpClientAuthenticationProvider } from "./HttpClientAuthenticationProvider";
type BodyFactory = () => BodyInit | null;
interface IHttpClientRequestConfig {
    signal?: AbortSignal;
    bodyFactory?: BodyFactory;
    acceptCodes: HttpStatusCode[];
    httpStatusHandlers: HttpStatusHandlers;
    queryParams: QueryParamsObject;
    contentType?: string;
}
export interface IExceptionLoggerParams {
    fetchUrl?: string;
}
export type ExceptionLogger = (error: unknown, params?: IExceptionLoggerParams) => void;
export declare class HttpClientResponse implements IHttpClientResponse {
    private readonly fetchResponse;
    constructor(fetchResponse: Response);
    get status(): HttpStatusCode;
    get headers(): Headers;
    json<T>(): Promise<T>;
    blob(): Promise<Blob>;
    text(): Promise<string>;
}
type RequestExecutionFunction = (config: IHttpClientRequestConfig) => Promise<HttpClientResponse | Response>;
export declare class HttpClientRequestWrapper implements IHttpClientRequestWrapper {
    private readonly executeRequest;
    private readonly acceptCodes;
    private httpStatusHandlers;
    private queryParams;
    private signal;
    private bodyFactory?;
    private contentType?;
    constructor(execute: RequestExecutionFunction);
    withAbortSignal(signal: undefined | AbortSignal): this;
    withContentType(contentType: string): this;
    withBodyFromBuffer(body: false | Buffer | undefined): this;
    withJsonBody(serializableBody: unknown | {} | unknown[] | undefined): this;
    withQueryParams(queryParams: QueryParamsObject | false | undefined): this;
    withRawBody(body: Buffer | string | false | undefined): this;
    withHttpStatusHandler(statusCode: HttpStatusCode, handler: HttpStatusHandler): this;
    withHttpStatusHandlers(httpStatusHandlers: HttpStatusHandlers): this;
    json<T>(): Promise<T>;
    blob(): Promise<Blob>;
    acceptCode(code: HttpStatusCode): this;
    readonly then: Promise<HttpClientResponse>["then"];
    private createConfig;
    static create(fn: RequestExecutionFunction): HttpClientRequestWrapper;
}
export declare class HttpClient implements IHttpClient {
    private readonly fetchImpl;
    private readonly baseUrl;
    private readonly authenticationProvider;
    private readonly defaultParams?;
    private readonly exceptionLogger?;
    private readonly httpStatusHandlers;
    constructor(fetchImpl: Fetch, baseUrl: undefined | string, authenticationProvider: HttpClientAuthenticationProvider, defaultQueryParams?: QueryParamsObject, exceptionLogger?: ExceptionLogger, httpStatusHandlers?: HttpStatusHandlers);
    get(url: string): IHttpClientRequestWrapper;
    post(url: string): IHttpClientRequestWrapper;
    put(url: string): IHttpClientRequestWrapper;
    patch(url: string): IHttpClientRequestWrapper;
    delete(url: string): IHttpClientRequestWrapper;
    private method;
    private appendDefaultRequestHeaders;
    url(path: string, params: undefined | QueryParamsObject): string;
    private checkStatus;
    postSync(url: string, params: undefined | QueryParamsObject, data: {}): IHttpClientResponseSync;
    private makeParamValueString;
    private createQueryString;
    createFetchImpl(): Fetch;
}
export declare class HttpError extends Error {
    readonly statusCode: number;
    constructor(message: string, statusCode: number);
    get message(): string;
}
export declare const getContentType: (response: IHttpClientResponse) => string | null;
export declare const getFileName: (response: IHttpClientResponse) => string | undefined;
export {};
