import { MinLengthArray } from "./TypingUtils";
import { CompareFunc } from "./Comparers";
export declare function reverseArray<T>(array: readonly T[]): IterableIterator<T>;
export declare function toAsyncIterator<T>(generator: () => AsyncGenerator<T>): AsyncIterableIterator<T>;
export declare function findLast<T>(items: Iterable<T>, condition: (item: T) => boolean): T | undefined;
export declare function concat<T>(...iterables: Array<Iterable<T>>): IterableIterator<T>;
export declare function iterateValues<TK extends string | symbol | number, TV>(obj: {
    [key in TK]: TV;
}): Iterable<TV>;
export declare function iterateKeys<TO>(obj: Readonly<TO>): Iterable<Extract<keyof TO, string>>;
export declare function iterateKeys<TV>(obj: Readonly<{
    [key: string]: TV;
}>): Iterable<string>;
export declare function iterateKeys(obj: Readonly<{}>): Iterable<string>;
export declare function iterateEntries<TO>(obj: Readonly<TO>): Iterable<[Extract<keyof TO, string>, TO[keyof TO]]>;
export declare function iterateEntries<TV>(obj: Readonly<{
    [key: string]: TV;
}>): Iterable<[string, TV]>;
export declare function iterateEntries(obj: Readonly<{}>): Iterable<[string, unknown]>;
export declare function count<T>(items: Iterable<T>): number;
export declare class IterUtils {
    static count: typeof count;
    static concat: typeof concat;
    static isEmpty<T>(iterable: Iterable<T>): boolean;
    static groupBy<T, TKey>(iterable: Iterable<T>, selectKey: (item: T) => TKey): Map<TKey, T[]>;
    static sortBy<T>(items: Iterable<T>, selector: (item: T) => string | number): T[];
    static last<T>(items: MinLengthArray<T, 1>): T;
    static last<T>(items: Iterable<T> | readonly T[]): undefined | T;
    static first<T>(items: [T, ...unknown[]]): T;
    static first<T>(items: MinLengthArray<T, 1>): T;
    static first<T>(items: Iterable<T> | readonly T[]): undefined | T;
    static single<T>(items: readonly T[] | Iterable<T>, singleAssertMessage?: string): T;
    static iterateEntries<T>(items: readonly T[] | T[] | Iterable<T>): Iterable<[number, T]>;
    static skipLast<T>(items: Iterable<T>, skipCount: number): Iterable<T>;
    static minBy<T>(items: readonly T[] | T[] | Iterable<T>, compareFunc: CompareFunc<T>): T;
    static minByOrUndefined<T>(items: readonly T[] | T[] | Iterable<T>, compareFunc: CompareFunc<T>): T | undefined;
    static range(itemCount: number): Iterable<number>;
    static range(start: number, itemCount: number): Iterable<number>;
    /**
     * @summary Вставляет между элементами массива заданный элемент и возвращает новый массив
     */
    static intersperse<TI, TS>(items: TI[], item: TS): Array<TI | TS>;
    /**
     * @summary Вставляет между элементами массива элемент, создаваемый функцией, и возвращает новый массив
     */
    static intersperseFn<TI, TS>(items: TI[], item: (nextItem: TI, index: number) => TS): Array<TI | TS>;
    static zip<T1, T2>(items1: Iterable<T1>, items2: Iterable<T2>): Iterable<[T1 | undefined, T2 | undefined]>;
    static distinctBy<T>(items: Iterable<T>, idSelector: (item: T) => string): Iterable<T>;
    static reduce<T, U>(items: Iterable<T>, reducer: (accumulator: U, item: T, index: number) => U, initialValue: U): U;
    static batch<T>(items: Iterable<T>, batchSize: number): Iterable<Iterable<T>>;
}
