"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KCLangRuntimeUtils = void 0;
const big_js_1 = __importDefault(require("big.js"));
const ValidationHelper_1 = require("../Engine/src/Engine/ValidationHelper");
const AutocalcCommonFunctions_1 = require("../Engine/src/Helpers/AutocalcCommonFunctions");
const DateHelpers_1 = require("../Engine/src/Helpers/DateHelpers");
const TypingUtils_1 = require("./TypingUtils");
const isBigSource = (value) => {
    if (typeof value === "number" || typeof value === "string" || value instanceof big_js_1.default) {
        return true;
    }
    return false;
};
const isBigOrNumber = (value) => typeof value === "number" || value instanceof big_js_1.default;
class KCLangRuntimeUtils {
    static kcLangUtilsName = "kcLangUtils";
    and(left, right) {
        if (left == undefined || right == undefined) {
            return undefined;
        }
        return left && right;
    }
    division(left, right) {
        if (isBigSource(left) && isBigSource(right)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).div(right);
        }
        return undefined;
    }
    eq(left, right) {
        if (typeof left === "string" && typeof right === "string") {
            return left === right;
        }
        if (isBigSource(left) && isBigSource(right)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).cmp(right) === 0;
        }
        return false;
    }
    ge(left, right) {
        if (isBigSource(left) && isBigSource(right)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).gte(right);
        }
        return false;
    }
    gt(left, right) {
        if (isBigSource(left) && isBigSource(right)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).gt(right);
        }
        return false;
    }
    le(left, right) {
        if (isBigSource(left) && isBigSource(right)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).lte(right);
        }
        return false;
    }
    lt(left, right) {
        if (isBigSource(left) && isBigSource(right)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).lt(right);
        }
        return false;
    }
    minus(argument) {
        if (isBigSource(argument)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(argument).mul(-1);
        }
        return undefined;
    }
    multiply(left, right) {
        if (isBigSource(left) && isBigSource(right)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).mul(right);
        }
        return undefined;
    }
    not(left) {
        return left != undefined ? !left : undefined;
    }
    noteq(left, right) {
        return !this.eq(left, right);
    }
    or(left, right) {
        if (typeof left == "boolean" || typeof right == "boolean") {
            return left || right;
        }
        return undefined;
    }
    round(value, fractionDigits) {
        if (isBigSource(value)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value).round(fractionDigits);
        }
        return undefined;
    }
    floor(value) {
        if (isBigSource(value)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value).round(undefined, 0));
        }
        return undefined;
    }
    getYear(value) {
        return new Date(value)?.getFullYear();
    }
    dateTime(value) {
        const date = (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, DateHelpers_1.convertDate)(value) : undefined;
        if (!date) {
            return undefined;
        }
        return date;
    }
    sum(left, right) {
        if (!isBigOrNumber(left) && !isBigOrNumber(right)) {
            return undefined;
        }
        if (isBigSource(left) && isBigSource(right)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).add(right);
        }
        if (isBigSource(left)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left);
        }
        if (isBigSource(right)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(right);
        }
        return undefined;
    }
    abs(argument) {
        if (isBigSource(argument)) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(argument).abs();
        }
        return undefined;
    }
    xabs(argument) {
        if (isBigSource(argument)) {
            const value = (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(argument);
            return value.lt(0) ? new big_js_1.default(0) : value;
        }
        return undefined;
    }
    length(argument) {
        if (typeof argument === "string") {
            return argument.length;
        }
        return undefined;
    }
    exists(valueSelector, path) {
        if (valueSelector == undefined) {
            return undefined;
        }
        const value = valueSelector(path);
        return value != undefined && value != "";
    }
    isSnils(valueSelector, path) {
        if (valueSelector == undefined) {
            return undefined;
        }
        const value = valueSelector(path);
        const normalizedSnils = value?.replace(/[^0-9]/g, "");
        if ((0, TypingUtils_1.isNullOrWhiteSpace)(normalizedSnils)) {
            return false;
        }
        const result = ValidationHelper_1.ValidationHelper.isSnils(normalizedSnils);
        return result;
    }
    isRegNumSfr(valueSelector, path) {
        if (valueSelector == undefined) {
            return undefined;
        }
        const value = valueSelector(path);
        const normalizedRegNum = value?.replace(/[^0-9]/g, "");
        if ((0, TypingUtils_1.isNullOrWhiteSpace)(normalizedRegNum)) {
            return false;
        }
        const result = ValidationHelper_1.ValidationHelper.isRegNumSfr(normalizedRegNum);
        return result;
    }
    isInn(valueSelector, path) {
        if (valueSelector == undefined) {
            return undefined;
        }
        const value = valueSelector(path);
        const normalizedSnils = value?.replace(/[^0-9]/g, "");
        if ((0, TypingUtils_1.isNullOrWhiteSpace)(normalizedSnils)) {
            return false;
        }
        const result = ValidationHelper_1.ValidationHelper.isInn(normalizedSnils);
        return result;
    }
    isOgrn(valueSelector, path) {
        if (valueSelector == undefined) {
            return undefined;
        }
        const value = valueSelector(path);
        const normalizedOgrn = value?.replace(/[^0-9]/g, "");
        if ((0, TypingUtils_1.isNullOrWhiteSpace)(normalizedOgrn)) {
            return false;
        }
        const result = ValidationHelper_1.ValidationHelper.isOgrn(normalizedOgrn);
        return result;
    }
    isValidAccountNumber(valueSelector, bik, account) {
        if (valueSelector == undefined) {
            return undefined;
        }
        const bikValue = valueSelector(bik);
        const accountValue = valueSelector(account);
        if ((0, TypingUtils_1.isNullOrWhiteSpace)(bikValue) || (0, TypingUtils_1.isNullOrWhiteSpace)(accountValue)) {
            return false;
        }
        const result = ValidationHelper_1.ValidationHelper.isValidAccountNumber(bikValue, accountValue);
        return result;
    }
    isValidMirCardNumber(valueSelector, path) {
        if (valueSelector == undefined) {
            return undefined;
        }
        const value = valueSelector(path);
        if ((0, TypingUtils_1.isNullOrWhiteSpace)(value)) {
            return false;
        }
        const result = ValidationHelper_1.ValidationHelper.isValidMirCardNumber(value);
        return result;
    }
    convertToDecimal(argument) {
        return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(argument);
    }
    convertToString(argument) {
        if (typeof argument === "string") {
            return argument;
        }
        if (typeof argument === "number") {
            return argument.toString();
        }
        if (argument instanceof big_js_1.default) {
            return argument.toString();
        }
        return undefined;
    }
    regexMatch(pattern, value) {
        if (pattern == undefined || value == undefined) {
            return undefined;
        }
        if (typeof value === "string") {
            return new RegExp(pattern).test(value);
        }
        return undefined;
    }
    oneof(values, value) {
        if (values == undefined) {
            return undefined;
        }
        if (typeof value === "string") {
            return values.indexOf(value) !== -1;
        }
        if (isBigOrNumber(value)) {
            const valueAsBig = new big_js_1.default(value);
            return values.some(item => isBigOrNumber(item) && valueAsBig.eq(item));
        }
        return false;
    }
    substring(str, start, length) {
        if (str == undefined || start == undefined || length == undefined) {
            return undefined;
        }
        if (length <= 0) {
            return "";
        }
        const end = start + length;
        return str.substring(start, end);
    }
    dateNow() {
        return new Date().toLocaleDateString("ru-RU");
    }
}
exports.KCLangRuntimeUtils = KCLangRuntimeUtils;
