"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbsoluteModelFieldPath = void 0;
const CoreTypes_1 = require("../../Engine/src/Engine/Core/CoreTypes");
const hashUtils_1 = require("../hashUtils");
const TypingUtils_1 = require("../TypingUtils");
const IterableUtils_1 = require("../IterableUtils");
const EachCurrentCollision_1 = require("./EachCurrentCollision");
const ModelPath_1 = require("./ModelPath");
class AbsoluteModelFieldPath {
    static extraFieldNames = ["\\$errorCount", "\\$warningCount"];
    static fieldExtractRegex = new RegExp(`^(.*)\\.(${AbsoluteModelFieldPath.extraFieldNames
        .concat(...(0, CoreTypes_1.getKnownViewModelFieldNames)().values())
        .join("|")})$`);
    path;
    field;
    constructor(path, fieldName) {
        this.path = path;
        this.field = fieldName;
    }
    getHash() {
        return this.path.getHash() + (0, hashUtils_1.getSimpleHashByString)(this.field);
    }
    isEquals(modelPath, eachCurrentCollision = EachCurrentCollision_1.EachCurrentCollision.AreSame) {
        return this.path.isEquals(modelPath.path, eachCurrentCollision) && this.field === modelPath.field;
    }
    static createFromString(pathWithFieldName, defaultFieldName) {
        return this.createFrom(pathWithFieldName, path => (0, ModelPath_1.createAbsolute)(path), defaultFieldName);
    }
    static createFromMask(pathWithFieldName, multiplicitySymbol, defaultFieldName) {
        return this.createFrom(pathWithFieldName, path => (0, ModelPath_1.createAbsoluteFromMask)(path, multiplicitySymbol), defaultFieldName);
    }
    static createFrom(pathWithFieldName, modelPathFactory, defaultFieldName = "value") {
        const match = AbsoluteModelFieldPath.fieldExtractRegex.exec(pathWithFieldName);
        const path = match != undefined ? (match[1] ?? (0, TypingUtils_1.reject)()) : pathWithFieldName;
        const field = match != undefined ? (match[2] ?? (0, TypingUtils_1.reject)()) : defaultFieldName;
        return new AbsoluteModelFieldPath(modelPathFactory(path), field);
    }
    toString(includeLeadingSlash = false) {
        return `${includeLeadingSlash ? "/" : ""}${this.path.toLegacyPath()}.${this.field}`;
    }
    toLegacyPath() {
        const lastToken = IterableUtils_1.IterUtils.last(this.path.getPathParts());
        const isLastTokenAnInstanceOrMultiplicity = ModelPath_1.PathTokens.isInstanceTokenOrInstancesSpec(lastToken) || ModelPath_1.PathTokens.isMultiToken(lastToken);
        const slashIfRequired = isLastTokenAnInstanceOrMultiplicity && (0, TypingUtils_1.isNotNullOrEmpty)(this.field) ? "/" : "";
        const dotIfRequired = (0, TypingUtils_1.isNotNullOrEmpty)(this.field) ? "." : "";
        return `${this.path.toLegacyPath()}${slashIfRequired}${dotIfRequired}${this.field}`;
    }
}
exports.AbsoluteModelFieldPath = AbsoluteModelFieldPath;
