"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildIterationSequence = buildIterationSequence;
exports.buildIterationSequenceWithInstances = buildIterationSequenceWithInstances;
const ModelPath_1 = require("./ModelPath");
function buildIterationSequence(modelPath) {
    const result = [];
    const tokens = modelPath.getPathPartsAsArray();
    const createPathFunc = modelPath.isAbsolute() ? ModelPath_1.createAbsoluteFromTokens : ModelPath_1.createRelativeFromTokens;
    for (let i = 0; i < tokens.length; i++) {
        if (tokens[i] === ModelPath_1.PathTokens.each) {
            result.push({
                iterable: createPathFunc(tokens.slice(0, i)),
                suffix: (0, ModelPath_1.createRelativeFromTokens)(tokens.slice(i + 1)),
            });
        }
    }
    return result;
}
function buildIterationSequenceWithInstances(modelPath) {
    const result = [];
    const tokens = modelPath.getPathPartsAsArray();
    const createPathFunc = modelPath.isAbsolute() ? ModelPath_1.createAbsoluteFromTokens : ModelPath_1.createRelativeFromTokens;
    let currentInstances = [];
    for (let i = 0; i < tokens.length; i++) {
        const token = tokens[i];
        if (token === ModelPath_1.PathTokens.each || ModelPath_1.PathTokens.isInstanceToken(token)) {
            currentInstances = currentInstances.concat(token);
            result.push({
                iterable: createPathFunc(tokens.slice(0, i).map(x => (ModelPath_1.PathTokens.isInstanceToken(x) ? ModelPath_1.PathTokens.each : x))),
                suffix: (0, ModelPath_1.createRelativeFromTokens)(tokens.slice(i + 1)),
                instances: currentInstances,
            });
        }
    }
    return result;
}
