"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Map2 = void 0;
const TypingUtils_1 = require("../../TypingUtils");
const IterableUtils_1 = require("../../IterableUtils");
class Map2 {
    hashMap = new Map();
    constructor(iterable) {
        if (iterable != undefined) {
            for (const [key, value] of iterable) {
                this.set(key, value);
            }
        }
    }
    set(key, value) {
        this.setOrUpdate(key, () => value, () => value);
    }
    getOrSet(key, defaultValueFactory) {
        return this.setOrUpdate(key, defaultValueFactory, (k, v) => v);
    }
    get(key) {
        return this.getEntryOrUndefined(key)?.[1];
    }
    has(key) {
        return this.getEntryOrUndefined(key) != undefined;
    }
    remove(key) {
        const hash = this.getKeyHash(key);
        const list = this.hashMap.get(hash);
        if (list != undefined) {
            const removingEntry = Iterator.from(IterableUtils_1.IterUtils.iterateEntries(list)).find(([index, [itemKey, itemValue]]) => this.areKeyEqual(itemKey, key));
            if (removingEntry != undefined) {
                list.splice(removingEntry[0], 1);
                return true;
            }
        }
        return false;
    }
    get isEmpty() {
        return this.hashMap.size === 0;
    }
    /**
     * Обновляет значение в словаре по ключю, если значения нет -- устнавит новое
     * @param key ключ словаря для робновления
     * @param newValueFactory Если значения нет -- будет установлен результат этой функции
     * @param updateValueFactory Если значение есть -- будет установлен результат этой функции
     */
    setOrUpdate(key, newValueFactory, updateValueFactory) {
        const hash = this.getKeyHash(key);
        const list = this.hashMap.get(hash);
        if (list == undefined) {
            const result = newValueFactory(key);
            this.hashMap.set(hash, [[key, result]]);
            return result;
        }
        else {
            const itemIndex = list.findIndex(x => this.areKeyEqual(x[0], key));
            if (itemIndex == -1) {
                const result = newValueFactory(key);
                list.push([key, result]);
                return result;
            }
            else {
                const entry = list[itemIndex] ?? (0, TypingUtils_1.reject)();
                entry[1] = updateValueFactory(key, entry[1]);
                return entry[1];
            }
        }
    }
    keys() {
        return Iterator.from(this.entries()).map(e => e[0]);
    }
    values() {
        return Iterator.from(this.entries()).map(e => e[1]);
    }
    entries() {
        return Iterator.from(this.hashMap.values()).flatMap(x => x);
    }
    clear() {
        this.hashMap.clear();
    }
    getKeyHash(key) {
        return key.getHash();
    }
    areKeyEqual(left, right) {
        return left.isEquals(right);
    }
    getEntryOrUndefined(key) {
        const hash = this.getKeyHash(key);
        return this.hashMap.get(hash)?.find(x => this.areKeyEqual(x[0], key));
    }
}
exports.Map2 = Map2;
