import { BindingPath } from "../../Engine/src/Engine/Core/CoreTypes";
import { EachCurrentCollision } from "./EachCurrentCollision";
import { PathMatching } from "./PathMatcher";
import { IEquatable } from "./IEquatable";
declare const EachSymbol: unique symbol;
declare const CurrentSymbol: unique symbol;
export declare class PathTokens {
    static each: typeof EachSymbol;
    static current: typeof CurrentSymbol;
    static isMultiToken(part: undefined | PathToken): part is typeof EachSymbol | typeof CurrentSymbol;
    static isSimpleToken(part: undefined | PathToken): part is string;
    static isInstanceToken(part: undefined | PathToken): part is string;
    static isInstanceTokenOrInstancesSpec(part: undefined | PathToken): part is string;
}
export type SimplePathToken = string;
export type PathToken = SimplePathToken | typeof EachSymbol | typeof CurrentSymbol;
export declare function isModelPath(path: BindingPath | ModelPath | unknown): path is ModelPath;
export declare function convertToLegacyPath(path: BindingPath | ModelPath): BindingPath;
export declare function convertToAbsoluteModelPath<R extends boolean, A extends boolean>(path: BindingPath | IModelPath<R, A>): IModelPath<R, true>;
export declare function convertToRelativeModelPath<R extends boolean, A extends boolean>(path: BindingPath | IModelPath<R, A>): IModelPath<R, false>;
export declare function emptyModelPath(): RelativeResolvedModelPath;
export declare function emptyAbsoluteModelPath(): AbsoluteResolvedModelPath;
export declare function createModelPath(path: string, absolute: true): AbsoluteResolvedModelPath;
export declare function createModelPath(path: string, absolute: false): RelativeResolvedModelPath;
export declare function createModelPath(path: string, absolute: "auto"): ResolvedModelPath;
export declare function createFromMask(path: string, absolute: true, multiplicitySymbol: PathToken): AbsoluteModelPath;
export declare function createFromMask(path: string, absolute: false, multiplicitySymbol: PathToken): ResolvedModelPath;
export declare function createFromMask(path: string, absolute: "auto", multiplicitySymbol: PathToken): ModelPath;
export declare function createAbsoluteFromMask(path: string, multiplicitySymbol: PathToken): AbsoluteModelPath;
export declare function createRelativeFromMask(path: string, multiplicitySymbol: PathToken): RelativeModelPath;
export declare function createAbsolute(path: string): AbsoluteResolvedModelPath;
export declare function createRelativeResolvedModelPath(path: string): RelativeResolvedModelPath;
export declare function createAbsoluteFromTokens(tokens: PathToken[]): AbsoluteModelPath;
export declare function createAbsoluteResolvedFromTokens(tokens: readonly string[]): AbsoluteResolvedModelPath;
export declare function createRelativeFromTokens(tokens: readonly string[]): RelativeResolvedModelPath;
export declare function createRelativeFromTokens(tokens: readonly PathToken[]): RelativeModelPath;
export declare function joinModelPaths(...parts: Array<ModelPath | PathToken | PathToken[]>): ModelPath;
export interface IModelPath<TResolved extends boolean, TAbsolute extends boolean> extends IEquatable<IModelPath<boolean, boolean>> {
    readonly resolved: TResolved;
    readonly absolute: TAbsolute;
    isEquals(modelPath: ModelPath, eachCurrentCollision?: EachCurrentCollision): boolean;
    getHash(eachCurrentCollision?: EachCurrentCollision): number;
    isResolved(): this is IModelPath<true, TAbsolute>;
    isAbsolute(): this is IModelPath<TResolved, true>;
    toAbsolute(): IModelPath<TResolved, true>;
    joinWith(...path: string[]): IModelPath<TResolved, TAbsolute>;
    joinWith(...path: PathToken[]): IModelPath<boolean, TAbsolute>;
    joinWith(...path: Array<ResolvedModelPath | string>): IModelPath<TResolved, TAbsolute>;
    joinWith(...path: Array<AbsoluteResolvedModelPath | string>): IModelPath<TResolved, true>;
    joinWith(...path: Array<ModelPath | PathToken>): IModelPath<boolean, TAbsolute>;
    getPathParts(): Iterable<PathTokenSpec<TResolved>>;
    getPathPartsAsArray(): ReadonlyArray<PathTokenSpec<TResolved>>;
    getSimpleTokensStrict(): Iterable<SimplePathToken>;
    getSimpleTokensAsArrayStrict(): readonly SimplePathToken[];
    resolveAll(instance: string): ModelPath;
    isContainIteration(): boolean;
    isContainCurrentIteration(): boolean;
    isContainCurrentOrEachIteration(): boolean;
    getLastListPath(): PathToken[];
    getPathWithoutLastIteration(): ModelPath;
    getLastIndexOfCurrentOrEachIteration(): number | -1;
    isEndsWithIteration(): boolean;
    isEndsWithCurrentIteration(): boolean;
    isIncludes(modelPath: ModelPath): boolean;
    getIterationRank(eachCurrentCollision?: EachCurrentCollision): number;
    isEmpty(): boolean;
    matchFor(modelPath: ModelPath): PathMatching;
    startsWith(modelPath: ModelPath): boolean;
    endsWith(modelPath: ModelPath): boolean;
    splitByMultiplicity(): Iterable<ResolvedModelPath | typeof EachSymbol | typeof CurrentSymbol>;
    normalize(): IModelPath<TResolved, TAbsolute>;
    toCurrentIteration(): IModelPath<TResolved, TAbsolute>;
    toEachIteration(): IModelPath<TResolved, TAbsolute>;
    getAllParentPaths(): Iterable<ModelPath>;
    trimLastStarIfLastToken(): IModelPath<TResolved, TAbsolute>;
    trimFirstStarIfLastToken(): IModelPath<TResolved, TAbsolute>;
    getSuffixPathFrom(modelFieldPath: ModelPath): RelativeModelPath;
    getParentPath(): IModelPath<TResolved, TAbsolute>;
    applyInstancesDirty(instances: SimplePathToken[], fillWithMultiplicityIfHasNoInstance?: boolean): IModelPath<TResolved, TAbsolute>;
    getFirstMultiplicityInstanceNumber(): number;
    toString(): string;
    toLegacyPath(): string;
}
export type ModelPath = IModelPath<boolean, boolean>;
export type ResolvedModelPath = IModelPath<true, boolean>;
export type AbsoluteModelPath = IModelPath<boolean, true>;
export type AbsoluteResolvedModelPath = IModelPath<true, true>;
export type RelativeModelPath = IModelPath<boolean, false>;
export type RelativeResolvedModelPath = IModelPath<true, false>;
type PathTokenSpec<TResolved extends boolean> = TResolved extends true ? string : PathToken;
type ReadonlyPathTokensSpec<TResolved extends boolean> = ReadonlyArray<PathTokenSpec<TResolved>>;
export declare class ModelPathImpl<TResolved extends boolean, TAbsolute extends boolean> implements IEquatable<ModelPath>, IModelPath<TResolved, TAbsolute> {
    private readonly tokens;
    readonly absolute: TAbsolute;
    private legacyPath;
    private cachedHash;
    constructor(pathParts: ReadonlyPathTokensSpec<TResolved>, absolute: TAbsolute);
    get resolved(): TResolved;
    getPathParts(): Iterable<PathTokenSpec<TResolved>>;
    getPathPartsAsArray(): ReadonlyArray<PathTokenSpec<TResolved>>;
    getSimpleTokensStrict(): Iterable<SimplePathToken>;
    getSimpleTokensAsArrayStrict(): readonly SimplePathToken[];
    joinWith(...path: string[]): IModelPath<TResolved, TAbsolute>;
    joinWith(...path: PathToken[]): IModelPath<boolean, TAbsolute>;
    joinWith(...path: Array<ResolvedModelPath | string>): IModelPath<TResolved, TAbsolute>;
    joinWith(...path: Array<AbsoluteResolvedModelPath | string>): IModelPath<TResolved, true>;
    joinWith(...path: Array<ModelPath | PathToken>): IModelPath<boolean, TAbsolute>;
    getFirstMultiplicityInstanceNumber(): number;
    isResolved(): this is IModelPath<true, TAbsolute>;
    resolveAll(instance: string): ModelPath;
    isContainIteration(): boolean;
    isContainCurrentIteration(): boolean;
    isContainCurrentOrEachIteration(): boolean;
    getLastListPath(): PathToken[];
    getPathWithoutLastIteration(): ModelPath;
    getLastIndexOfCurrentOrEachIteration(): number | -1;
    isEndsWithIteration(): boolean;
    isEndsWithCurrentIteration(): boolean;
    isAbsolute(): this is IModelPath<TResolved, true>;
    toString(): string;
    toLegacyPath(): string;
    isEquals(modelPath: ModelPath, eachCurrentCollision?: EachCurrentCollision): boolean;
    /**
     * Всегда целое положительное беззнаковое 32-битное число
     */
    getHash(eachCurrentCollision?: EachCurrentCollision): number;
    isIncludes(modelPath: ModelPath): boolean;
    getIterationRank(eachCurrentCollision?: EachCurrentCollision): number;
    isEmpty(): boolean;
    matchFor(modelPath: ModelPath): PathMatching;
    startsWith(modelPath: ModelPath): boolean;
    endsWith(modelPath: ModelPath): boolean;
    splitByMultiplicity(): Iterable<ResolvedModelPath | typeof EachSymbol | typeof CurrentSymbol>;
    normalize(): IModelPath<TResolved, TAbsolute>;
    /**
     * @summary Спецификаторы множественности Each заменяет на Current
     */
    toCurrentIteration(): IModelPath<TResolved, TAbsolute>;
    toEachIteration(): IModelPath<TResolved, TAbsolute>;
    toAbsolute(): IModelPath<TResolved, true>;
    getAllParentPaths(): Iterable<ModelPath>;
    trimLastStarIfLastToken(): IModelPath<TResolved, TAbsolute>;
    trimFirstStarIfLastToken(): IModelPath<TResolved, TAbsolute>;
    getSuffixPathFrom(modelFieldPath: ModelPath): RelativeModelPath;
    getParentPath(): IModelPath<TResolved, TAbsolute>;
    applyInstancesDirty(instances: SimplePathToken[], fillWithMultiplicityIfHasNoInstance?: boolean): IModelPath<TResolved, TAbsolute>;
    private pathPartToString;
}
export {};
