"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelPathBuilder = void 0;
const ModelPath_1 = require("./ModelPath");
class ModelPathBuilder {
    parts;
    constructor(forceAbsolute = false) {
        this.parts = forceAbsolute ? [(0, ModelPath_1.emptyAbsoluteModelPath)()] : [];
    }
    get empty() {
        return this.parts.length === 0;
    }
    append(pathPart) {
        if (Array.isArray(pathPart)) {
            this.parts.push(...pathPart);
        }
        else {
            this.parts.push(pathPart);
        }
        return this;
    }
    toPath() {
        return (0, ModelPath_1.joinModelPaths)(...this.parts);
    }
    toResolvedPath() {
        const result = this.toPath();
        if (!result.isResolved()) {
            throw new Error("Path cannot be casted to resolved");
        }
        return result;
    }
    toAbsolutePath() {
        return this.toPath().toAbsolute();
    }
    toString() {
        return this.toPath().toString();
    }
}
exports.ModelPathBuilder = ModelPathBuilder;
