"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAggregationPath = isAggregationPath;
exports.getMask = getMask;
exports.getMaskWithInstances = getMaskWithInstances;
exports.replaceInstancesWithMultiplicity = replaceInstancesWithMultiplicity;
exports.checkIsReservedPath = checkIsReservedPath;
const PathConstants_1 = require("../PathConstants");
const ModelPath_1 = require("./ModelPath");
function isAggregationPath(modelPath) {
    const tokens = modelPath.getPathPartsAsArray();
    const lastToken = tokens[tokens.length - 1];
    return lastToken === "Sum" || lastToken === "Count";
}
function getMask(exactPath) {
    return (0, ModelPath_1.createAbsoluteFromTokens)(Iterator.from(exactPath.getPathParts())
        .map(x => (ModelPath_1.PathTokens.isInstanceToken(x) ? ModelPath_1.PathTokens.each : x))
        .toArray());
}
function getMaskWithInstances(exactPath) {
    const reduceResult = Iterator.from(exactPath.getPathParts()).reduce((result, token) => {
        if (ModelPath_1.PathTokens.isInstanceToken(token) || ModelPath_1.PathTokens.isMultiToken(token)) {
            result.instances.push(token);
        }
        result.tokens.push(ModelPath_1.PathTokens.isInstanceToken(token) ? ModelPath_1.PathTokens.each : token);
        return result;
    }, {
        tokens: [],
        instances: [],
    });
    return {
        mask: (0, ModelPath_1.createAbsoluteFromTokens)(reduceResult.tokens),
        instances: reduceResult.instances,
    };
}
function replaceInstancesWithMultiplicity(path, replaceToken = ModelPath_1.PathTokens.each) {
    const isAbsolute = path.isAbsolute();
    const resultTokens = path.getPathPartsAsArray().map(x => (ModelPath_1.PathTokens.isInstanceToken(x) ? replaceToken : x));
    return isAbsolute ? (0, ModelPath_1.createAbsoluteFromTokens)(resultTokens) : (0, ModelPath_1.createRelativeFromTokens)(resultTokens);
}
function checkIsReservedPath(path) {
    return PathConstants_1.reservedForTechnicalUsageFields.some(field => path.endsWith((0, ModelPath_1.createModelPath)(field, false)));
}
