"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathMatcher = exports.MatchKind = void 0;
const ModelPath_1 = require("./ModelPath");
const ModelPathBuilder_1 = require("./ModelPathBuilder");
var MatchKind;
(function (MatchKind) {
    MatchKind["MatchedPath"] = "MatchedPath";
    MatchKind["DifferentSuffix"] = "DifferentSuffix";
    MatchKind["MatchedMultiplicity"] = "MatchedMultiplicity";
})(MatchKind || (exports.MatchKind = MatchKind = {}));
class PathMatcher {
    static match(leftParts, rightParts) {
        const result = [];
        let samePrefix = new ModelPathBuilder_1.ModelPathBuilder(true);
        let differentSuffix;
        const partCount = Math.max(leftParts.length, rightParts.length);
        for (let i = 0; i < partCount; i++) {
            const sourcePart = leftParts[i];
            const targetPart = rightParts[i];
            if (ModelPath_1.PathTokens.isSimpleToken(sourcePart) &&
                ModelPath_1.PathTokens.isSimpleToken(targetPart) &&
                sourcePart === targetPart) {
                samePrefix.append(targetPart);
                continue;
            }
            if (ModelPath_1.PathTokens.isMultiToken(sourcePart) && ModelPath_1.PathTokens.isMultiToken(targetPart)) {
                if (!samePrefix.empty) {
                    result.push({
                        kind: MatchKind.MatchedPath,
                        prefix: samePrefix.toPath(),
                    });
                    samePrefix = new ModelPathBuilder_1.ModelPathBuilder(false);
                }
                result.push({
                    kind: MatchKind.MatchedMultiplicity,
                    source: sourcePart,
                    target: targetPart,
                });
                continue;
            }
            if (ModelPath_1.PathTokens.isMultiToken(sourcePart) && ModelPath_1.PathTokens.isInstanceToken(targetPart)) {
                if (!samePrefix.empty) {
                    result.push({
                        kind: MatchKind.MatchedPath,
                        prefix: samePrefix.toPath(),
                    });
                    samePrefix = new ModelPathBuilder_1.ModelPathBuilder(false);
                }
                result.push({
                    kind: MatchKind.MatchedMultiplicity,
                    source: sourcePart,
                    target: targetPart,
                });
                continue;
            }
            if (ModelPath_1.PathTokens.isInstanceToken(sourcePart) && ModelPath_1.PathTokens.isMultiToken(targetPart)) {
                if (!samePrefix.empty) {
                    result.push({
                        kind: MatchKind.MatchedPath,
                        prefix: samePrefix.toPath(),
                    });
                    samePrefix = new ModelPathBuilder_1.ModelPathBuilder(false);
                }
                result.push({
                    kind: MatchKind.MatchedMultiplicity,
                    source: sourcePart,
                    target: targetPart,
                });
                continue;
            }
            const sourcePartDifference = new ModelPathBuilder_1.ModelPathBuilder(false).append(leftParts.slice(i));
            const targetPartDifference = new ModelPathBuilder_1.ModelPathBuilder(false).append(rightParts.slice(i));
            differentSuffix = {
                kind: MatchKind.DifferentSuffix,
                source: !sourcePartDifference.empty ? sourcePartDifference.toPath() : undefined,
                target: !targetPartDifference.empty ? targetPartDifference.toPath() : undefined,
            };
            break;
        }
        if (!samePrefix.empty) {
            result.push({
                kind: MatchKind.MatchedPath,
                prefix: samePrefix.toPath(),
            });
        }
        if (differentSuffix != undefined) {
            result.push(differentSuffix);
        }
        return result;
    }
}
exports.PathMatcher = PathMatcher;
