/***
 * Строготипизированная замена для Boolean(...)
 * Обычно используется для [...].filter(isNotNullOrUndefined)
 */
export declare function isNotNullOrUndefined<T>(input: T): input is NonNullable<T>;
export declare function stringifyUnknownError(error: unknown): undefined | string;
export declare function ensureNotNullOrUndefined<T>(input: T): asserts input is NonNullable<T>;
export declare function noop(): void;
export type ErrorFirstCallback<T> = (...args: [Error, null | undefined] | [null | undefined, T] | [Error]) => void;
export declare function reject(message?: string | Error): never;
type ArrayOfLength<TItem, TLength extends number, TRest extends TItem[] = []> = TRest extends {
    length: TLength;
} ? TRest : ArrayOfLength<TItem, TLength, [TItem, ...TRest]>;
export type MinLengthArray<TItem, TLength extends number, TRest extends TItem[] = []> = TRest extends {
    length: TLength;
} ? [...TRest, ...TItem[]] : MinLengthArray<TItem, TLength, [...TRest, TItem]>;
export declare function isLengthGreaterThenOrEqual<T, TNum extends number>(items: readonly T[], minLength: TNum): items is MinLengthArray<T, TNum>;
export declare function minLengthOrUndefined<T, TNum extends number>(items: T[], minLength: TNum): undefined | MinLengthArray<T, TNum>;
export declare function ensureIsArray<T>(items: unknown): T[];
export declare function arrayHasLength<T, TNum extends number>(items: readonly T[], length: TNum): items is ArrayOfLength<T, TNum>;
export declare function arrayIsNotEmpty<T>(items: readonly T[]): items is MinLengthArray<T, 1>;
export declare function arrayHasAtLeastOneItem<T>(items: readonly T[]): items is MinLengthArray<T, 1>;
export declare function ensureArrayIsNotEmpty<T>(items: readonly T[], message?: string): asserts items is MinLengthArray<T, 1>;
export declare function ensureArrayHasLength<T, TNum extends number>(items: readonly T[], length: TNum): asserts items is ArrayOfLength<T, TNum>;
export declare function ensureArrayLengthGreaterThenOrEqual<T, TNum extends number>(items: readonly T[], length: TNum): asserts items is ArrayOfLength<T, TNum>;
export declare function ensureNever(neverArgument: never): void;
export declare function ensureAndGetString(neverArgument: never): string;
export declare function runAsyncAction(action: () => Promise<void>): void;
export declare function isNullOrEmpty(input: string | undefined): input is undefined;
export declare function isNotNullOrEmpty(input: string | undefined): input is string;
export declare function isNullOrWhiteSpace(input: string | undefined | null): input is undefined | null;
export declare function trimLastSlash(url: string): string;
export declare function assertIs<T>(value: T): void;
export {};
