"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isNotNullOrUndefined = isNotNullOrUndefined;
exports.stringifyUnknownError = stringifyUnknownError;
exports.ensureNotNullOrUndefined = ensureNotNullOrUndefined;
exports.noop = noop;
exports.reject = reject;
exports.isLengthGreaterThenOrEqual = isLengthGreaterThenOrEqual;
exports.minLengthOrUndefined = minLengthOrUndefined;
exports.ensureIsArray = ensureIsArray;
exports.arrayHasLength = arrayHasLength;
exports.arrayIsNotEmpty = arrayIsNotEmpty;
exports.arrayHasAtLeastOneItem = arrayHasAtLeastOneItem;
exports.ensureArrayIsNotEmpty = ensureArrayIsNotEmpty;
exports.ensureArrayHasLength = ensureArrayHasLength;
exports.ensureArrayLengthGreaterThenOrEqual = ensureArrayLengthGreaterThenOrEqual;
exports.ensureNever = ensureNever;
exports.ensureAndGetString = ensureAndGetString;
exports.runAsyncAction = runAsyncAction;
exports.isNullOrEmpty = isNullOrEmpty;
exports.isNotNullOrEmpty = isNotNullOrEmpty;
exports.isNullOrWhiteSpace = isNullOrWhiteSpace;
exports.trimLastSlash = trimLastSlash;
exports.assertIs = assertIs;
/***
 * Строготипизированная замена для Boolean(...)
 * Обычно используется для [...].filter(isNotNullOrUndefined)
 */
function isNotNullOrUndefined(input) {
    return input != undefined;
}
function stringifyUnknownError(error) {
    // @ts-ignore
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return typeof error === "object" && error?.["message"];
}
function ensureNotNullOrUndefined(input) {
    if (!isNotNullOrUndefined(input)) {
        throw new Error(`Value must be defined`);
    }
}
function noop() {
    // noop
}
function reject(message) {
    if (typeof message === "string" || message == undefined) {
        throw new Error(message ?? "Undefined is not allowed");
    }
    else {
        throw message;
    }
}
function isLengthGreaterThenOrEqual(items, minLength) {
    return items.length >= minLength;
}
function minLengthOrUndefined(items, minLength) {
    if (isLengthGreaterThenOrEqual(items, minLength)) {
        return items;
    }
    return undefined;
}
function ensureIsArray(items) {
    if (!Array.isArray(items)) {
        throw new Error(`Must be an array!`);
    }
    return items;
}
function arrayHasLength(items, length) {
    return items.length === length;
}
function arrayIsNotEmpty(items) {
    return items.length > 0;
}
function arrayHasAtLeastOneItem(items) {
    return items.length > 0;
}
function ensureArrayIsNotEmpty(items, message) {
    if (!arrayIsNotEmpty(items)) {
        throw new Error(message ?? `Array must be not empty`);
    }
}
function ensureArrayHasLength(items, length) {
    if (!arrayHasLength(items, length)) {
        throw new Error(`Array should has length ${length}`);
    }
}
function ensureArrayLengthGreaterThenOrEqual(items, length) {
    if (!isLengthGreaterThenOrEqual(items, length)) {
        throw new Error(`Array must have minimal length ${length}`);
    }
}
function ensureNever(neverArgument) {
    // noop
}
function ensureAndGetString(neverArgument) {
    return `${neverArgument}`;
}
function runAsyncAction(action) {
    // eslint-disable-next-line @typescript-eslint/no-floating-promises
    action();
}
function isNullOrEmpty(input) {
    return input == undefined || input === "";
}
function isNotNullOrEmpty(input) {
    return !isNullOrEmpty(input);
}
function isNullOrWhiteSpace(input) {
    return input == undefined || input.trim() === "";
}
function trimLastSlash(url) {
    return url.slice(-1) === "/" ? url.slice(0, -1) : url;
}
function assertIs(value) {
    // noop
}
