"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.unitWithoutInstance = unitWithoutInstance;
exports.isUnitIdEquals = isUnitIdEquals;
exports.sectionNameToUnitId = sectionNameToUnitId;
exports.sectionNameToUnitIdForNavigation = sectionNameToUnitIdForNavigation;
exports.getAllUnitIdInstances = getAllUnitIdInstances;
exports.unitIdToLegacyUnitId = unitIdToLegacyUnitId;
exports.unitIdWithInstance = unitIdWithInstance;
exports.createUnitId = createUnitId;
const areEqual_1 = __importDefault(require("fbjs/lib/areEqual"));
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
const IterableUtils_1 = require("../../../../../../Common/IterableUtils");
function unitWithoutInstance(unitId) {
    if (unitId.instance == undefined) {
        return unitId;
    }
    return {
        ...unitId,
        instance: undefined,
    };
}
function isUnitIdEquals(left, right) {
    if (left == undefined && right == undefined) {
        return true;
    }
    if (left == undefined || right == undefined) {
        return false;
    }
    return (left.pageId === right.pageId &&
        (left.instance ?? undefined) === (right.instance ?? undefined) &&
        (0, areEqual_1.default)(left.parentInstances ?? [], right.parentInstances ?? []));
}
function sectionNameToUnitId(sectionName) {
    const [id, ...instances] = sectionName.split("-");
    if (instances.length === 0) {
        return {
            pageId: id ?? (0, TypingUtils_1.reject)(),
        };
    }
    else if (instances.length === 1) {
        return {
            pageId: id ?? (0, TypingUtils_1.reject)(),
            instance: instances[0] ?? (0, TypingUtils_1.reject)(),
        };
    }
    else {
        return {
            pageId: id ?? (0, TypingUtils_1.reject)(),
            parentInstances: instances.slice(0, -1),
            instance: IterableUtils_1.IterUtils.last(instances) ?? (0, TypingUtils_1.reject)(),
        };
    }
}
function sectionNameToUnitIdForNavigation(sectionName) {
    const [id, ...instances] = sectionName.split("-");
    if (instances.length === 0) {
        return {
            pageId: id ?? (0, TypingUtils_1.reject)(),
        };
    }
    else {
        return {
            pageId: id ?? (0, TypingUtils_1.reject)(),
            instance: instances[0] ?? (0, TypingUtils_1.reject)(),
        };
    }
}
function getAllUnitIdInstances(unitId) {
    return (unitId.parentInstances ?? []).concat(unitId.instance ?? []);
}
function unitIdToLegacyUnitId(unitId) {
    if (unitId == undefined) {
        return undefined;
    }
    return [unitId.pageId, ...(unitId.parentInstances || []), unitId.instance].filter(x => x != undefined).join("-");
}
function unitIdWithInstance(unitId, instance) {
    if (unitId.instance) {
        throw new Error(`${unitId.pageId} already has instance`);
    }
    return { ...unitId, instance: instance };
}
function createUnitId(id, instance) {
    return id == undefined ? undefined : { pageId: id, instance: instance };
}
