"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.crossFitTableMiddleware = crossFitTableMiddleware;
const lodash_debounce_1 = __importDefault(require("lodash.debounce"));
const crossFitTableActions_1 = require("./crossFitTableActions");
function crossFitTableMiddleware() {
    const result = (store) => (next) => {
        let actionBatch = [];
        let isFirstAction = true;
        function flushBatch() {
            const actionBatchToProcess = actionBatch;
            actionBatch = [];
            if (actionBatchToProcess.length > 0) {
                next({
                    type: crossFitTableActions_1.CrossFitTableActionType.ColumnGroupingBatch,
                    payload: { events: actionBatchToProcess },
                });
            }
        }
        const flushTrottled = (0, lodash_debounce_1.default)(flushBatch, 1000);
        const enableThrottling = (0, lodash_debounce_1.default)(() => {
            isFirstAction = false;
        }, 500);
        return (action) => {
            if (typeof action === "function") {
                return next(action);
            }
            if (isFirstAction) {
                const actionResult = next(action);
                enableThrottling();
                return actionResult;
            }
            else {
                if (action.type === crossFitTableActions_1.CrossFitTableActionType.SetColumnGrouping) {
                    actionBatch.push(action);
                    if (action.payload.doNotThrottle) {
                        flushBatch();
                    }
                    else {
                        flushTrottled();
                    }
                    return undefined;
                }
                else if (action.type === crossFitTableActions_1.CrossFitTableActionType.SetHiddenColumnGrouping) {
                    actionBatch.push(action);
                    if (action.payload.doNotThrottle) {
                        flushBatch();
                    }
                    else {
                        flushTrottled();
                    }
                    return undefined;
                }
                else {
                    return next(action);
                }
            }
        };
    };
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return,@typescript-eslint/no-explicit-any
    return result;
}
