"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.clearAllFiltersSessions = exports.safeGetFilter = exports.removeFilter = exports.setFilter = void 0;
const ALL_FILTERS_IDS = "all-filters-ids";
const buildFilterSessionId = (filteredUniqKey, bindingPath) => `filters-${filteredUniqKey}-${bindingPath}`;
const setFilter = (filteredUniqKey, bindingPath, value) => {
    window.sessionStorage.setItem(buildFilterSessionId(filteredUniqKey, bindingPath), value);
    addToAllFiltersSessionIds(filteredUniqKey, bindingPath);
};
exports.setFilter = setFilter;
const removeFilter = (filteredUniqKey, bindingPath) => {
    window.sessionStorage.removeItem(buildFilterSessionId(filteredUniqKey, bindingPath));
    removeFromAllFiltersSessionIds(filteredUniqKey, bindingPath);
};
exports.removeFilter = removeFilter;
const safeGetFilter = (filteredUniqKey, bindingPath) => {
    try {
        return JSON.parse(window.sessionStorage.getItem(buildFilterSessionId(filteredUniqKey, bindingPath)) ?? "undefined");
    }
    catch {
        return undefined;
    }
};
exports.safeGetFilter = safeGetFilter;
const addToAllFiltersSessionIds = (filteredUniqKey, bindingPath) => {
    const filterSessionId = buildFilterSessionId(filteredUniqKey, bindingPath);
    const allFiltersSessionIds = getAllFiltersSessionIds()
        .filter(it => it !== filterSessionId)
        .concat(filterSessionId);
    window.sessionStorage.setItem(ALL_FILTERS_IDS, JSON.stringify(allFiltersSessionIds));
};
const removeFromAllFiltersSessionIds = (filteredUniqKey, bindingPath) => {
    const allFiltersSessionIds = getAllFiltersSessionIds().filter(it => it !== buildFilterSessionId(filteredUniqKey, bindingPath));
    window.sessionStorage.setItem(ALL_FILTERS_IDS, JSON.stringify(allFiltersSessionIds));
};
const getAllFiltersSessionIds = () => {
    try {
        return JSON.parse(window.sessionStorage.getItem(ALL_FILTERS_IDS) ?? "[]");
    }
    catch {
        return [];
    }
};
const clearAllFiltersSessions = () => {
    for (const filterSessionId of getAllFiltersSessionIds()) {
        window.sessionStorage.removeItem(filterSessionId);
    }
    window.sessionStorage.removeItem(ALL_FILTERS_IDS);
};
exports.clearAllFiltersSessions = clearAllFiltersSessions;
