import { ReadonlyModelChunk } from "../../../../Engine/Core/ModelStoring/ModelChunk";
import { SortDataType, SortType } from "../MultiLineTypes";
import { MultiLineFilter } from "../MultiLineFilter";
import { FilterExpression } from "../../../../Helpers/FilterExpressions/FilterExpression";
export declare enum MultiLineActionType {
    AddInstance = "ADD_INSTANCE",
    RemoveInstance = "REMOVE_INSTANCE",
    GoToPage = "GO_TO_PAGE",
    Init = "INIT",
    Detach = "DETACH",
    Sort = "SORT",
    Filter = "FILTER",
    FilterAll = "FILTER_ALL",
    RemoveFilter = "RemoveFilter",
    ResetFilters = "RESET_FILTERS",
    SetLoading = "SET_MULTILINE_LOADING",
    SetFilteredData = "SET_FILTERED_DATA"
}
export type MultiLineAction = {
    type: MultiLineActionType.AddInstance;
    payload: {
        bindingPath: string;
        filteredUniqKey: string;
        data?: ReadonlyModelChunk;
        insertAfterInstance?: string;
        insertBeforeInstance?: string;
        doNotCopyPaths?: string[];
        instance?: string;
    };
} | {
    type: MultiLineActionType.RemoveInstance;
    payload: {
        bindingPath: string;
        indices: readonly string[];
        filteredUniqKey: string;
    };
} | {
    type: MultiLineActionType.GoToPage;
    payload: {
        bindingPath: string;
        page: number;
        filteredUniqKey: string;
    };
} | {
    type: MultiLineActionType.Init;
    payload: {
        bindingPath: string;
        storeFiltersWithinSession: boolean;
        currentPage: number;
        instancesPerPage: number;
        filteredUniqKey: string;
        defaultFilter?: FilterExpression | undefined;
        useServerFilters?: boolean;
    };
} | {
    type: MultiLineActionType.Detach;
    payload: {
        bindingPath: string;
        filteredUniqKey: string;
    };
} | {
    type: MultiLineActionType.SetFilteredData;
    payload: {
        bindingPath: string;
        filteredUniqKey: string;
        instances: string[];
        currentPage: number;
    };
} | {
    type: MultiLineActionType.Sort;
    payload: {
        stickyTablePath: string;
        multilinePath: string;
        dataPath: string;
        dataType: SortDataType;
        dataSave: boolean;
        value: SortType;
        filteredUniqKey: string;
    };
} | {
    type: MultiLineActionType.Filter;
    payload: {
        stickyTablePath: string;
        multilinePath: string;
        multilineFilter: MultiLineFilter;
        filteredUniqKey: string;
    };
} | {
    type: MultiLineActionType.FilterAll;
} | {
    type: MultiLineActionType.RemoveFilter;
    payload: {
        stickyTablePath: string;
        multilinePath: string;
        filterSourceId: string;
        filteredUniqKey: string;
    };
} | {
    type: MultiLineActionType.ResetFilters;
    payload: {
        stickyTablePath: string;
        multilinePath: string;
        filteredUniqKey: string;
    };
} | {
    type: MultiLineActionType.SetLoading;
    payload: {
        bindingPath: string;
        filteredUniqKey: string;
        loading: boolean;
    };
};
