"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultMultiLinePathState = void 0;
exports.multiLineReducer = multiLineReducer;
const FilteringHelpers_1 = require("../../../../Helpers/FilteringHelpers");
const MultilineHelpers_1 = require("../../../../Helpers/StateAccessors/MultilineHelpers");
const MultilineStateSelect_1 = require("../../../../Helpers/StateAccessors/MultilineStateSelect");
const MultiLineFilter_1 = require("../MultiLineFilter");
const multiLineActions_1 = require("./multiLineActions");
const FiltersSessionHelpers_1 = require("./FiltersSessionHelpers");
exports.defaultMultiLinePathState = {
    storeFiltersWithinSession: false,
    currentPage: 1,
    instancesPerPage: 10,
    instances: [],
};
function multiLineReducer(state, action) {
    switch (action.type) {
        case multiLineActions_1.MultiLineActionType.Init: {
            const { bindingPath, storeFiltersWithinSession, currentPage, instancesPerPage, filteredUniqKey, useServerFilters, } = action.payload;
            let filters;
            if (storeFiltersWithinSession) {
                filters = (0, FiltersSessionHelpers_1.safeGetFilter)(filteredUniqKey, bindingPath)?.filter(x => x.filterSourceInfo.type !== MultiLineFilter_1.FilterSourceInfoType.DefaultFilter);
            }
            const instances = (0, MultilineStateSelect_1.getMultilineState)(state.multiLineInfos, bindingPath, filteredUniqKey, exports.defaultMultiLinePathState)?.instances;
            const nextState = {
                ...state,
                multiLineInfos: {
                    ...state.multiLineInfos,
                    [bindingPath]: {
                        ...state.multiLineInfos[bindingPath],
                        [filteredUniqKey]: {
                            storeFiltersWithinSession: storeFiltersWithinSession,
                            currentPage: currentPage,
                            instancesPerPage: instancesPerPage,
                            instances: instances,
                            filters: filters,
                            useServerFilter: useServerFilters,
                        },
                    },
                },
            };
            return nextState;
        }
        case multiLineActions_1.MultiLineActionType.AddInstance: {
            const { bindingPath, insertBeforeInstance, insertAfterInstance, instance, filteredUniqKey = "default", } = action.payload;
            if (!state.multiLineInfos[bindingPath]) {
                return state;
            }
            const { instancesPerPage, filters, sort } = (0, MultilineStateSelect_1.getMultilineState)(state.multiLineInfos, bindingPath, filteredUniqKey, exports.defaultMultiLinePathState);
            const displayedInstances = (0, MultilineHelpers_1.getDisplayedInstances)(state, bindingPath, filteredUniqKey);
            const instanceIndex = insertAfterInstance
                ? displayedInstances.indexOf(insertAfterInstance) + 1
                : insertBeforeInstance
                    ? displayedInstances.indexOf(insertBeforeInstance)
                    : displayedInstances.length;
            let newInstances = displayedInstances;
            if ((0, FilteringHelpers_1.isNotEmptyFilters)(filters) || sort) {
                newInstances = (0, MultilineHelpers_1.getInstancesAfterAdd)(displayedInstances, instance, instanceIndex);
            }
            const instanceCount = newInstances.length;
            const newCurrentPage = insertAfterInstance
                ? Math.ceil((Number(instanceIndex) + 1) / instancesPerPage)
                : insertBeforeInstance
                    ? Math.ceil(Number(instanceIndex) / instancesPerPage)
                    : Math.ceil(instanceCount / instancesPerPage);
            return {
                ...state,
                multiLineInfos: {
                    ...state.multiLineInfos,
                    [bindingPath]: {
                        ...state.multiLineInfos[bindingPath],
                        [filteredUniqKey]: {
                            ...exports.defaultMultiLinePathState,
                            ...state.multiLineInfos[bindingPath]?.[filteredUniqKey],
                            currentPage: newCurrentPage,
                            instances: newInstances,
                        },
                    },
                },
            };
        }
        case multiLineActions_1.MultiLineActionType.RemoveInstance: {
            const { bindingPath, indices, filteredUniqKey } = action.payload;
            if (!state.multiLineInfos[bindingPath]) {
                return state;
            }
            const { currentPage, instancesPerPage, filters, sort } = (0, MultilineStateSelect_1.getMultilineState)(state.multiLineInfos, bindingPath, filteredUniqKey, exports.defaultMultiLinePathState);
            const displayedInstances = (0, MultilineHelpers_1.getDisplayedInstances)(state, bindingPath, filteredUniqKey);
            let newInstances = displayedInstances;
            if ((0, FilteringHelpers_1.isNotEmptyFilters)(filters) || sort) {
                newInstances = (0, MultilineHelpers_1.getInstancesAfterRemove)(displayedInstances, indices);
            }
            const instanceCount = newInstances.length;
            const isLastInstanceOnCurrentPage = instanceCount - (currentPage - 1) * instancesPerPage <= 0;
            const newCurrentPage = isLastInstanceOnCurrentPage ? Math.max(1, currentPage - 1) : currentPage;
            return {
                ...state,
                multiLineInfos: {
                    ...state.multiLineInfos,
                    [bindingPath]: {
                        ...state.multiLineInfos[bindingPath],
                        [filteredUniqKey]: {
                            ...exports.defaultMultiLinePathState,
                            ...state.multiLineInfos[bindingPath]?.[filteredUniqKey],
                            currentPage: newCurrentPage,
                            instances: newInstances,
                        },
                    },
                },
            };
        }
        case multiLineActions_1.MultiLineActionType.GoToPage: {
            const { page, bindingPath, filteredUniqKey } = action.payload;
            const { instancesPerPage } = (0, MultilineStateSelect_1.getMultilineState)(state.multiLineInfos, bindingPath, filteredUniqKey, exports.defaultMultiLinePathState);
            const displayedInstancesCount = (0, MultilineHelpers_1.getDisplayedInstancesCount)(state, bindingPath, filteredUniqKey);
            const instanceCount = displayedInstancesCount;
            const pageCount = Math.ceil(instanceCount / instancesPerPage);
            const currentPage = Math.max(1, Math.min(page, pageCount));
            return {
                ...state,
                multiLineInfos: {
                    ...state.multiLineInfos,
                    [bindingPath]: {
                        ...state.multiLineInfos[bindingPath],
                        [filteredUniqKey]: {
                            ...exports.defaultMultiLinePathState,
                            ...state.multiLineInfos[bindingPath]?.[filteredUniqKey],
                            currentPage: currentPage,
                        },
                    },
                },
            };
        }
        case multiLineActions_1.MultiLineActionType.Sort: {
            const { multilinePath, dataPath, dataType, value, filteredUniqKey } = action.payload;
            const multilineSort = { dataPath: dataPath, dataType: dataType, value: value };
            return {
                ...state,
                multiLineInfos: {
                    ...state.multiLineInfos,
                    [multilinePath]: {
                        ...state.multiLineInfos[multilinePath],
                        [filteredUniqKey]: {
                            ...exports.defaultMultiLinePathState,
                            ...state.multiLineInfos[multilinePath]?.[filteredUniqKey],
                            sort: multilineSort,
                        },
                    },
                },
            };
        }
        case multiLineActions_1.MultiLineActionType.RemoveFilter: {
            const { multilinePath, filterSourceId, filteredUniqKey } = action.payload;
            const { filters, storeFiltersWithinSession } = (0, MultilineStateSelect_1.getMultilineState)(state.multiLineInfos, multilinePath, filteredUniqKey, exports.defaultMultiLinePathState);
            const updatedFilters = [...(filters ?? [])].filter(x => x.filterSourceId !== filterSourceId);
            if (storeFiltersWithinSession) {
                (0, FiltersSessionHelpers_1.setFilter)(filteredUniqKey, multilinePath, JSON.stringify(filters));
            }
            const result = {
                ...state,
                multiLineInfos: {
                    ...state.multiLineInfos,
                    [multilinePath]: {
                        ...state.multiLineInfos[multilinePath],
                        [filteredUniqKey]: {
                            ...exports.defaultMultiLinePathState,
                            ...state.multiLineInfos[multilinePath]?.[filteredUniqKey],
                            filters: updatedFilters,
                            currentPage: 1,
                        },
                    },
                },
            };
            return result;
        }
        case multiLineActions_1.MultiLineActionType.SetFilteredData: {
            const { bindingPath: multilinePath, filteredUniqKey, instances, currentPage } = action.payload;
            const result = {
                ...state,
                multiLineInfos: {
                    ...state.multiLineInfos,
                    [multilinePath]: {
                        ...state.multiLineInfos[multilinePath],
                        [filteredUniqKey]: {
                            ...exports.defaultMultiLinePathState,
                            ...state.multiLineInfos[multilinePath]?.[filteredUniqKey],
                            instances: instances,
                            currentPage: currentPage,
                            filteringResultVersion: (state.multiLineInfos[multilinePath]?.[filteredUniqKey]?.filteringResultVersion ?? 0) +
                                1,
                        },
                    },
                },
            };
            return result;
        }
        case multiLineActions_1.MultiLineActionType.Filter: {
            const { multilinePath, multilineFilter, filteredUniqKey } = action.payload;
            const { filters, storeFiltersWithinSession } = (0, MultilineStateSelect_1.getMultilineState)(state.multiLineInfos, multilinePath, filteredUniqKey, exports.defaultMultiLinePathState);
            const updatedFilters = filters ? [...filters] : [];
            const index = updatedFilters.findIndex(x => x.filterSourceId === multilineFilter.filterSourceId);
            if (index >= 0) {
                updatedFilters[index] = multilineFilter;
            }
            else {
                updatedFilters.push(multilineFilter);
            }
            if (storeFiltersWithinSession) {
                (0, FiltersSessionHelpers_1.setFilter)(filteredUniqKey, multilinePath, JSON.stringify(filters));
            }
            const result = {
                ...state,
                multiLineInfos: {
                    ...state.multiLineInfos,
                    [multilinePath]: {
                        ...state.multiLineInfos[multilinePath],
                        [filteredUniqKey]: {
                            ...exports.defaultMultiLinePathState,
                            ...state.multiLineInfos[multilinePath]?.[filteredUniqKey],
                            filters: updatedFilters,
                        },
                    },
                },
            };
            return result;
        }
        case multiLineActions_1.MultiLineActionType.ResetFilters: {
            const { multilinePath, filteredUniqKey } = action.payload;
            (0, FiltersSessionHelpers_1.removeFilter)(filteredUniqKey, multilinePath);
            return {
                ...state,
                multiLineInfos: {
                    ...state.multiLineInfos,
                    [multilinePath]: {
                        ...state.multiLineInfos[multilinePath],
                        [filteredUniqKey]: {
                            ...exports.defaultMultiLinePathState,
                            ...state.multiLineInfos[multilinePath]?.[filteredUniqKey],
                            filters: undefined,
                            currentPage: 1,
                        },
                    },
                },
            };
        }
        case multiLineActions_1.MultiLineActionType.SetLoading: {
            const { bindingPath, filteredUniqKey, loading } = action.payload;
            const multilineState = (0, MultilineStateSelect_1.getMultilineState)(state.multiLineInfos, bindingPath, filteredUniqKey);
            if (multilineState == undefined) {
                return state;
            }
            const loadingCounter = multilineState.loadingCounter;
            const nextLoadingCounter = Math.max((loadingCounter ?? 0) + (loading ? 1 : -1), 0);
            return {
                ...state,
                multiLineInfos: {
                    ...state.multiLineInfos,
                    [bindingPath]: {
                        ...state.multiLineInfos[bindingPath],
                        [filteredUniqKey]: {
                            ...exports.defaultMultiLinePathState,
                            ...state.multiLineInfos[bindingPath]?.[filteredUniqKey],
                            loading: nextLoadingCounter > 0,
                            loadingCounter: nextLoadingCounter,
                        },
                    },
                },
            };
        }
        default:
            return state;
    }
}
