"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTestingMode = isTestingMode;
exports.isScreenshotTestingMode = isScreenshotTestingMode;
exports.setTestingMode = setTestingMode;
exports.setExposedForm = setExposedForm;
exports.exposeIntoWindowTestingModeHandleForIntegrationTests = exposeIntoWindowTestingModeHandleForIntegrationTests;
const FakeErrorLog_1 = require("../Engine/Fakes/FakeErrorLog");
const UploadToDriveTask_1 = require("../Engine/Externals/Attachments/Tasks/UploadToDriveTask");
const ITestLogger_1 = require("../Engine/Testing/TestLogging/ITestLogger");
let testingMode = typeof window === "undefined" ? false : Boolean(window.__CANDY_FORMS_TESTING_MODE__);
let screenshotTestingMode = false;
function getCookieValue(name) {
    if (typeof document === "undefined") {
        return undefined;
    }
    const value = "; " + document.cookie;
    const parts = value.split("; " + name + "=");
    if (parts.length === 2) {
        const lastPart = parts.pop();
        if (lastPart == undefined) {
            throw new Error("IvalidProgramState");
        }
        return lastPart.split(";").shift();
    }
    return undefined;
}
const testingModeViaCookies = Boolean(getCookieValue("candy.testing-mode"));
const screenshotTestingModeViaCookies = Boolean(getCookieValue("candy.screenshots-testing-mode"));
function isTestingMode() {
    return testingMode || testingModeViaCookies;
}
function isScreenshotTestingMode() {
    return screenshotTestingMode || screenshotTestingModeViaCookies;
}
function setTestingMode(value) {
    testingMode = value;
    window.document.body.classList.add("testing-mode");
}
let exposedForm;
function setExposedForm(form) {
    exposedForm = form;
}
function exposeIntoWindowTestingModeHandleForIntegrationTests() {
    window.__CANDY_FORMS_ENABLE_TESTING_MODE__ = () => setTestingMode(true);
    window.__CANDY_FORMS_ENABLE_TESTING_MODE_SCREENSHOTS__ = () => {
        screenshotTestingMode = true;
        window.document.body.classList.add("testing-mode");
    };
    window.__CANDY_FORMS_TESTING_MODE__SuspendNextUpload = UploadToDriveTask_1.suspendNextUpload;
    window.__CANDY_FORMS_TESTING_MODE__ResumeLastSuspendedUpload = UploadToDriveTask_1.resumeLastSuspendedUpload;
    window.__CANDY_FORMS_TESTING_MODE__SetFakeErrorLogForDraft = FakeErrorLog_1.setFakeErrorLogForDraft;
    window.__CANDY_FORMS_TESTING_MODE__GetStoredLogItems = ITestLogger_1.getStoredLogItems;
    window.__CANDY_FORMS_TESTING_MODE__FormObject = () => exposedForm;
}
