"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isNodeOrChildrenContainsError = isNodeOrChildrenContainsError;
exports.isNodeOrChildrenContainsNotEmptyValue = isNodeOrChildrenContainsNotEmptyValue;
exports.iterateModelEntries = iterateModelEntries;
exports.iterateViewModels = iterateViewModels;
const ModelPath_1 = require("../../../Common/ModelPath/ModelPath");
const FormViewModel_1 = require("../Engine/Core/Store/FormViewModel");
function isNodeOrChildrenContainsError(node) {
    const viewModel = (0, FormViewModel_1.getViewModel)(node);
    if (viewModel != undefined && viewModel.error != undefined && Array.isArray(viewModel.error)) {
        if (viewModel.error.length > 0) {
            return true;
        }
    }
    return (0, FormViewModel_1.getNestedNodeViewModels)(node).some(([, nestedNode]) => isNodeOrChildrenContainsError(nestedNode));
}
function isNodeOrChildrenContainsNotEmptyValue(object) {
    const viewModel = (0, FormViewModel_1.getViewModel)(object);
    if (viewModel != undefined && Boolean(viewModel.value)) {
        return true;
    }
    for (const [, value] of (0, FormViewModel_1.getNestedNodeViewModels)(object)) {
        if (isNodeOrChildrenContainsNotEmptyValue(value)) {
            return true;
        }
    }
    return false;
}
function* iterateModelEntries(node, path = (0, ModelPath_1.emptyModelPath)()) {
    const viewModel = (0, FormViewModel_1.getViewModel)(node);
    if (viewModel != undefined) {
        yield [path, viewModel];
    }
    for (const [childPathToken, value] of (0, FormViewModel_1.getNestedNodeViewModels)(node)) {
        yield* iterateModelEntries(value, path.joinWith(childPathToken));
    }
}
function iterateViewModels(node) {
    return Iterator.from(iterateModelEntries(node)).map(([_path, viewModel]) => viewModel);
}
