"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LastInstanceBehaviour = void 0;
exports.removeMultilineField = removeMultilineField;
exports.addMultilineField = addMultilineField;
exports.setValues = setValues;
exports.clearModel = clearModel;
exports.setToModel = setToModel;
exports.setToModelBatch = setToModelBatch;
exports.getChildren = getChildren;
exports.setChildrenToModel = setChildrenToModel;
exports.aggregatedSetToModel = aggregatedSetToModel;
exports.copySubtree = copySubtree;
exports.revalidateChildrenPaths = revalidateChildrenPaths;
exports.removeSingleAttachment = removeSingleAttachment;
exports.setColGrouping = setColGrouping;
exports.setHiddenColGrouping = setHiddenColGrouping;
exports.setStateHiddenColGrouping = setStateHiddenColGrouping;
exports.setLoading = setLoading;
exports.init = init;
exports.detach = detach;
exports.setFilteredData = setFilteredData;
exports.sort = sort;
exports.filter = filter;
exports.filterAll = filterAll;
exports.removeFilterAction = removeFilterAction;
exports.resetFilters = resetFilters;
exports.setMultilineLoading = setMultilineLoading;
exports.goToPage = goToPage;
exports.addInstance = addInstance;
exports.copyInstanceAction = copyInstanceAction;
exports.removeInstance = removeInstance;
exports.removeInstances = removeInstances;
exports.fillMultiline = fillMultiline;
const ModelPath_1 = require("../../../../Common/ModelPath/ModelPath");
const crossFitTableActions_1 = require("../../Controls/MultiControls/CrossfitTable/Store/crossFitTableActions");
const MultilineHelpers_1 = require("../../Helpers/StateAccessors/MultilineHelpers");
const ViewModelSelect_1 = require("../../Helpers/StateAccessors/ViewModelSelect");
const ModelHunk_1 = require("../Core/ModelStoring/ModelHunk");
const modelActions_1 = require("../Core/ModelStoring/Store/modelActions");
const PathUtils_1 = require("../Core/PathUtils/PathUtils");
const IterableUtils_1 = require("../../../../Common/IterableUtils");
const multiLineActions_1 = require("../../Controls/MultiControls/MultiLine/Store/multiLineActions");
const SortingHelpers_1 = require("../../Helpers/SortingHelpers");
/*
 * ====================================================
 *
 *          Это свалка -- подлежит разбору!!!
 *
 * ====================================================
 */
function removeMultilineField(bindingPath, index) {
    return (dispatch, getState, form) => {
        const instances = ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
        form.setChildren(bindingPath, (0, MultilineHelpers_1.getInstancesAfterRemove)(instances, [index]));
    };
}
function addMultilineField(bindingPath) {
    return (dispatch, getState, form) => {
        const instances = ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
        const instance = (0, MultilineHelpers_1.findFreeInstanceNumber)(instances);
        const newInstances = (0, MultilineHelpers_1.getInstancesAfterAdd)(instances, instance, instances.length);
        form.setChildren(bindingPath, newInstances);
    };
}
function setValues(values, deleted) {
    return {
        type: modelActions_1.ModelActionType.SetValues,
        payload: { values: values, deleted: deleted },
    };
}
function clearModel() {
    return {
        type: modelActions_1.ModelActionType.ClearModel,
    };
}
function setToModel(bindingPath, field, value) {
    return (dispatch, getState, form) => {
        form.setViewModelField(bindingPath, field, value);
    };
}
function setToModelBatch(modelUpdate) {
    return (dispatch, getState, form) => {
        form.updateModel(model => {
            for (const [path, viewModel] of modelUpdate.iterateEntries()) {
                for (const [fieldName, fieldValue] of (0, IterableUtils_1.iterateEntries)(viewModel)) {
                    model.setViewModelField(path.toAbsolute(), fieldName, fieldValue);
                }
            }
        });
    };
}
function getChildren(bindingPath) {
    return (dispatch, getState, form) => ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
}
function setChildrenToModel(path, childValues, field) {
    return (dispatch, getState, form) => {
        form.updateModel(model => {
            model.setChildren(path, childValues.map((_, index) => `${index}`));
            for (let index = 0; index < childValues.length; index++) {
                model.setViewModelField(PathUtils_1.PathUtils.join([path, index.toString()]), field ?? "value", childValues[index]);
            }
        });
    };
}
function aggregatedSetToModel(bindingPath, field, value) {
    return {
        type: modelActions_1.ModelActionType.AggregatedSetToModel,
        payload: {
            bindingPath: (0, ModelPath_1.convertToLegacyPath)(bindingPath),
            field: field,
            value: value,
        },
    };
}
function copySubtree(form, sourceBindingPath, targetBindingPath, doNotCopyPaths) {
    form.updateModel(model => {
        model.copySubtree(sourceBindingPath, targetBindingPath, doNotCopyPaths);
    });
}
function revalidateChildrenPaths(parentPath) {
    return async (d, s, form) => {
        const iterableChildrenPaths = form.getChildrenPaths(parentPath);
        await form.revalidatePaths(iterableChildrenPaths);
    };
}
function removeSingleAttachment(bindingPath) {
    return (dispatch, getState, form) => {
        form.remove(bindingPath);
    };
}
function setColGrouping(stickyTablePath, groupKey, isShowColumns, doNotThrottle = false) {
    return {
        type: crossFitTableActions_1.CrossFitTableActionType.SetColumnGrouping,
        payload: {
            stickyTablePath: stickyTablePath,
            groupKey: groupKey,
            isShowColumns: isShowColumns,
            doNotThrottle: doNotThrottle,
        },
    };
}
function setHiddenColGrouping(stickyTablePath, groupKey, isShowColumns, doNotThrottle = false) {
    return {
        type: crossFitTableActions_1.CrossFitTableActionType.SetHiddenColumnGrouping,
        payload: {
            stickyTablePath: stickyTablePath,
            groupKey: groupKey,
            isShowColumns: isShowColumns,
            doNotThrottle: doNotThrottle,
        },
    };
}
function setStateHiddenColGrouping(stickyTablePath, groupKey, isShowColumns) {
    return {
        type: crossFitTableActions_1.CrossFitTableActionType.SetStateHiddenColumnGrouping,
        payload: {
            stickyTablePath: stickyTablePath,
            groupKey: groupKey,
            isShowColumns: isShowColumns,
        },
    };
}
function setLoading(stickyTablePath, loading) {
    return {
        type: crossFitTableActions_1.CrossFitTableActionType.SetLoading,
        payload: {
            stickyTablePath: stickyTablePath,
            loading: loading,
        },
    };
}
function init(bindingPath, currentPage, instancesPerPage, storeFiltersWithinSession, filteredUniqKey, enableFilterTracking, useShortDebounce, defaultFilter) {
    return (dispatch, getState, form) => {
        const useServerFilters = form.dataDeclarationHelper.getEntry(bindingPath)?.useServerFilters ?? false;
        dispatch({
            type: multiLineActions_1.MultiLineActionType.Init,
            payload: {
                bindingPath: (0, ModelPath_1.convertToLegacyPath)(bindingPath),
                currentPage: currentPage,
                instancesPerPage: instancesPerPage,
                storeFiltersWithinSession: storeFiltersWithinSession,
                filteredUniqKey: filteredUniqKey,
                defaultFilter: defaultFilter,
                useServerFilters: useServerFilters,
            },
        });
        form.filteringManager.subscribe(bindingPath, filteredUniqKey, useServerFilters, enableFilterTracking, useShortDebounce, defaultFilter);
    };
}
function detach(bindingPath, filteredUniqKey) {
    return (dispatch, getState, form) => {
        dispatch({
            type: multiLineActions_1.MultiLineActionType.Detach,
            payload: {
                bindingPath: (0, ModelPath_1.convertToLegacyPath)(bindingPath),
                filteredUniqKey: filteredUniqKey,
            },
        });
        form.filteringManager.unsubscribe(bindingPath, filteredUniqKey);
    };
}
function setFilteredData(bindingPath, filteredUniqKey, instances, currentPage = 1) {
    return (dispatch, getState, form) => {
        dispatch({
            type: multiLineActions_1.MultiLineActionType.SetFilteredData,
            payload: {
                bindingPath: (0, ModelPath_1.convertToLegacyPath)(bindingPath),
                filteredUniqKey: filteredUniqKey,
                instances: instances,
                currentPage: currentPage,
            },
        });
    };
}
function sort(stickyTablePath, multilinePath, dataPath, dataType, dataSave, value, filteredUniqKey) {
    return (dispatch, getState, form) => {
        if (dataSave) {
            const state = getState();
            const multilineState = ViewModelSelect_1.currentViewModelSelect.node(state, multilinePath);
            const multilineSort = { dataPath: dataPath, dataType: dataType, value: value };
            const children = ViewModelSelect_1.currentViewModelSelect.children(state, multilinePath);
            const sortedChildren = (0, SortingHelpers_1.sortInstances)(children, multilineState, multilineSort);
            form.setChildren(multilinePath, sortedChildren);
        }
        dispatch({
            type: multiLineActions_1.MultiLineActionType.Sort,
            payload: {
                stickyTablePath: stickyTablePath,
                multilinePath: multilinePath,
                dataPath: dataPath,
                dataType: dataType,
                dataSave: dataSave,
                value: value,
                filteredUniqKey: filteredUniqKey,
            },
        });
        form.filteringManager.requestFiltering(multilinePath, filteredUniqKey);
    };
}
function filter(stickyTablePath, multilinePath, multilineFilter, filteredUniqKey) {
    return (dispatch, getState, form) => {
        dispatch({
            type: multiLineActions_1.MultiLineActionType.Filter,
            payload: {
                stickyTablePath: stickyTablePath,
                multilinePath: multilinePath,
                multilineFilter: multilineFilter,
                filteredUniqKey: filteredUniqKey,
            },
        });
        form.filteringManager.requestFiltering(multilinePath, filteredUniqKey);
    };
}
function filterAll() {
    return async (dispatch, getState, form) => {
        dispatch({
            type: multiLineActions_1.MultiLineActionType.FilterAll,
        });
        return await form.filteringManager.requestFilteringAllMultilines();
    };
}
function removeFilterAction(stickyTablePath, multilinePath, filterSourceId, filteredUniqKey) {
    return (dispatch, getState, form) => {
        dispatch({
            type: multiLineActions_1.MultiLineActionType.RemoveFilter,
            payload: {
                stickyTablePath: stickyTablePath,
                multilinePath: multilinePath,
                filterSourceId: filterSourceId,
                filteredUniqKey: filteredUniqKey,
            },
        });
        form.filteringManager.requestFiltering(multilinePath, filteredUniqKey);
    };
}
function resetFilters(stickyTablePath, multilinePath, filteredUniqKey) {
    return (dispatch, getState, form) => {
        dispatch({
            type: multiLineActions_1.MultiLineActionType.ResetFilters,
            payload: {
                stickyTablePath: stickyTablePath,
                multilinePath: multilinePath,
                filteredUniqKey: filteredUniqKey,
            },
        });
        form.filteringManager.requestFiltering(multilinePath, filteredUniqKey);
    };
}
function setMultilineLoading(bindingPath, filteredUniqKey, loading) {
    return {
        type: multiLineActions_1.MultiLineActionType.SetLoading,
        payload: {
            bindingPath: bindingPath,
            filteredUniqKey: filteredUniqKey,
            loading: loading,
        },
    };
}
function goToPage(bindingPath, page, filteredUniqKey) {
    return async (dispatch, getState, form) => {
        await form.filteringManager.activePageChanged(bindingPath, filteredUniqKey, page);
    };
}
function addInstance(bindingPath, data, doNotCopyPaths, insertAfterInstance, insertBeforeInstance, filteredUniqKey = "default") {
    return (dispatch, getState, form) => {
        const instances = ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
        const instance = (0, MultilineHelpers_1.findFreeInstanceNumber)(instances);
        const instanceIndex = insertAfterInstance
            ? instances.indexOf(insertAfterInstance) + 1
            : insertBeforeInstance
                ? instances.indexOf(insertBeforeInstance)
                : instances.length;
        const newInstances = (0, MultilineHelpers_1.getInstancesAfterAdd)(instances, instance, instanceIndex);
        if (instanceIndex !== instances.length && doNotCopyPaths && doNotCopyPaths.length && data) {
            for (const path of doNotCopyPaths) {
                ModelHunk_1.ModelHunk.set(data, path, { value: "" });
            }
        }
        form.setChildren(bindingPath, newInstances);
        if (data) {
            form.setBatchByPrefix(`${bindingPath}/${instance}`, data);
        }
        dispatch({
            type: multiLineActions_1.MultiLineActionType.AddInstance,
            payload: {
                bindingPath: (0, ModelPath_1.convertToLegacyPath)(bindingPath),
                data: data,
                insertAfterInstance: insertAfterInstance,
                insertBeforeInstance: insertBeforeInstance,
                doNotCopyPaths: doNotCopyPaths,
                instance: instance,
                filteredUniqKey: filteredUniqKey,
            },
        });
        return instance;
    };
}
function copyInstanceAction(bindingPath, sourceInstance, doNotCopyPaths, insertAfterInstance, filteredUniqKey = "default") {
    return (dispatch, getState, form) => {
        function getInstanceData() {
            const instancePath = (0, ModelPath_1.convertToAbsoluteModelPath)(bindingPath).joinWith(sourceInstance);
            const dataWithRelativePaths = ModelHunk_1.ModelHunk.create();
            const instanceData = form.getBatch(instancePath);
            const doNotCopyPathsSet = new Set(doNotCopyPaths);
            const pathsToCopy = ModelHunk_1.ModelHunk.paths(instanceData)
                .map(absolutePath => [
                absolutePath,
                absolutePath.replace(`${instancePath.toLegacyPath()}/`, ""),
            ])
                .filter(([, relativePath]) => doNotCopyPaths == undefined || !doNotCopyPathsSet.has(relativePath));
            for (const [absolutePath, relativePath] of pathsToCopy) {
                ModelHunk_1.ModelHunk.set(dataWithRelativePaths, relativePath, ModelHunk_1.ModelHunk.get(instanceData, absolutePath));
            }
            return dataWithRelativePaths;
        }
        const data = getInstanceData();
        const instances = ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
        const instance = (0, MultilineHelpers_1.findFreeInstanceNumber)(instances);
        const instanceIndex = insertAfterInstance ? instances.indexOf(insertAfterInstance) + 1 : instances.length;
        const newInstances = (0, MultilineHelpers_1.getInstancesAfterAdd)(instances, instance, instanceIndex);
        if (instanceIndex !== instances.length && doNotCopyPaths && doNotCopyPaths.length && data) {
            for (const path of doNotCopyPaths) {
                ModelHunk_1.ModelHunk.set(data, path, { value: "" });
            }
        }
        form.updateModel(model => {
            model.setChildren(bindingPath, newInstances);
            if (data) {
                model.setBatchByPrefix(`${bindingPath}/${instance}`, data);
            }
        });
        dispatch({
            type: multiLineActions_1.MultiLineActionType.AddInstance,
            payload: {
                bindingPath: (0, ModelPath_1.convertToLegacyPath)(bindingPath),
                data: data,
                insertAfterInstance: insertAfterInstance,
                doNotCopyPaths: doNotCopyPaths,
                instance: instance,
                filteredUniqKey: filteredUniqKey,
            },
        });
    };
}
var LastInstanceBehaviour;
(function (LastInstanceBehaviour) {
    LastInstanceBehaviour[LastInstanceBehaviour["Remove"] = 0] = "Remove";
    LastInstanceBehaviour[LastInstanceBehaviour["Clear"] = 1] = "Clear";
})(LastInstanceBehaviour || (exports.LastInstanceBehaviour = LastInstanceBehaviour = {}));
function removeInstance(bindingPath, instancesToRemove, lastInstanceBehaviour = LastInstanceBehaviour.Remove, filteredUniqKey = "default") {
    return removeInstances(bindingPath, [instancesToRemove], lastInstanceBehaviour, filteredUniqKey);
}
function removeInstances(bindingPath, instancesToRemove, lastInstanceBehaviour = LastInstanceBehaviour.Remove, filteredUniqKey = "default") {
    return (dispatch, getState, form) => {
        const instances = ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
        const instancesAfterRemove = (0, MultilineHelpers_1.getInstancesAfterRemove)(instances, instancesToRemove);
        form.setChildren(bindingPath, instancesAfterRemove);
        dispatch({
            type: multiLineActions_1.MultiLineActionType.RemoveInstance,
            payload: {
                bindingPath: (0, ModelPath_1.convertToLegacyPath)(bindingPath),
                indices: instancesToRemove,
                filteredUniqKey: filteredUniqKey,
            },
        });
        if (lastInstanceBehaviour === LastInstanceBehaviour.Clear) {
            if (instancesAfterRemove.length === 0) {
                form.setChildren(bindingPath, ["0"]);
                dispatch({
                    type: multiLineActions_1.MultiLineActionType.AddInstance,
                    payload: {
                        bindingPath: (0, ModelPath_1.convertToLegacyPath)(bindingPath),
                        instance: "0",
                        filteredUniqKey: filteredUniqKey,
                    },
                });
            }
        }
    };
}
function fillMultiline(bindingPath, values, newInstanceCount, filteredUniqKey = "default") {
    return (dispatch, getState, form) => {
        let instancesAfterAdd = [];
        form.updateModel(model => {
            let instances = model.getChildren(bindingPath);
            if (instances != undefined) {
                for (let i = 0; i < newInstanceCount; i++) {
                    const instance = (0, MultilineHelpers_1.findFreeInstanceNumber)(instances);
                    instances = [...instances, instance];
                }
                instancesAfterAdd = instances;
                model.setChildren(bindingPath, instances);
            }
            model.setBatchByPrefix(bindingPath, values.toModelChunk());
        });
        if (newInstanceCount !== 0) {
            dispatch({
                type: multiLineActions_1.MultiLineActionType.AddInstance,
                payload: {
                    bindingPath: (0, ModelPath_1.convertToLegacyPath)(bindingPath),
                    instance: instancesAfterAdd[instancesAfterAdd?.length - 1],
                    filteredUniqKey: filteredUniqKey,
                },
            });
        }
    };
}
