"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getErrorLog = exports.getExpertErrorLog = exports.checkByPopd = void 0;
exports.changeErrorLogModelStateAction = changeErrorLogModelStateAction;
exports.requestErrorLog = requestErrorLog;
exports.receivedErrorlog = receivedErrorlog;
exports.requestFailed = requestFailed;
exports.requestExpertErrorLog = requestExpertErrorLog;
exports.receivedExpertErrorlog = receivedExpertErrorlog;
exports.requestExpertErrorLogFailed = requestExpertErrorLogFailed;
exports.requestPopdProtocol = requestPopdProtocol;
exports.receivedPopdProtocol = receivedPopdProtocol;
exports.requestPopdProtocolFailed = requestPopdProtocolFailed;
exports.setAttachmentErrorLog = setAttachmentErrorLog;
exports.clearAttachmentErrorLog = clearAttachmentErrorLog;
exports.navigateInsideAttachment = navigateInsideAttachment;
exports.navigateToAttachment = navigateToAttachment;
exports.navigateByBindingPathsOrHighlanderPathActionCreator = navigateByBindingPathsOrHighlanderPathActionCreator;
const metricsAction_1 = require("../../Controls/metricsAction");
const MetricsProvider_1 = require("../../Controls/Internals/Metrics/MetricsProvider");
const InitializeSentry_1 = require("../../Forms/Sentry/InitializeSentry");
const ErrorLogActions_1 = require("../Core/ErrorManager/Store/ErrorLogActions");
const TypingUtils_1 = require("../../../../Common/TypingUtils");
const Utils_1 = require("../Core/ErrorManager/Utils/Utils");
function changeErrorLogModelStateAction(modelChanged) {
    return {
        type: ErrorLogActions_1.ErrorLogActionsType.UpdateModelState,
        payload: {
            modelChanged: modelChanged,
        },
    };
}
function requestErrorLog() {
    return { type: ErrorLogActions_1.ErrorLogActionsType.requestErrorlog };
}
function receivedErrorlog(errorLog) {
    return {
        type: ErrorLogActions_1.ErrorLogActionsType.receivedErrorLog,
        payload: errorLog,
    };
}
function requestFailed() {
    return { type: ErrorLogActions_1.ErrorLogActionsType.requestFailed };
}
function requestExpertErrorLog() {
    return { type: ErrorLogActions_1.ErrorLogActionsType.requestExpertErrorlog };
}
function receivedExpertErrorlog(errorLog) {
    return {
        type: ErrorLogActions_1.ErrorLogActionsType.receivedExpertErrorLog,
        payload: errorLog,
    };
}
function requestExpertErrorLogFailed() {
    return {
        type: ErrorLogActions_1.ErrorLogActionsType.requestExpertErrorLogFailed,
    };
}
function requestPopdProtocol() {
    return { type: ErrorLogActions_1.ErrorLogActionsType.requestPopdProtocol };
}
function receivedPopdProtocol(errorLog) {
    return {
        type: ErrorLogActions_1.ErrorLogActionsType.receivedPopdProtocol,
        payload: errorLog,
    };
}
function requestPopdProtocolFailed() {
    return { type: ErrorLogActions_1.ErrorLogActionsType.requestPopdProtocolFailed };
}
const checkByPopd = async (dispatch, getState, form) => {
    dispatch(requestPopdProtocol());
    try {
        await form.changeSet.send();
        const popdProtocol = await form.candyApiDraftClient.checkByPopd();
        (0, MetricsProvider_1.track)("candy", metricsAction_1.POPD_PROTOCOL_COUNTER, {
            resultCode: popdProtocol.resultCode,
            errors: popdProtocol.errorsCount,
            warnings: popdProtocol.warningsCount,
        });
        dispatch(receivedPopdProtocol(popdProtocol));
    }
    catch (e) {
        dispatch(requestPopdProtocolFailed());
        // eslint-disable-next-line no-console
        console.error(e);
    }
};
exports.checkByPopd = checkByPopd;
//Возможно легаси и можно выпилить. Кто будет делать следующую таску по еррорлогу - поглядите
const getExpertErrorLog = async (dispatch, getState, form) => {
    dispatch(requestExpertErrorLog());
    try {
        await form.changeSet.send();
        const noPathMapping = form.lazyLoadingManager.isLazyLoadingEnabled();
        const expertErrorLog = await form.candyApiDraftClient.getExpertErrorLog();
        const errorLogWithoutLinks = (0, Utils_1.parseErrorLog)((0, Utils_1.serverErrorLogToGroupedAdapter)(expertErrorLog), () => form.navigationPathMapper, true, noPathMapping);
        (0, MetricsProvider_1.track)("candy", metricsAction_1.EXPERT_ERROR_LOG_COUNTER, {
            errors: expertErrorLog.errors.length,
            warnings: expertErrorLog.warnings.length,
        });
        dispatch(receivedExpertErrorlog(errorLogWithoutLinks));
    }
    catch (e) {
        (0, MetricsProvider_1.track)("candy", metricsAction_1.EXPERT_ERROR_LOG_REQUEST_FAILED, { error: (0, TypingUtils_1.stringifyUnknownError)(e) });
        dispatch(requestExpertErrorLogFailed());
        // eslint-disable-next-line no-console
        console.error(e);
    }
};
exports.getExpertErrorLog = getExpertErrorLog;
const getErrorLog = async (dispatch, getState, form) => {
    function getPathMapperForGroup(groupAttachmentId) {
        if (groupAttachmentId) {
            const attachmentPublicForms = form.attachmentsPublicForms[groupAttachmentId];
            if (attachmentPublicForms) {
                return attachmentPublicForms.getNavigationPathMapper();
            }
        }
        return form.navigationPathMapper;
    }
    dispatch(requestErrorLog());
    try {
        await form.changeSet.send();
        await form.normalizeDirtyMasksAndReloadInnerIfRequired();
        const noPathMapping = form.lazyLoadingManager.isLazyLoadingEnabled();
        const errorLog = await form.errorLogApi.getErrorLog(attId => getState().attachments.byId[attId]);
        dispatch(receivedErrorlog((0, Utils_1.parseErrorLog)(errorLog, getPathMapperForGroup, false, noPathMapping)));
    }
    catch (e) {
        if (sessionStorage.getItem("FakeErrorLog." + form.settings.draftId) == undefined) {
            // Пропустим ошибку ибо сюда бывает подсовывают ошибочный error log
            (0, InitializeSentry_1.getCandyHub)().captureException(e);
        }
        form.errorLogApi.resetStatuses();
        dispatch(requestFailed());
        // eslint-disable-next-line no-console
        console.error(e);
    }
};
exports.getErrorLog = getErrorLog;
function setAttachmentErrorLog(attachmentErrorLog) {
    return {
        type: ErrorLogActions_1.ErrorLogActionsType.setAttachmentErrors,
        payload: attachmentErrorLog,
    };
}
function clearAttachmentErrorLog() {
    return {
        type: ErrorLogActions_1.ErrorLogActionsType.clearAttachmentErrors,
    };
}
function navigateInsideAttachment(attachmentPublicForm, highlanderPath) {
    return (d, s, form) => attachmentPublicForm.navigateByHighlanderPaths([highlanderPath]);
}
function navigateToAttachment(attachmentId, highlanderPath) {
    return (d, s, form) => {
        const attachmentPublicForm = form.getAttachmentsPublicFormsById(attachmentId);
        return attachmentPublicForm.navigateByHighlanderPaths([highlanderPath]);
    };
}
function navigateByBindingPathsOrHighlanderPathActionCreator(attachmentId) {
    return (d, s, form) => {
        if (attachmentId) {
            const attachmentInfo = s().attachments.byId[attachmentId];
            const attachmentPath = attachmentInfo?.meta.bindingPath;
            if (attachmentPath) {
                return async (paths, highlanderPath, errorItemId) => {
                    if (highlanderPath) {
                        d(setAttachmentErrorLog({
                            attachmentId: attachmentId,
                            path: highlanderPath,
                            errorItemId: errorItemId,
                        }));
                    }
                    await form.activeLogHelper.navigateByHighlanderPaths([attachmentPath]);
                    return undefined;
                };
            }
        }
        return (paths) => form.lazyLoadingManager.isLazyLoadingEnabled()
            ? form.activeLogHelper.navigateByHighlanderPaths(paths)
            : form.activeLogHelper.navigateByBindingPaths(paths);
    };
}
