"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertDraftFromPrevGfvAction = exports.interdocumentTransferHandler = exports.showTourHandler = exports.enableAutoValuesHandler = exports.printHandler = exports.innerSaveHandler = exports.fufSaveHandler = exports.clearFormHandler = exports.feedbackHandler = exports.formSendToClientForSignAction = exports.formCloseAction = exports.formSendAction = exports.formSilentSaveAction = exports.formSaveAction = exports.formSaveAction2 = void 0;
exports.setFormSaveStatus = setFormSaveStatus;
exports.setFormReloading = setFormReloading;
const FormSaveStatusActions_1 = require("../../Controls/FormParts/Panel/FormSaveStatus/Store/FormSaveStatusActions");
const ExternalEvents_1 = require("../Core/EventManagement/ExternalEvents/ExternalEvents");
const FiltersSessionHelpers_1 = require("../../Controls/MultiControls/MultiLine/Store/FiltersSessionHelpers");
const FormActions_1 = require("../Form/Store/FormActions");
const ActionPanelActions_1 = require("../../Controls/FormParts/Panel/ActionPanel/Store/ActionPanelActions");
const formSaveAction2 = async (dispatch, getState, form) => {
    try {
        await form.changeSet.send();
        form.triggerExternalEvent((0, ExternalEvents_1.formSaveEvent)());
    }
    catch {
        // eslint-disable-next-line no-alert
        alert("Не удалось сохранить данные. Попробуйте еще раз.");
    }
};
exports.formSaveAction2 = formSaveAction2;
const formSaveAction = (dispatch, getState, form) => {
    // eslint-disable-next-line @typescript-eslint/no-floating-promises
    form.changeSet.send({ force: true, operationName: undefined });
};
exports.formSaveAction = formSaveAction;
const formSilentSaveAction = (dispatch, getState, form) => form.changeSet.send();
exports.formSilentSaveAction = formSilentSaveAction;
const formSendAction = async (d, s, form) => {
    try {
        await form.changeSet.send();
        form.triggerExternalEvent((0, ExternalEvents_1.formSendEvent)());
    }
    catch {
        // eslint-disable-next-line no-alert
        if (confirm("Не удалось сохранить последние изменения. Перейти к отправке формы?")) {
            form.triggerExternalEvent((0, ExternalEvents_1.formSendEvent)());
        }
    }
};
exports.formSendAction = formSendAction;
const formCloseAction = async (d, s, form) => {
    await form.changeSet.send();
    form.triggerExternalEvent((0, ExternalEvents_1.formCloseEvent)());
};
exports.formCloseAction = formCloseAction;
const formSendToClientForSignAction = async (d, s, form) => {
    form.triggerExternalEvent((0, ExternalEvents_1.formSendToClientForSign)());
};
exports.formSendToClientForSignAction = formSendToClientForSignAction;
const feedbackHandler = (d, s, form) => {
    form.triggerExternalEvent((0, ExternalEvents_1.formShowFeedbackEvent)());
};
exports.feedbackHandler = feedbackHandler;
const clearFormHandler = async (d, s, form) => {
    // eslint-disable-next-line no-alert
    const permission = confirm("При очистке все данные этой формы будут потеряны. Очистить форму?");
    if (!permission) {
        return;
    }
    try {
        (0, FiltersSessionHelpers_1.clearAllFiltersSessions)();
        await form.changeSet.send();
        await form.candyApiDraftClient.clearDraft();
        window.location.reload();
    }
    catch {
        // eslint-disable-next-line no-alert
        alert("Не удалось очистить форму. Попробуйте позже.");
    }
};
exports.clearFormHandler = clearFormHandler;
const fufSaveHandler = async (d, s, form) => {
    d({ type: ActionPanelActions_1.ActionPanelActionType.BeginDownloadFuf });
    await form.changeSet.send();
    await form.normalizeDirtyMasksAndReloadInnerIfRequired();
    await form.candyApiDraftClient.downloadFuf();
    d({ type: ActionPanelActions_1.ActionPanelActionType.EndDownloadFuf });
};
exports.fufSaveHandler = fufSaveHandler;
const innerSaveHandler = async (d, s, form) => {
    d({ type: ActionPanelActions_1.ActionPanelActionType.BeginDownloadInner });
    await form.changeSet.send();
    await form.normalizeDirtyMasksAndReloadInnerIfRequired();
    await form.candyApiDraftClient.downloadInner();
    d({ type: ActionPanelActions_1.ActionPanelActionType.EndDownloadInner });
};
exports.innerSaveHandler = innerSaveHandler;
const printHandler = async (d, s, form) => {
    await form.changeSet.send();
    await form.normalizeDirtyMasksAndReloadInnerIfRequired();
    form.triggerExternalEvent((0, ExternalEvents_1.formPrintEvent)());
};
exports.printHandler = printHandler;
const enableAutoValuesHandler = async (d, s, form) => {
    await form.customNormalizationHelper.fillAutoFieldsWithAutoValuesAndReload([], true);
};
exports.enableAutoValuesHandler = enableAutoValuesHandler;
const showTourHandler = (d, s, form) => {
    form.triggerExternalEvent((0, ExternalEvents_1.formShowTourEvent)());
};
exports.showTourHandler = showTourHandler;
const interdocumentTransferHandler = (d, s, form) => {
    // eslint-disable-next-line @typescript-eslint/no-floating-promises
    form.changeSet.send().then(() => form.triggerExternalEvent((0, ExternalEvents_1.formInterdocumentTransferEvent)()));
};
exports.interdocumentTransferHandler = interdocumentTransferHandler;
const convertDraftFromPrevGfvAction = async (d, s, form) => {
    await form.changeSet.send();
    await form.normalizeDirtyMasksAndReloadInnerIfRequired();
    await form.candyApiDraftClient.convertDraftFromPrevGfv({
        // @ts-ignore Я хз, кажется prevGfvDraftId должен быть undefined -- надо вставить проверку и кинуть Error, но я не увереын
        prevGfvDraftId: form.settings.prevGfvDraftId,
        gfv: form.settings.gfv,
    });
    window.location.reload();
};
exports.convertDraftFromPrevGfvAction = convertDraftFromPrevGfvAction;
function setFormSaveStatus(status) {
    return { type: FormSaveStatusActions_1.FormSaveStatusActionType.SetSavingStatus, payload: { status: status } };
}
function setFormReloading(isFormReloading) {
    return { type: FormActions_1.FormActionType.SetFormReloading, payload: { isFormReloading: isFormReloading } };
}
