"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClearPageMethod = void 0;
exports.addPageAction = addPageAction;
exports.removePageAction = removePageAction;
exports.clearPageAction = clearPageAction;
exports.addMultipleUnitInstanceAction = addMultipleUnitInstanceAction;
exports.clearMultipleUnitInstanceAction = clearMultipleUnitInstanceAction;
exports.removeMultipleUnitInstanceAction = removeMultipleUnitInstanceAction;
exports.copyMultipleUnitInstanceAction = copyMultipleUnitInstanceAction;
exports.openModalUnitAction = openModalUnitAction;
exports.closeModalUnitAction = closeModalUnitAction;
const unitActions_1 = require("../../Controls/FormParts/Unit/Store/unitActions");
const MultilineHelpers_1 = require("../../Helpers/StateAccessors/MultilineHelpers");
const ViewModelSelect_1 = require("../../Helpers/StateAccessors/ViewModelSelect");
const ExternalEvents_1 = require("../Core/EventManagement/ExternalEvents/ExternalEvents");
const UnitId_1 = require("../../Controls/FormParts/Unit/UnitId/UnitId");
const ModelHunk_1 = require("../Core/ModelStoring/ModelHunk");
const ModelPath_1 = require("../../../../Common/ModelPath/ModelPath");
const AsyncUtils_1 = require("../../../../Common/AsyncUtils");
const ModelPathHelper_1 = require("../../../../Common/ModelPath/ModelPathHelper");
const UIUtil_1 = require("../Form/CustomizationContexts/HelperFunctionContext/UIUtil/UIUtil");
const multiLineActions_1 = require("../../Controls/MultiControls/MultiLine/Store/multiLineActions");
const TypingUtils_1 = require("../../../../Common/TypingUtils");
const IterableUtils_1 = require("../../../../Common/IterableUtils");
const NavigationActionCreators_1 = require("./NavigationActionCreators");
var ClearPageMethod;
(function (ClearPageMethod) {
    ClearPageMethod["OnlyParent"] = "OnlyParent";
    ClearPageMethod["ParentAndChildren"] = "ParentAndChildren";
})(ClearPageMethod || (exports.ClearPageMethod = ClearPageMethod = {}));
function addPageAction(unitType) {
    return async (dispatch, getState, form) => {
        await form.normalizeSection(unitType);
        form.triggerExternalEvent((0, ExternalEvents_1.formPageChangedEvent)(unitType));
    };
}
const UNITS_COUNT_SERVER_REMOVE_THRESHOLD = 10;
function removePageAction(unitType, bindingPath) {
    return async (dispatch, getState, form) => {
        if (bindingPath && form.settings.useServerSideFLangNormalization) {
            const instances = ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
            if (instances?.length > UNITS_COUNT_SERVER_REMOVE_THRESHOLD) {
                const uiUtil = new UIUtil_1.UIUtil(form.settings.theme);
                await uiUtil.executeLongOperation({ title: "Удаление раздела", modalSpecialCase: "InfiniteProcess", allowCancel: false }, async () => {
                    await form.candyApiDraftClient.deleteInstances([
                        {
                            multiplicityPath: bindingPath.toLegacyPath(),
                            instanceNumbers: instances,
                            prefixesToDelete: [],
                        },
                    ]);
                    await form.reloadInner();
                });
            }
        }
        form.removeSection(unitType, bindingPath);
        form.triggerExternalEvent((0, ExternalEvents_1.formPageChangedEvent)(unitType));
    };
}
function clearPageAction(unitType, bindingPath, method) {
    return async (dispatch, getState, form) => {
        await form.clearSection(unitType, bindingPath, method);
    };
}
function addMultipleUnitInstanceAction(bindingPath, unitId, newIndex) {
    return (dispatch, getState, form) => {
        const instances = ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
        const newInstance = (0, MultilineHelpers_1.findFreeInstanceNumber)(instances ?? []);
        const newInstances = (0, MultilineHelpers_1.getInstancesAfterAdd)(instances, newInstance, newIndex ? newIndex : instances.length);
        form.setChildren(bindingPath, newInstances);
        dispatch({
            type: unitActions_1.UnitActionType.AddUnit,
            payload: {
                bindingPath: bindingPath.toLegacyPath(),
                newInstance: newInstance,
                newIndex: newIndex,
                unitId: unitId,
            },
        });
        setTimeout(() => {
            const id = (0, UnitId_1.unitIdWithInstance)(unitId, newInstance);
            dispatch((0, NavigationActionCreators_1.scrollToUnitAction)(id));
            dispatch((0, NavigationActionCreators_1.unitHighlightAction)(id));
        }, 0);
    };
}
function clearMultipleUnitInstanceAction(unitType, bindingPath, removingInstanceId, method) {
    return async (d, getState, form) => {
        if (method === ClearPageMethod.ParentAndChildren) {
            const instances = ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
            const oldInstances = [...instances];
            const newInstances = (0, MultilineHelpers_1.getInstancesAfterRemove)(instances, [removingInstanceId]);
            form.setChildren(bindingPath, newInstances);
            form.setChildren(bindingPath, oldInstances);
            const parentInstances = Iterator.from(bindingPath.getPathParts())
                .filter(part => ModelPath_1.PathTokens.isInstanceToken(part))
                .toArray()
                .reverse();
            const changedSections = form.getSectionsByPath(unitType, bindingPath, [removingInstanceId].concat(parentInstances));
            for (const section of changedSections) {
                form.triggerExternalEvent((0, ExternalEvents_1.formPageChangedEvent)(section));
            }
        }
        else {
            const instancePath = bindingPath.joinWith(removingInstanceId);
            await form.renormalizePage(unitType, instancePath);
        }
    };
}
function removeMultipleUnitInstanceAction(bindingPath, instance, unitId, scrollToNearestInstance, operationName) {
    return async (dispatch, getState, form) => {
        const instances = ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
        const unitIdToScrollAfter = scrollToNearestInstance
            ? findScrolledUnitAfterRemoveUnit(instances, unitId, instance)
            : undefined;
        form.setChildren(bindingPath, (0, MultilineHelpers_1.getInstancesAfterRemove)(instances, [instance]));
        dispatch({
            type: multiLineActions_1.MultiLineActionType.RemoveInstance,
            payload: {
                bindingPath: bindingPath.toLegacyPath(),
                indices: [instance],
                filteredUniqKey: "default",
            },
        });
        dispatch({
            type: unitActions_1.UnitActionType.RemoveUnit,
            payload: {
                bindingPath: bindingPath.toLegacyPath(),
                index: instance,
                unitId: unitId,
            },
        });
        if (unitIdToScrollAfter != undefined) {
            dispatch((0, NavigationActionCreators_1.scrollToUnitAction)(unitIdToScrollAfter));
            dispatch((0, NavigationActionCreators_1.unitHighlightAction)(unitIdToScrollAfter));
        }
    };
}
function copyMultipleUnitInstanceAction(bindingPath, sourceInstance, unitId) {
    return async (dispatch, getState, form) => {
        const instancePath = bindingPath.joinWith(sourceInstance);
        const instances = ViewModelSelect_1.currentViewModelSelect.children(getState(), bindingPath);
        const instance = (0, MultilineHelpers_1.findFreeInstanceNumber)(instances);
        const instanceIndex = instances.length;
        const newInstances = (0, MultilineHelpers_1.getInstancesAfterAdd)(instances, instance, instanceIndex);
        const prefix = bindingPath.joinWith(instance);
        const dataWithRelativePaths = ModelHunk_1.ModelHunk.create();
        const secondDataWithRelativePaths = ModelHunk_1.ModelHunk.create();
        const instanceData = form.getBatch(instancePath);
        const pathsToCopy = ModelHunk_1.ModelHunk.paths(instanceData).map(absolutePath => [
            absolutePath,
            absolutePath.replace(instancePath.toLegacyPath() + "/", ""),
        ]);
        for (const [absolutePath, relativePath] of pathsToCopy) {
            if (!(0, ModelPathHelper_1.isAggregationPath)((0, ModelPath_1.createModelPath)(absolutePath, true))) {
                const viewModel = { ...ModelHunk_1.ModelHunk.get(instanceData, absolutePath) };
                if (viewModel.autoValue == undefined) {
                    ModelHunk_1.ModelHunk.set(dataWithRelativePaths, prefix.joinWith((0, ModelPath_1.createModelPath)(relativePath, false)), viewModel);
                }
                else {
                    ModelHunk_1.ModelHunk.set(secondDataWithRelativePaths, prefix.joinWith((0, ModelPath_1.createModelPath)(relativePath, false)), viewModel);
                }
            }
        }
        form.updateModel(model => {
            model.setChildren(bindingPath, newInstances);
        });
        form.updateModel(model => {
            model.setBatch(dataWithRelativePaths);
        });
        form.updateModel(model => {
            model.setBatch(secondDataWithRelativePaths);
        });
        form.sections.resolveOptionalSections(dataWithRelativePaths);
        form.sections.resolveOptionalSections(secondDataWithRelativePaths);
        dispatch({
            type: unitActions_1.UnitActionType.AddUnit,
            payload: {
                bindingPath: bindingPath.toLegacyPath(),
                newInstance: instance,
                newIndex: instanceIndex,
                unitId: unitId,
            },
        });
        await (0, AsyncUtils_1.nextTick)();
        const copiedUnitId = (0, UnitId_1.createUnitId)(unitId.pageId, instance);
        copiedUnitId.parentInstances = unitId.parentInstances ? [...unitId.parentInstances] : [];
        dispatch((0, NavigationActionCreators_1.scrollToUnitAction)(copiedUnitId));
        dispatch((0, NavigationActionCreators_1.unitHighlightAction)(copiedUnitId));
        return copiedUnitId;
    };
}
function findScrolledUnitAfterRemoveUnit(instances, unitId, instanceToRemove) {
    if (instances.length > 1) {
        const instanceIndex = instances.findIndex(ins => ins === instanceToRemove);
        return instanceIndex === 0
            ? (0, UnitId_1.unitIdWithInstance)(unitId, instances[instanceIndex + 1])
            : (0, UnitId_1.unitIdWithInstance)(unitId, instances[instanceIndex - 1]);
    }
    return undefined;
}
function openModalUnitAction(helperContextualBindingPath, unitId, modalInstancePageId) {
    return (dispatch, getState, form) => {
        const unitInfos = getState().unitInfos;
        const nodalUnitInfo = unitInfos?.find(x => x.id === modalInstancePageId) ??
            (0, TypingUtils_1.reject)(`Modal instance page [${modalInstancePageId}] is not found !`);
        const bindingPathParts = helperContextualBindingPath.getPathPartsAsArray();
        const nodalUnitInfoBindingPathLength = IterableUtils_1.IterUtils.count(nodalUnitInfo.bindingPath.getPathParts());
        const instanceSource = bindingPathParts[nodalUnitInfoBindingPathLength];
        const instance = ModelPath_1.PathTokens.isInstanceToken(instanceSource) ? instanceSource : (0, TypingUtils_1.reject)("Instance is not found!");
        const modalUnitId = { pageId: modalInstancePageId, instance: instance };
        form.openModalUnit(unitId, modalUnitId);
    };
}
function closeModalUnitAction(unitId) {
    return (dispatch, getState, form) => {
        form.closeModalUnit(unitId);
    };
}
