"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VerifyAndFixFnsRequirementsTask = exports.CancellationError = void 0;
const utils_1 = require("../../utils");
const Tasks_1 = require("../../../../../Common/CandyApiClient/Types/Tasks/Tasks");
class CancellationError {
    code = "Cancelled";
    message;
    constructor(message) {
        this.message = message;
    }
}
exports.CancellationError = CancellationError;
class VerifyAndFixFnsRequirementsTask {
    candyApiDraftTasksClient;
    progressListeners = [];
    id;
    attachmentId;
    canceled = false;
    constructor(candyApiDraftTasksClient, id, attachmentId) {
        this.candyApiDraftTasksClient = candyApiDraftTasksClient;
        this.id = id;
        this.attachmentId = attachmentId;
    }
    async startTask() {
        await this.candyApiDraftTasksClient.enqueueTask(this.id, {
            attachmentId: this.attachmentId,
            type: "VerifyAndFixFnsRequirements",
        });
        await this.poll();
    }
    async cancel() {
        this.canceled = true;
        await this.candyApiDraftTasksClient.cancelTask(this.id);
    }
    progress(callback) {
        this.progressListeners.push(callback);
    }
    poll = async () => {
        if (this.canceled) {
            return;
        }
        const taskInfo = await this.candyApiDraftTasksClient.getTaskById(this.id);
        if (taskInfo == undefined) {
            return;
        }
        switch (taskInfo.status) {
            case Tasks_1.TaskStatus.Cancelled:
                // eslint-disable-next-line @typescript-eslint/only-throw-error
                throw new CancellationError("Cancel task");
            case Tasks_1.TaskStatus.Error: {
                // eslint-disable-next-line @typescript-eslint/only-throw-error
                throw taskInfo.error;
            }
            case Tasks_1.TaskStatus.InProgress: {
                for (const callback of this.progressListeners) {
                    callback(taskInfo.progress.done, taskInfo.progress.total);
                }
                await (0, utils_1.delay)(1000);
                await this.poll();
                break;
            }
            case Tasks_1.TaskStatus.Success:
                break;
            default:
                break;
        }
    };
}
exports.VerifyAndFixFnsRequirementsTask = VerifyAndFixFnsRequirementsTask;
