"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OperandsGetter = void 0;
const AutocalcCommonFunctions_1 = require("../../../Helpers/AutocalcCommonFunctions");
const ModelEvent_1 = require("../EventManagement/ModelEvents/ModelEvent");
const MaskResolver_1 = require("../PathUtils/MaskResolver");
const PathUtils_1 = require("../PathUtils/PathUtils");
class OperandsGetter {
    model;
    declHelper;
    basePath;
    eventArgs;
    constructor(model, dataDeclarationHelper, basePath, eventArgs) {
        this.basePath = basePath;
        this.eventArgs = eventArgs;
        this.model = model;
        this.declHelper = dataDeclarationHelper;
    }
    get = (relativeOperandNodePath, valueName, type) => {
        const operandPath = PathUtils_1.PathUtils.getAbsolutePath(this.basePath, relativeOperandNodePath);
        if (operandPath.lastIndexOf("*") === -1) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return valueName == undefined
                ? // @ts-ignore
                    this.model.getViewModel(operandPath)
                : this.model.getViewModelField(operandPath, valueName);
        }
        if (valueName == undefined) {
            throw new Error(`For multiple elements valueName must be specified`);
        }
        if (type === "sum") {
            const path = PathUtils_1.PathUtils.join(PathUtils_1.PathUtils.splitMask(operandPath));
            const value = this.model.getViewModelField(path, type);
            if (value !== undefined && value !== "") {
                if (this.eventArgs &&
                    this.declHelper.getMask(this.eventArgs.path) === this.declHelper.getMask(operandPath)) {
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                    return this.calculateSumIncremental(value, path);
                }
                // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value);
            }
            const { values } = this.getValues(operandPath, valueName);
            return (0, AutocalcCommonFunctions_1.getSum)(values);
        }
        else if (type === "count") {
            const path = PathUtils_1.PathUtils.join(PathUtils_1.PathUtils.splitMask(operandPath));
            const value = this.model.getViewModelField(path, type);
            if (value !== undefined &&
                this.eventArgs &&
                this.declHelper.getMask(this.eventArgs.path) === this.declHelper.getMask(operandPath)) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                return this.calculateCountIncremental(value);
            }
            const { values } = this.getValues(operandPath, valueName);
            return this.getCount(values);
        }
        return this.getValues(operandPath, valueName);
    };
    getValues(operandPath, valueName) {
        const nodePaths = MaskResolver_1.MaskResolver.resolveMask(this.model, operandPath);
        const result = {
            paths: [],
            values: [],
        };
        for (const path of nodePaths) {
            result.paths.push(PathUtils_1.PathUtils.getValuePath(path, valueName));
            // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
            result.values.push(this.model.getViewModelField(path, valueName));
        }
        return result;
    }
    getCount(values) {
        return values.filter(val => val).length;
    }
    calculateCountIncremental(count) {
        if (this.eventArgs == undefined) {
            throw new Error("EventArgs must be not null");
        }
        const { oldValue, value } = this.eventArgs;
        let newCount = Number(count);
        if (oldValue && !value) {
            newCount -= 1;
        }
        if (!oldValue && value) {
            newCount += 1;
        }
        return newCount;
    }
    calculateSumIncremental(sumValue, path) {
        const countValue = this.model.getViewModelField(path, "count");
        if (!countValue) {
            return "";
        }
        if (this.eventArgs == undefined) {
            throw new Error("EventArgs must be not null");
        }
        if (this.eventArgs.processedBySum) {
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(sumValue);
        }
        (0, ModelEvent_1.patchModelEventInfo)(this.eventArgs, x => {
            x.processedBySum = true;
        });
        return ((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(sumValue)
            // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
            .minus((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(this.eventArgs.oldValue))
            // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
            .plus((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(this.eventArgs.value)));
    }
}
exports.OperandsGetter = OperandsGetter;
