"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AggregatedValidationFunction = void 0;
const CalculationFunctionBase_1 = require("./AutoCalculations/CalculationFunctionBase");
const CalculationTarget_1 = require("./CalculationTarget");
const LegacyValidationFunctionAdapter_1 = require("./Validations/LegacyValidationFunctionAdapter");
const ValidationFunction_1 = require("./Validations/ValidationFunction");
class AggregatedValidationFunction extends CalculationFunctionBase_1.CalculationFunctionBase {
    validationFunctions = [];
    target;
    dependencies;
    constructor(target) {
        super(target.modelPath);
        this.target = target;
    }
    asDebugString() {
        return "[" + this.validationFunctions.map(x => x.asDebugString()).join(", ") + "]";
    }
    addValidationFunction(validationFunction) {
        this.validationFunctions.push(validationFunction);
        this.dependencies = undefined;
    }
    async execute(context) {
        const validationFunctions = this.validationFunctions;
        for (const validationFunction of validationFunctions) {
            await validationFunction.execute(context);
        }
    }
    getDependencies() {
        return this.dependencies ?? (this.dependencies = this.buildDependencies());
    }
    getTarget() {
        return this.target;
    }
    getValidationFunctionDebugInfo() {
        const result = { valueValidators: [], validationFunctions: [] };
        for (const validationFunction of this.validationFunctions) {
            if (validationFunction instanceof LegacyValidationFunctionAdapter_1.LegacyValidationFunctionAdapter) {
                result.valueValidators.push(validationFunction.asDebugString());
            }
            if (validationFunction instanceof ValidationFunction_1.ValidationFunction) {
                result.validationFunctions.push(validationFunction.asDebugString());
            }
        }
        return result;
    }
    isDependsOn(modelPath) {
        return (modelPath.targetAttribute === CalculationTarget_1.CalculationTargetAttribute.Value &&
            this.getDependencies().some(x => x.isMatchToPath(modelPath.modelPath)));
    }
    buildDependencies() {
        const result = [];
        for (const validationFunction of this.validationFunctions) {
            result.push(validationFunction.getDependencies());
        }
        return result.flat();
    }
}
exports.AggregatedValidationFunction = AggregatedValidationFunction;
