"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CalculationFunction = void 0;
const CalculationFunctionDependency_1 = require("../CalculationFunctionDependency");
const CalculationTarget_1 = require("../CalculationTarget");
const PathMatcher_1 = require("../../../../../../Common/ModelPath/PathMatcher");
const CalculationFunctionBase_1 = require("./CalculationFunctionBase");
class CalculationFunction extends CalculationFunctionBase_1.CalculationFunctionBase {
    calculationTarget;
    dependencies;
    evaluator;
    isDisabled;
    isAutoField;
    isOptional;
    constructor(targetPath, dependenciesPaths, evaluator, isAutoField = undefined, isDisabled = undefined, isOptional = undefined) {
        super(targetPath);
        this.isOptional = isOptional ?? false;
        this.isDisabled = isDisabled ?? false;
        this.isAutoField = isAutoField ?? false;
        this.calculationTarget = new CalculationTarget_1.CalculationTarget(targetPath, CalculationTarget_1.CalculationTargetAttribute.Value);
        this.dependencies = dependenciesPaths.map(x => new CalculationFunctionDependency_1.CalculationFunctionDependency(targetPath, x));
        this.evaluator = evaluator;
    }
    getDependencies() {
        return this.dependencies;
    }
    execute(context) {
        const evaluator = this.evaluator;
        const targetPath = this.targetPath;
        if (context.isIgnoredPath(targetPath)) {
            return;
        }
        const autoFlag = context.getAutoFlag(targetPath);
        const ignoreAutoFlag = context.getIgnoreAutoFlag(targetPath);
        const evaluationResult = this.checkIfCalculationRequired(context) ? evaluator(context) : undefined;
        if (this.isAutoField) {
            const prevValue = context.get(targetPath);
            const prevAutoValue = context.getAutoValue(targetPath);
            context.setAutoValue(targetPath, evaluationResult);
            const isValueAbsentInSource = !this.isOptional && prevValue == prevAutoValue && autoFlag == undefined;
            if ((!ignoreAutoFlag && (this.isDisabled || autoFlag)) || isValueAbsentInSource) {
                context.set(targetPath, evaluationResult);
            }
        }
        else {
            context.set(targetPath, evaluationResult);
        }
    }
    checkIfCalculationRequired(context) {
        const hasDeletedItems = this.dependencies.some(x => Iterator.from(context.relatedEvents).some(evt => x
            .getDependencyPath()
            .matchFor(evt.path)
            .every(match => match.kind === PathMatcher_1.MatchKind.MatchedPath || match.kind === PathMatcher_1.MatchKind.MatchedMultiplicity) && evt.type == "remove"));
        const isDependsOnEmptySum = () => {
            const dep = this.dependencies[0]?.getDependencyPath();
            const depParts = dep?.getPathPartsAsArray() ?? [];
            return (dep != undefined &&
                this.dependencies.length === 1 &&
                depParts[depParts.length - 1] === "Sum" &&
                context.get(dep) === "");
        };
        return !((hasDeletedItems || isDependsOnEmptySum()) &&
            context.getAutoValue(this.targetPath) == undefined &&
            context.get(this.targetPath) == undefined);
    }
    isDependsOn(calculationTarget) {
        return (calculationTarget.targetAttribute === CalculationTarget_1.CalculationTargetAttribute.Value &&
            this.dependencies.some(x => x.isMatchToPath(calculationTarget.modelPath)));
    }
    getTarget() {
        return this.calculationTarget;
    }
    asDebugString() {
        return "CalculationFunction(N/A)";
    }
}
exports.CalculationFunction = CalculationFunction;
