"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CalculationFunctionBase = void 0;
const ModelPath_1 = require("../../../../../../Common/ModelPath/ModelPath");
const PathMatcher_1 = require("../../../../../../Common/ModelPath/PathMatcher");
const CalculationFunctionExecutionPlan_1 = require("../ExecutionPlan/CalculationFunctionExecutionPlan");
const IterateOverChildrenPlanStep_1 = require("../ExecutionPlan/IterateOverChildrenPlanStep");
const PushIterationContextExecutionPlanStep_1 = require("../ExecutionPlan/PushIterationContextExecutionPlanStep");
const SimpleExecutionPlanStep_1 = require("../ExecutionPlan/SimpleExecutionPlanStep");
const ModelPathBuilder_1 = require("../../../../../../Common/ModelPath/ModelPathBuilder");
const Map2_1 = require("../../../../../../Common/ModelPath/Map/Map2");
class CalculationFunctionBase {
    targetPath;
    pathIsOutsideDependenciesListCache = new Map2_1.Map2();
    hasDependencyOutsideTargetMultiplicityCache;
    constructor(targetPath) {
        this.targetPath = targetPath;
    }
    getFullPlan() {
        const executionPlan = new CalculationFunctionExecutionPlan_1.CalculationFunctionExecutionPlan(this);
        const currentPrefix = new ModelPathBuilder_1.ModelPathBuilder(false);
        for (const pathPart of this.targetPath.splitByMultiplicity()) {
            if ((0, ModelPath_1.isModelPath)(pathPart)) {
                currentPrefix.append(pathPart);
            }
            else {
                executionPlan.addStep(new IterateOverChildrenPlanStep_1.IterateOverChildrenPlanStep(currentPrefix.toAbsolutePath()));
                currentPrefix.append(pathPart);
            }
        }
        executionPlan.addStep(new SimpleExecutionPlanStep_1.SimpleExecutionPlanStep());
        return executionPlan;
    }
    getPlanForTarget(targetPath2, iterationData, sync) {
        const executionPlan = new CalculationFunctionExecutionPlan_1.CalculationFunctionExecutionPlan(this);
        const currentPrefix = new ModelPathBuilder_1.ModelPathBuilder(false);
        const pathMatching = this.targetPath.matchFor(targetPath2);
        for (const match of pathMatching) {
            if (match.kind === PathMatcher_1.MatchKind.MatchedPath) {
                currentPrefix.append(match.prefix);
            }
            else if (match.kind === PathMatcher_1.MatchKind.MatchedMultiplicity) {
                if (ModelPath_1.PathTokens.isMultiToken(match.source) && ModelPath_1.PathTokens.isMultiToken(match.target)) {
                    const currentPrefixPath = currentPrefix.toAbsolutePath();
                    executionPlan.addStep(new IterateOverChildrenPlanStep_1.IterateOverChildrenPlanStep(currentPrefixPath, iterationData != undefined && currentPrefixPath.isEquals(iterationData.path)
                        ? iterationData.instancesToIterate
                        : undefined, sync));
                    currentPrefix.append(match.source);
                }
                else if (ModelPath_1.PathTokens.isSimpleToken(match.source) && ModelPath_1.PathTokens.isMultiToken(match.target)) {
                    currentPrefix.append(match.source);
                }
                else if (ModelPath_1.PathTokens.isMultiToken(match.source) && ModelPath_1.PathTokens.isSimpleToken(match.target)) {
                    executionPlan.addStep(new PushIterationContextExecutionPlanStep_1.PushIterationContextExecutionPlanStep(currentPrefix.toPath(), match.target));
                    currentPrefix.append(match.source);
                }
            }
            else {
                throw new Error(`Cannot build execution plan of function with target '${this.targetPath.toString()}' to evaluate '${targetPath2.toString()}'`);
            }
        }
        executionPlan.addStep(new SimpleExecutionPlanStep_1.SimpleExecutionPlanStep());
        return executionPlan;
    }
    pathIsOutsideDependenciesList(path) {
        return this.pathIsOutsideDependenciesListCache.getOrSet(path, () => Iterator.from(this.getDependencies())
            .map(x => x.getDependencyPath())
            .every(dependencyPath => {
            const matching = dependencyPath.matchFor(path);
            return (dependencyPath.isContainCurrentIteration() &&
                matching.find(matchingPart => matchingPart.kind === PathMatcher_1.MatchKind.DifferentSuffix) != undefined);
        }));
    }
}
exports.CalculationFunctionBase = CalculationFunctionBase;
