"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CalculationFunctionBuilder = exports.CalculationsBuilder = void 0;
const ModelPath_1 = require("../../../../../../Common/ModelPath/ModelPath");
const CalculationFunction_1 = require("./CalculationFunction");
class CalculationsBuilder {
    calculationFunctionsBuilders = [];
    set(target) {
        const result = new CalculationFunctionBuilder(target);
        this.calculationFunctionsBuilders.push(result);
        return result;
    }
    applyTo(calculator2) {
        for (const calculationFunctionsBuilder of this.calculationFunctionsBuilders) {
            calculator2.addCalculationFunction(calculationFunctionsBuilder.build());
        }
    }
}
exports.CalculationsBuilder = CalculationsBuilder;
class CalculationFunctionBuilder {
    target;
    dependencies = [];
    targetEvaluator;
    constructor(target) {
        this.target = target;
    }
    dependsOn(...dependencies) {
        this.dependencies.push(...dependencies);
        return this;
    }
    as(targetEvaluator) {
        this.targetEvaluator = targetEvaluator;
        return this;
    }
    build() {
        const targetPath = this.resolveModelPath(this.target);
        if (this.dependencies.length === 0) {
            throw new Error(`У функции автовычисления для ${targetPath.toString()} не объявлено зависимостей`);
        }
        const targetEvaluator = this.targetEvaluator;
        if (targetEvaluator == undefined) {
            throw new Error(`У функции автовычисления для ${targetPath.toString()} не объявлен способ вычисления`);
        }
        return new CalculationFunction_1.CalculationFunction(targetPath, this.dependencies.map(x => this.resolveModelPath(x)), context => targetEvaluator({ get: (path) => context.get(this.resolveModelPath(path)) }));
    }
    resolveModelPath(modelPathAsString) {
        return (0, ModelPath_1.createModelPath)(modelPathAsString, true);
    }
}
exports.CalculationFunctionBuilder = CalculationFunctionBuilder;
