"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KCCountCalculation = void 0;
const PathMatcher_1 = require("../../../../../../Common/ModelPath/PathMatcher");
const AutocalcCommonFunctions_1 = require("../../../../Helpers/AutocalcCommonFunctions");
const KCExpressionRuntime_1 = require("../KCExpressionRuntime");
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
class KCCountCalculation {
    static execute(context, targetPath, dependencyPath, isDependencyAggregation, defaultValue) {
        if ((0, TypingUtils_1.arrayHasLength)(context.relatedEvents, 0)) {
            const expressionRuntime = new KCExpressionRuntime_1.KCExpressionRuntime(context);
            return expressionRuntime.count(dependencyPath.toLegacyPath()) ?? defaultValue;
        }
        const currentValue = (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(context.get(targetPath));
        const nextValue = Iterator.from(context.relatedEvents)
            .filter(evt => dependencyPath
            .matchFor(evt.path)
            .every(x => x.kind === PathMatcher_1.MatchKind.MatchedPath || x.kind === PathMatcher_1.MatchKind.MatchedMultiplicity))
            .reduce((sum, curr) => {
            const oldValue = (0, TypingUtils_1.isNotNullOrEmpty)(curr.oldValue)
                ? isDependencyAggregation
                    ? (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(curr.oldValue)
                    : 1
                : 0;
            const value = (0, TypingUtils_1.isNotNullOrEmpty)(curr.value)
                ? isDependencyAggregation
                    ? (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(curr.value)
                    : 1
                : 0;
            return sum.minus(oldValue).add(value);
        }, currentValue);
        return nextValue.toFixed();
    }
}
exports.KCCountCalculation = KCCountCalculation;
