"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KCSumCalculation = void 0;
const PathMatcher_1 = require("../../../../../../Common/ModelPath/PathMatcher");
const AutocalcCommonFunctions_1 = require("../../../../Helpers/AutocalcCommonFunctions");
const KCExpressionRuntime_1 = require("../KCExpressionRuntime");
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
class KCSumCalculation {
    static execute(context, targetPath, dependencyPath, countPath, defaultValue) {
        if ((0, TypingUtils_1.arrayHasLength)(context.relatedEvents, 0)) {
            const expressionRuntime = new KCExpressionRuntime_1.KCExpressionRuntime(context);
            return expressionRuntime.argument(dependencyPath.toLegacyPath()) ?? defaultValue;
        }
        const countValue = (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(context.get(countPath));
        if (countValue.eq(0)) {
            return defaultValue;
        }
        const currentValue = (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(context.get(context.resolvePath(targetPath)));
        const nextValue = Iterator.from(context.relatedEvents)
            .filter(evt => dependencyPath
            .matchFor(evt.path)
            .every(x => x.kind === PathMatcher_1.MatchKind.MatchedPath || x.kind === PathMatcher_1.MatchKind.MatchedMultiplicity))
            .reduce((sum, curr) => {
            const oldValue = (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(curr.oldValue);
            const value = (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(curr.value);
            return sum.add(value.minus(oldValue));
        }, currentValue);
        return nextValue.toFixed();
    }
}
exports.KCSumCalculation = KCSumCalculation;
