"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CalculationFunctionDependency = void 0;
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const PathMatcher_1 = require("../../../../../Common/ModelPath/PathMatcher");
const ModelPathBuilder_1 = require("../../../../../Common/ModelPath/ModelPathBuilder");
class CalculationFunctionDependency {
    targetPath;
    dependencyPath;
    constructor(targetPath, dependencyPath) {
        this.targetPath = targetPath;
        this.dependencyPath = dependencyPath;
    }
    isMatchToPath(modelPath) {
        return this.dependencyPath.isIncludes(modelPath);
    }
    getDependencyPath() {
        return this.dependencyPath;
    }
    getTargetByChangedPath(changedPath) {
        const changedDependencyPath = new ModelPathBuilder_1.ModelPathBuilder(false);
        const dependencyMatching = this.dependencyPath.matchFor(changedPath);
        for (const dependencyMatch of dependencyMatching) {
            if (dependencyMatch.kind === PathMatcher_1.MatchKind.MatchedPath) {
                changedDependencyPath.append(dependencyMatch.prefix);
            }
            else if (dependencyMatch.kind === PathMatcher_1.MatchKind.MatchedMultiplicity) {
                if (ModelPath_1.PathTokens.isInstanceToken(dependencyMatch.target)) {
                    if (dependencyMatch.source === ModelPath_1.PathTokens.current) {
                        changedDependencyPath.append(dependencyMatch.target);
                    }
                    else if (dependencyMatch.source === ModelPath_1.PathTokens.each) {
                        changedDependencyPath.append(ModelPath_1.PathTokens.each);
                    }
                }
                else {
                    throw new Error("Changed path must be resolved path");
                }
            }
            else {
                return undefined;
            }
        }
        return this.getTargetByChangedChildrenPath(changedDependencyPath.toPath());
    }
    getTargetByChangedChildrenPath(changedPath) {
        const resultPath = new ModelPathBuilder_1.ModelPathBuilder(false);
        const pathMatching = this.targetPath.matchFor(changedPath);
        for (const match of pathMatching) {
            if (match.kind === PathMatcher_1.MatchKind.MatchedPath) {
                resultPath.append(match.prefix);
            }
            else if (match.kind === PathMatcher_1.MatchKind.MatchedMultiplicity) {
                if (ModelPath_1.PathTokens.isMultiToken(match.source) && ModelPath_1.PathTokens.isMultiToken(match.target)) {
                    resultPath.append(match.source);
                }
                else if (ModelPath_1.PathTokens.isSimpleToken(match.source) && ModelPath_1.PathTokens.isMultiToken(match.target)) {
                    resultPath.append(match.source);
                }
                else if (ModelPath_1.PathTokens.isMultiToken(match.source) && ModelPath_1.PathTokens.isSimpleToken(match.target)) {
                    resultPath.append(match.target);
                }
            }
            else {
                if (match.source != undefined) {
                    resultPath.append(match.source);
                }
                break;
            }
        }
        return resultPath.toPath();
    }
}
exports.CalculationFunctionDependency = CalculationFunctionDependency;
