import { AbsoluteModelPath } from "../../../../../Common/ModelPath/ModelPath";
import { BindingPath } from "../CoreTypes";
import { IErrorManager } from "../ErrorManager/ErrorManager";
import { ModelEvent } from "../EventManagement/ModelEvents/ModelEvent";
import { IModel } from "../ModelStoring/Model";
import { Predicate } from "../../../../../Common/Funcs";
import { IFormReadonlyModelAccessor } from "../../Form/IFormModelAccessor";
import { DataDeclarationHelper } from "../DataDeclaration/DataDeclarationHelper";
import { IPicklistsProvider } from "../Picklists/IPicklistsProvider";
import { IExternalPropertiesProvider } from "../../Form/IExternalPropertiesProvider";
import { ICalculationFunction } from "./ICalculationFunction";
import { IIgnoredPathChecker } from "./IIgnoredPathChecker";
export interface ICalculator2 {
    addCalculationFunction(calculationFunction: ICalculationFunction): void;
    process(model: IModel, eventArg: ModelEvent, acceptFunction?: Predicate<ICalculationFunction>): Promise<void>;
    processSync(model: IModel, eventArg: ModelEvent, acceptFunction?: Predicate<ICalculationFunction>): void;
    processEvents(model: IModel, modelChangeEventInfos: ModelEvent[]): Promise<void>;
    calculateAll(model: IModel, acceptTarget: Predicate<AbsoluteModelPath>, acceptFunction?: Predicate<ICalculationFunction>): Promise<void>;
    calculateFunctions(model: IModel, acceptFunction: Predicate<ICalculationFunction>): Promise<void>;
    recalculateTargets(model: IModel, paths: readonly BindingPath[]): Promise<void>;
    revalidateTargets(model: IModel, paths: AbsoluteModelPath[]): Promise<void>;
    getArgumentsPaths(model: IFormReadonlyModelAccessor, calcPath: BindingPath): BindingPath[];
    ensurePrepared(): void;
    getFunctionsByTarget(path: AbsoluteModelPath): ICalculationFunction[];
    clearFunctions(): void;
}
export declare class Calculator2 implements ICalculator2 {
    private calculationFunctions;
    private readonly picklistsProvider;
    private readonly ignoredPathChecker;
    private readonly errorManager;
    private readonly dataDeclarationHelper;
    private cachedNextFunctions;
    private readonly externalPropertiesProvider;
    constructor(errorManager: IErrorManager, dataDeclarationHelper: DataDeclarationHelper, ignoredPathChecker: IIgnoredPathChecker, picklistsProvider: IPicklistsProvider, externalPropertiesProvider: IExternalPropertiesProvider);
    clearFunctions(): void;
    getFunctionsByTarget(path: AbsoluteModelPath): ICalculationFunction[];
    addCalculationFunction(calculationFunction: ICalculationFunction): void;
    processEvents(model: IModel, modelChangeEventInfos: ModelEvent[]): Promise<void>;
    calculateFunctions(model: IModel, acceptFunction: Predicate<ICalculationFunction>): Promise<void>;
    calculateAll(model: IModel, acceptTarget: Predicate<AbsoluteModelPath>, acceptFunction?: Predicate<ICalculationFunction>): Promise<void>;
    revalidateTargets(model: IModel, targetPaths: AbsoluteModelPath[]): Promise<void>;
    recalculateTargets(model: IModel, targetLegacyPaths: readonly string[]): Promise<void>;
    processEvent(model: IModel, modelChangeEventInfo: ModelEvent, acceptFunction: Predicate<ICalculationFunction>): Promise<void>;
    processEventSync(model: IModel, modelChangeEventInfo: ModelEvent, acceptFunction: Predicate<ICalculationFunction>): void;
    process(model: IModel, eventArg: ModelEvent, acceptFunction?: Predicate<ICalculationFunction>): Promise<void>;
    processSync(model: IModel, eventArg: ModelEvent, acceptFunction?: Predicate<ICalculationFunction>): void;
    private getAddedChildren;
    private getExecutionPlan;
    private dfs;
    private linkTargetNodeByDependencyToNode;
    private addToGraph;
    private buildFunctionsGraphAndGetDependencyMap;
    private buildCachedOrderedFunctions;
    ensurePrepared(): void;
    private getOrderedFunctions;
    private getMaskDirty;
    private getOrderedFunctionsByGraph;
    private getNextFunctions;
    private getAllOrderedFunctions;
    private dropCache;
    getArgumentsPaths(model: IModel, calcPath: BindingPath): BindingPath[];
    private getUnwrappedAggregationTargets;
    private createEvaluationContext;
}
