"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runCalculatorFloatingAsyncAction = runCalculatorFloatingAsyncAction;
exports.createSimpleAsyncActionQueue = createSimpleAsyncActionQueue;
exports.createActionQueue = createActionQueue;
const utils_1 = require("../../utils");
const Deferred_1 = require("../../../Helpers/Deferred");
const InitializeSentry_1 = require("../../../Forms/Sentry/InitializeSentry");
function runCalculatorFloatingAsyncAction(action) {
    // eslint-disable-next-line @typescript-eslint/no-floating-promises
    action();
}
function createSimpleAsyncActionQueue(errorHandler, queueDelay) {
    const activeTaskQueue = [];
    const enqueue = (action) => {
        activeTaskQueue.push(action);
        if (activeTaskQueue.length === 1) {
            runCalculatorFloatingAsyncAction(async () => {
                while (activeTaskQueue.length > 0) {
                    const currentTask = activeTaskQueue[0];
                    if (currentTask != undefined) {
                        try {
                            await currentTask();
                        }
                        catch (e) {
                            errorHandler(e);
                        }
                        if (queueDelay != undefined) {
                            await (0, utils_1.delay)(queueDelay);
                        }
                    }
                    activeTaskQueue.shift();
                }
            });
        }
    };
    const enqueueWithPromise = (action) => {
        const deferred = new Deferred_1.Deferred();
        const wrappedAction = async () => {
            try {
                deferred.resolve(await action());
            }
            catch (e) {
                deferred.reject(e);
            }
        };
        enqueue(wrappedAction);
        return deferred.promise;
    };
    return {
        enqueue: enqueue,
        enqueueWithPromise: enqueueWithPromise,
        getQueueLength: () => activeTaskQueue.length,
    };
}
function createActionQueue(name, queueDelay) {
    return createSimpleAsyncActionQueue(error => (0, InitializeSentry_1.getCandyHub)().captureException(error, { data: { queueName: name } }), queueDelay);
}
