"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IterateOverChildrenPlanStep = void 0;
const IterationContext_1 = require("../ModelPath/IterationContext");
const Errors_1 = require("../../../../../../Common/Errors");
class IterateOverChildrenPlanStep {
    pathToList;
    sync;
    instancesToIterate;
    constructor(pathToList, instancesToIterate, sync) {
        this.sync = sync;
        this.pathToList = pathToList;
        this.instancesToIterate = instancesToIterate;
    }
    getStepExecutor() {
        return (this.sync ?? false)
            ? this.getSyncCalculationActionDecorator()
            : this.getAsyncCalculationActionDecorator();
    }
    getAsyncCalculationActionDecorator() {
        const pathToList = this.pathToList;
        return (action) => async (context) => {
            const instances = this.instancesToIterate ?? context.getInstances(pathToList) ?? [];
            for (const instance of instances) {
                const newContext = context.pushIterationContext(new IterationContext_1.IterationContext(pathToList, instance));
                await action(newContext);
            }
        };
    }
    getSyncCalculationActionDecorator() {
        const pathToList = this.pathToList;
        return (action) => (context) => {
            const instances = this.instancesToIterate ?? context.getInstances(pathToList) ?? [];
            for (const instance of instances) {
                const newContext = context.pushIterationContext(new IterationContext_1.IterationContext(pathToList, instance));
                const result = action(newContext);
                if (result != undefined) {
                    throw new Errors_1.InvalidProgramStateError();
                }
            }
        };
    }
    toDebugString() {
        return `iterate(${this.pathToList.toString()})`;
    }
}
exports.IterateOverChildrenPlanStep = IterateOverChildrenPlanStep;
