import { AbsoluteModelPath } from "../../../../../Common/ModelPath/ModelPath";
import { IEvaluationContext } from "./IEvaluationContext";
export type PlainModelValue = string | undefined;
export interface IKCExpressionRuntime {
    when(condition: boolean | undefined, body: () => boolean | undefined | Promise<boolean | undefined>): boolean | undefined | Promise<boolean | undefined>;
    or(conditions: Array<boolean | undefined>): boolean | undefined;
    and(conditions: Array<boolean | undefined>): boolean | undefined;
    not(condition: boolean | undefined): boolean | undefined;
    eq(left: PlainModelValue | Date, right: PlainModelValue | Date): boolean | undefined;
    gt(left: PlainModelValue | Date, right: PlainModelValue | Date): boolean | undefined;
    lt(left: PlainModelValue | Date, right: PlainModelValue | Date): boolean | undefined;
    ge(left: PlainModelValue | Date, right: PlainModelValue | Date): boolean | undefined;
    le(left: PlainModelValue | Date, right: PlainModelValue | Date): boolean | undefined;
    const(value: PlainModelValue): PlainModelValue;
    argument(modelPath: string | AbsoluteModelPath): PlainModelValue;
    count(modelPath: string | AbsoluteModelPath, withNullOrEmptyValues?: boolean): PlainModelValue;
    instanceCount(modelPath: string | AbsoluteModelPath, withNullOrEmptyValues?: boolean): PlainModelValue;
    exists(modelPath: string | AbsoluteModelPath): boolean;
    isValueEqlAutoValue(modelPath: string | AbsoluteModelPath): boolean | undefined;
    isSnils(modelPath: string | AbsoluteModelPath): boolean;
    isRegNumSfr(modelPath: string | AbsoluteModelPath): boolean;
    isInn(modelPath: string | AbsoluteModelPath): boolean;
    isOgrn(modelPath: string | AbsoluteModelPath): boolean;
    isValidMirCardNumber(modelPath: string | AbsoluteModelPath): boolean;
    isValidAccountNumber(bikPath: string | AbsoluteModelPath, accountPath: string | AbsoluteModelPath): boolean;
    length(value: PlainModelValue): PlainModelValue;
    regexMatch(value: PlainModelValue, pattern: string): boolean;
    multySum(modelPath: string | AbsoluteModelPath): PlainModelValue;
    groupSum(targetKeys: PlainModelValue | undefined, sourceKeyPath: string, sourceValuePath: string): PlainModelValue;
    groupCount(targetKeys: PlainModelValue | undefined, sourceKeyPath: string): PlainModelValue;
    multiply(values: PlainModelValue[]): PlainModelValue;
    division(values: PlainModelValue[]): PlainModelValue;
    sum(values: PlainModelValue[]): PlainModelValue;
    minus(value: PlainModelValue): PlainModelValue;
    abs(value: PlainModelValue): PlainModelValue;
    xabs(value: PlainModelValue): PlainModelValue;
    choose(condition: boolean | undefined, thenBlock: () => PlainModelValue, elseBlock: () => PlainModelValue): PlainModelValue;
    substring(modelPath: PlainModelValue, start: number, length: number): PlainModelValue;
    concat(values: PlainModelValue[]): PlainModelValue;
    dateNow(): Date;
    getDaysInMonth(value: PlainModelValue): PlainModelValue;
    getDay(value: PlainModelValue): PlainModelValue;
    getMonth(value: PlainModelValue): PlainModelValue;
    getYear(value: PlainModelValue): PlainModelValue;
    isDate(value: PlainModelValue): boolean | undefined;
    differenceInMonths(values: PlainModelValue[]): PlainModelValue;
    differenceInDays(values: PlainModelValue[]): PlainModelValue;
    addDays(value: PlainModelValue, amount: number): Date | undefined;
    addMonths(value: PlainModelValue, amount: number): Date | undefined;
    addYears(value: PlainModelValue, amount: number): Date | undefined;
    dateToString(value: Date | undefined, format: string): PlainModelValue;
    dateTime(value: PlainModelValue): Date | undefined;
    oneOf(value: PlainModelValue, items: PlainModelValue[]): boolean;
    join(values: PlainModelValue[]): string;
    getProperty(name: string): string;
    getPicklistValues(gid: string, filterColumns: string[], filterValues: PlainModelValue[], resultColumnName: string): Promise<string[]>;
    round(value: PlainModelValue, fractional?: number): PlainModelValue;
    floor(value: PlainModelValue): PlainModelValue;
}
export declare class KCExpressionRuntime implements IKCExpressionRuntime {
    private readonly context;
    constructor(context: IEvaluationContext);
    join(values: PlainModelValue[]): string;
    getProperty(name: string): string;
    getPicklistValues(gid: string, filterColumns: string[], filterValues: PlainModelValue[], resultColumnName: string): Promise<string[]>;
    argument(modelPath: string): string | undefined;
    count(modelPath: string | AbsoluteModelPath, withNullOrEmptyValues?: boolean): string | undefined;
    instanceCount(modelPath: string | AbsoluteModelPath): string | undefined;
    const(value: PlainModelValue): PlainModelValue;
    eq(left: PlainModelValue | Date, right: PlainModelValue | Date): boolean | undefined;
    gt(left: PlainModelValue | Date, right: PlainModelValue | Date): boolean | undefined;
    lt(left: PlainModelValue | Date, right: PlainModelValue | Date): boolean | undefined;
    ge(left: PlainModelValue | Date, right: PlainModelValue | Date): boolean | undefined;
    le(left: PlainModelValue | Date, right: PlainModelValue | Date): boolean | undefined;
    when(condition: boolean | undefined, body: () => boolean | undefined | Promise<boolean | undefined>): boolean | undefined | Promise<boolean | undefined>;
    or(conditions: Array<boolean | undefined>): boolean | undefined;
    and(conditions: Array<boolean | undefined>): boolean | undefined;
    not(condition: boolean | undefined): boolean | undefined;
    exists(path: string | AbsoluteModelPath): boolean;
    isValueEqlAutoValue(path: string | AbsoluteModelPath): boolean | undefined;
    isSnils(path: string | AbsoluteModelPath): boolean;
    isInn(path: string | AbsoluteModelPath): boolean;
    isOgrn(path: string | AbsoluteModelPath): boolean;
    isRegNumSfr(path: string | AbsoluteModelPath): boolean;
    isValidAccountNumber(bikPath: string | AbsoluteModelPath, accountPath: string | AbsoluteModelPath): boolean;
    isValidMirCardNumber(path: string | AbsoluteModelPath): boolean;
    length(value: PlainModelValue): PlainModelValue;
    regexMatch(value: PlainModelValue, pattern: string): boolean;
    multySum(modelPath: string | AbsoluteModelPath): PlainModelValue;
    private unpackPlainModelValue;
    private unpackGroupKeys;
    groupCount(targetKeysSource: PlainModelValue | undefined, sourceKeyPath: string): PlainModelValue;
    groupSum(targetKeysSource: PlainModelValue | undefined, sourceKeyPath: string, sourceValuePath: string): PlainModelValue;
    multiply(values: PlainModelValue[]): PlainModelValue;
    division(values: PlainModelValue[]): PlainModelValue;
    sum(values: PlainModelValue[]): PlainModelValue;
    minus(value: PlainModelValue): PlainModelValue;
    abs(value: PlainModelValue): PlainModelValue;
    xabs(value: PlainModelValue): PlainModelValue;
    round(value: PlainModelValue, fractional?: number): PlainModelValue;
    floor(value: PlainModelValue): PlainModelValue;
    choose(condition: boolean | undefined, thenBlock: () => PlainModelValue, elseBlock: () => PlainModelValue): PlainModelValue;
    dateNow(): Date;
    getDaysInMonth(value: PlainModelValue): PlainModelValue;
    getDay(value: PlainModelValue): PlainModelValue;
    getMonth(value: PlainModelValue): PlainModelValue;
    getYear(value: PlainModelValue): PlainModelValue;
    isDate(value: PlainModelValue): boolean | undefined;
    differenceInMonths(values: PlainModelValue[]): PlainModelValue;
    differenceInDays(values: PlainModelValue[]): PlainModelValue;
    addDays(value: PlainModelValue, amount: number): Date | undefined;
    addMonths(value: PlainModelValue, amount: number): Date | undefined;
    addYears(value: PlainModelValue, amount: number): Date | undefined;
    dateToString(date: Date | undefined, format: string): PlainModelValue;
    dateTime(value: PlainModelValue): Date | undefined;
    substring(value: PlainModelValue | Date, start: number, length: number): PlainModelValue;
    concat(values: PlainModelValue[]): PlainModelValue;
    oneOf(value: PlainModelValue, items: PlainModelValue[]): boolean;
    private getSingleValueOrSumByAllInstances;
    private joinIfArray;
    private getCountByAllInstances;
    private checkExistsInAllInstances;
}
