"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KCExpressionRuntime = void 0;
const big_js_1 = __importDefault(require("big.js"));
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const AutocalcCommonFunctions_1 = require("../../../Helpers/AutocalcCommonFunctions");
const ModelPathHelper_1 = require("../../../../../Common/ModelPath/ModelPathHelper");
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const ValidationHelper_1 = require("../../ValidationHelper");
const DateHelpers_1 = require("../../../Helpers/DateHelpers");
const DigestQueryFilter_1 = require("../../../../../Common/CandyApiClient/DigestApiClient/Types/DigestQueryFilter");
class KCExpressionRuntime {
    context;
    constructor(context) {
        this.context = context;
    }
    join(values) {
        return values.join("");
    }
    getProperty(name) {
        const settingValue = this.context.getExternalPropertiesProvider().getExternalProperty(`@settings/${name}`);
        if (settingValue != null && settingValue != "") {
            return settingValue.toString();
        }
        const featureFlag = this.context.getExternalPropertiesProvider().getExternalProperty(`@featureFlags/${name}`);
        if (featureFlag != null && featureFlag != "") {
            return featureFlag.toString();
        }
        const requisiteProperty = this.context.getExternalPropertiesProvider().getExternalProperty(`@${name}`);
        if (requisiteProperty != null && requisiteProperty != "") {
            return requisiteProperty.toString();
        }
        return "";
    }
    async getPicklistValues(gid, filterColumns, filterValues, resultColumnName) {
        // NOTE В общем, за каким-то хером при компиляции массива его делают JSON.stringify, я подозреваю, зачем это, но
        // сейчас не хочу с этим разбираться, на тот случай, если это исправят тут так, через if
        if (typeof filterColumns === "string") {
            filterColumns = JSON.parse(filterColumns);
        }
        if (typeof filterValues === "string") {
            filterValues = JSON.parse(filterValues);
        }
        const queryResult = (await this.context
            .getPicklistProvider()
            .getPicklist({ gId: gid })
            ?.query({
            filter: {
                operator: DigestQueryFilter_1.DigestLogicalOperator.And,
                operands: filterColumns.map((column, index) => ({
                    operator: DigestQueryFilter_1.DigestFilterOperator.Equal,
                    column: column,
                    value: filterValues[index]?.toString() ?? "",
                })),
            },
        })) ?? (0, TypingUtils_1.reject)("Failed to query picklist");
        return queryResult.items.map(x => x[resultColumnName]?.toString() ?? "");
    }
    argument(modelPath) {
        return this.getSingleValueOrSumByAllInstances(modelPath);
    }
    count(modelPath, withNullOrEmptyValues) {
        return this.getCountByAllInstances((0, ModelPath_1.convertToAbsoluteModelPath)(modelPath), withNullOrEmptyValues);
    }
    instanceCount(modelPath) {
        return this.context
            .getInstanceCount(this.context.resolvePath((0, ModelPath_1.convertToAbsoluteModelPath)(modelPath)))
            .toString();
    }
    const(value) {
        return value;
    }
    eq(left, right) {
        if (left instanceof Date || right instanceof Date) {
            if (!(left instanceof Date && right instanceof Date)) {
                return undefined;
            }
            return left.getTime() === right.getTime();
        }
        return left === right;
    }
    gt(left, right) {
        if (left instanceof Date || right instanceof Date) {
            if (!(left instanceof Date && right instanceof Date)) {
                return undefined;
            }
            return left > right;
        }
        return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).gt((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(right));
    }
    lt(left, right) {
        if (left instanceof Date || right instanceof Date) {
            if (!(left instanceof Date && right instanceof Date)) {
                return undefined;
            }
            return left < right;
        }
        return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).lt((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(right));
    }
    ge(left, right) {
        if (left instanceof Date || right instanceof Date) {
            if (!(left instanceof Date && right instanceof Date)) {
                return undefined;
            }
            return left >= right;
        }
        return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).gte((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(right));
    }
    le(left, right) {
        if (left instanceof Date || right instanceof Date) {
            if (!(left instanceof Date && right instanceof Date)) {
                return undefined;
            }
            return left <= right;
        }
        return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(left).lte((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(right));
    }
    when(condition, body) {
        if (condition) {
            const b = body();
            return b;
        }
        return true;
    }
    or(conditions) {
        return conditions.reduce((acc, current) => (acc == undefined || current == undefined ? undefined : acc || current), false);
    }
    and(conditions) {
        return conditions.reduce((acc, current) => (acc == undefined || current == undefined ? undefined : acc && current), true);
    }
    not(condition) {
        return !condition;
    }
    exists(path) {
        return this.checkExistsInAllInstances((0, ModelPath_1.convertToAbsoluteModelPath)(path));
    }
    isValueEqlAutoValue(path) {
        const value = this.context.get((0, ModelPath_1.convertToAbsoluteModelPath)(path));
        const autoValue = this.context.getAutoValue((0, ModelPath_1.convertToAbsoluteModelPath)(path));
        return this.eq(value, autoValue);
    }
    isSnils(path) {
        const value = this.context.get((0, ModelPath_1.convertToAbsoluteModelPath)(path));
        const normalizedSnils = value?.replace(/[^0-9]/g, "");
        if ((0, TypingUtils_1.isNullOrEmpty)(normalizedSnils)) {
            return false;
        }
        const result = ValidationHelper_1.ValidationHelper.isSnils(normalizedSnils);
        return result;
    }
    isInn(path) {
        const value = this.context.get((0, ModelPath_1.convertToAbsoluteModelPath)(path));
        const result = ValidationHelper_1.ValidationHelper.isInn(value);
        return result;
    }
    isOgrn(path) {
        const value = this.context.get((0, ModelPath_1.convertToAbsoluteModelPath)(path));
        const result = ValidationHelper_1.ValidationHelper.isOgrn(value);
        return result;
    }
    isRegNumSfr(path) {
        const value = this.context.get((0, ModelPath_1.convertToAbsoluteModelPath)(path));
        const normalizedRegNum = value?.replace(/[^0-9]/g, "");
        if ((0, TypingUtils_1.isNullOrEmpty)(normalizedRegNum)) {
            return false;
        }
        const result = ValidationHelper_1.ValidationHelper.isRegNumSfr(normalizedRegNum);
        return result;
    }
    isValidAccountNumber(bikPath, accountPath) {
        const bikValue = this.context.get((0, ModelPath_1.convertToAbsoluteModelPath)(bikPath));
        const accountValue = this.context.get((0, ModelPath_1.convertToAbsoluteModelPath)(accountPath));
        if ((0, TypingUtils_1.isNullOrEmpty)(bikValue) || (0, TypingUtils_1.isNullOrEmpty)(accountValue)) {
            return false;
        }
        const result = ValidationHelper_1.ValidationHelper.isValidAccountNumber(bikValue, accountValue);
        return result;
    }
    isValidMirCardNumber(path) {
        const value = this.context.get((0, ModelPath_1.convertToAbsoluteModelPath)(path));
        const normalizedCard = value?.replace(/[^0-9]/g, "");
        if ((0, TypingUtils_1.isNullOrEmpty)(normalizedCard)) {
            return false;
        }
        const result = ValidationHelper_1.ValidationHelper.isValidMirCardNumber(normalizedCard);
        return result;
    }
    length(value) {
        return (0, TypingUtils_1.isNotNullOrUndefined)(value) ? value.length.toString() : "0";
    }
    regexMatch(value, pattern) {
        return value != undefined && new RegExp(pattern).test(value);
    }
    multySum(modelPath) {
        return this.getSingleValueOrSumByAllInstances(modelPath);
    }
    unpackPlainModelValue(source) {
        if (source == undefined) {
            return source;
        }
        if (IterableUtils_1.IterUtils.first(source) === "[" && IterableUtils_1.IterUtils.last(source) === "]") {
            return JSON.parse(source);
        }
        return source;
    }
    unpackGroupKeys(source) {
        const targetKeys = this.unpackPlainModelValue(source);
        const groupKeys = (Array.isArray(targetKeys) ? targetKeys : [targetKeys])
            .map(x => {
            const value = this.unpackPlainModelValue(x?.toString());
            return Array.isArray(value) ? value : [value];
        })
            .flat()
            .map(x => (x != undefined ? `${x}` : undefined));
        return groupKeys;
    }
    groupCount(targetKeysSource, sourceKeyPath) {
        const groupKeys = new Set(this.unpackGroupKeys(targetKeysSource));
        const sourceKeyUnresolvedPath = (0, ModelPath_1.createFromMask)(sourceKeyPath, true, ModelPath_1.PathTokens.each);
        const contextForSource = this.context.resolvePath(sourceKeyUnresolvedPath).isContainIteration()
            ? this.context
            : this.context.popIterationContext();
        const sourceKeyModelPath = contextForSource.resolvePath(sourceKeyUnresolvedPath);
        const count = Iterator.from(IterableUtils_1.IterUtils.iterateEntries(contextForSource.getAllResolvedValues(sourceKeyModelPath)))
            .filter(([index, key]) => groupKeys.has(key))
            .reduce((acc, _) => acc.add(1), (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(0));
        return count?.toFixed();
    }
    groupSum(targetKeysSource, sourceKeyPath, sourceValuePath) {
        const groupKeys = new Set(this.unpackGroupKeys(targetKeysSource));
        const sourceKeyUnresolvedPath = (0, ModelPath_1.createFromMask)(sourceKeyPath, true, ModelPath_1.PathTokens.each);
        const sourceValueUnresolvedPath = (0, ModelPath_1.createFromMask)(sourceValuePath, true, ModelPath_1.PathTokens.each);
        const contextForSource = this.context.resolvePath(sourceKeyUnresolvedPath).isContainIteration()
            ? this.context
            : this.context.popIterationContext();
        const sourceKeyModelPath = contextForSource.resolvePath(sourceKeyUnresolvedPath);
        const sourceValueModelPath = contextForSource.resolvePath(sourceValueUnresolvedPath);
        const keysMatch = {};
        for (const [index, key] of IterableUtils_1.IterUtils.iterateEntries(contextForSource.getAllResolvedValues(sourceKeyModelPath))) {
            keysMatch[index] = groupKeys.has(key);
        }
        return IterableUtils_1.IterUtils.reduce(contextForSource.getAllResolvedValues(sourceValueModelPath), (prev, value, i) => {
            if (!keysMatch[i]) {
                return prev;
            }
            return (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(prev).add((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value));
        }, undefined)?.toFixed();
    }
    multiply(values) {
        const result = values.map(x => (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(x)).reduce((prev, value) => prev.mul(value), (0, big_js_1.default)(1));
        return result?.toFixed();
    }
    division(values) {
        const first = values[0];
        const rest = values.slice(1);
        const result = rest
            .map(x => (x == undefined || x === "" ? undefined : (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(x)))
            .reduce((prev, value) => {
            if (prev == undefined || value == undefined || value.eq(0)) {
                return undefined;
            }
            return prev?.div(value);
        }, first != undefined ? (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(first) : (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(0));
        return result?.toFixed();
    }
    sum(values) {
        const result = values.reduce((prev, value) => (value === "" || value == undefined) && prev == undefined
            ? undefined
            : (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value).add((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(prev)), undefined);
        return result?.toFixed();
    }
    minus(value) {
        return (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, big_js_1.default)(0).minus((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value)).toFixed() : undefined;
    }
    abs(value) {
        return (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value).abs().toFixed() : undefined;
    }
    xabs(value) {
        if ((0, TypingUtils_1.isNullOrEmpty)(value)) {
            return undefined;
        }
        const wrappedValue = (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value);
        return wrappedValue.lt(0) ? "0" : wrappedValue.abs().toFixed();
    }
    round(value, fractional) {
        return (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value).round(fractional).toFixed() : undefined;
    }
    floor(value) {
        return (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value).round(undefined, 0).toFixed() : undefined;
    }
    choose(condition, thenBlock, elseBlock) {
        return condition != undefined ? (condition ? thenBlock() : elseBlock()) : undefined;
    }
    dateNow() {
        return new Date();
    }
    getDaysInMonth(value) {
        const date = (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, DateHelpers_1.convertDate)(value) : undefined;
        if (!date) {
            return undefined;
        }
        return (0, DateHelpers_1.getLastDateOfMonth)(date).toString();
    }
    getDay(value) {
        const date = (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, DateHelpers_1.convertDate)(value) : undefined;
        if (!date) {
            return undefined;
        }
        return date.getDate().toString();
    }
    getMonth(value) {
        const date = (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, DateHelpers_1.convertDate)(value) : undefined;
        if (!date) {
            return undefined;
        }
        return (date.getMonth() + 1).toString();
    }
    getYear(value) {
        const date = (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, DateHelpers_1.convertDate)(value) : undefined;
        if (!date) {
            return undefined;
        }
        return date.getFullYear().toString();
    }
    isDate(value) {
        const date = (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, DateHelpers_1.convertDate)(value) : undefined;
        return (0, DateHelpers_1.isDate)(date);
    }
    differenceInMonths(values) {
        const leftDate = values[0] ? (0, DateHelpers_1.convertDate)(values[0]) : undefined;
        const rightDate = values[1] ? (0, DateHelpers_1.convertDate)(values[1]) : undefined;
        if (!leftDate || !rightDate) {
            return undefined;
        }
        return (0, DateHelpers_1.differenceInMonths)(leftDate, rightDate).toString();
    }
    differenceInDays(values) {
        const leftDate = values[0] ? (0, DateHelpers_1.convertDate)(values[0]) : undefined;
        const rightDate = values[1] ? (0, DateHelpers_1.convertDate)(values[1]) : undefined;
        if (!leftDate || !rightDate) {
            return undefined;
        }
        return (0, DateHelpers_1.differenceInDays)(leftDate, rightDate).toString();
    }
    addDays(value, amount) {
        const date = (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, DateHelpers_1.convertDate)(value) : undefined;
        if (!date) {
            return undefined;
        }
        return (0, DateHelpers_1.addDays)(date, amount);
    }
    addMonths(value, amount) {
        const date = (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, DateHelpers_1.convertDate)(value) : undefined;
        if (!date) {
            return undefined;
        }
        return (0, DateHelpers_1.addMonths)(date, amount);
    }
    addYears(value, amount) {
        const date = (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, DateHelpers_1.convertDate)(value) : undefined;
        if (!date) {
            return undefined;
        }
        return (0, DateHelpers_1.addYears)(date, amount);
    }
    dateToString(date, format) {
        let dateFormat = format;
        if (format === "fss") {
            dateFormat = "yyyy-MM-dd";
        }
        return date != undefined ? (0, DateHelpers_1.dateToString)(date, dateFormat) : undefined;
    }
    dateTime(value) {
        const date = (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, DateHelpers_1.convertDate)(value) : undefined;
        if (!date) {
            return undefined;
        }
        return date;
    }
    substring(value, start, length) {
        if (value instanceof Date) {
            return value.toLocaleDateString("ru-RU").substr(start, length);
        }
        return value?.substr(start, length);
    }
    concat(values) {
        const result = values.join("");
        return result;
    }
    oneOf(value, items) {
        return items.some(x => x === value);
    }
    getSingleValueOrSumByAllInstances(path) {
        const modelPath = this.context.resolvePath((0, ModelPath_1.convertToAbsoluteModelPath)(path));
        if (modelPath.isResolved()) {
            return this.joinIfArray(this.context.get(modelPath));
        }
        const finalValue = modelPath.isContainIteration()
            ? Iterator.from(this.context.getAllResolvedValues(modelPath))
                .reduce((prev, value) => (0, TypingUtils_1.isNotNullOrEmpty)(value) ? (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(value).add((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(prev)) : prev, undefined)
                ?.toFixed()
            : this.context.get(modelPath);
        return finalValue;
    }
    joinIfArray(value) {
        try {
            const array = JSON.parse(value ?? (0, TypingUtils_1.reject)());
            return array.join(", ");
        }
        catch {
            return value;
        }
    }
    getCountByAllInstances(path, withNullOrEmptyValues) {
        const modelPath = this.context.resolvePath(path);
        // if (IterUtils.last(modelPath.getPathParts()) == PathTokens.each) {
        //     let number = IterUtils.count(this.context.getInstances(modelPath.getParentPath()));
        //     return number.toString();
        // }
        const finalValue = modelPath.isContainIteration()
            ? Iterator.from(this.context.getAllResolvedValues(modelPath))
                .reduce((prev, value) => withNullOrEmptyValues === true || (0, TypingUtils_1.isNotNullOrEmpty)(value)
                ? (0, AutocalcCommonFunctions_1.wrapDecimalOperand)(1).add((0, AutocalcCommonFunctions_1.wrapDecimalOperand)(prev))
                : prev, undefined)
                ?.toFixed()
            : (0, ModelPathHelper_1.isAggregationPath)(modelPath)
                ? this.context.get(modelPath)
                : (0, TypingUtils_1.isNotNullOrEmpty)(this.context.get(modelPath))
                    ? "1"
                    : "0";
        return finalValue;
    }
    checkExistsInAllInstances(modelPath) {
        const resolvedModelPath = this.context.resolvePath(modelPath);
        const isMultiplicityPath = resolvedModelPath.isContainIteration();
        return Iterator.from(this.context.getAllResolvedValues(modelPath)).some(x => isMultiplicityPath || !(0, TypingUtils_1.isNullOrEmpty)(x));
    }
}
exports.KCExpressionRuntime = KCExpressionRuntime;
