"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DependenciesRequiredAreEmpty = void 0;
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
const MaskResolver_1 = require("../../PathUtils/MaskResolver");
const ValidationFunction_1 = require("./ValidationFunction");
class DependenciesRequiredAreEmpty {
    static checkDependenciesAreEmpty = (context, dependencies) => dependencies.every(x => {
        const resolvedPath = context.resolvePath(x).toAbsolute();
        if (!resolvedPath.isResolved()) {
            return true;
        }
        const allResolvedPaths = MaskResolver_1.MaskResolver.resolvePath(context.model, resolvedPath);
        for (const path of allResolvedPaths) {
            if (path.isContainCurrentIteration()) {
                const allInstancesEmpty = Iterator.from(context.getAllResolvedValues(path.toEachIteration())).every(TypingUtils_1.isNullOrEmpty);
                if (!allInstancesEmpty) {
                    return false;
                }
            }
            else {
                const value = context.model.getValue(path);
                if (!(0, TypingUtils_1.isNullOrEmpty)(value)) {
                    return false;
                }
            }
        }
        return true;
    });
    static checkValueNotEmpty = (context, path) => Iterator.from(context.getAllResolvedValues(path)).some(v => !(0, TypingUtils_1.isNullOrEmpty)(v));
    static getMessage(localizer, message) {
        if (message == undefined) {
            return undefined;
        }
        if (typeof message === "string") {
            return message;
        }
        return message(localizer);
    }
    static execute(context, path, dependencies, message, severity, localization) {
        const isValid = this.checkValueNotEmpty(context, path) || this.checkDependenciesAreEmpty(context, dependencies);
        if (isValid) {
            return ValidationFunction_1.ValidationFunctionResults.Valid();
        }
        else {
            return ValidationFunction_1.ValidationFunctionResults.Invalid(this.getMessage(localization, message) ?? "Поле должно быть заполнено", severity);
        }
    }
}
exports.DependenciesRequiredAreEmpty = DependenciesRequiredAreEmpty;
