import { AbsoluteModelPath, ModelPath } from "../../../../../../Common/ModelPath/ModelPath";
import { CalculationFunctionBase } from "../AutoCalculations/CalculationFunctionBase";
import { CalculationFunctionDependency } from "../CalculationFunctionDependency";
import { CalculationTarget } from "../CalculationTarget";
import { IEvaluationContext } from "../IEvaluationContext";
import { ModelAccessor, ModelCountAccessor } from "../../../../Controls/withEvaluators";
export type ValidationFunctionEvaluator = (evaluationContext: IEvaluationContext, targetPath: ModelPath) => ValidationFunctionResult | Promise<ValidationFunctionResult>;
export type ValidationFunctionResult = {
    valid: true;
} | {
    valid: false;
    message: string;
    severity: ValidationSeverity;
};
export declare enum ValidationSeverity {
    Error = "Error",
    Warning = "Warning"
}
export declare class ValidationFunctionResults {
    static Valid(): ValidationFunctionResult;
    static Invalid(message: string, severity?: ValidationSeverity): ValidationFunctionResult;
}
export declare class ValidationFunction extends CalculationFunctionBase {
    private readonly calculationTarget;
    private readonly dependencies;
    private readonly evaluator;
    private readonly source?;
    private readonly allDependencies;
    constructor(targetPath: AbsoluteModelPath, dependenciesPaths: AbsoluteModelPath[], evaluator: ValidationFunctionEvaluator, source?: string);
    getDependencies(): readonly CalculationFunctionDependency[];
    static conditionalValidatorEvaluator(validateIfCondition: undefined | ((evaluationContext: IEvaluationContext) => boolean), evaluator: ValidationFunctionEvaluator): ValidationFunctionEvaluator;
    static javascriptSugarExpressionToEvaluator(form: IForm, conditionContextAndDeps: Array<[
        condition: (path: ModelAccessor, count: ModelCountAccessor) => boolean,
        conditionContext: AbsoluteModelPath
    ]>): undefined | ((evaluationContext: IEvaluationContext) => boolean);
    execute(context: IEvaluationContext): Promise<void>;
    isDependsOn(calculationTarget: CalculationTarget): boolean;
    getTarget(): CalculationTarget;
    asDebugString(): string;
}
