"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidationFunction = exports.ValidationFunctionResults = exports.ValidationSeverity = void 0;
const ModelPath_1 = require("../../../../../../Common/ModelPath/ModelPath");
const CalculationFunctionBase_1 = require("../AutoCalculations/CalculationFunctionBase");
const CalculationFunctionDependency_1 = require("../CalculationFunctionDependency");
const CalculationTarget_1 = require("../CalculationTarget");
const GetValueByPath_1 = require("../../../../Helpers/StateAccessors/GetValueByPath");
const IterableUtils_1 = require("../../../../../../Common/IterableUtils");
var ValidationSeverity;
(function (ValidationSeverity) {
    ValidationSeverity["Error"] = "Error";
    ValidationSeverity["Warning"] = "Warning";
})(ValidationSeverity || (exports.ValidationSeverity = ValidationSeverity = {}));
class ValidationFunctionResults {
    static Valid() {
        return { valid: true };
    }
    static Invalid(message, severity = ValidationSeverity.Error) {
        return { valid: false, message: message, severity: severity };
    }
}
exports.ValidationFunctionResults = ValidationFunctionResults;
class ValidationFunction extends CalculationFunctionBase_1.CalculationFunctionBase {
    calculationTarget;
    dependencies;
    evaluator;
    source;
    allDependencies;
    constructor(targetPath, dependenciesPaths, evaluator, source) {
        super(targetPath);
        this.source = source;
        this.calculationTarget = new CalculationTarget_1.CalculationTarget(targetPath, CalculationTarget_1.CalculationTargetAttribute.Validation);
        this.dependencies = dependenciesPaths.map(x => new CalculationFunctionDependency_1.CalculationFunctionDependency(targetPath, x));
        this.evaluator = evaluator;
        this.allDependencies = [
            ...this.dependencies,
            new CalculationFunctionDependency_1.CalculationFunctionDependency(this.targetPath, this.targetPath.toCurrentIteration()),
        ];
    }
    getDependencies() {
        return this.allDependencies;
    }
    static conditionalValidatorEvaluator(validateIfCondition, evaluator) {
        if (validateIfCondition == null) {
            return evaluator;
        }
        return (context, targetPath) => {
            if (validateIfCondition(context)) {
                return evaluator(context, targetPath);
            }
            return ValidationFunctionResults.Valid();
        };
    }
    static javascriptSugarExpressionToEvaluator(form, conditionContextAndDeps) {
        if (conditionContextAndDeps.length == 0) {
            return undefined;
        }
        const condition = (evaluationContext) => conditionContextAndDeps
            .map(([generatedExpressionFn, contextCurrentElement]) => {
            const getValueByPathGeneratedFn = (expressionPath, fieldName) => {
                if (expressionPath.startsWith("@")) {
                    const state = form.store.getState();
                    return (0, GetValueByPath_1.getValueByPath)(state, () => "", expressionPath, fieldName);
                }
                const generatedAbsoluteModelPath = contextCurrentElement
                    .joinWith((0, ModelPath_1.createModelPath)(expressionPath, "auto"))
                    .normalize();
                if (fieldName === "children") {
                    const instances = evaluationContext.getInstances(generatedAbsoluteModelPath);
                    return Array.isArray(instances) ? instances : [...instances];
                }
                return evaluationContext.get(generatedAbsoluteModelPath) ?? "";
            };
            const getCountByPathGeneratedFn = expressionPath => {
                const generatedAbsoluteModelPath = contextCurrentElement
                    .joinWith((0, ModelPath_1.createModelPath)(expressionPath, "auto"))
                    .normalize();
                return (0, IterableUtils_1.count)(evaluationContext.getInstances(generatedAbsoluteModelPath));
            };
            return generatedExpressionFn(getValueByPathGeneratedFn, getCountByPathGeneratedFn);
        })
            .reduce((x, y) => x && y, true);
        return condition;
    }
    async execute(context) {
        const evaluator = this.evaluator;
        const targetPath = this.targetPath;
        if (context.isIgnoredPath(targetPath)) {
            return;
        }
        const evaluationResult = await evaluator(context, context.resolvePath(targetPath));
        if (evaluationResult.valid || evaluationResult.severity === ValidationSeverity.Error) {
            context.setError(targetPath, evaluationResult.valid ? undefined : evaluationResult.message);
        }
        else {
            context.setWarning(targetPath, evaluationResult.message);
        }
    }
    isDependsOn(calculationTarget) {
        return (calculationTarget.targetAttribute === CalculationTarget_1.CalculationTargetAttribute.Value &&
            this.dependencies.some(x => x.isMatchToPath(calculationTarget.modelPath)));
    }
    getTarget() {
        return this.calculationTarget;
    }
    asDebugString() {
        return `KCXmlValidationFunction(\n${this.source}\n)`;
    }
}
exports.ValidationFunction = ValidationFunction;
