"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChangeSet = void 0;
const areEqual_1 = __importDefault(require("fbjs/lib/areEqual"));
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
class ChangeSet {
    changeSet;
    includeOldValues;
    constructor(includeOldValues) {
        this.includeOldValues = includeOldValues;
        this.changeSet = {
            added: {},
            changed: {},
            removed: {},
            oldValues: {},
        };
    }
    appendEventInfo(eventArgs) {
        const value = eventArgs.value === undefined ? true : eventArgs.value;
        const oldValue = eventArgs.oldValue === undefined ? true : eventArgs.oldValue;
        const path = eventArgs.fullPath || eventArgs.path.toLegacyPath();
        const requiredToSave = !(ModelPath_1.PathTokens.isInstanceToken(IterableUtils_1.IterUtils.last(eventArgs.path.getPathParts())) && eventArgs.name == undefined);
        if (requiredToSave) {
            if (eventArgs.type === "change") {
                delete this.changeSet.removed[path];
                this.changeSet.changed[path] = value;
            }
            else if (eventArgs.type === "remove") {
                delete this.changeSet.added[path];
                delete this.changeSet.changed[path];
                this.changeSet.removed[path] = value;
            }
            if (this.includeOldValues) {
                this.changeSet.oldValues[path] =
                    this.changeSet.oldValues[path] == undefined ? oldValue : this.changeSet.oldValues[path];
            }
        }
    }
    canBeMerged(event) {
        const path = event.fullPath || event.path.toLegacyPath();
        const isConflictedChange = event.name === "children" &&
            event.type == "change" &&
            ((this.changeSet.changed[path] != undefined && !(0, areEqual_1.default)(this.changeSet.changed[path], event.value)) ||
                this.changeSet.removed[path] != undefined);
        return !isConflictedChange;
    }
    isEmpty() {
        const data = this.toServerChangeSet();
        return data.added.length === 0 && data.removed.length === 0 && !Object.keys(data.changed).length;
    }
    toServerChangeSet() {
        return {
            added: Object.keys(this.changeSet.added),
            changed: this.changeSet.changed,
            removed: Object.keys(this.changeSet.removed),
            oldValues: this.changeSet.oldValues,
        };
    }
    prependWithChangeSet(source) {
        const targetChangeSet = this.changeSet;
        const sourceChangeSet = source.changeSet;
        for (const [path, sourceAddedKey] of (0, IterableUtils_1.iterateEntries)(sourceChangeSet.added)) {
            if (targetChangeSet.removed[path] != undefined) {
                continue;
            }
            targetChangeSet.added[path] = sourceAddedKey;
        }
        for (const [path, sourceChangedValue] of (0, IterableUtils_1.iterateEntries)(sourceChangeSet.changed)) {
            if (targetChangeSet.removed[path] != undefined) {
                continue;
            }
            targetChangeSet.changed[path] = sourceChangedValue;
        }
        for (const [path, sourceChangedValue] of (0, IterableUtils_1.iterateEntries)(sourceChangeSet.oldValues)) {
            if (targetChangeSet.removed[path] != undefined) {
                continue;
            }
            if (this.includeOldValues) {
                if (targetChangeSet.oldValues[path] == undefined) {
                    targetChangeSet.oldValues[path] = sourceChangedValue;
                }
            }
        }
        for (const [path, sourceRemovedKey] of (0, IterableUtils_1.iterateEntries)(sourceChangeSet.removed)) {
            if (targetChangeSet.added[path] != undefined) {
                continue;
            }
            if (targetChangeSet.changed[path] != undefined) {
                continue;
            }
            targetChangeSet.removed[path] = sourceRemovedKey;
        }
    }
}
exports.ChangeSet = ChangeSet;
