"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ControlRegistry = void 0;
const PathUtils_1 = require("../PathUtils/PathUtils");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
class ControlRegistry {
    controls = new Map();
    registerComponentHandle(bindingPath, control, exclusiveRegistration) {
        const normalizedId = this.getNormalizedId(bindingPath);
        if (exclusiveRegistration) {
            this.controls.set(normalizedId, [{ control: control, isExclusive: true }]);
            return;
        }
        if (!this.controls.has(normalizedId)) {
            this.controls.set(normalizedId, [{ control: control, isExclusive: false }]);
            return;
        }
        const { index } = this.find(normalizedId, x => x.isExclusive === true);
        if (index > -1) {
            return;
        }
        this.getHandleList(normalizedId)?.push({ control: control, isExclusive: false });
    }
    unregisterComponentHandle(bindingPath, control) {
        const normalizedId = this.getNormalizedId(bindingPath);
        const { handleList, index } = this.find(normalizedId, x => x.control === control);
        if (index > -1) {
            handleList.splice(index, 1);
        }
        if (handleList.length === 0) {
            this.controls.delete(normalizedId);
        }
    }
    getBindingPaths() {
        return this.controls.keys();
    }
    getComponentHandles(bindingPath) {
        let path = bindingPath;
        while ((0, TypingUtils_1.isNotNullOrUndefined)(path)) {
            //NOTE: почему-то здесь не нормализуется bindingPath сначала
            const handleList = this.getHandleList(bindingPath)?.map(x => x.control);
            if ((0, TypingUtils_1.isNotNullOrUndefined)(handleList)) {
                return handleList;
            }
            else {
                path = PathUtils_1.PathUtils.getParentPath(path);
            }
        }
        return undefined;
    }
    find(bindingPath, predicate) {
        const handleList = this.getHandleList(bindingPath);
        if (handleList == undefined) {
            return { handleList: [], index: -1 };
        }
        const index = handleList.findIndex(predicate);
        if (index > -1) {
            return { handleList: handleList, index: index };
        }
        return { handleList: handleList, index: -1 };
    }
    getNormalizedId(bindingPath) {
        return bindingPath.replace(/\/\*$/g, "");
    }
    getHandleList(bindingPath) {
        return this.controls.get(bindingPath);
    }
}
exports.ControlRegistry = ControlRegistry;
