"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataDeclarationHelper = void 0;
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const MaskRegistrator_1 = require("../PathUtils/MaskRegistrator");
const AbsoluteModelPathSet_1 = require("../../../../../Common/ModelPath/Set/AbsoluteModelPathSet");
class DataDeclarationHelper {
    fetchFunctionCallContextProvider;
    dataDeclaration;
    optionalSections;
    allSingleSections;
    dataDeclarationMasks;
    allPaths;
    constructor(fetchFunctionCallContextProvider, dataDeclaration, formRoot) {
        this.fetchFunctionCallContextProvider = fetchFunctionCallContextProvider;
        this.dataDeclaration = dataDeclaration;
        this.dataDeclarationMasks = new MaskRegistrator_1.MaskRegistrator();
        this.optionalSections = this.dataDeclaration[formRoot]?.optionalSections ?? [];
        this.allSingleSections = this.dataDeclaration[formRoot]?.allSingleSections ?? [];
        const allPaths = [];
        for (const declMask of (0, IterableUtils_1.iterateKeys)(this.dataDeclaration)) {
            this.dataDeclarationMasks.registerMask(declMask, undefined);
            allPaths.push((0, ModelPath_1.createAbsoluteFromMask)(declMask, ModelPath_1.PathTokens.each));
        }
        this.allPaths = allPaths;
    }
    reloadDataDeclaration(formRoot, dataDeclaration) {
        this.dataDeclaration = dataDeclaration;
        this.dataDeclarationMasks = new MaskRegistrator_1.MaskRegistrator();
        this.allSingleSections = this.dataDeclaration[formRoot]?.allSingleSections ?? [];
        this.optionalSections = this.dataDeclaration[formRoot]?.optionalSections ?? [];
        const allPaths = [];
        for (const declMask of (0, IterableUtils_1.iterateKeys)(this.dataDeclaration)) {
            this.dataDeclarationMasks.registerMask(declMask, undefined);
            allPaths.push((0, ModelPath_1.createAbsoluteFromMask)(declMask, ModelPath_1.PathTokens.each));
        }
        this.allPaths = allPaths;
    }
    getAllPaths() {
        return this.allPaths;
    }
    hasMask(nodePath) {
        return this.getMask(nodePath) != undefined;
    }
    getMask(nodePath) {
        return this.dataDeclarationMasks.getMasks(nodePath)[0];
    }
    isNodeHasChildrenEntry(nodePath) {
        return this.getEntry(nodePath).children != undefined;
    }
    isNodeHasAutoFlagEntry(nodePath) {
        return this.getEntry(nodePath).autoFlag != undefined;
    }
    getType(nodePath) {
        return this.getEntry(nodePath)?.type;
    }
    isErrorsAndWarningsCountRequired(nodePath) {
        return this.getEntry(nodePath).errorAndWarningCountRequired?.[0] ?? false;
    }
    isDisableValueAutoInit(nodePath) {
        return this.getEntry(nodePath).disableValueAutoInit;
    }
    getEntry(nodePath) {
        const mask = this.getMask(nodePath);
        return (mask != undefined ? this.dataDeclaration[mask] : undefined) ?? {};
    }
    getSections(nodePath) {
        return (this.getSectionsStrict(nodePath) ?? this.getSections((0, ModelPath_1.convertToAbsoluteModelPath)(nodePath).getParentPath()));
    }
    getSectionsStrict(nodePath) {
        const node = this.getEntry(nodePath);
        let sections;
        const section = (node.section || [])[0];
        if (typeof section === "string") {
            sections = [section];
        }
        else if (Array.isArray(section)) {
            sections = section.slice(0);
            sections = sections
                .map(x => {
                if (x != undefined && typeof x === "object" && "fn" in x && typeof x.fn === "function") {
                    const fnResult = x.fn.call(this.fetchFunctionCallContextProvider.getFetchFunctionCallContext(undefined), (0, ModelPath_1.convertToLegacyPath)(nodePath));
                    if (Array.isArray(fnResult)) {
                        return fnResult.slice(0);
                    }
                    return fnResult;
                }
                return x;
            })
                .flat();
        }
        else if (section != undefined &&
            typeof section === "object" &&
            "fn" in section &&
            typeof section.fn === "function") {
            const fnResult = section.fn.call(this.fetchFunctionCallContextProvider.getFetchFunctionCallContext(undefined), (0, ModelPath_1.convertToLegacyPath)(nodePath));
            sections = Array.isArray(fnResult) ? fnResult : [fnResult];
        }
        else {
            return undefined;
        }
        // @ts-ignore
        return [...new Set(sections.filter(TypingUtils_1.isNotNullOrUndefined))];
    }
    isFnsConvertFile(path, fileName) {
        const extension = fileName?.split(".").slice(-1)[0];
        const mask = this.getMask(path);
        const { fnsConvertFileExtentions } = (mask != undefined ? this.dataDeclaration[mask] : undefined) ?? {};
        if (extension == undefined) {
            return false;
        }
        return fnsConvertFileExtentions?.includes(extension) ?? false;
    }
    getOptionalSections() {
        return this.optionalSections;
    }
    getAllSingleSections() {
        return this.allSingleSections;
    }
    buildSectionToMasksMap() {
        const section2masks = new Map();
        for (const path of this.getAllPaths()) {
            const sections = this.getSections(path);
            for (const sectionName of sections) {
                if (!section2masks.has(sectionName)) {
                    section2masks.set(sectionName, new AbsoluteModelPathSet_1.AbsoluteModelPathSet());
                }
                section2masks.get(sectionName)?.add(path);
            }
        }
        return section2masks;
    }
    *getMasksByPrefix(nodePathPrefix) {
        const prefixTokens = nodePathPrefix.getPathPartsAsArray();
        for (const dataDeclarationPath of this.getAllPaths()) {
            const dataDeclarationPathTokens = [...dataDeclarationPath.getPathParts()];
            if (dataDeclarationPathTokens.length < prefixTokens.length) {
                continue;
            }
            let matches = true;
            for (let i = 0; i < prefixTokens.length; ++i) {
                const prefToken = prefixTokens[i];
                const declToken = dataDeclarationPathTokens[i];
                if (prefToken === declToken) {
                    continue;
                }
                if (prefToken != undefined &&
                    ModelPath_1.PathTokens.isInstanceToken(prefToken) &&
                    ModelPath_1.PathTokens.isMultiToken(declToken)) {
                    dataDeclarationPathTokens[i] = prefToken;
                    continue;
                }
                matches = false;
                break;
            }
            if (matches) {
                yield (0, ModelPath_1.createAbsoluteFromTokens)(dataDeclarationPathTokens);
            }
        }
    }
}
exports.DataDeclarationHelper = DataDeclarationHelper;
