"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorLogApi = void 0;
const ExternalEvents_1 = require("../EventManagement/ExternalEvents/ExternalEvents");
const utils_1 = require("../../utils");
const ErrorLogActionCreators_1 = require("../../ActionCreators/ErrorLogActionCreators");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const FakeErrorLog_1 = require("../../Fakes/FakeErrorLog");
const MetricsProvider_1 = require("../../../Controls/Internals/Metrics/MetricsProvider");
const metricsAction_1 = require("../../../Controls/metricsAction");
const TypingUtils_2 = require("../../../Helpers/TypingUtils");
const Utils_1 = require("./Utils/Utils");
const Models_1 = require("./Utils/Models");
class ErrorLogApi {
    form;
    taskId = "";
    taskStatus;
    constructor(form) {
        this.form = form;
    }
    async getErrorLog(getAttachInfo) {
        let errorLog;
        let type;
        const start = Date.now();
        const fakeErrorLog = this.getFakeErrorLog();
        if (fakeErrorLog) {
            if ((0, Models_1.isHeavyErrorLog)(fakeErrorLog)) {
                errorLog = (0, Utils_1.errorlogFromKewebToServerAdapter)(getAttachInfo, fakeErrorLog);
            }
            else {
                errorLog = (0, Utils_1.serverErrorLogToGroupedAdapter)(fakeErrorLog);
            }
        }
        else if (this.form.settings.isReadOnly === true) {
            const serverErrorLog = await this.form.candyApiDraftClient.getErrorLogWithCheckScenarios(["readonly"]);
            errorLog = (0, Utils_1.serverErrorLogToGroupedAdapter)(serverErrorLog);
        }
        else if ((this.form.settings.useHeavyErrorLog || this.form.settings.isFormInFormNds) &&
            this.form.hasExternalEventSubscriber(ExternalEvents_1.EXTERN_CHECK_TASK_EVENT_PATH)) {
            // Флаг для активации нового протокола
            type = "New";
            errorLog = await this.getErrorLogFromKeweb(getAttachInfo);
        }
        else {
            // Флаг для активации нового протокола
            type = "Old";
            const serverErrorLog = await this.form.candyApiDraftClient.getErrorLog();
            errorLog = (0, Utils_1.serverErrorLogToGroupedAdapter)(serverErrorLog);
        }
        const end = Date.now();
        (0, MetricsProvider_1.track)("candy", metricsAction_1.ERROR_LOG_COUNTER, {
            errors: errorLog.groups.reduce((acc, it) => acc + it.errors.length, 0),
            warnings: errorLog.groups.reduce((acc, it) => acc + it.warnings.length, 0),
            timeSeconds: (end - start) / 1000,
            type: type,
        });
        this.resetStatuses();
        return errorLog;
    }
    getFakeErrorLog() {
        return (0, FakeErrorLog_1.getFakeErrorLogForDraft)(this.form.settings.draftId);
    }
    async getErrorLogFromKeweb(getAttachInfo) {
        await this.setErrorLogTask();
        await this.waitBeforeTaskFinished();
        // От этих неочевидных зависимостей избавимся, как только начнем ходить в апи, а не в кевеб
        // Если есть идеи как можно лучше - буду рад критике
        return (0, Utils_1.errorlogFromKewebToServerAdapter)(getAttachInfo, this.taskStatus?.result);
    }
    async setErrorLogTask() {
        // Тут вместо вызова колбека будет запрос на апи. Как только Стас вынесет в отдельный сервис
        // https://yt.skbkontur.ru/issue/FP-92/Vstroit-vse-proverki-v-Kendi.-2
        this.form.triggerExternalEvent((0, ExternalEvents_1.formStartExternCheckTask)(this.setTaskId.bind(this)));
        //-- Пока туда ходит кевеб - мы будем мониторить
        await this.waitBeforeTaskStands();
    }
    async waitBeforeTaskStands() {
        while ((0, TypingUtils_1.isNullOrEmpty)(this.taskId)) {
            await (0, utils_1.delay)(500);
        }
    }
    async waitBeforeTaskFinished() {
        await this.checkErrorLogStatus();
        while (!this.taskStatus?.isFinished) {
            await (0, utils_1.delay)(500);
        }
    }
    async checkErrorLogStatus() {
        await (0, utils_1.delay)(1000);
        if (this.taskId) {
            const params = {
                taskId: this.taskId,
                setTaskStatus: this.setTaskStatus.bind(this),
            };
            this.form.triggerExternalEvent((0, ExternalEvents_1.formGetExternCheckTask)(params));
            //-- Пока туда ходит кевеб - мы будем сбоку мониторить
        }
    }
    //Это отдаём для кевеба. Потом адаптируем под себя
    setTaskId(taskId) {
        if ((0, TypingUtils_1.isNullOrEmpty)(taskId)) {
            this.setRequestFailedStatus();
        }
        else {
            this.taskId = taskId;
        }
    }
    async setTaskStatus(taskStatus) {
        if (taskStatus == undefined || (0, TypingUtils_1.isNotNullOrEmpty)(taskStatus.errorMessage)) {
            this.setRequestFailedStatus();
        }
        else if (!taskStatus.isFinished) {
            (0, TypingUtils_2.runFloatingAsyncAction)(this.checkErrorLogStatus.bind(this));
        }
        else {
            this.taskStatus = taskStatus;
        }
    }
    // Конец кевеба
    setRequestFailedStatus() {
        this.resetStatuses();
        this.form.store.dispatch((0, ErrorLogActionCreators_1.requestFailed)());
    }
    resetStatuses() {
        this.taskId = "";
        this.taskStatus = undefined;
    }
}
exports.ErrorLogApi = ErrorLogApi;
