"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorManager = exports.errorsFallback = exports.KREC_ERROR_CONVERT = exports.KREC_ERROR_CHECK = void 0;
exports.KREC_ERROR_CHECK = "KREC_ERROR_CHECK";
exports.KREC_ERROR_CONVERT = "KREC_ERROR_CONVERT";
exports.errorsFallback = {
    [exports.KREC_ERROR_CHECK]: "Не удалось проверить документ на соответствие требованиям ФНС." +
        "Проверьте вручную или загрузите ещё раз в течение часа",
    [exports.KREC_ERROR_CONVERT]: "Не удалось привести документ в соответствие требованиям ФНС." +
        "Исправьте вручную или загрузите ещё раз в течение часа",
};
class ErrorManager {
    clearErrorsCached(model, nodePath) {
        return this.internalClear(model, nodePath, "error");
    }
    clearWarningsCached(model, nodePath) {
        return this.internalClear(model, nodePath, "warning");
    }
    addErrorCached(model, nodePath, message) {
        return this.internalAdd(model, nodePath, message, "error");
    }
    addWarningCached(model, nodePath, message) {
        this.internalAdd(model, nodePath, message, "warning");
    }
    removeWarning(model, nodePath, message) {
        return this.internalRemove(model, "warning", nodePath, message);
    }
    clearCached(model, path) {
        this.clearErrorsCached(model, path);
        this.clearWarningsCached(model, path);
    }
    internalClear(model, nodePath, type) {
        const nodeData = model.getViewModel(nodePath);
        if (!nodeData) {
            return;
        }
        const items = nodeData[type];
        if (!items) {
            return;
        }
        model.setViewModelField(nodePath, type, []);
    }
    internalAdd(model, nodePath, message, type) {
        const nodeData = model.getViewModel(nodePath);
        if (nodeData !== undefined) {
            model.setViewModelField(nodePath, type, [message]);
        }
    }
    internalRemove(model, type, nodePath, message) {
        let nodeData = model.getViewModel(nodePath);
        if (nodeData === undefined) {
            nodeData = {};
        }
        const container = nodeData[type] ?? [];
        const msgIdx = container.lastIndexOf(message);
        if (msgIdx !== -1) {
            const newContainer = container.slice(0);
            model.setViewModelField(nodePath, type, newContainer);
        }
    }
}
exports.ErrorManager = ErrorManager;
