"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultErrorLogState = void 0;
exports.errorLogModelReducer = errorLogModelReducer;
exports.errorLogReducer = errorLogReducer;
const IterableUtils_1 = require("../../../../../../Common/IterableUtils");
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
const modelActions_1 = require("../../ModelStoring/Store/modelActions");
const CandyPopdProtocol_1 = require("../../../../../../Common/CandyApiClient/Types/ErrorLog/CandyPopdProtocol");
const Utils_1 = require("../Utils/Utils");
const ErrorLogActions_1 = require("./ErrorLogActions");
exports.defaultErrorLogState = {
    modelChangedAfterCheck: false,
    form: { errorsTotalCount: 0, warningsTotalCount: 0, successesTotalCount: 0, errorGroupInfos: [] },
};
function updateErrorItemsTouchedState(errorItems, touchedPaths) {
    if (errorItems == undefined) {
        return errorItems;
    }
    const touchedBindingPaths = new Set(Iterator.from(touchedPaths).map(x => x.toLegacyPath()));
    let nextErrorItems = errorItems;
    for (const item of Iterator.from((0, IterableUtils_1.iterateValues)(errorItems)).filter(TypingUtils_1.isNotNullOrUndefined)) {
        const itemBindingPaths = Iterator.from((0, Utils_1.getErrorItemBindingPaths)(item));
        if (itemBindingPaths.some(x => touchedBindingPaths.has(x))) {
            nextErrorItems = {
                ...nextErrorItems,
                [item.id]: { ...item, touched: true },
            };
        }
    }
    return nextErrorItems;
}
function errorLogModelReducer(state, action) {
    let nextState = state;
    if (action.type === ErrorLogActions_1.ErrorLogActionsType.UpdateModelState) {
        nextState = {
            ...nextState,
            errorLog: {
                ...nextState.errorLog,
                modelChangedAfterCheck: action.payload.modelChanged,
            },
        };
    }
    if (action.type === modelActions_1.ModelActionType.SetValues) {
        if (!nextState.errorLog.modelChangedAfterCheck) {
            nextState = {
                ...nextState,
                errorLog: {
                    ...nextState.errorLog,
                    modelChangedAfterCheck: true,
                },
            };
        }
        if (nextState.errorLog.form.errors != undefined && nextState.errorLog.form.warnings != undefined) {
            const pathWithChangedValue = Iterator.from(action.payload.values.iterateEntries())
                .filter(([path, update]) => "value" in update)
                .map(([path]) => path)
                .toArray();
            const deletedPaths = action.payload.deleted;
            const touchedBindingPaths = [...pathWithChangedValue, ...deletedPaths];
            const nextErrors = updateErrorItemsTouchedState(nextState.errorLog.form.errors, touchedBindingPaths);
            const nextWarning = updateErrorItemsTouchedState(nextState.errorLog.form.warnings, touchedBindingPaths);
            if (nextState.errorLog.form.errors !== nextErrors || nextState.errorLog.form.warnings !== nextWarning) {
                nextState = {
                    ...nextState,
                    errorLog: {
                        ...nextState.errorLog,
                        form: {
                            ...nextState.errorLog.form,
                            errors: nextErrors,
                            warnings: nextWarning,
                        },
                    },
                };
            }
        }
    }
    return nextState;
}
function errorLogReducer(state, action) {
    switch (action.type) {
        case ErrorLogActions_1.ErrorLogActionsType.requestErrorlog:
            return {
                ...state,
                modelChangedAfterCheck: false,
                form: {
                    ...state.form,
                    isChecking: true,
                },
                popdProtocol: undefined,
            };
        case ErrorLogActions_1.ErrorLogActionsType.receivedErrorLog: {
            const { errorsTotalCount = 0, warningsTotalCount = 0 } = action.payload;
            const counterEnabled = state.form.counterEnabled || Boolean(errorsTotalCount + warningsTotalCount);
            return {
                ...state,
                form: {
                    isChecking: false,
                    errorLogWasRequested: true,
                    counterEnabled: counterEnabled,
                    ...action.payload,
                },
            };
        }
        case ErrorLogActions_1.ErrorLogActionsType.requestFailed:
            return {
                ...state,
                form: {
                    ...exports.defaultErrorLogState.form,
                    requestFailed: true,
                    isChecking: false,
                },
                popdProtocol: undefined,
            };
        case ErrorLogActions_1.ErrorLogActionsType.requestExpertErrorlog: {
            return {
                ...state,
                expert: {
                    ...state.form,
                    isChecking: true,
                },
            };
        }
        case ErrorLogActions_1.ErrorLogActionsType.receivedExpertErrorLog: {
            return {
                ...state,
                expert: action.payload,
            };
        }
        case ErrorLogActions_1.ErrorLogActionsType.requestExpertErrorLogFailed:
            return {
                ...state,
                expert: {
                    errorsTotalCount: 0,
                    warningsTotalCount: 0,
                    successesTotalCount: 0,
                    errorGroupInfos: [],
                    requestFailed: true,
                },
            };
        case ErrorLogActions_1.ErrorLogActionsType.requestPopdProtocol:
            return {
                ...state,
                popdProtocol: {
                    ...state.popdProtocol,
                    isChecking: true,
                },
            };
        case ErrorLogActions_1.ErrorLogActionsType.receivedPopdProtocol: {
            return {
                ...state,
                popdProtocol: {
                    ...action.payload,
                    isChecking: false,
                },
            };
        }
        case ErrorLogActions_1.ErrorLogActionsType.requestPopdProtocolFailed:
            return {
                ...state,
                popdProtocol: {
                    resultCode: CandyPopdProtocol_1.PopdResultCode.Error,
                    isChecking: false,
                },
            };
        case ErrorLogActions_1.ErrorLogActionsType.setAttachmentErrors:
            return {
                ...state,
                attachmentErrorLogs: action.payload,
            };
        case ErrorLogActions_1.ErrorLogActionsType.clearAttachmentErrors:
            return {
                ...state,
                attachmentErrorLogs: undefined,
            };
        default:
            return state;
    }
}
