import { BindingPath } from "../../CoreTypes";
import { Guid } from "../../../Types/Guid";
import { ServerErrorLog } from "../../../../../../Common/CandyApiClient/Types/ErrorLog/ServerErrorLog";
export declare enum ItemErrorMessageType {
    external = "external",
    navLink = "navLink",
    text = "text"
}
export interface NavLinkErrorMessage {
    type: ItemErrorMessageType.navLink;
    text: string;
    highlanderUrl: string;
    bindingPaths: BindingPath[];
}
export type ItemErrorMessage = {
    type: ItemErrorMessageType.external;
    text: string;
    url?: string;
} | NavLinkErrorMessage | {
    type: ItemErrorMessageType.text;
    text: string;
    hideLinks?: boolean;
    url?: string;
};
export interface ErrorLogItem {
    id: Guid;
    description: ItemErrorMessage[];
    highlanderPath: null | undefined | string;
    bindingPaths?: undefined | BindingPath[];
    source: string[];
    highlanderUsedArguments: string[] | undefined;
    usedArgumentBindingPaths: undefined | BindingPath[];
    touched: boolean;
    justification?: string;
}
export interface ErrorLogItems {
    [key: string]: ErrorLogItem;
}
export interface ErrorLogGroupInfo {
    groupName: string | null;
    errorIds: string[];
    warningIds: string[];
    successIds: string[];
    groupAttachmentId?: string;
    tid?: string;
}
export interface ErrorLog {
    errorsTotalCount: number;
    warningsTotalCount: number;
    successesTotalCount: number;
    errorLogName?: null | string;
    propsName?: null | string;
    errorGroupInfos: ErrorLogGroupInfo[];
    errors?: ErrorLogItems;
    warnings?: ErrorLogItems;
    successes?: ErrorLogItems;
    interdocumentCheckerResult?: InterdocumentCheckerResult | null;
    isChecking?: boolean;
    requestFailed?: true;
    errorLogWasRequested?: true;
    counterEnabled?: boolean;
}
export interface IErrorLogTaskInfo {
    errorMessage?: string;
    isFinished: boolean;
    taskId: Guid;
    progress: number;
    result: IErrorLogTaskResult;
}
export declare enum InterdocumentCheckStatus {
    Fail = "fail",
    Success = "success"
}
export declare enum InterdocumentType {
    Fine = "fine",
    Warning = "warning",
    Error = "error",
    NotFound = "notfound"
}
export interface InterdocumentIssue {
    id: string;
    description: string;
}
export interface CheckedReport {
    id: string;
    url: string;
    caption: string;
}
export interface DiscrepancyInfo {
    discrepanciesCount: number;
    issues: InterdocumentIssue[];
}
export interface InterdocumentCheck {
    type: InterdocumentType;
    title: string;
    userDescription?: string;
    issues?: InterdocumentIssue[];
    checkedReports?: CheckedReport[];
    discrepancyInfo: DiscrepancyInfo | null;
}
export interface InterdocumentCheckerResult {
    status: InterdocumentCheckStatus;
    failReason?: string;
    checks?: InterdocumentCheck[];
}
export interface IErrorLogTaskResult {
    errorLogName?: null | string;
    propsName?: null | string;
    mainReportCheckLog: ServerErrorLog;
    mainReportCrossCheckLog: ServerErrorLog;
    warrantCheckLog: ServerErrorLog | null;
    digitalWarrantCheckLog: ServerErrorLog | null;
    interdocumentsCheckLog: InterdocumentCheckerResult | null;
    attachmentsCheckLogsWithIds: {
        [attachmentId: string]: ServerErrorLog;
    };
}
export declare function isHeavyErrorLog(errorLog: ServerErrorLog | IErrorLogTaskResult): errorLog is IErrorLogTaskResult;
