"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEFAULT_SUCCESS_DESCRIPTION = exports.splitPathWithComma = void 0;
exports.parseErrorDescription = parseErrorDescription;
exports.errorlogFromKewebToServerAdapter = errorlogFromKewebToServerAdapter;
exports.serverErrorLogToGroupedAdapter = serverErrorLogToGroupedAdapter;
exports.parseErrorLog = parseErrorLog;
exports.isNavLinkErrorMessage = isNavLinkErrorMessage;
exports.getErrorItemBindingPaths = getErrorItemBindingPaths;
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
const Utils_1 = require("../../../../Controls/FormParts/EmbeddedLog/ErrorList/ErrorItemDescription/Utils/Utils");
const Models_1 = require("./Models");
// ,     ищем запятую
// (?!   Negative look-ahead open
// [^(]* Любые номера или символы кроме '(', 0 и более раз
// /]    закрывашка ']'
// )     Close the lookahead.
const errorDescriptionRegexp = /(\[([^[]+)\]\(([^)]+)\)|\[?[^[]+)/g;
function processErrorSourcePart(sourcePart) {
    return sourcePart.replace(/\|\|/g, "/");
}
// В путях, может прийти вот такое [справНал=4,6]
exports.splitPathWithComma = /,(?![^\[]*\])/;
function parseErrorDescription(description, pathMapper, noLinks, noPathMapping) {
    const parsedDescription = [];
    while (true) {
        const match = errorDescriptionRegexp.exec(description);
        if (match == undefined || !(0, TypingUtils_1.isLengthGreaterThenOrEqual)(match, 4)) {
            break;
        }
        const text = match[1];
        const title = match[2];
        let url = match[3];
        if (!noLinks && (title || url)) {
            const urlBeginPosition = errorDescriptionRegexp.lastIndex - url.length - 2;
            url = parseUrl(description, urlBeginPosition) || url;
            errorDescriptionRegexp.lastIndex = urlBeginPosition + `(${url})`.length;
            const message = url.startsWith("http")
                ? {
                    type: Models_1.ItemErrorMessageType.external,
                    text: title,
                    url: url,
                }
                : {
                    type: Models_1.ItemErrorMessageType.navLink,
                    text: title,
                    highlanderUrl: url,
                    bindingPaths: (noPathMapping ?? false)
                        ? url.split(exports.splitPathWithComma)
                        : url.split(exports.splitPathWithComma).map(x => pathMapper.mapPath(x)),
                };
            parsedDescription.push(message);
        }
        else {
            const message = {
                type: Models_1.ItemErrorMessageType.text,
                text: text,
            };
            if ((0, Utils_1.hasHiddenText)(text)) {
                message.hideLinks = true;
            }
            parsedDescription.push(message);
        }
    }
    return parsedDescription;
}
function parseUrl(baseText, urlBeginPosition) {
    let counter = 0;
    for (let i = urlBeginPosition; i < baseText.length; i++) {
        if (baseText[i] === "(") {
            counter++;
        }
        if (baseText[i] === ")") {
            counter--;
        }
        if (counter === 0) {
            return baseText.substring(urlBeginPosition + 1, i);
        }
    }
    return null;
}
function parseError(error, pathMapper, noLinks, noPathMapping) {
    return {
        id: error.id,
        highlanderPath: error.path,
        bindingPaths: (noPathMapping ?? false)
            ? error.path?.split(exports.splitPathWithComma)
            : error.path?.split(exports.splitPathWithComma)?.map(path => pathMapper.mapPath(path)),
        source: (error.source ?? "").split("/").map(processErrorSourcePart),
        highlanderUsedArguments: error.usedArguments,
        usedArgumentBindingPaths: (noPathMapping ?? false) ? error.usedArguments : error.usedArguments?.map(x => pathMapper.mapPath(x)),
        justification: error.justification ?? undefined,
        description: parseErrorDescription(error.description, pathMapper, noLinks, noPathMapping),
        touched: false,
    };
}
// yarkeev: Можно было бы сделать тут всё по-умнее - такая информация есть на бекенде, и есть в ресурсах в элементах <attachmentform />
// Но решил пока не заморачиваться, и вот так вот захардкодить. Кажется ещё 10 лет не придётся менять :)
function getAttachmentGroupNameByAttachmentInfo(attachInfo) {
    const gfv = attachInfo?.meta.gfv;
    if (!gfv) {
        return "Приложение";
    }
    const gfToNameMap = {
        ["1103"]: "Книга покупок (Раздел 8)",
        ["1105"]: "Дополнительные листы книги покупок (Раздел 8.1)",
        ["1102"]: "Книга продаж (Раздел 9)",
        ["1106"]: "Дополнительные листы книги продаж (Раздел 9.1)",
        ["1104"]: "Сведения из журнала выставленных счетов-фактур (Раздел 10)",
        ["1108"]: "Сведения из журнала полученных счетов-фактур (Раздел 11)",
        ["1107"]: "Сведения из счетов-фактур, выставленных лицами, указанными в п. 5 ст. 173 НК РФ (Раздел 12)",
    };
    const gf = gfv.substring(0, 4);
    const name = gfToNameMap[gf];
    if (!name) {
        return "Приложение";
    }
    return name;
}
exports.DEFAULT_SUCCESS_DESCRIPTION = "Ошибок и предупреждений нет";
function errorlogFromKewebToServerAdapter(getAttachInfo, result) {
    const mainLogErrors = (result?.mainReportCheckLog.errors ?? []).concat(result?.mainReportCrossCheckLog.errors ?? []);
    const mainLogWarnings = (result?.mainReportCheckLog.warnings ?? []).concat(result?.mainReportCrossCheckLog.warnings ?? []);
    const mainLogSuccesses = (result?.mainReportCheckLog.successes ?? []).concat(result?.mainReportCrossCheckLog.successes ?? []);
    const groups = [
        {
            groupName: "",
            errors: mainLogErrors,
            warnings: mainLogWarnings,
            successes: mainLogSuccesses,
        },
    ];
    if (result?.warrantCheckLog) {
        groups.push({
            groupName: "Результат проверки СоП",
            errors: result?.warrantCheckLog.errors ?? [],
            warnings: result?.warrantCheckLog.warnings ?? [],
            successes: result?.warrantCheckLog.successes ?? [],
            tid: "protocol-warrant",
        });
    }
    if (result?.digitalWarrantCheckLog) {
        groups.push({
            groupName: "Результат проверки Электронной доверенности",
            errors: result?.digitalWarrantCheckLog.errors ?? [],
            warnings: result?.digitalWarrantCheckLog.warnings ?? [],
            successes: result?.digitalWarrantCheckLog.successes ?? [],
            tid: "protocol-digital-warrant",
        });
    }
    const attachmentCheckLogs = result?.attachmentsCheckLogsWithIds || {};
    for (const attachmentId in attachmentCheckLogs) {
        const attachmentCheckLog = attachmentCheckLogs[attachmentId];
        groups.push({
            groupName: getAttachmentGroupNameByAttachmentInfo(getAttachInfo(attachmentId)),
            errors: attachmentCheckLog?.errors ?? [],
            warnings: attachmentCheckLog?.warnings ?? [],
            successes: attachmentCheckLog?.successes ?? [],
            groupAttachmentId: attachmentId,
            tid: "protocol-attachment",
        });
    }
    const groupedErrorLog = {
        propsName: result?.propsName,
        errorLogName: result?.errorLogName,
        groups: groups,
        interdocumentCheckerResult: result?.interdocumentsCheckLog,
    };
    return groupedErrorLog;
}
function serverErrorLogToGroupedAdapter(serverErrorLog) {
    return {
        propsName: serverErrorLog.propsName,
        errorLogName: serverErrorLog.errorLogName,
        groups: [
            {
                groupName: null,
                errors: serverErrorLog.errors,
                warnings: serverErrorLog.warnings,
                successes: serverErrorLog.successes ?? [],
            },
        ],
    };
}
function parseErrorLog(errorLog, pathMapperGetter, noLinks, noPathMapping) {
    const parsedErrors = {};
    const parsedWarnings = {};
    const parsedSuccesses = {};
    errorLog.groups.forEach(group => {
        group.errors.forEach(error => (parsedErrors[error.id] = parseError(error, pathMapperGetter(group.groupAttachmentId), noLinks, noPathMapping)));
        group.warnings.forEach(error => (parsedWarnings[error.id] = parseError(error, pathMapperGetter(group.groupAttachmentId), noLinks, noPathMapping)));
        group.successes.forEach(error => (parsedSuccesses[error.id] = parseError(error, pathMapperGetter(group.groupAttachmentId), noLinks, noPathMapping)));
    });
    const interdocumentErrorsCount = errorLog.interdocumentCheckerResult?.checks
        ?.filter(check => check.type === Models_1.InterdocumentType.Error)
        .reduce((acc, check) => (acc += check.issues?.length ?? 0), 0) ?? 0;
    const interdocumentWarningsCount = errorLog.interdocumentCheckerResult?.checks
        ?.filter(check => check.type === Models_1.InterdocumentType.Warning)
        .reduce((acc, check) => (acc += check.issues?.length ?? 0), 0) ?? 0;
    const interdocumentSuccessesCount = errorLog.interdocumentCheckerResult?.checks
        ?.filter(check => check.type === Models_1.InterdocumentType.Fine)
        .reduce((acc, check) => (acc += check.issues?.length ?? 0), 0) ?? 0;
    return {
        propsName: errorLog.propsName,
        errorLogName: errorLog.errorLogName,
        errorsTotalCount: Object.keys(parsedErrors).length + interdocumentErrorsCount,
        warningsTotalCount: Object.keys(parsedWarnings).length + interdocumentWarningsCount,
        successesTotalCount: Object.keys(parsedSuccesses).length + interdocumentSuccessesCount,
        errorGroupInfos: errorLog.groups.map(group => ({
            groupName: group.groupName,
            errorIds: group.errors.map(it => it.id),
            warningIds: group.warnings.map(it => it.id),
            successIds: group.successes.map(it => it.id),
            groupAttachmentId: group.groupAttachmentId,
            tid: group.tid,
        })),
        errors: parsedErrors,
        warnings: parsedWarnings,
        successes: parsedSuccesses,
        interdocumentCheckerResult: errorLog.interdocumentCheckerResult,
    };
}
function isNavLinkErrorMessage(itemErrorMessage) {
    return itemErrorMessage.type === Models_1.ItemErrorMessageType.navLink;
}
function* getErrorItemBindingPaths(error) {
    yield* Iterator.from(error.description)
        .filter(isNavLinkErrorMessage)
        .flatMap(x => x.bindingPaths);
    if (error.bindingPaths != undefined) {
        yield* error.bindingPaths;
    }
    if (error.usedArgumentBindingPaths != undefined) {
        yield* error.usedArgumentBindingPaths;
    }
}
