"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExternalEventManager = void 0;
const IterableUtils_1 = require("../../../../../../Common/IterableUtils");
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
const ExternalEvents_1 = require("./ExternalEvents");
class ExternalEventManager {
    hookStorage = { children: {}, hooks: [] };
    eventsQueue = [];
    queueProcessing = false;
    onExternalEvent(pathArr, callback) {
        const hookTreeItem = this.getPathNodes(pathArr, true).pop();
        if (hookTreeItem != undefined) {
            hookTreeItem.hooks.push(callback);
        }
    }
    onceExternalEvent(pathArr, callback) {
        const hook = data => {
            this.offExternalEvent(pathArr, hook);
            callback(data);
        };
        const hookTreeItem = this.getPathNodes(pathArr, true).pop();
        if (hookTreeItem != undefined) {
            hookTreeItem.hooks.push(hook);
        }
        return hook;
    }
    offExternalEvent(pathArr, callback) {
        const nodes = this.getPathNodes(pathArr);
        if (nodes.length !== pathArr.length + 1) {
            return;
        }
        (0, TypingUtils_1.ensureArrayIsNotEmpty)(nodes);
        const node = IterableUtils_1.IterUtils.last(nodes);
        const h = node.hooks;
        for (let i = 0; i < h.length; ++i) {
            if (h[i] === callback) {
                node.hooks.splice(i, 1);
                for (let j = pathArr.length - 1; j >= 0; --j) {
                    if (nodes[j + 1]?.hooks.length || Object.keys(nodes[j + 1]?.children ?? {}).length) {
                        break;
                    }
                    const childName = pathArr[j];
                    if (childName != undefined) {
                        delete nodes[j]?.children[childName];
                    }
                }
                break;
            }
        }
    }
    triggerExternalEvent(event) {
        this.triggerExternalEvents([event]);
    }
    triggerExternalEvents(events) {
        for (const event of events) {
            this.eventsQueue.push(event);
        }
        this.processQueue();
    }
    hasExternalEventSubscriber(pathArr) {
        const nodes = this.getPathNodes(pathArr);
        return nodes.length === pathArr.length + 1;
    }
    processQueue() {
        if (this.queueProcessing) {
            return;
        }
        this.queueProcessing = true;
        while (this.eventsQueue.length !== 0) {
            const event = this.eventsQueue.shift();
            if (event == undefined) {
                break;
            }
            const pathArr = (0, ExternalEvents_1.getExternalEventPath)(event);
            const eventData = (0, ExternalEvents_1.getExternalEventInfo)(event);
            const nodes = this.getPathNodes(pathArr);
            for (const node of (0, IterableUtils_1.reverseArray)(nodes)) {
                for (const hook of node.hooks) {
                    hook(eventData);
                }
            }
        }
        this.queueProcessing = false;
    }
    getPathNodes(pathArr, createIfNotExists = false) {
        let currentNode = this.hookStorage;
        const res = [currentNode];
        for (const pathItem of pathArr) {
            let nextNode = currentNode.children[pathItem];
            if (!nextNode) {
                if (createIfNotExists) {
                    nextNode = { children: {}, hooks: [] };
                    currentNode.children[pathItem] = nextNode;
                }
                else {
                    break;
                }
            }
            currentNode = nextNode;
            res.push(currentNode);
        }
        return res;
    }
}
exports.ExternalEventManager = ExternalEventManager;
