"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerSideFilteringStrategy = void 0;
const MultiLineTypes_1 = require("../../../Controls/MultiControls/MultiLine/MultiLineTypes");
const FilterExpressionToQueryCollectionExpressionVisitor_1 = require("../../../Helpers/FilterExpressions/FilterExpressionToQueryCollectionExpressionVisitor");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const AndExpression_1 = require("../../../../../Common/CandyApiClient/Types/QueryCollecton/Expressions/AndExpression");
const InnerQueryRequest_1 = require("../../../../../Common/CandyApiClient/Types/QueryCollecton/InnerQueryRequest");
const SelectClause_1 = require("../../../../../Common/CandyApiClient/Types/QueryCollecton/SelectClause");
const InstanceIdCallExpression_1 = require("../../../../../Common/CandyApiClient/Types/QueryCollecton/Expressions/InstanceIdCallExpression");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const SortingRule_1 = require("../../../../../Common/CandyApiClient/Types/QueryCollecton/SortingRule");
const AbsoluteModelFieldPath_1 = require("../../../../../Common/ModelPath/AbsoluteModelFieldPath");
const ViewModelSelect_1 = require("../../../Helpers/StateAccessors/ViewModelSelect");
const MaxInstancesForServerRequest = 1000000000;
class ServerSideFilteringStrategy {
    maxInstancesForServerRequest = MaxInstancesForServerRequest;
    getState;
    candyApiDraftClient;
    constructor(getState, candyApiDraftClient, maxInstancesForServerRequest = MaxInstancesForServerRequest) {
        this.getState = getState;
        this.candyApiDraftClient = candyApiDraftClient;
        this.maxInstancesForServerRequest = maxInstancesForServerRequest;
    }
    async performFilteringAndSorting(multilinePath, filteredUniqKey, filters, multilineSort) {
        const filterConverter = new FilterExpressionToQueryCollectionExpressionVisitor_1.FilterExpressionToQueryCollectionExpressionVisitor((0, ModelPath_1.createAbsolute)(multilinePath));
        const convertedFilters = filters.map(x => filterConverter.transformExpression(x.expression));
        const combinedFilter = convertedFilters.length > 1 ? new AndExpression_1.AndExpression(convertedFilters) : convertedFilters?.[0];
        const request = new InnerQueryRequest_1.InnerQueryRequest(multilinePath, [new SelectClause_1.SelectClause("instance", new InstanceIdCallExpression_1.InstanceIdCallExpression())], combinedFilter, multilineSort != undefined
            ? [this.multilineSortToSortingRule(multilineSort, (0, ModelPath_1.createAbsolute)(multilinePath))]
            : undefined, undefined, 0, this.maxInstancesForServerRequest);
        const response = await this.candyApiDraftClient.queryCollection(request);
        const childrenSet = new Set(ViewModelSelect_1.currentViewModelSelect.children(this.getState(), multilinePath));
        const instances = Iterator.from(response.data)
            .map(x => x[0])
            .filter(TypingUtils_1.isNotNullOrUndefined)
            .filter(x => childrenSet.has(x))
            .toArray();
        return instances;
    }
    multilineSortToSortingRule(sort, multiLinePath) {
        const pathAndField = AbsoluteModelFieldPath_1.AbsoluteModelFieldPath.createFromMask(sort.dataPath, ModelPath_1.PathTokens.each);
        const tailPath = multiLinePath.getSuffixPathFrom(pathAndField.path).trimFirstStarIfLastToken();
        return new SortingRule_1.SortingRule(`${tailPath.toString()}.${pathAndField.field}`, sort.value === MultiLineTypes_1.SortEnum.Desc ? "Descending" : "Ascending");
    }
}
exports.ServerSideFilteringStrategy = ServerSideFilteringStrategy;
