import { AbsoluteModelPath, AbsoluteResolvedModelPath } from "../../../../../Common/ModelPath/ModelPath";
import { ModelHunk } from "../ModelStoring/ModelHunk";
import { PathLoadSpecification } from "../../../../../Common/CandyApiClient/ICandyApiClient";
import { IReadonlyModelHunk } from "../ModelStoring/IReadonlyModelHunk";
import { BindingPath, ViewModel } from "../CoreTypes";
import { AbsoluteModelFieldPath } from "../../../../../Common/ModelPath/AbsoluteModelFieldPath";
import { LazyLoadingDeclaration } from "./LazyLoadingDeclaration";
export declare enum LazyLoadingInstanceState {
    OnlyInitialData = 0,
    OnlyRequiredData = 1,
    FullyLoaded = 2,
    Pending = 3
}
export interface LazyLoadRule {
    modelPath: AbsoluteModelPath | AbsoluteModelFieldPath;
    skip: boolean;
}
export type InstancesExtractedFromPath = string[];
export type LazyLoadedPathResolveResult = InstancesExtractedFromPath[];
export interface ILazyLoadingManager {
    requestInstancesLoad(multiplicityPath: AbsoluteModelPath, ...instances: string[]): Promise<void>;
    requestFullInstanceLoad(multiplicityPath: AbsoluteModelPath, ...instances: string[]): Promise<void>;
    requestFullInstanceLoadForCustomPath(multiplicityPath: AbsoluteModelPath): Promise<void>;
    getInstanceLazyLoadingState(multiplicityPath: AbsoluteModelPath, instance: string): LazyLoadingInstanceState;
    getLazyLoadingStateByPath(path: AbsoluteModelPath): LazyLoadingInstanceState;
    resolvePathByLoadedInstances(path: AbsoluteModelPath | AbsoluteModelFieldPath): LazyLoadedPathResolveResult;
    resolvePathByInitialRules(sourcePath: AbsoluteModelPath | AbsoluteModelFieldPath): LazyLoadedPathResolveResult;
    getServerOnlyRules(): Set<LazyLoadRule>;
    getInitialLazyLoadRules(): Iterable<LazyLoadRule>;
    hasLazyLoadRules(): boolean;
    isLazyLoadingEnabled(): boolean;
    setLazyLoadingEnabledMode(enabled: boolean): void;
    clearState(): Promise<void>;
    waitForAllPendingRequests(): Promise<void>;
}
export interface IServerDataProvider {
    loadInner(paths: PathLoadSpecification[]): Promise<ModelHunk>;
}
export interface IUpdatedDataTarget {
    apply(data: IReadonlyModelHunk): void;
}
export interface ICurrentStoreDataProvider {
    sendCurrentChanges(): Promise<void>;
    getViewModel(path: BindingPath | AbsoluteResolvedModelPath): ViewModel | undefined;
}
export declare class LazyLoadingManager implements ILazyLoadingManager {
    private readonly originalLazyLoadingDeclaration;
    private readonly lazyLoadingElementInfo;
    private readonly serverDataProvider;
    private readonly updatedDataTarget;
    private readonly currentStoreDataProvider;
    private lazyLoadingEnabled;
    private isInCleaningState;
    constructor(lazyLoadingDeclaration: LazyLoadingDeclaration, serverDataProvider: IServerDataProvider, updatedDataTarget: IUpdatedDataTarget, currentStoreDataProvider: ICurrentStoreDataProvider);
    resolvePathByInitialRules(sourcePath: AbsoluteModelPath | AbsoluteModelFieldPath): LazyLoadedPathResolveResult;
    resolvePathByLoadedInstances(sourcePath: AbsoluteModelPath | AbsoluteModelFieldPath): LazyLoadedPathResolveResult;
    getInstanceLazyLoadingState(multiplicityPath: AbsoluteModelPath, instance: string): LazyLoadingInstanceState;
    getLazyLoadingStateByPath(path: AbsoluteModelPath): LazyLoadingInstanceState;
    requestInstancesLoad(multiplicityPath: AbsoluteModelPath, ...instances: string[]): Promise<void>;
    private getFirstLazyDeclaration;
    private requestInstancesLoadInternal;
    private getLazyLoadingElementInfo;
    private loadDataWithChangesCheck;
    getServerOnlyRules(): Set<LazyLoadRule>;
    getInitialLazyLoadRules(): Iterable<LazyLoadRule>;
    requestFullInstanceLoad(bindingPath: AbsoluteModelPath, ...instances: string[]): Promise<void>;
    requestFullInstanceLoadForCustomPath(mappedPath: AbsoluteModelPath): Promise<void>;
    setLazyLoadingEnabledMode(enabled: boolean): void;
    isLazyLoadingEnabled(): boolean;
    hasLazyLoadRules(): boolean;
    clearState(): Promise<void>;
    waitForAllPendingRequests(): Promise<void>;
}
