"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LazyLoadingManager = exports.LazyLoadingInstanceState = void 0;
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const Deferred_1 = require("../../../Helpers/Deferred");
const StringArrayKey_1 = require("../../../../../Common/StringArrayKey");
const Map2_1 = require("../../../../../Common/ModelPath/Map/Map2");
const BuildIterationSequence_1 = require("../../../../../Common/ModelPath/BuildIterationSequence");
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const ModelPathHelper_1 = require("../../../../../Common/ModelPath/ModelPathHelper");
const AbsoluteModelFieldPath_1 = require("../../../../../Common/ModelPath/AbsoluteModelFieldPath");
var LazyLoadingInstanceState;
(function (LazyLoadingInstanceState) {
    LazyLoadingInstanceState[LazyLoadingInstanceState["OnlyInitialData"] = 0] = "OnlyInitialData";
    LazyLoadingInstanceState[LazyLoadingInstanceState["OnlyRequiredData"] = 1] = "OnlyRequiredData";
    LazyLoadingInstanceState[LazyLoadingInstanceState["FullyLoaded"] = 2] = "FullyLoaded";
    LazyLoadingInstanceState[LazyLoadingInstanceState["Pending"] = 3] = "Pending";
})(LazyLoadingInstanceState || (exports.LazyLoadingInstanceState = LazyLoadingInstanceState = {}));
class LazyLoadingManager {
    originalLazyLoadingDeclaration;
    lazyLoadingElementInfo = new Map2_1.Map2();
    serverDataProvider;
    updatedDataTarget;
    currentStoreDataProvider;
    lazyLoadingEnabled = false;
    isInCleaningState = false;
    constructor(lazyLoadingDeclaration, serverDataProvider, updatedDataTarget, currentStoreDataProvider) {
        this.currentStoreDataProvider = currentStoreDataProvider;
        this.updatedDataTarget = updatedDataTarget;
        this.serverDataProvider = serverDataProvider;
        this.originalLazyLoadingDeclaration = Object.entries(lazyLoadingDeclaration).reduce((acc, [bindingPath, declarationNode]) => {
            const pathWithField = AbsoluteModelFieldPath_1.AbsoluteModelFieldPath.createFromMask(bindingPath, ModelPath_1.PathTokens.each, "");
            acc.set(pathWithField, declarationNode);
            return acc;
        }, new Map2_1.Map2());
    }
    resolvePathByInitialRules(sourcePath) {
        const path = (0, ModelPath_1.isModelPath)(sourcePath) ? sourcePath : sourcePath.path;
        const maskWithInstances = (0, ModelPathHelper_1.getMaskWithInstances)(path);
        const isInitialRule = sourcePath instanceof AbsoluteModelFieldPath_1.AbsoluteModelFieldPath &&
            this.originalLazyLoadingDeclaration.get(new AbsoluteModelFieldPath_1.AbsoluteModelFieldPath(maskWithInstances.mask, sourcePath.field))?.lazyLoadMode === "initial";
        if (isInitialRule) {
            const firstDeclaration = this.getFirstLazyDeclaration(path);
            const concreteInstances = maskWithInstances.instances.filter(ModelPath_1.PathTokens.isInstanceToken);
            if (isInitialRule &&
                firstDeclaration != undefined &&
                firstDeclaration.iterable.getIterationRank() >= concreteInstances.length) {
                return [concreteInstances];
            }
        }
        return [];
    }
    resolvePathByLoadedInstances(sourcePath) {
        const path = (0, ModelPath_1.isModelPath)(sourcePath) ? sourcePath : sourcePath.path;
        const maskWithInstances = (0, ModelPathHelper_1.getMaskWithInstances)(path);
        const firstDeclaration = this.getFirstLazyDeclaration(path);
        const concreteInstances = maskWithInstances.instances.filter(ModelPath_1.PathTokens.isInstanceToken);
        if (firstDeclaration == undefined || !this.lazyLoadingEnabled) {
            return [concreteInstances];
        }
        const multiElementInfo = this.getLazyLoadingElementInfo(firstDeclaration.iterable);
        const result = Iterator.from(multiElementInfo.instanceIds.keys())
            .filter(key => {
            for (const [index, instance] of IterableUtils_1.IterUtils.iterateEntries(concreteInstances)) {
                if (index >= key.items.length) {
                    return true;
                }
                if (key.items[index] !== instance) {
                    return false;
                }
            }
            return true;
        })
            .map(x => x.items.concat(concreteInstances.slice(x.items.length)))
            .toArray();
        return result;
    }
    getInstanceLazyLoadingState(multiplicityPath, instance) {
        const path = multiplicityPath.joinWith(instance);
        return this.getLazyLoadingStateByPath(path);
    }
    getLazyLoadingStateByPath(path) {
        const firstDeclaration = this.getFirstLazyDeclaration(path);
        if (firstDeclaration != undefined && firstDeclaration.lazyLoadMode == "deferred") {
            const multiElementInfo = this.getLazyLoadingElementInfo(firstDeclaration.iterable);
            const key = new StringArrayKey_1.StringArrayKey(firstDeclaration.instances.map(x => (ModelPath_1.PathTokens.isInstanceToken(x) ? x : "*")));
            const elementInstanceData = multiElementInfo.instanceIds.get(key);
            if (elementInstanceData != undefined) {
                return elementInstanceData.state;
            }
        }
        return firstDeclaration?.lazyLoadMode === "deferred"
            ? LazyLoadingInstanceState.OnlyInitialData
            : LazyLoadingInstanceState.FullyLoaded;
    }
    async requestInstancesLoad(multiplicityPath, ...instances) {
        // eslint-disable-next-line no-warning-comments
        // TODO: Временный костыль! Грузим сразу весь инстанс несмотря на то, есть у него required поля
        return await this.requestInstancesLoadInternal(LazyLoadingInstanceState.FullyLoaded, multiplicityPath, ...instances);
    }
    getFirstLazyDeclaration(path) {
        const iterationSequences = (0, BuildIterationSequence_1.buildIterationSequenceWithInstances)(path);
        const revertedIterationSequences = (0, IterableUtils_1.reverseArray)(iterationSequences);
        const firstDeclaration = revertedIterationSequences
            .map(x => ({
            lazyLoadMode: this.originalLazyLoadingDeclaration.get(new AbsoluteModelFieldPath_1.AbsoluteModelFieldPath(x.iterable.joinWith(ModelPath_1.PathTokens.each), ""))?.lazyLoadMode,
            iterable: x.iterable,
            instances: x.instances,
        }))
            .find(x => (0, TypingUtils_1.isNotNullOrUndefined)(x.lazyLoadMode) && x.lazyLoadMode != "request");
        return firstDeclaration;
    }
    async requestInstancesLoadInternal(requestLoadingMode, multiplicityPath, ...instances) {
        if (!this.lazyLoadingEnabled || this.isInCleaningState) {
            return;
        }
        const firstDeclaration = this.getFirstLazyDeclaration(multiplicityPath.joinWith(ModelPath_1.PathTokens.each));
        if (firstDeclaration != undefined && firstDeclaration.lazyLoadMode == "deferred") {
            if (firstDeclaration.iterable.getPathPartsAsArray().length < multiplicityPath.getPathPartsAsArray().length) {
                return;
            }
            const prevInstances = firstDeclaration.instances
                .slice(0, firstDeclaration.instances.length - 1)
                .map(x => (ModelPath_1.PathTokens.isInstanceToken(x) ? x : (0, TypingUtils_1.reject)()));
            const multiElementInfo = this.getLazyLoadingElementInfo(firstDeclaration.iterable);
            const instanceKeys = instances.map(instance => new StringArrayKey_1.StringArrayKey([...prevInstances, instance]));
            const instanceInfos = instanceKeys.map(key => {
                const instanceInfo = multiElementInfo.instanceIds.get(key);
                return instanceInfo;
            });
            const instancesInPendingStateTasks = instanceInfos
                .map(x => x?.loadingDeferred)
                .filter(TypingUtils_1.isNotNullOrUndefined);
            const deferred = new Deferred_1.Deferred();
            const requiredInstanceInfos = instanceKeys
                .map(key => {
                const instanceInfo = multiElementInfo.instanceIds.getOrSet(key, () => ({
                    state: LazyLoadingInstanceState.Pending,
                    loadingDeferred: deferred,
                }));
                return {
                    instanceInfo: instanceInfo,
                    key: key,
                };
            })
                .filter(x => x.instanceInfo.state === LazyLoadingInstanceState.Pending &&
                x.instanceInfo.loadingDeferred === deferred);
            const requiredInstances = requiredInstanceInfos.map(x => IterableUtils_1.IterUtils.last(x.key.items));
            const requestInstancesPathPart = `[${requiredInstances.join(",")}]`;
            if (requiredInstances.length == 0) {
                return;
            }
            const pathToLoad = {
                pathOrPathPrefix: multiplicityPath.joinWith(requestInstancesPathPart),
                skip: false,
            };
            const paths = [pathToLoad];
            try {
                if ((0, TypingUtils_1.arrayIsNotEmpty)(instancesInPendingStateTasks)) {
                    await Promise.all(instancesInPendingStateTasks);
                }
                if ((0, TypingUtils_1.arrayIsNotEmpty)(paths)) {
                    const data = await this.loadDataWithChangesCheck(paths);
                    this.updatedDataTarget.apply(data);
                }
            }
            finally {
                for (const requiredInstanceInfo of requiredInstanceInfos) {
                    requiredInstanceInfo.instanceInfo.state = requestLoadingMode;
                    requiredInstanceInfo.instanceInfo.loadingDeferred = undefined;
                }
                deferred.resolve();
            }
        }
    }
    getLazyLoadingElementInfo(multiplicityPath) {
        return this.lazyLoadingElementInfo.getOrSet(multiplicityPath, () => ({
            instanceIds: new Map2_1.Map2(),
        }));
    }
    async loadDataWithChangesCheck(paths) {
        await this.currentStoreDataProvider.sendCurrentChanges();
        const data = await this.serverDataProvider.loadInner(paths);
        for (const [path, viewModel] of [...data.iterateEntries()]) {
            const currentViewModel = this.currentStoreDataProvider.getViewModel(path) ?? {};
            data.set(path, { ...viewModel, ...currentViewModel });
        }
        return data;
    }
    getServerOnlyRules() {
        const serverOnlyRules = Iterator.from(this.originalLazyLoadingDeclaration.entries())
            .map(x => ({ path: x[0], declaration: x[1] }))
            .filter(x => x.declaration?.lazyLoadMode === "serverOnly")
            .map(x => ({
            modelPath: x.path,
            skip: true,
        }));
        return new Set(serverOnlyRules);
    }
    *getInitialLazyLoadRules() {
        yield* Iterator.from(this.originalLazyLoadingDeclaration.entries())
            .map(x => ({ path: x[0], declaration: x[1] }))
            .filter(x => x.declaration?.lazyLoadMode === "initial" || x.declaration?.lazyLoadMode === "deferred")
            .map(x => ({
            modelPath: x.path,
            skip: x.declaration?.lazyLoadMode === "deferred",
        }));
    }
    async requestFullInstanceLoad(bindingPath, ...instances) {
        return await this.requestInstancesLoadInternal(LazyLoadingInstanceState.FullyLoaded, bindingPath, ...instances);
    }
    async requestFullInstanceLoadForCustomPath(mappedPath) {
        const mappedPathTokens = mappedPath.getPathPartsAsArray();
        const instanceTokenIndices = Iterator.from(IterableUtils_1.IterUtils.iterateEntries(mappedPath.getPathParts()))
            .filter(([index, item]) => ModelPath_1.PathTokens.isInstanceToken(item))
            .map(([index, item]) => index);
        for (const instanceTokenIndex of instanceTokenIndices) {
            const multiplicityItems = mappedPathTokens.slice(0, instanceTokenIndex);
            const multiplicityPath = (0, ModelPath_1.createAbsoluteFromTokens)(multiplicityItems);
            const instance = mappedPathTokens[instanceTokenIndex];
            if (instance != undefined && ModelPath_1.PathTokens.isInstanceToken(instance)) {
                await this.requestFullInstanceLoad(multiplicityPath, instance);
            }
        }
    }
    setLazyLoadingEnabledMode(enabled) {
        this.lazyLoadingEnabled = enabled;
    }
    isLazyLoadingEnabled() {
        return this.lazyLoadingEnabled;
    }
    hasLazyLoadRules() {
        return !this.originalLazyLoadingDeclaration.isEmpty;
    }
    async clearState() {
        this.isInCleaningState = true;
        try {
            const promisesToAwait = Iterator.from(this.lazyLoadingElementInfo.values())
                .map(x => Iterator.from(x.instanceIds.values())
                .map(child => child.loadingDeferred?.promise)
                .filter(TypingUtils_1.isNotNullOrUndefined)
                .toArray())
                .flatMap(x => x)
                .map(x => x?.catch(e => null))
                .toArray();
            await Promise.all(promisesToAwait);
            this.lazyLoadingElementInfo.clear();
        }
        finally {
            this.isInCleaningState = false;
        }
    }
    async waitForAllPendingRequests() {
        const allPendingRequests = Iterator.from(this.lazyLoadingElementInfo.values())
            .flatMap(x => x.instanceIds.values())
            .map(x => x.loadingDeferred)
            .filter(TypingUtils_1.isNotNullOrUndefined);
        await Promise.all(allPendingRequests);
    }
}
exports.LazyLoadingManager = LazyLoadingManager;
