import { AbsoluteModelPath, AbsoluteResolvedModelPath } from "../../../../../Common/ModelPath/ModelPath";
import { BindingPath, ModelValue, ViewModel, ViewModelFieldName } from "../CoreTypes";
import { DataDeclarationHelper } from "../DataDeclaration/DataDeclarationHelper";
import { ModelEvent } from "../EventManagement/ModelEvents/ModelEvent";
import { ModelNormalizer } from "../Normalization/ModelNormalizer";
import { FormStore } from "../Store/CandyStore";
import { IFormReadonlyModelAccessor, IUpdateFormModelAccessor } from "../../Form/IFormModelAccessor";
import { IReadonlyModelHunk } from "./IReadonlyModelHunk";
import { ModelChunk, ReadonlyModelChunk } from "./ModelChunk";
export interface IModel extends IUpdateFormModelAccessor {
    setChildrenWithoutNormalization(path: BindingPath | AbsoluteResolvedModelPath, instances: string[]): void;
    flushIntoStore(store: FormStore): void;
    beginGetEvents(): IEventCollectionContext;
    removeByPrefix(nodePathPrefix: AbsoluteResolvedModelPath): void;
}
export declare class Model implements IModel {
    private readonly parentModel;
    private readonly modelNormalizer;
    private readonly pathWithTouchedErrors;
    private readonly eventCollectionContexts;
    private readonly pathWithTouchedWarnings;
    private readonly useShallowChildrenRemove;
    private updates;
    private deletedPaths;
    constructor(parentModel: IFormReadonlyModelAccessor, modelNormalizer: ModelNormalizer, dataDeclarationHelper: DataDeclarationHelper, useShallowChildrenRemove: boolean);
    addEvent(event: ModelEvent): void;
    beginGetEvents(): IEventCollectionContext;
    getViewModel(bindingPath: BindingPath | AbsoluteResolvedModelPath): ViewModel | undefined;
    setChildren(inPath: BindingPath | AbsoluteResolvedModelPath, children: string[]): void;
    setValue(bindingPath: BindingPath | AbsoluteResolvedModelPath, value: string | undefined): void;
    setViewModelField(inPath: BindingPath | AbsoluteResolvedModelPath, name: ViewModelFieldName, value: unknown): void;
    setBatch(batch: ReadonlyModelChunk | IReadonlyModelHunk): void;
    setBatchByPrefix(prefix: BindingPath | AbsoluteModelPath, batch: ReadonlyModelChunk): void;
    addInstance(path: BindingPath): string;
    removeInstance(path: BindingPath, instance: string): void;
    removeInstances(path: BindingPath, instances: readonly string[]): void;
    copySubtree(sourceBindingPath: BindingPath | AbsoluteResolvedModelPath, targetBindingPath: BindingPath | AbsoluteResolvedModelPath, doNotCopyPaths?: string[]): void;
    getViewModelField(bindingPath: AbsoluteResolvedModelPath | BindingPath, name: string): ModelValue | undefined;
    getValue(bindingPath: BindingPath | AbsoluteResolvedModelPath): string | undefined;
    getChildren(bindingPath: BindingPath | AbsoluteResolvedModelPath): string[] | undefined;
    getBatch(nodePathPrefix: BindingPath | AbsoluteResolvedModelPath): ModelChunk;
    setChildrenWithoutNormalization(path: BindingPath | AbsoluteResolvedModelPath, instances: string[]): void;
    remove(path: AbsoluteResolvedModelPath): void;
    flushIntoStore(store: FormStore): void;
    private removeInternal;
    private removeInternalSilently;
    private setViewModelFieldInternal;
    private getViewModelFieldInternal;
    removeByPrefix(nodePathPrefix: AbsoluteResolvedModelPath): void;
}
export interface IEventCollectionContext {
    releaseAndGetEvent(): ModelEvent[];
}
