"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sections = void 0;
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const MaskResolver_1 = require("../PathUtils/MaskResolver");
const UnitId_1 = require("../../../Controls/FormParts/Unit/UnitId/UnitId");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const PathMatcher_1 = require("../../../../../Common/ModelPath/PathMatcher");
const Set2_1 = require("../../../../../Common/ModelPath/Set/Set2");
const hashUtils_1 = require("../../../../../Common/hashUtils");
const ExternalEvents_1 = require("../EventManagement/ExternalEvents/ExternalEvents");
const ModelHunk_1 = require("./ModelHunk");
class OptionalSectionKey {
    instances;
    sectionName;
    constructor(sectionName, instances) {
        this.sectionName = sectionName;
        this.instances = instances;
    }
    getHash() {
        return ((0, hashUtils_1.getSimpleHashByString)(this.sectionName) +
            this.instances.reduce((acc, current) => acc + (0, hashUtils_1.getSimpleHashByString)(current), 0));
    }
    isEquals(right) {
        if (this.instances.length != right.instances.length || this.sectionName !== right.sectionName) {
            return false;
        }
        return this.instances.every((item, index) => item === right.instances[index]);
    }
}
class Sections {
    section2masks;
    dataDeclarationHelper;
    externalEventManager;
    allOptionalSections;
    visibleOptionalSections;
    allSingleSections;
    constructor(dataDeclarationHelper, externalEventManager) {
        this.dataDeclarationHelper = dataDeclarationHelper;
        this.externalEventManager = externalEventManager;
        const optionalSections = dataDeclarationHelper.getOptionalSections().map(x => {
            const path = (0, ModelPath_1.createAbsoluteFromMask)(x.path, ModelPath_1.PathTokens.each);
            return [
                x.id,
                {
                    path: path,
                    requiredInstancesCount: path.getIterationRank(),
                    optional: true,
                },
            ];
        });
        // @ts-ignore
        this.allOptionalSections = new Map(optionalSections);
        const singleSections = dataDeclarationHelper.getAllSingleSections().map(x => {
            const path = (0, ModelPath_1.createAbsoluteFromMask)(x.path, ModelPath_1.PathTokens.each);
            return [
                x.id,
                {
                    path: path,
                    requiredInstancesCount: path.getIterationRank(),
                    dependsOnSection: x.dependsOnSection,
                    optional: x.optional,
                },
            ];
        });
        // @ts-ignore
        this.allSingleSections = new Map(singleSections);
        this.section2masks = dataDeclarationHelper.buildSectionToMasksMap();
        this.visibleOptionalSections = new Set2_1.Set2();
    }
    isPrefixPath(pagePath, prefix) {
        const prefixParts = prefix.getPathPartsAsArray();
        const pagePathParts = pagePath.getPathPartsAsArray();
        if (pagePathParts.length < prefixParts.length) {
            return false;
        }
        for (const [index, prefixPart] of prefixParts.entries()) {
            const pathPart = pagePathParts[index];
            if (!(pathPart === prefixPart ||
                (ModelPath_1.PathTokens.isMultiToken(pathPart) && ModelPath_1.PathTokens.isInstanceToken(prefixPart)))) {
                return false;
            }
        }
        return true;
    }
    markSectionsAsIgnoredByPrefix(prefixes) {
        if (this.allOptionalSections.size > 0) {
            for (const prefix of prefixes) {
                for (const [pageId, info] of this.allOptionalSections.entries()) {
                    if (this.isPrefixPath(info.path, prefix)) {
                        const instances = info.path.matchFor(prefix).reduce((acc, curr) => {
                            if (curr.kind === PathMatcher_1.MatchKind.MatchedMultiplicity &&
                                ModelPath_1.PathTokens.isInstanceToken(curr.target)) {
                                return [...acc, curr.target];
                            }
                            return acc;
                        }, []);
                        this.markSectionAsIgnoredForced(pageId, info, instances);
                    }
                }
            }
        }
    }
    markSectionAsIgnoredForced(rootSectionName, sectionInfo, allInstances) {
        if (allInstances.length < sectionInfo.requiredInstancesCount) {
            return;
        }
        const optionalSectionKey = new OptionalSectionKey(rootSectionName, allInstances.slice(0, sectionInfo.requiredInstancesCount));
        this.visibleOptionalSections.remove(optionalSectionKey);
    }
    getSingleSectionByNameOrDependencySection(sectionName) {
        const sectionInfo = this.allSingleSections.get(sectionName);
        if (sectionInfo == undefined) {
            return undefined;
        }
        if (sectionInfo.dependsOnSection != undefined) {
            const dependencySection = this.allSingleSections.get(sectionInfo.dependsOnSection);
            return dependencySection?.optional === true
                ? {
                    ...dependencySection,
                    sectionName: sectionInfo.dependsOnSection,
                }
                : undefined;
        }
        return sectionInfo.optional
            ? {
                ...sectionInfo,
                sectionName: sectionName,
            }
            : undefined;
    }
    markSectionAsIgnored(sectionName) {
        const unitId = (0, UnitId_1.sectionNameToUnitId)(sectionName);
        const rootSectionName = unitId.pageId;
        const sectionInfo = this.getSingleSectionByNameOrDependencySection(rootSectionName);
        if (sectionInfo == undefined) {
            return;
        }
        const allInstances = (0, UnitId_1.getAllUnitIdInstances)(unitId);
        if (allInstances.length < sectionInfo.requiredInstancesCount) {
            return;
        }
        const optionalSectionKey = new OptionalSectionKey(sectionInfo.sectionName, allInstances.slice(0, sectionInfo.requiredInstancesCount));
        this.visibleOptionalSections.remove(optionalSectionKey);
    }
    removeSectionFromIgnore(sectionName) {
        const unitId = (0, UnitId_1.sectionNameToUnitId)(sectionName);
        const rootSectionName = unitId.pageId;
        const sectionInfo = this.getSingleSectionByNameOrDependencySection(rootSectionName);
        if (sectionInfo == undefined) {
            return;
        }
        const allInstances = (0, UnitId_1.getAllUnitIdInstances)(unitId);
        if (allInstances.length < sectionInfo.requiredInstancesCount) {
            return;
        }
        const optionalSectionKey = new OptionalSectionKey(sectionInfo.sectionName, allInstances.slice(0, sectionInfo.requiredInstancesCount));
        this.visibleOptionalSections.add(optionalSectionKey);
        this.externalEventManager.triggerExternalEvent((0, ExternalEvents_1.formPageChangedEvent)(sectionInfo.sectionName));
    }
    markAllAsIgnored() {
        this.visibleOptionalSections.clear();
    }
    isSectionIgnored(sectionName) {
        const unitId = (0, UnitId_1.sectionNameToUnitId)(sectionName);
        const rootSectionName = unitId.pageId;
        const sectionInfo = this.getSingleSectionByNameOrDependencySection(rootSectionName);
        if (sectionInfo == undefined) {
            return false;
        }
        const allInstances = (0, UnitId_1.getAllUnitIdInstances)(unitId);
        if (allInstances.length < sectionInfo.requiredInstancesCount) {
            return false;
        }
        const optionalSectionKey = new OptionalSectionKey(sectionInfo.sectionName, allInstances.slice(0, sectionInfo.requiredInstancesCount));
        return !this.visibleOptionalSections.has(optionalSectionKey);
    }
    isIgnoredPathByMasks(path) {
        if (this.allOptionalSections.size === 0) {
            return false;
        }
        const sections = this.dataDeclarationHelper.getSectionsStrict(path);
        if (sections == undefined) {
            return false;
        }
        return sections.some(x => this.isSectionIgnored(x));
    }
    resolveOptionalSections(serverSideModel) {
        if (this.allOptionalSections.size > 0) {
            const pathsWithMask = Iterator.from(ModelHunk_1.ModelHunk.iterateEntries(serverSideModel))
                .filter(
            /* eslint-disable no-warning-comments */
            // TODO: временно ставлю сюда этот костыль т.к. при включенной серверной нормализации
            // TODO: не удается корректно заполнять children для опциональных разделов и
            // TODO: поэтому отключаем просмотр children полей до тех пора не сделаем https://yt.skbkontur.ru/issue/FS-7374
            /* eslint-enable no-warning-comments */
            ([path, viewModel]) => this.dataDeclarationHelper.hasMask(path) && !this.hasChildrenField(viewModel))
                .map(([path, viewModel]) => path);
            for (const path of pathsWithMask) {
                const sections = this.dataDeclarationHelper.getSections(path);
                for (const sectionName of sections) {
                    this.removeSectionFromIgnore(sectionName);
                }
            }
        }
    }
    hasChildrenField(viewModel) {
        return viewModel.children != null;
    }
    paths(model, section) {
        return Iterator.from(this.masks(section)).flatMap(x => MaskResolver_1.MaskResolver.resolvePath(model, x));
    }
    masks(section) {
        const sectionMask = getSectionMaskBySectionName(section);
        const [_sectionName, ...instances] = section.split("-");
        const sectionMasks = this.section2masks.get(sectionMask)?.values() ?? [];
        return Iterator.from(sectionMasks)
            .map(mask => mask.applyInstancesDirty(instances))
            .filter(mask => !this.isIgnoredPathByMasks(mask));
    }
    getSectionPathsForCalculations(model, section) {
        const unitId = (0, UnitId_1.sectionNameToUnitId)(section);
        const rootSectionName = unitId.pageId;
        const sectionInfo = this.allSingleSections.get(rootSectionName);
        if (sectionInfo == undefined) {
            return Iterator.from(this.masks(section)).flatMap(x => MaskResolver_1.MaskResolver.resolvePath(model, x));
        }
        const allInstances = (0, UnitId_1.getAllUnitIdInstances)(unitId);
        if (allInstances.length < sectionInfo.requiredInstancesCount) {
            return [];
        }
        const sections = Iterator.from(this.allSingleSections.entries())
            .filter(([key, value]) => value.path.startsWith(sectionInfo.path) &&
            value.requiredInstancesCount == sectionInfo.requiredInstancesCount)
            .map(([sectionId]) => [sectionId, ...allInstances].join("-"));
        return sections.flatMap(x => this.masks(x)).flatMap(x => MaskResolver_1.MaskResolver.resolvePath(model, x));
    }
    getSectionPrefix(section) {
        const unitId = (0, UnitId_1.sectionNameToUnitId)(section);
        const rootSectionName = unitId.pageId;
        const sectionInfo = this.allSingleSections.get(rootSectionName);
        if (sectionInfo == undefined) {
            return undefined;
        }
        const allInstances = (0, UnitId_1.getAllUnitIdInstances)(unitId);
        if (allInstances.length < sectionInfo.requiredInstancesCount) {
            return undefined;
        }
        const resolvedPath = sectionInfo.path.applyInstancesDirty(allInstances);
        return (0, ModelPath_1.createAbsoluteResolvedFromTokens)(resolvedPath.getSimpleTokensAsArrayStrict());
    }
}
exports.Sections = Sections;
function getSectionMaskBySectionName(sectionName) {
    const [sectionId, ...instances] = sectionName.split("-");
    let result;
    if (instances.length == 0) {
        result = sectionId ?? (0, TypingUtils_1.reject)();
    }
    else {
        result = `${sectionId}-${Array(instances.length).fill("*").join("-")}`;
    }
    return result;
}
