"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MaskRegistrator = void 0;
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const PathUtils_1 = require("./PathUtils");
class MaskRegistrator {
    masks = { nodes: {} };
    cache = new Map();
    registerMask(nodePath, valueName) {
        const tokens = PathUtils_1.PathUtils.split(nodePath);
        let currentNode = this.masks;
        for (const [i, token] of IterableUtils_1.IterUtils.iterateEntries(tokens)) {
            const nodes = currentNode.nodes;
            const tokenNode = nodes[token] || { nodes: {} };
            if (i === tokens.length - 1) {
                if (!tokenNode.names) {
                    tokenNode.names = {};
                }
                tokenNode.names[valueName ?? ""] = true;
            }
            if (token[0] === "*") {
                const multiple = (currentNode.multiple = currentNode.multiple || []);
                if (!multiple.includes(token)) {
                    multiple.push(token);
                }
            }
            nodes[token] = tokenNode;
            currentNode = tokenNode;
        }
        this.cache.clear();
    }
    unregisterMask(mask) {
        const [nodePath, valueName] = PathUtils_1.PathUtils.splitToPathAndValueName(mask);
        const tokens = PathUtils_1.PathUtils.split(nodePath);
        this.cache.clear();
        const nodes = [];
        let node = this.masks;
        for (const token of tokens) {
            const nextNode = node.nodes[token];
            if (nextNode == undefined) {
                return;
            }
            nodes.push(node);
            node = nextNode;
        }
        if (!node.names || !node.names[valueName]) {
            return;
        }
        delete node.names[valueName];
        nodes.push(node);
        for (let i = nodes.length - 1; i >= 1; --i) {
            const currNode = nodes[i] ?? (0, TypingUtils_1.reject)();
            if (!Object.keys(currNode.nodes || {}).length && !Object.keys(currNode.names || {}).length) {
                const token = tokens[i - 1];
                if (token != undefined) {
                    delete nodes[i - 1]?.nodes[token];
                }
            }
            else {
                break;
            }
        }
    }
    getMasks(path, valueName) {
        let result = [];
        const cacheKey = (0, ModelPath_1.convertToLegacyPath)(path) + ((0, TypingUtils_1.isNullOrEmpty)(valueName) ? "" : "." + valueName);
        const cacheValue = this.cache.get(cacheKey);
        if (cacheValue != undefined) {
            result = [...cacheValue];
        }
        else {
            let tokens;
            if ((0, ModelPath_1.isModelPath)(path)) {
                if (path.isEmpty()) {
                    tokens = [""];
                }
                else {
                    tokens = path.getPathPartsAsArray();
                }
            }
            else {
                tokens = PathUtils_1.PathUtils.split(path);
            }
            const stack = [[this.masks, "", 0]];
            while (stack.length) {
                const t = stack.pop();
                if (t == undefined) {
                    throw new Error("InvalidProgramStateError");
                }
                const node = t[0];
                if (!node) {
                    continue;
                }
                let accum = t[1];
                const pos = t[2];
                if (pos === tokens.length) {
                    if (node.names && node.names[valueName ?? ""]) {
                        result.push(accum);
                    }
                    continue;
                }
                if (accum !== "") {
                    accum = accum + "/";
                }
                const token = tokens[pos];
                if (node.multiple && (ModelPath_1.PathTokens.isMultiToken(token) || ModelPath_1.PathTokens.isInstanceToken(token))) {
                    for (const multipleToken of node.multiple) {
                        const nextMultipleNode = node.nodes[multipleToken];
                        if (nextMultipleNode == undefined) {
                            throw new Error("InvalidProgramStateError");
                        }
                        stack.push([nextMultipleNode, accum + multipleToken, pos + 1]);
                    }
                }
                if (node.nodes) {
                    if (ModelPath_1.PathTokens.isSimpleToken(token)) {
                        const nextNode = node.nodes[token];
                        if (nextNode != undefined) {
                            stack.push([nextNode, accum + token, pos + 1]);
                        }
                    }
                }
            }
            this.cache.set(cacheKey, [...result]);
        }
        if (valueName != undefined && valueName !== "") {
            for (let i = 0; i < result.length; ++i) {
                result[i] += "." + valueName;
            }
        }
        return result;
    }
}
exports.MaskRegistrator = MaskRegistrator;
