"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MaskResolver = void 0;
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const BuildIterationSequence_1 = require("../../../../../Common/ModelPath/BuildIterationSequence");
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const Errors_1 = require("../../../../../Common/Errors");
class MaskResolver {
    static resolveMask(model, mask) {
        const path = (0, ModelPath_1.createAbsoluteFromMask)(mask, ModelPath_1.PathTokens.each);
        return Iterator.from(this.resolvePath(model, path))
            .map(x => x.toLegacyPath())
            .toArray();
    }
    static getPathsByPrefix(dataDeclarationHelper, model, nodePathPrefix) {
        return Iterator.from(dataDeclarationHelper.getMasksByPrefix(nodePathPrefix)).flatMap(m => MaskResolver.resolvePath(model, m));
    }
    static *resolvePath(model, path) {
        if (path.isResolved()) {
            yield path;
        }
        else {
            const iterationSequence = (0, BuildIterationSequence_1.buildIterationSequence)(path);
            (0, TypingUtils_1.ensureArrayIsNotEmpty)(iterationSequence);
            const { iterable, suffix } = IterableUtils_1.IterUtils.first(iterationSequence);
            if (!iterable.isResolved()) {
                throw new Errors_1.InvalidProgramStateError();
            }
            const children = model.getChildren(iterable) ?? [];
            for (const childInstance of children) {
                yield* this.resolvePath(model, iterable.joinWith(childInstance, suffix));
            }
        }
    }
}
exports.MaskResolver = MaskResolver;
