"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathUtils = void 0;
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const PathMatcher_1 = require("../../../../../Common/ModelPath/PathMatcher");
const ModelPathBuilder_1 = require("../../../../../Common/ModelPath/ModelPathBuilder");
class PathUtils {
    static getAbsolutePath(parentPath, relativePath) {
        const parentSymbol = "-";
        let pathSuffix = relativePath;
        let parentLength = parentPath.length;
        let i = 0;
        while (i < pathSuffix.length && pathSuffix[i] === parentSymbol) {
            if (parentLength === -1) {
                throw new Error("Argument exception");
            }
            parentLength = parentPath.lastIndexOf("/", parentLength - 1);
            i += 1;
        }
        parentPath = parentPath.substr(0, parentLength);
        pathSuffix = pathSuffix.substr(i);
        return ((parentPath.length && pathSuffix.length && pathSuffix[0] !== "." ? parentPath + "/" : parentPath) +
            pathSuffix);
    }
    static getParentPath(nodePath) {
        const delimIdx = nodePath.lastIndexOf("/");
        return delimIdx === -1 ? null : nodePath.substr(0, delimIdx);
    }
    static splitToPathAndValueName(path) {
        for (let i = path.length - 1; i >= 0; --i) {
            if (path[i] === "/") {
                return [path, ""];
            }
            else if (path[i] === ".") {
                return [path.substr(0, i), path.substr(i + 1)];
            }
        }
        return [path, ""];
    }
    static getLastToken(nodePath) {
        const delimIdx = nodePath.lastIndexOf("/");
        return delimIdx === -1 ? nodePath : nodePath.substr(delimIdx + 1);
    }
    static getValuePath(nodePath, valueName) {
        return nodePath + "." + valueName;
    }
    static isMask(path) {
        return path.includes("*");
    }
    static trimLastStarIfLastToken(path) {
        if ((0, ModelPath_1.isModelPath)(path)) {
            return path.trimLastStarIfLastToken().toLegacyPath();
        }
        if (PathUtils.getLastToken(path) === "*") {
            return PathUtils.getParentPath(path) ?? "";
        }
        return path;
    }
    static isPrefix(prefix, path) {
        return path.indexOf(prefix) === 0 && (path.length === prefix.length || path[prefix.length] === "/");
    }
    static split(path) {
        return path.split("/");
    }
    static splitMask(mask) {
        return mask.split("*").map(subPath => this.join(this.split(subPath).filter(x => x)));
    }
    static join(tokens) {
        return tokens.join("/");
    }
    static resolve(pathWithMask, instances = []) {
        if (typeof pathWithMask === "string") {
            let instanceIndex = 0;
            const tokens = this.split(pathWithMask);
            const resolvedTokens = tokens.map(token => {
                if (token !== "*") {
                    return token;
                }
                const instance = instances[instanceIndex];
                instanceIndex++;
                if (instance === undefined) {
                    return "*";
                }
                return instance;
            });
            return this.join(resolvedTokens);
        }
        else {
            const resultBuilder = new ModelPathBuilder_1.ModelPathBuilder(pathWithMask.isAbsolute());
            let instanceIndex = 0;
            for (const token of pathWithMask.getPathParts()) {
                if (ModelPath_1.PathTokens.isSimpleToken(token)) {
                    resultBuilder.append(token);
                }
                else {
                    const instance = instances[instanceIndex];
                    instanceIndex++;
                    if (instance === undefined) {
                        resultBuilder.append(token);
                    }
                    else {
                        resultBuilder.append(instance);
                    }
                }
            }
            if (pathWithMask.isAbsolute()) {
                return resultBuilder.toAbsolutePath();
            }
            return resultBuilder.toPath();
        }
    }
    static resolveMaskToDeepestCommonInstance(fullPath, mask) {
        const fullPathTokens = this.split(fullPath);
        const maskTokens = this.split(mask);
        const deepestCommonInstancePathTokens = [];
        let tokensBeforeLastMultiplicity = [];
        for (let index = 0; maskTokens[index] && fullPathTokens[index]; index++) {
            if (maskTokens[index] === fullPathTokens[index]) {
                tokensBeforeLastMultiplicity.push(fullPathTokens[index]);
            }
            else if (maskTokens[index] === "*") {
                const instanceNumber = fullPathTokens[index];
                deepestCommonInstancePathTokens.push(...tokensBeforeLastMultiplicity, instanceNumber);
                tokensBeforeLastMultiplicity = [];
            }
            else {
                break;
            }
        }
        const deepestCommonInstancePathTokensLength = deepestCommonInstancePathTokens.length;
        if (deepestCommonInstancePathTokensLength === 0) {
            return null;
        }
        return deepestCommonInstancePathTokens
            .concat(maskTokens.slice(deepestCommonInstancePathTokensLength))
            .join("/");
    }
    static isPicklistFieldValueName(valueName) {
        return valueName.startsWith("picklist@");
    }
    static getPicklistFieldName(valueName) {
        if (!valueName.startsWith("picklist@")) {
            throw new Error(`${valueName} is not picklist field name`);
        }
        const splitValueName = valueName.split("@");
        if (!(0, TypingUtils_1.arrayHasLength)(splitValueName, 2)) {
            throw new Error(`${valueName} is not a valid picklist field name`);
        }
        return splitValueName[1];
    }
    /**
     * Используется снаружи. Извлкается номер инстанса для пути. Например, если передать `pathWithMask` /A/B*, а
     * `path` /A/B/2, то функция вернёт `"2"`
     * @param pathWithMask
     * @param path
     */
    static extractInstance(pathWithMask, path) {
        const modelPathWithMask = (0, ModelPath_1.createFromMask)(pathWithMask, true, ModelPath_1.PathTokens.each);
        const modelPath = (0, ModelPath_1.createFromMask)(path, true, ModelPath_1.PathTokens.each);
        const matchedMultiplicity = modelPathWithMask
            .matchFor(modelPath)
            .filter(x => x.kind === PathMatcher_1.MatchKind.MatchedMultiplicity && ModelPath_1.PathTokens.isInstanceToken(x.target));
        if ((0, TypingUtils_1.arrayHasLength)(matchedMultiplicity, 1) && matchedMultiplicity[0].kind === PathMatcher_1.MatchKind.MatchedMultiplicity) {
            if (ModelPath_1.PathTokens.isInstanceToken(matchedMultiplicity[0].target)) {
                return matchedMultiplicity[0].target;
            }
        }
        throw new Error(`Unable to extract single index while match paths '${modelPathWithMask.toString()}' and '${modelPath.toString()}'`);
    }
    static extractPathWithInstance(pathWithMask, path) {
        const prefix = PathUtils.trimLastStarIfLastToken(pathWithMask);
        if (!PathUtils.isPrefix(prefix, path)) {
            return undefined;
        }
        const instance = PathUtils.extractInstance(pathWithMask, path);
        const pathWithInstance = PathUtils.join([prefix, instance]);
        return pathWithInstance;
    }
}
exports.PathUtils = PathUtils;
