"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildFilterPredicate = buildFilterPredicate;
exports.gPathFilter = gPathFilter;
const DigestQueryFilter_1 = require("../../../../../Common/CandyApiClient/DigestApiClient/Types/DigestQueryFilter");
function buildFilterPredicate(filter) {
    if (filter == undefined) {
        return () => true;
    }
    switch (filter?.operator) {
        case DigestQueryFilter_1.DigestFilterOperator.Equal: {
            const filterValue = filter.value.toLowerCase();
            return item => (item[filter.column]?.toString().toLowerCase() ?? "") === filterValue;
        }
        case DigestQueryFilter_1.DigestFilterOperator.Prefix: {
            const filterValue = filter.value.toLowerCase();
            return item => (item[filter.column]?.toString().toLowerCase() ?? "")?.startsWith(filterValue) ?? false;
        }
        case DigestQueryFilter_1.DigestFilterOperator.Substring: {
            const filterValue = filter.value.toLowerCase();
            return item => (item[filter.column]?.toString().toLowerCase() ?? "")?.includes(filterValue) ?? false;
        }
        case DigestQueryFilter_1.DigestFilterOperator.ContainsInList: {
            const filterValue = filter.value.toLowerCase();
            return item => (item[filter.column]?.toString().toLowerCase() ?? "")?.split(",").includes(filterValue) ?? false;
        }
        case DigestQueryFilter_1.DigestLogicalOperator.Or: {
            if (filter.operands.length === 0) {
                return () => true;
            }
            const operandPredicates = filter.operands.map(x => buildFilterPredicate(x));
            return item => operandPredicates.some(x => x(item));
        }
        case DigestQueryFilter_1.DigestLogicalOperator.And: {
            const operandPredicates = filter.operands.map(x => buildFilterPredicate(x));
            return item => operandPredicates.every(x => x(item));
        }
        default:
            return () => true;
    }
}
function gPathFilter(items, filterQuery) {
    let firstItem;
    let suitableCount = 0;
    const filterPredicate = buildFilterPredicate(filterQuery?.filter);
    const result = Iterator.from(items)
        .map(item => {
        firstItem = firstItem ?? item;
        return item;
    })
        .filter(filterPredicate)
        .map(item => {
        suitableCount++;
        return item;
    })
        .toArray();
    const slicedResult = Iterator.from(result)
        .drop(filterQuery?.skip ?? 0)
        .take(filterQuery?.take ?? Infinity)
        .toArray();
    return {
        items: slicedResult,
        suitableCount: suitableCount,
        columnMetas: Object.keys(firstItem ?? {}).map(x => ({ name: x, withMarkdown: false })),
    };
}
