"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emptyPicklistQueryResponse = void 0;
exports.isPicklistFilterColumn = isPicklistFilterColumn;
exports.isPicklistLogicalFilter = isPicklistLogicalFilter;
exports.isPicklistFilter = isPicklistFilter;
exports.isPicklistLogicalOperator = isPicklistLogicalOperator;
exports.unionWithAndFilters = unionWithAndFilters;
exports.unionWithOrFilters = unionWithOrFilters;
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const DigestQueryFilter_1 = require("../../../../../Common/CandyApiClient/DigestApiClient/Types/DigestQueryFilter");
const emptyPicklistQueryResponse = () => ({
    items: [],
    suitableCount: 0,
    columnMetas: [],
});
exports.emptyPicklistQueryResponse = emptyPicklistQueryResponse;
function isPicklistFilterColumn(filter) {
    return (filter.operator === DigestQueryFilter_1.DigestFilterOperator.Equal ||
        filter.operator === DigestQueryFilter_1.DigestFilterOperator.Substring ||
        filter.operator === DigestQueryFilter_1.DigestFilterOperator.Prefix ||
        filter.operator === DigestQueryFilter_1.DigestFilterOperator.ContainsInList);
}
function isPicklistLogicalFilter(filter) {
    return ((filter.operator === DigestQueryFilter_1.DigestLogicalOperator.Or || filter.operator === DigestQueryFilter_1.DigestLogicalOperator.And) &&
        Array.isArray(filter.operands));
}
function isPicklistFilter(filter) {
    if (!isPicklistLogicalOperator(filter)) {
        return isPicklistFilterColumn(filter) || isPicklistLogicalFilter(filter);
    }
    return false;
}
function isPicklistLogicalOperator(operator) {
    return operator === DigestQueryFilter_1.DigestLogicalOperator.Or || operator === DigestQueryFilter_1.DigestLogicalOperator.And;
}
function unionWithAndFilters(filter1, filter2) {
    return unionFilters([filter1, filter2], DigestQueryFilter_1.DigestLogicalOperator.And);
}
function unionWithOrFilters(filter1, filter2) {
    return unionFilters([filter1, filter2], DigestQueryFilter_1.DigestLogicalOperator.Or);
}
function unionFilters(filters, operator) {
    const resultFilters = filters.filter(TypingUtils_1.isNotNullOrUndefined);
    if (resultFilters.length === 0) {
        return undefined;
    }
    if (resultFilters.length === 1) {
        return resultFilters[0];
    }
    return {
        operator: operator,
        operands: resultFilters,
    };
}
