"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PicklistsProvider = void 0;
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const DigestApiPicklist_1 = require("./DigestApiPicklist");
const InnerPathsPicklist_1 = require("./InnerPathsPicklist");
class PicklistsProvider {
    modelAccessor;
    digestApiClient;
    digestApiPicklists = new Map();
    innerPathsPicklists = new Map();
    constructor(digestApiClient, modelAccessor) {
        this.digestApiClient = digestApiClient;
        this.modelAccessor = modelAccessor;
    }
    getDigestApiPicklist(gId) {
        return getOrSet(this.digestApiPicklists, gId, () => new DigestApiPicklist_1.DigestApiPicklist(this.digestApiClient, gId));
    }
    getInnerPathsPicklist(paths) {
        return getOrSet(this.innerPathsPicklists, paths.map(x => x.toString()).join(""), () => new InnerPathsPicklist_1.InnerPathsPicklist(this.modelAccessor, paths));
    }
    getPicklist(source) {
        if (source.gId) {
            return this.getDigestApiPicklist(source.gId);
        }
        else if (source.gPaths != undefined) {
            const paths = source.gPaths.map(x => (0, ModelPath_1.createAbsoluteFromMask)(x, ModelPath_1.PathTokens.each));
            return this.getInnerPathsPicklist(paths);
        }
        return undefined;
    }
}
exports.PicklistsProvider = PicklistsProvider;
function getOrSet(map, key, initValueFactory) {
    const result = map.get(key);
    if (result != undefined) {
        return result;
    }
    const newResult = initValueFactory();
    map.set(key, newResult);
    return newResult;
}
