"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.aggregateValuesMiddleware = aggregateValuesMiddleware;
const lodash_debounce_1 = __importDefault(require("lodash.debounce"));
const ModelHunk_1 = require("../../ModelStoring/ModelHunk");
const modelActions_1 = require("../../ModelStoring/Store/modelActions");
function aggregateValuesMiddleware(form) {
    const result = () => (next) => {
        let data = ModelHunk_1.ModelHunk.create();
        function flushBatch() {
            const filledData = data;
            data = {};
            form.setBatch(filledData);
        }
        const flushThrottled = (0, lodash_debounce_1.default)(flushBatch, 100);
        return (action) => {
            if (typeof action === "function") {
                return next(action);
            }
            if (action.type === modelActions_1.ModelActionType.AggregatedSetToModel) {
                const { bindingPath, field, value } = action.payload;
                if (ModelHunk_1.ModelHunk.hasPath(data, bindingPath)) {
                    // @ts-ignore
                    ModelHunk_1.ModelHunk.updateIfExists(data, bindingPath, x => (x[field] = value));
                }
                else {
                    ModelHunk_1.ModelHunk.set(data, bindingPath, { [field]: value });
                }
                flushThrottled();
            }
            else {
                return next(action);
            }
            return undefined;
        };
    };
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return,@typescript-eslint/no-explicit-any
    return result;
}
