"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.modelChangeHookMiddleware = void 0;
const IterableUtils_1 = require("../../../../../../Common/IterableUtils");
const ModelPath_1 = require("../../../../../../Common/ModelPath/ModelPath");
const modelActions_1 = require("../../ModelStoring/Store/modelActions");
const findPathInHooks = (hooks, targetPath, viewModel) => hooks.entries().flatMap(([listener, pathList]) => pathList.map(bindingPath => {
    const modelPath = (0, ModelPath_1.createFromMask)(bindingPath, "auto", ModelPath_1.PathTokens.current);
    return modelPath.isIncludes(targetPath)
        ? {
            listener: listener,
            path: targetPath,
            viewModel: viewModel,
        }
        : undefined;
}));
const modelChangeHookMiddleware = (form) => {
    const result = (store) => (next) => (action) => {
        // @ts-ignore
        const res = next(action);
        if ("type" in action && action?.type === modelActions_1.ModelActionType.SetValues) {
            const hooks = form.getModelChangeEventListenerHooks();
            const deletedTargets = Iterator.from(action.payload.deleted).flatMap(deletedPath => findPathInHooks(hooks, deletedPath, undefined));
            const valuesTargets = Iterator.from(action.payload.values.iterateEntries()).flatMap(([payloadPath, viewModel]) => findPathInHooks(hooks, payloadPath, viewModel));
            for (const notifyTarget of (0, IterableUtils_1.concat)(deletedTargets, valuesTargets)) {
                if (notifyTarget) {
                    notifyTarget.listener({
                        path: notifyTarget.path.toLegacyPath(),
                        value: notifyTarget.viewModel?.value ?? "",
                    });
                }
            }
        }
        return res;
    };
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return,@typescript-eslint/no-explicit-any
    return result;
};
exports.modelChangeHookMiddleware = modelChangeHookMiddleware;
