"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sentryBreadcrumbsMiddleware = sentryBreadcrumbsMiddleware;
const FeatureFlagsActions_1 = require("../../Externals/FeatureFlags/Store/FeatureFlagsActions");
const unitActions_1 = require("../../../Controls/FormParts/Unit/Store/unitActions");
const actionTypeBlacklist = [FeatureFlagsActions_1.FeatureFlagsActionType.LoadInitialFeatureFlags, unitActions_1.UnitActionType.InitUnitInfo];
function sentryBreadcrumbsMiddleware(hub) {
    const result = (store) => (next) => (action) => {
        if (typeof action !== "function") {
            try {
                if (actionTypeBlacklist.includes(action.type)) {
                    return next(action);
                }
                const actionPayloadToLog = 
                // @ts-ignore
                action["payload"] === undefined
                    ? undefined
                    : // @ts-ignore
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                        Object.keys(action["payload"]).length > 10
                            ? "[LargePayload]"
                            : // @ts-ignore
                                action["payload"];
                hub.addBreadcrumb({
                    level: "log",
                    data: { type: action.type, payload: actionPayloadToLog },
                    category: "ReduxAction",
                });
            }
            catch (e) {
                hub.addBreadcrumb({
                    level: "error",
                    message: `Redux action breadcrumb build failed: ${e instanceof Error ? e?.message : "unknown reason"}`,
                    category: "ReduxAction",
                });
            }
        }
        // @ts-ignore
        return next(action);
    };
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return,@typescript-eslint/no-explicit-any
    return result;
}
