"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFormStore = createFormStore;
const redux_1 = require("redux");
const redux_thunk_1 = __importDefault(require("redux-thunk"));
const constants_1 = require("../../../Controls/constants");
const unitReducer_1 = require("../../../Controls/FormParts/Unit/Store/unitReducer");
const crossFitTableMiddleware_1 = require("../../../Controls/MultiControls/CrossfitTable/Store/crossFitTableMiddleware");
const crossFitTableReducer_1 = require("../../../Controls/MultiControls/CrossfitTable/Store/crossFitTableReducer");
const multiLineReducer_1 = require("../../../Controls/MultiControls/MultiLine/Store/multiLineReducer");
const InitializeSentry_1 = require("../../../Forms/Sentry/InitializeSentry");
const AttachmentsReducer_1 = require("../../Externals/Attachments/Store/AttachmentsReducer");
const TooltipsResolveReducer_1 = require("../../TooltipsResolveReducer");
const ErrorLogReducer_1 = require("../ErrorManager/Store/ErrorLogReducer");
const CustomErrorLogTooltipReducer_1 = require("../../../Controls/FormParts/EmbeddedLog/ErrorList/ErrorLogTooltip/CustomErrorLogTooltipReducer");
const BrowserHelpers_1 = require("../../../Helpers/BrowserHelpers");
const ImportEmployeesReducer_1 = require("../../../StaffServiceIntegration/Employees/State/ImportEmployeesReducer");
const ViewModes_1 = require("../../../Controls/Internals/ViewModeSelector/ViewModes");
const ViewModesReducer_1 = require("../../../Controls/Internals/ViewModeSelector/Store/ViewModesReducer");
const RequisitesReducer_1 = require("../../Externals/Requisites/Store/RequisitesReducer");
const FeatureFlagsReducer_1 = require("../../Externals/FeatureFlags/Store/FeatureFlagsReducer");
const ModalStateReducer_1 = require("../ModalManager/ModalStateReducer");
const PanelTypeStroing_1 = require("../../ActionCreators/PanelTypeStroing");
const UnitIdImmutableSet_1 = require("../../../Controls/FormParts/Unit/UnitId/UnitIdImmutableSet");
const EmployerDocumentsReducer_1 = require("../../../StaffServiceIntegration/Documents/State/EmployerDocumentsReducer");
const TabInfosReducer_1 = require("../../../Controls/MultiControls/Tabs/Store/TabInfosReducer");
const EmployeeRelativesReducer_1 = require("../../../StaffServiceIntegration/Relatives/State/EmployeeRelativesReducer");
const FormVisualStateReducer_1 = require("../../FormVisualStateReducer");
const instanceNumberReducer_1 = require("../../../Controls/MultiControls/InstanceNumber/Store/instanceNumberReducer");
const aggregateValuesMiddleware_1 = require("./Middleware/aggregateValuesMiddleware");
const ReduxDevToolsExtensions_1 = require("./ReduxDevToolsExtensions");
const SentryBreadcrumbsMiddleware_1 = require("./SentryBreadcrumbsMiddleware");
const LocalStorageUtils_1 = require("./LocalStorageUtils");
const modelChangeHookMiddleware_1 = require("./Middleware/modelChangeHookMiddleware");
function createReducer(modelReducer) {
    return (state, action) => {
        let nextState = state;
        nextState = (0, multiLineReducer_1.multiLineReducer)(nextState, action);
        nextState = (0, crossFitTableReducer_1.crossFitTableReducer)(nextState, action);
        nextState = (0, unitReducer_1.unitReducer)(nextState, action);
        nextState = (0, ModalStateReducer_1.ModalStateReducer)(nextState, action);
        nextState = modelReducer(nextState, action);
        nextState = (0, ViewModesReducer_1.viewModesReducer)(nextState, action);
        nextState = {
            ...nextState,
            attachments: (0, AttachmentsReducer_1.attachmentsReducer)(state.attachments, state, action),
            requisites: (0, RequisitesReducer_1.requisitesReducer)(state.requisites, action),
            featureFlags: (0, FeatureFlagsReducer_1.featureFlagsReducer)(state.featureFlags, action),
            tabInfos: (0, TabInfosReducer_1.tabInfosReducer)(state.tabInfos, action),
            errorLog: (0, ErrorLogReducer_1.errorLogReducer)(state.errorLog, action),
            customErrorLogTooltip: (0, CustomErrorLogTooltipReducer_1.customErrorLogTooltipReducer)(state.customErrorLogTooltip, action),
            importEmployees: (0, ImportEmployeesReducer_1.importEmployeesReducer)(state.importEmployees, action),
            employerDocuments: (0, EmployerDocumentsReducer_1.employerDocumentsReducer)(state.employerDocuments, action),
            employeeRelatives: (0, EmployeeRelativesReducer_1.employeeRelativesReducer)(state.employeeRelatives, action),
            formVisualState: (0, FormVisualStateReducer_1.formVisualStateReducer)(state.formVisualState, action),
        };
        nextState = (0, ErrorLogReducer_1.errorLogModelReducer)(nextState, action);
        nextState = (0, TooltipsResolveReducer_1.tooltipReducer)(nextState, action);
        nextState = (0, instanceNumberReducer_1.instanceNumberReducer)(nextState, action);
        return nextState;
    };
}
function getInitialState(formSettings, featureFlags) {
    const panelState = (0, PanelTypeStroing_1.getPanelStateToLocalStorage)(formSettings) ??
        (BrowserHelpers_1.isBrowser && document.body.offsetWidth > constants_1.MAX_WIDTH ? "pin" : "hide");
    const captionsViewMode = (0, LocalStorageUtils_1.getFromLocalStorage)(ViewModes_1.captionsViewModeKey, ViewModes_1.CaptionsViewMode.Long);
    const hintsViewMode = (0, LocalStorageUtils_1.getFromLocalStorage)(ViewModes_1.hintsViewModeKey, ViewModes_1.HintsViewMode.Show);
    return {
        attachments: AttachmentsReducer_1.defaultAttachmentsState,
        units: {},
        viewModel: {},
        errorLog: ErrorLogReducer_1.defaultErrorLogState,
        settings: formSettings,
        featureFlags: featureFlags,
        modalUnits: [],
        emptyUnits: UnitIdImmutableSet_1.UnitIdSet.create(),
        viewModes: {
            captions: captionsViewMode,
            hints: hintsViewMode,
        },
        multiLineInfos: {},
        tableInfos: {},
        tabInfos: {
            currentInstance: {},
            currentTabInstanceIndex: {},
        },
        formVisualState: {
            savingStatus: "success",
            panelActiveTab: "menu",
            isWidePanel: false,
            panelState: panelState,
        },
    };
}
function createFormStore(form, modelReducer, state = {}) {
    // @ts-ignore
    const reducer = createReducer(modelReducer);
    // @ts-ignore
    return (0, redux_1.createStore)(reducer, { ...getInitialState(form.settings, form.featureFlags), ...state }, (0, ReduxDevToolsExtensions_1.composeEnchancers)((0, redux_1.applyMiddleware)((0, crossFitTableMiddleware_1.crossFitTableMiddleware)(), redux_thunk_1.default.withExtraArgument(form), (0, aggregateValuesMiddleware_1.aggregateValuesMiddleware)(form), (0, modelChangeHookMiddleware_1.modelChangeHookMiddleware)(form), (0, SentryBreadcrumbsMiddleware_1.sentryBreadcrumbsMiddleware)((0, InitializeSentry_1.getCandyHub)()))));
}
