"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Validator = void 0;
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const LegacyValidationFunctionAdapter_1 = require("../Calculator2/Validations/LegacyValidationFunctionAdapter");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const AggregatedValidationFunction_1 = require("../Calculator2/AggregatedValidationFunction");
const GetValueByPath_1 = require("../../../Helpers/StateAccessors/GetValueByPath");
class Validator {
    calculator2;
    customValidationFunctionCallContextProvider;
    localization;
    store;
    errorManager;
    picklistsProvider;
    validationsDisabled = false;
    constructor(customValidationFunctionCallContextProvider, localization, store, errorManager, picklistsProvider, calculator2, validationFunctions, enableLegacyValidators) {
        this.customValidationFunctionCallContextProvider = customValidationFunctionCallContextProvider;
        this.localization = localization;
        this.store = store;
        this.errorManager = errorManager;
        this.picklistsProvider = picklistsProvider;
        this.calculator2 = calculator2;
        this.loadValidationFunctions(validationFunctions ?? {}, enableLegacyValidators);
    }
    loadValidationFunctions(validatorResources, enableLegacyValidators = false) {
        for (const [validatorPathMask, validator] of (0, IterableUtils_1.iterateEntries)(validatorResources)) {
            if (validatorPathMask === "") {
                // NOTE В approval тестах на Engine есть по всей видимости забагованные валидации
                continue;
            }
            if (Array.isArray(validator)) {
                for (const validatorItem of validator) {
                    const conditionContext = validatorItem[1];
                    const conditionContextAndDeps = validatorItem[0];
                    const conditionDeps = conditionContextAndDeps.map(x => x[2]).flat();
                    const condition = (evaluationContext) => conditionContextAndDeps
                        .map(([generatedExpressionFn, contextCurrentElement]) => {
                        const getValueByPathGeneratedFn = (expressionPath, fieldName) => {
                            if (expressionPath.startsWith("@")) {
                                const state = this.store.getState();
                                return (0, GetValueByPath_1.getValueByPath)(state, () => "", expressionPath, fieldName);
                            }
                            const generatedAbsoluteModelPath = contextCurrentElement
                                .joinWith((0, ModelPath_1.createModelPath)(expressionPath, "auto"))
                                .normalize();
                            if (fieldName === "children") {
                                const instances = evaluationContext.getInstances(generatedAbsoluteModelPath);
                                return Array.isArray(instances) ? instances : [...instances];
                            }
                            return evaluationContext.get(generatedAbsoluteModelPath) ?? "";
                        };
                        const getCountByPathGeneratedFn = expressionPath => {
                            const generatedAbsoluteModelPath = contextCurrentElement
                                .joinWith((0, ModelPath_1.createModelPath)(expressionPath, "auto"))
                                .normalize();
                            return (0, IterableUtils_1.count)(evaluationContext.getInstances(generatedAbsoluteModelPath));
                        };
                        return generatedExpressionFn(getValueByPathGeneratedFn, getCountByPathGeneratedFn);
                    })
                        .reduce((x, y) => x && y, true);
                    this.calculator2.addCalculationFunction(new LegacyValidationFunctionAdapter_1.LegacyValidationFunctionAdapter(validatorPathMask, conditionContext, this.localization, this.store, this.errorManager, this.picklistsProvider, this.customValidationFunctionCallContextProvider, enableLegacyValidators, () => this.validationsDisabled, conditionDeps, condition));
                }
            }
            else {
                this.calculator2.addCalculationFunction(new LegacyValidationFunctionAdapter_1.LegacyValidationFunctionAdapter(validatorPathMask, validator, this.localization, this.store, this.errorManager, this.picklistsProvider, this.customValidationFunctionCallContextProvider, enableLegacyValidators, () => this.validationsDisabled, [], undefined));
            }
        }
    }
    getAggregatedValidationFunctionByPath(path) {
        const functions = this.calculator2
            .getFunctionsByTarget(path)
            .filter((x) => x instanceof AggregatedValidationFunction_1.AggregatedValidationFunction);
        return functions[0];
    }
    disableAllValidations() {
        this.validationsDisabled = true;
    }
    enableAllValidations() {
        this.validationsDisabled = false;
    }
}
exports.Validator = Validator;
