"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttachmentsValueValidator = void 0;
const IterableUtils_1 = require("../../../../../../Common/IterableUtils");
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
const IValueValidator_1 = require("./Bases/IValueValidator");
const ValueValidatorBase_1 = require("./Bases/ValueValidatorBase");
class AttachmentsValueValidator extends ValueValidatorBase_1.ValueValidatorBase {
    checkType;
    options;
    message;
    constructor(options, checkType, message) {
        super();
        this.checkType = checkType;
        this.options = options;
        this.message = message;
    }
    validate(context, nodeViewModel) {
        const state = context.store.getState();
        const { error, value } = nodeViewModel;
        if (value == undefined) {
            return IValueValidator_1.ValidationResults.Valid;
        }
        if (error && error.length) {
            return IValueValidator_1.ValidationResults.Invalid(this.getMessage(context, this.message), this.checkType);
        }
        const { extentions, maxSize } = this.options;
        const meta = state.attachments.byId[value]?.meta;
        if (meta == undefined) {
            return IValueValidator_1.ValidationResults.Valid;
        }
        const { fileName, fileSize } = meta;
        const extensionError = this.checkExtension(fileName, extentions);
        if (extensionError != undefined) {
            return IValueValidator_1.ValidationResults.Invalid(extensionError, this.checkType);
        }
        const sizeError = this.checkSize(fileSize, maxSize);
        if (sizeError != undefined) {
            return IValueValidator_1.ValidationResults.Invalid(sizeError, this.checkType);
        }
        return IValueValidator_1.ValidationResults.Valid;
    }
    checkExtension(fileName, acceptedExtensions) {
        const fileNameTokens = fileName.split(".");
        const extension = (0, TypingUtils_1.isLengthGreaterThenOrEqual)(fileNameTokens, 2) ? IterableUtils_1.IterUtils.last(fileNameTokens) : "";
        if (!acceptedExtensions.includes(extension.toLowerCase())) {
            return "Неправильный формат файла";
        }
        return undefined;
    }
    checkSize(fileSize, maxSize) {
        if (fileSize < 0) {
            return "Не удалось получить размер файла";
        }
        if (fileSize === 0) {
            return "Содержимое файла не может быть пустым";
        }
        if (fileSize > maxSize * 1024 * 1024) {
            return this.options.multiple
                ? "Файл слишком большой, попробуйте отправить частями"
                : "Файл слишком большой, попробуйте уменьшить размер и прикрепить заново";
        }
        return undefined;
    }
}
exports.AttachmentsValueValidator = AttachmentsValueValidator;
