"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CompositeValueValidatorWithForcedValidators = void 0;
const ValidationHelper_1 = require("../../../ValidationHelper");
const IndentString_1 = require("../../../../../../Common/IndentString");
const IValueValidator_1 = require("./Bases/IValueValidator");
class CompositeValueValidatorWithForcedValidators {
    optional;
    forcedValidators;
    allValidators;
    constructor(optional, validators, forcedValidators) {
        this.optional = optional;
        this.forcedValidators = forcedValidators;
        this.allValidators = [...validators, ...forcedValidators];
    }
    async validate(context, nodeViewModel) {
        const events = [];
        if (this.optional && ValidationHelper_1.ValidationHelper.isBlank(nodeViewModel.value)) {
            for (const validator of this.forcedValidators) {
                const result = await validator.validate(context, nodeViewModel);
                events.push(...(result.additionalEvents ?? []));
                if (result.valid && result.forceValid) {
                    return IValueValidator_1.ValidationResults.ValidWithEvents(events);
                }
                if (!result.valid) {
                    return { ...result, additionalEvents: events };
                }
            }
            return IValueValidator_1.ValidationResults.ValidWithEvents(events);
        }
        for (const validator of this.allValidators) {
            const result = await validator.validate(context, nodeViewModel);
            events.push(...(result.additionalEvents ?? []));
            if (result.valid && result.forceValid) {
                return IValueValidator_1.ValidationResults.ValidWithEvents(events);
            }
            if (!result.valid) {
                return { ...result, additionalEvents: events };
            }
        }
        return IValueValidator_1.ValidationResults.ValidWithEvents(events);
    }
    asDebugString() {
        return [
            `new CompositeValueValidatorWithForcedValidators {`,
            `  optional: ${this.optional},`,
            // eslint-disable-next-line @typescript-eslint/no-base-to-string
            `  forcedValidators: ${this.forcedValidators},`,
            `  allValidators: [`,
            `${(0, IndentString_1.indent)(this.allValidators.map(x => x.asDebugString()).join(",\n"), 4)}`,
            `  ]`,
            `}`,
        ].join("\n");
    }
}
exports.CompositeValueValidatorWithForcedValidators = CompositeValueValidatorWithForcedValidators;
