"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DigestConditionValueValidator = void 0;
const FilterBuilder_1 = require("../../../../Controls/ValueEditors/Picklist/Common/FilterBuilder");
const DigestQueryFilter_1 = require("../../../../../../Common/CandyApiClient/DigestApiClient/Types/DigestQueryFilter");
const GPathFilter_1 = require("../../Picklists/GPathFilter");
const Funcs_1 = require("../../../../../../Common/Funcs");
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
const ValueValidatorBase_1 = require("./Bases/ValueValidatorBase");
const IValueValidator_1 = require("./Bases/IValueValidator");
class DigestConditionValueValidator extends ValueValidatorBase_1.ValueValidatorBase {
    picklistSourceId;
    condition;
    checkType;
    message;
    constructor(picklistSourceId, condition, checkType, message) {
        super();
        this.picklistSourceId = picklistSourceId;
        this.checkType = checkType;
        this.condition = condition;
        this.message = message;
    }
    async validate(context, nodeViewModel) {
        if (nodeViewModel.value == undefined) {
            return IValueValidator_1.ValidationResults.Valid;
        }
        let digestCondition;
        const conditionBuilder = this.condition(new FilterBuilder_1.FilterBuilder(), () => (0, TypingUtils_1.reject)("Path expressions is not allowed in digestcondition"), FilterBuilder_1.resolveToNoneFilter, FilterBuilder_1.resolveToAllFilter);
        if (conditionBuilder == undefined || (0, FilterBuilder_1.isResolveToAllFilter)(conditionBuilder)) {
            digestCondition = Funcs_1.truePredicate;
        }
        else if ((0, FilterBuilder_1.isResolveToNoneFilter)(conditionBuilder)) {
            digestCondition = Funcs_1.falsePredicate;
        }
        else {
            digestCondition = (0, GPathFilter_1.buildFilterPredicate)(conditionBuilder.build());
        }
        const values = Array.isArray(nodeViewModel.value)
            ? nodeViewModel.value
            : [nodeViewModel.value];
        const data = await this.getValuesFromPicklist(values, context, values.length);
        const items = data?.items ?? [];
        if (items.length > 0) {
            if (!items.some(x => digestCondition(x))) {
                const description = this.getMessage(context, this.message) ?? "Значение справочника не соотвествует заданным условиям";
                return IValueValidator_1.ValidationResults.Invalid(context.localizer.translate(description), this.checkType);
            }
        }
        return IValueValidator_1.ValidationResults.Valid;
    }
    async getValuesFromPicklist(values, context, take) {
        const filter = {
            filter: {
                operator: DigestQueryFilter_1.DigestLogicalOperator.Or,
                operands: values.map((value) => ({
                    operator: DigestQueryFilter_1.DigestFilterOperator.Equal,
                    column: "code",
                    value: value ?? "",
                })),
            },
            take: take,
        };
        return context.picklistsProvider.getPicklist(this.picklistSourceId)?.query(filter);
    }
}
exports.DigestConditionValueValidator = DigestConditionValueValidator;
