"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultAttachmentsState = void 0;
exports.attachmentsReducer = attachmentsReducer;
const modelActions_1 = require("../../../Core/ModelStoring/Store/modelActions");
const PathConstants_1 = require("../../../../../../Common/PathConstants");
const ReduxStateHelpers_1 = require("../../../../Helpers/ReduxStateHelpers");
const AttachmentsActions_1 = require("./AttachmentsActions");
exports.defaultAttachmentsState = { allIds: [], byId: {} };
function attachmentsReducer(state = exports.defaultAttachmentsState, fullState, action) {
    switch (action.type) {
        case AttachmentsActions_1.AttachmentsActionType.LoadInitialAttachments: {
            return action.payload;
        }
        case AttachmentsActions_1.AttachmentsActionType.AddAttachments: {
            const nextState = {
                allIds: [...state.allIds, ...action.payload.map(({ id }) => id)],
                byId: { ...state.byId },
            };
            action.payload.forEach((attachment) => {
                nextState.byId[attachment.id] = attachment;
            });
            return nextState;
        }
        case AttachmentsActions_1.AttachmentsActionType.RemoveAttachment: {
            const { [action.payload.id]: removedAttachment, ...restAttachments } = state.byId;
            return {
                allIds: state.allIds.filter(id => removedAttachment == undefined || id !== removedAttachment.id),
                byId: restAttachments,
            };
        }
        case AttachmentsActions_1.AttachmentsActionType.UpdateAttachment: {
            const { id, attachment } = action.payload;
            const nextState = {
                allIds: state.allIds,
                byId: { ...state.byId, [id]: attachment },
            };
            if (!state.byId[id]) {
                nextState.allIds = [...state.allIds, id];
            }
            return nextState;
        }
        case AttachmentsActions_1.AttachmentsActionType.SetAttachmentProgress: {
            const { id, progress } = action.payload;
            const attachInfo = state.byId[id];
            if (attachInfo && Number(progress) > Number(attachInfo.meta.progress)) {
                return {
                    ...state,
                    byId: {
                        ...state.byId,
                        [id]: {
                            ...attachInfo,
                            meta: {
                                ...attachInfo.meta,
                                progress: progress,
                            },
                        },
                    },
                };
            }
            return state;
        }
        case modelActions_1.ModelActionType.SetValues: {
            const { deleted } = action.payload;
            if (deleted?.length > 0) {
                const newState = {
                    allIds: [...state.allIds],
                    byId: {
                        ...state.byId,
                    },
                };
                for (const deletedPath of deleted) {
                    if (deletedPath.endsWith(PathConstants_1.AttachmentInfoPath)) {
                        const deletedAttachmentId = (0, ReduxStateHelpers_1.getIn)(fullState, [
                            "viewModel",
                            ...deletedPath.getPathPartsAsArray(),
                            ".",
                            "value",
                        ]);
                        newState.allIds = newState.allIds.filter(it => it !== deletedAttachmentId);
                        delete newState.byId[deletedAttachmentId];
                    }
                }
                return newState;
            }
            return state;
        }
        default:
            return state;
    }
}
