"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConvertTask = void 0;
const metricsAction_1 = require("../../../../Controls/metricsAction");
const MetricsProvider_1 = require("../../../../Controls/Internals/Metrics/MetricsProvider");
const VerifyAndFixFnsRequirementsTask_1 = require("../../../Api/CandyApiClient/VerifyAndFixFnsRequirementsTask");
const ErrorManager_1 = require("../../../Core/ErrorManager/ErrorManager");
const AttachmentsActions_1 = require("../Store/AttachmentsActions");
const FileError_1 = require("../../../../Controls/ValueEditors/FileLoader/ViewFile/FileError");
class ConvertTask {
    task;
    attachmentId;
    bindingPath;
    modelAccessor;
    store;
    candyApiDraftClient;
    cancelled = false;
    taskPromise;
    constructor(store, modelAccessor, errorManager, candyApiDraftClient, attachmentId, bindingPath) {
        this.modelAccessor = modelAccessor;
        this.store = store;
        this.attachmentId = attachmentId;
        this.bindingPath = bindingPath;
        this.candyApiDraftClient = candyApiDraftClient;
        this.task = new VerifyAndFixFnsRequirementsTask_1.VerifyAndFixFnsRequirementsTask(candyApiDraftClient, attachmentId, attachmentId);
        this.taskPromise = this.executeTask();
    }
    cancel() {
        this.cancelled = true;
        // Пока так, если упадёт, то мы ничего не можем сделать.
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        this.task.cancel();
    }
    writeResult = async () => {
        const attachment = await this.candyApiDraftClient.getAttachmentById(this.attachmentId);
        const nextAttachment = {
            ...attachment,
            meta: {
                ...attachment.meta,
                progress: 100,
                isConverted: true,
            },
        };
        this.store.dispatch((0, AttachmentsActions_1.updateAttachment)(this.attachmentId, nextAttachment));
        await this.candyApiDraftClient.createOrUpdateAttachments([nextAttachment]);
        return undefined;
    };
    async executeTask() {
        try {
            this.task.progress(this.handleChangeTaskProgress);
            await this.task.startTask();
            if (this.cancelled) {
                return;
            }
            await this.writeResult();
        }
        catch (e) {
            // @ts-ignore
            this.handleTaskError(e);
        }
    }
    handleChangeTaskProgress = (current, total) => {
        const percent = total > 0 ? Math.floor((current / total) * 20) : 0;
        this.store.dispatch((0, AttachmentsActions_1.setAttachmentProgress)(this.attachmentId, percent + 80));
    };
    handleTaskError = (error) => {
        (0, MetricsProvider_1.track)("candy", metricsAction_1.FILE_LOADER_FILE_ERROR, {
            path: this.bindingPath,
            attachmentId: this.attachmentId,
        });
        // @ts-ignore Разобраться с ошибками
        if (error.code === "Cancelled") {
            return undefined;
        }
        // Когда `properties == null` - то проверка сломалась (с) Алексей Мухачев
        if (!error.properties) {
            const message = (0, FileError_1.createFileError)(ErrorManager_1.KREC_ERROR_CHECK);
            // @ts-ignore Ужасно, надо переделать, но пока лучше так
            this.modelAccessor.setWarning(this.bindingPath, message);
        }
        // Когда `properties != null && 'fileCheckResult' in properties` - то конвертация сломалась (с) Алексей Мухачев
        if (error.properties && "fileCheckResult" in error.properties) {
            const message = (0, FileError_1.createFileError)(ErrorManager_1.KREC_ERROR_CONVERT);
            // @ts-ignore Ужасно, надо переделать, но пока лучше так
            this.modelAccessor.setError(this.bindingPath, message);
        }
        return undefined;
    };
}
exports.ConvertTask = ConvertTask;
