"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UploadToDriveTask = void 0;
exports.suspendNextUpload = suspendNextUpload;
exports.resumeLastSuspendedUpload = resumeLastSuspendedUpload;
const file_1 = require("../../../../Helpers/file");
const Deferred_1 = require("../../../../Helpers/Deferred");
const AttachmentsActions_1 = require("../Store/AttachmentsActions");
let nextUploadDeferred;
function suspendNextUpload() {
    if (nextUploadDeferred != undefined) {
        throw new Error("There are suspended upload. Please resume one.");
    }
    nextUploadDeferred = new Deferred_1.Deferred();
}
function resumeLastSuspendedUpload() {
    if (nextUploadDeferred != undefined) {
        nextUploadDeferred.resolve();
        nextUploadDeferred = undefined;
    }
    else {
        throw new Error("There are no suspended uploads");
    }
}
class UploadToDriveTask {
    store;
    file;
    driveTask;
    attachmentId;
    isConvert;
    bindingPath;
    modelAccessor;
    taskPromise;
    cancelled = false;
    constructor(store, modelAccessor, attachmentId, bindingPath, file, isConvert, writeResultFunc) {
        this.modelAccessor = modelAccessor;
        this.store = store;
        this.bindingPath = bindingPath;
        this.file = file;
        this.attachmentId = attachmentId;
        this.isConvert = isConvert;
        this.taskPromise = this.startTask().then(contentUri => writeResultFunc(contentUri));
    }
    cancel() {
        if (this.driveTask) {
            this.driveTask.cancel();
        }
        this.cancelled = true;
    }
    startTask() {
        // Так-то тут надо просто uploadToDrive сделать так, чтобы он там был промисом
        // eslint-disable-next-line no-async-promise-executor
        return new Promise(async (resolve, reject) => {
            if (nextUploadDeferred != undefined) {
                await nextUploadDeferred.promise;
                if (this.cancelled) {
                    return;
                }
            }
            // Иначе Promise без cancel
            const { initDrive, getDriveTask } = (0, file_1.uploadToDrive)(this.file, this.handleChangeUploadToDriveProgress);
            await initDrive;
            this.driveTask = getDriveTask();
            this.driveTask.then(resolve, reject);
        }).catch((error) => {
            this.modelAccessor.addError(this.bindingPath, "Не удалось загрузить документ, попробуйте ещё раз через несколько минут");
            return undefined;
        });
    }
    handleChangeUploadToDriveProgress = (current, total) => {
        const maxLoad = this.isConvert ? 80 : 100;
        const percent = total > 0 ? Math.floor((current / total) * maxLoad) : 0;
        this.store.dispatch((0, AttachmentsActions_1.setAttachmentProgress)(this.attachmentId, percent));
    };
}
exports.UploadToDriveTask = UploadToDriveTask;
