import { AbsoluteResolvedModelPath } from "../../../../Common/ModelPath/ModelPath";
import { BindingPath, ModelValue, ViewModel, ViewModelFieldName } from "../Core/CoreTypes";
import { ModelChunk, ReadonlyModelChunk } from "../Core/ModelStoring/ModelChunk";
import { IReadonlyModelHunk } from "../Core/ModelStoring/IReadonlyModelHunk";
export interface IFormReadonlyModelAccessor {
    getViewModel(bindingPath: BindingPath | AbsoluteResolvedModelPath): ViewModel | undefined;
    getViewModelField(bindingPath: BindingPath | AbsoluteResolvedModelPath, fieldName: string): ModelValue | undefined;
    getValue(bindingPath: BindingPath | AbsoluteResolvedModelPath): string | undefined;
    getChildren(bindingPath: BindingPath | AbsoluteResolvedModelPath): string[] | undefined;
    getBatch(path: BindingPath | AbsoluteResolvedModelPath): ModelChunk;
}
export interface IUpdateFormModelAccessor extends IFormReadonlyModelAccessor {
    setChildren(bindingPath: BindingPath | AbsoluteResolvedModelPath, children: string[]): void;
    setValue(bindingPath: BindingPath | AbsoluteResolvedModelPath, value: string | undefined): void;
    remove(bindingPath: AbsoluteResolvedModelPath): void;
    setViewModelField(bindingPath: BindingPath | AbsoluteResolvedModelPath, fieldName: ViewModelFieldName, value: unknown): void;
    setBatch(batch: ReadonlyModelChunk | IReadonlyModelHunk): void;
    setBatchByPrefix(prefix: BindingPath | AbsoluteResolvedModelPath, batch: ReadonlyModelChunk): void;
    addInstance(path: BindingPath): string;
    removeInstance(path: BindingPath, instance: string): void;
    removeInstances(path: BindingPath, instances: readonly string[]): void;
    copySubtree(sourceBindingPath: BindingPath | AbsoluteResolvedModelPath, targetBindingPath: BindingPath | AbsoluteResolvedModelPath, doNotCopyPaths?: string[]): void;
}
export interface IModelValidationsAccessor {
    setError(path: BindingPath | AbsoluteResolvedModelPath, errorMessage: string): void;
    setWarning(bindingPath: BindingPath, param2: string): void;
    addError(nodePath: BindingPath | AbsoluteResolvedModelPath, message: string): void;
    addWarning(nodePath: string, message: string): void;
    clearWarnings(nodePath: string): void;
    clearErrors(nodePath: string): void;
    removeWarning(nodePath: string, message: string): void;
}
export interface IFormModelAccessor extends IUpdateFormModelAccessor, IModelValidationsAccessor {
    updateModel(updateFn: (model: IUpdateFormModelAccessor) => void): void;
}
