"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tooltipReducer = tooltipReducer;
const ViewModelSelect_1 = require("../Helpers/StateAccessors/ViewModelSelect");
const ModelHunk_1 = require("./Core/ModelStoring/ModelHunk");
const modelActions_1 = require("./Core/ModelStoring/Store/modelActions");
const TooltipsResolveActions_1 = require("./TooltipsResolveActions");
function tooltipReducer(state, action) {
    if (action.type === TooltipsResolveActions_1.TooltipsResolveActionType.DisableOwnTooltip) {
        return {
            ...state,
            errorLogTooltip: {
                errorId: action.payload.errorId,
                bindingPaths: action.payload.bindingPaths,
            },
        };
    }
    else if (action.type === TooltipsResolveActions_1.TooltipsResolveActionType.EnableOwnTooltip) {
        return {
            ...state,
            errorLogTooltip: undefined,
        };
    }
    else if (action.type === modelActions_1.ModelActionType.SetValues) {
        if (state.errorLogTooltip != undefined) {
            for (const [path, viewModelUpdate] of ModelHunk_1.ModelHunk.iterateEntries(action.payload.values)) {
                if (state.errorLogTooltip.bindingPaths.includes(path)) {
                    if ("error" in viewModelUpdate || "warning" in viewModelUpdate) {
                        const viewModel = ViewModelSelect_1.currentViewModelSelect.controlModel(state, path);
                        if (viewModel.error == undefined && viewModel.warning == undefined) {
                            return {
                                ...state,
                                errorLogTooltip: undefined,
                            };
                        }
                    }
                }
            }
        }
    }
    else if (action.type === TooltipsResolveActions_1.TooltipsResolveActionType.DisableOwnTooltipAttachment) {
        return {
            ...state,
            errorLogTooltipAttachment: {
                errorId: action.payload.errorId,
                bindingPaths: action.payload.bindingPaths,
                attachmentId: action.payload.attachmentId,
                highlanderPath: action.payload.highlanderPath,
            },
        };
    }
    else if (action.type === TooltipsResolveActions_1.TooltipsResolveActionType.EnableOwnTooltipAttachment) {
        return {
            ...state,
            errorLogTooltipAttachment: undefined,
        };
    }
    return state;
}
