"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidationHelper = void 0;
const TypingUtils_1 = require("../../../Common/TypingUtils");
class ValidationHelper {
    static isBlank(str) {
        return !str || /^\s*$/.test(str);
    }
    static isNumber(str) {
        return !this.isBlank(str) && str != undefined && /^-?\d+(([.]\d+$)|($))/.test(str);
    }
    static isDateFormat(date) {
        if (/^(0[1-9]|[12][0-9]|3[01])\.(0[1-9]|1[012])\.(19|20)[0-9]{2}$/.test(date)) {
            const v = date.split(".");
            const d = parseInt(v[0] ?? (0, TypingUtils_1.reject)(), 10);
            const m = parseInt(v[1] ?? (0, TypingUtils_1.reject)(), 10);
            const y = parseInt(v[2] ?? (0, TypingUtils_1.reject)(), 10);
            const o = new Date(Date.UTC(y, m - 1, d));
            return o.getUTCDate() === d && o.getUTCMonth() + 1 === m && o.getUTCFullYear() === y;
        }
        return false;
    }
    static isDateFormatIso(date) {
        if (/^(19|20)[0-9]{2}\-(0[1-9]|1[012])\-(0[1-9]|[12][0-9]|3[01])$/.test(date)) {
            const v = date.split("-");
            const d = parseInt(v[2] ?? (0, TypingUtils_1.reject)(), 10);
            const m = parseInt(v[1] ?? (0, TypingUtils_1.reject)(), 10);
            const y = parseInt(v[0] ?? (0, TypingUtils_1.reject)(), 10);
            const o = new Date(Date.UTC(y, m - 1, d));
            return o.getUTCDate() === d && o.getUTCMonth() + 1 === m && o.getUTCFullYear() === y;
        }
        return false;
    }
    static isInn(str) {
        if ((0, TypingUtils_1.isNullOrWhiteSpace)(str)) {
            return false;
        }
        const innBits = str.split("").map(x => parseInt(x, 10));
        if (innBits.length === 10) {
            const sum9Inn = (innBits[0] ?? (0, TypingUtils_1.reject)()) * 2 +
                (innBits[1] ?? (0, TypingUtils_1.reject)()) * 4 +
                (innBits[2] ?? (0, TypingUtils_1.reject)()) * 10 +
                (innBits[3] ?? (0, TypingUtils_1.reject)()) * 3 +
                (innBits[4] ?? (0, TypingUtils_1.reject)()) * 5 +
                (innBits[5] ?? (0, TypingUtils_1.reject)()) * 9 +
                (innBits[6] ?? (0, TypingUtils_1.reject)()) * 4 +
                (innBits[7] ?? (0, TypingUtils_1.reject)()) * 6 +
                (innBits[8] ?? (0, TypingUtils_1.reject)()) * 8;
            const k10 = (sum9Inn % 11) % 10;
            return k10 === innBits[9];
        }
        if (innBits.length === 12) {
            const sum10Inn = (innBits[0] ?? (0, TypingUtils_1.reject)()) * 7 +
                (innBits[1] ?? (0, TypingUtils_1.reject)()) * 2 +
                (innBits[2] ?? (0, TypingUtils_1.reject)()) * 4 +
                (innBits[3] ?? (0, TypingUtils_1.reject)()) * 10 +
                (innBits[4] ?? (0, TypingUtils_1.reject)()) * 3 +
                (innBits[5] ?? (0, TypingUtils_1.reject)()) * 5 +
                (innBits[6] ?? (0, TypingUtils_1.reject)()) * 9 +
                (innBits[7] ?? (0, TypingUtils_1.reject)()) * 4 +
                (innBits[8] ?? (0, TypingUtils_1.reject)()) * 6 +
                (innBits[9] ?? (0, TypingUtils_1.reject)()) * 8;
            const k11 = (sum10Inn % 11) % 10;
            const sum11Inn = (innBits[0] ?? (0, TypingUtils_1.reject)()) * 3 +
                (innBits[1] ?? (0, TypingUtils_1.reject)()) * 7 +
                (innBits[2] ?? (0, TypingUtils_1.reject)()) * 2 +
                (innBits[3] ?? (0, TypingUtils_1.reject)()) * 4 +
                (innBits[4] ?? (0, TypingUtils_1.reject)()) * 10 +
                (innBits[5] ?? (0, TypingUtils_1.reject)()) * 3 +
                (innBits[6] ?? (0, TypingUtils_1.reject)()) * 5 +
                (innBits[7] ?? (0, TypingUtils_1.reject)()) * 9 +
                (innBits[8] ?? (0, TypingUtils_1.reject)()) * 4 +
                (innBits[9] ?? (0, TypingUtils_1.reject)()) * 6 +
                (innBits[10] ?? (0, TypingUtils_1.reject)()) * 8;
            const k12 = (sum11Inn % 11) % 10;
            return k11 === innBits[10] && k12 === innBits[11];
        }
        return false;
    }
    static isOgrn(value) {
        if ((0, TypingUtils_1.isNullOrWhiteSpace)(value)) {
            return false;
        }
        const hasAppropriateLength = [PHYS_OGRN_LENGTH, IP_OGRN_LENGTH].includes(value.length);
        if (!hasAppropriateLength) {
            return false;
        }
        const matchingPattern = ogrnPattern.test(value);
        if (!matchingPattern) {
            return false;
        }
        const lastCharIndex = value.length - 1;
        const divisible = Number(value.slice(0, lastCharIndex));
        const divider = lastCharIndex - 1;
        const checkingDigit = (divisible % divider) % 10;
        const lastDigit = Number(value[lastCharIndex]);
        const isOgrn = checkingDigit == lastDigit;
        return isOgrn;
    }
    static isValidAccountNumber(bik, accountNumber) {
        if ((0, TypingUtils_1.isNullOrWhiteSpace)(bik) || (0, TypingUtils_1.isNullOrWhiteSpace)(accountNumber)) {
            return false;
        }
        if (bik.length != 9 || accountNumber.length != 20) {
            return false;
        }
        const allowedBicBase = ["000", "001", "002"];
        const coefs = {
            0: 7,
            1: 1,
            2: 3,
        };
        const conditionNumber = allowedBicBase.includes(bik.slice(-3)) ? "0" + bik.substring(4, 2) : bik.slice(-3);
        const extendenedAccountNumber = conditionNumber + accountNumber;
        const totalCheckSum = extendenedAccountNumber.split("").reduce((sum, number, index) => {
            const coefOrder = index % 3;
            const x = coefs[coefOrder] * parseInt(number, 10);
            sum += x % 10;
            return sum;
        }, 0);
        return totalCheckSum % 10 == 0;
    }
    static isValidMirCardNumber(cardNumber) {
        if ((0, TypingUtils_1.isNullOrWhiteSpace)(cardNumber)) {
            return false;
        }
        else if (cardNumber.length < 16) {
            return false;
        }
        return this.LuhnAlgorithmCheckSumVerifier(cardNumber);
    }
    static isSnils(snils) {
        if (!snils.length) {
            return false;
        }
        else if (/[^0-9]/.test(snils)) {
            return false; // СНИЛС может состоять только из цифр
        }
        else if (snils.length !== 11) {
            return false; // СНИЛС может состоять только из 11 цифр
        }
        else {
            let sum = 0;
            for (let i = 0; i < 9; i++) {
                const currentDigit = snils[i];
                if ((0, TypingUtils_1.isNotNullOrEmpty)(currentDigit)) {
                    sum += parseInt(currentDigit, 10) * (9 - i);
                }
            }
            let checkDigit = 0;
            if (sum < 100) {
                checkDigit = sum;
            }
            else if (sum > 101) {
                checkDigit = parseInt((sum % 101).toString(), 10);
                if (checkDigit === 100) {
                    checkDigit = 0;
                }
            }
            if (checkDigit === parseInt(snils.slice(-2), 10)) {
                return true;
            }
        }
        return false;
    }
    static isRegNumSfr(regNum) {
        if (!regNum.length) {
            return false;
        }
        else if (/[^0-9]/.test(regNum)) {
            return false; // может состоять только из цифр
        }
        else if (regNum.length !== 10) {
            return false; // может состоять только из 10 цифр
        }
        else if (regNum.startsWith("0")) {
            return false; // не может начинаться с лидирующего нуля
        }
        else {
            const coefs = [2, 1, 2, 1, 2, 1, 2, 1, 2, 1];
            let sum = 0;
            for (let i = 0; i < 10; i++) {
                const currentDigit = regNum[i];
                if ((0, TypingUtils_1.isNotNullOrEmpty)(currentDigit)) {
                    const typedCurrentDigit = parseInt(currentDigit, 10);
                    // @ts-ignore
                    const coefMul = typedCurrentDigit * coefs[i];
                    sum += coefMul > 9 ? coefMul - 9 : coefMul;
                }
            }
            if (sum % 10 === 0) {
                return true;
            }
        }
        return false;
    }
    static isKpp(str) {
        if (str.length !== 9) {
            return false;
        }
        else if (!/([0-9]{1}[1-9]{1}|[1-9]{1}[0-9]{1})([0-9]{2})([0-9A-Z]{2})([0-9]{3})/.test(str)) {
            return false;
        }
        return true;
    }
    static isOkved(str) {
        if (str.length < 2 || str.length > 8) {
            return false;
        }
        else if (!/[0-9]{2}/.test(str) &&
            !/[0-9]{2}\.[0-9]{1}/.test(str) &&
            !/[0-9]{2}\.[0-9]{2}/.test(str) &&
            !/[0-9]{2}\.[0-9]{2}\.[0-9]{1}/.test(str) &&
            !/[0-9]{2}\.[0-9]{2}\.[0-9]{2}/.test(str)) {
            return false;
        }
        return true;
    }
    static LuhnAlgorithmCheckSumVerifier(value) {
        const numbers = value.split("").map(num => parseInt(num, 10));
        const totalCheckSum = numbers.reduce((sum, number, index) => {
            let cardNumber = number;
            if ((numbers.length - index) % 2 == 0) {
                cardNumber *= 2;
                if (cardNumber > 9) {
                    cardNumber -= 9;
                }
            }
            sum += cardNumber;
            return sum;
        }, 0);
        return totalCheckSum % 10 === 0;
    }
}
exports.ValidationHelper = ValidationHelper;
const PHYS_OGRN_LENGTH = 13;
const IP_OGRN_LENGTH = 15;
const ogrnPattern = /^\d+$/;
