"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.delay = delay;
exports.declint = declint;
exports.detectIE = detectIE;
exports.parseQueryParams = parseQueryParams;
exports.smoothScrollBy = smoothScrollBy;
exports.smoothScroll = smoothScroll;
exports.scrollElementInCenter = scrollElementInCenter;
const IntRangeUtils_1 = require("../../../Common/IntRangeUtils");
const BrowserHelpers_1 = require("../Helpers/BrowserHelpers");
const FeatureFlags_1 = require("../Forms/Types/FeatureFlags");
const TypingUtils_1 = require("../../../Common/TypingUtils");
function delay(timeout) {
    return new Promise(resolve => {
        setTimeout(resolve, timeout);
    });
}
function declint(value, titles) {
    const numberFrom5to19 = value % 100 > 4 && value % 100 < 20;
    if (numberFrom5to19) {
        return titles[2].replace("%s", value.toString());
    }
    const cases = [2, 0, 1, 1, 1, 2];
    const pluralIndex = cases[(0, IntRangeUtils_1.clampUInt)(value % 10, 0, 5)];
    return titles[pluralIndex].replace("%s", value.toString());
}
let ieVersion;
/**
 * returns version of IE or false, if browser is not Internet Explorer
 */
function detectIE() {
    if (!BrowserHelpers_1.isBrowser) {
        return false;
    }
    if (ieVersion != undefined) {
        return ieVersion;
    }
    if (!navigator) {
        ieVersion = false;
        return ieVersion;
    }
    const userAgent = navigator.userAgent;
    const msie = userAgent.indexOf("MSIE ");
    if (msie > 0) {
        // IE 10 or older => return version number
        ieVersion = parseInt(userAgent.substring(msie + 5, userAgent.indexOf(".", msie)), 10);
        return ieVersion;
    }
    const trident = userAgent.indexOf("Trident/");
    if (trident > 0) {
        // IE 11 => return version number
        const rv = userAgent.indexOf("rv:");
        ieVersion = parseInt(userAgent.substring(rv + 3, userAgent.indexOf(".", rv)), 10);
        return ieVersion;
    }
    const edge = userAgent.indexOf("Edge/");
    if (edge > 0) {
        // Edge (IE 12+) => return version number
        ieVersion = parseInt(userAgent.substring(edge + 5, userAgent.indexOf(".", edge)), 10);
        return ieVersion;
    }
    // other browser
    ieVersion = false;
    return ieVersion;
}
function parseQueryParams() {
    const params = new URLSearchParams(window.location.search);
    const queryParams = {};
    const featureFlags = {};
    for (const [key, value] of params.entries()) {
        if (key === "gfv") {
            continue;
        }
        const decodedKey = decodeURIComponent(key);
        const decodedValue = decodeURIComponent(value);
        if (decodedKey.startsWith(FeatureFlags_1.featureFlagQueryParamPrefix)) {
            featureFlags[decodedKey.replace(FeatureFlags_1.featureFlagQueryParamPrefix, "")] = decodedValue;
        }
        else {
            if (decodedKey.includes(".")) {
                applyToSecondDepth(queryParams, decodedKey, decodedValue);
            }
            else {
                queryParams[decodedKey] = decodedValue;
            }
        }
    }
    return { queryParams: queryParams, featureFlags: featureFlags };
}
/** Разбиравем sendToSignButtonOptions.visible=true => {sendToSignButtonOptions : {visible: true}}
 * Используется только для тестов, делать глубюже 2 - не вижу смысла, тк сильно усложняется алгоритм, да и не нужно
 * */
function applyToSecondDepth(queryParams, key, value) {
    const splitByDot = key.split(".");
    (0, TypingUtils_1.ensureArrayHasLength)(splitByDot, 2);
    const [first, second] = splitByDot;
    const result = { [second]: value };
    // @ts-ignore
    const existDepth = queryParams[first];
    if (existDepth) {
        Object.assign(existDepth, result);
    }
    else {
        // @ts-ignore
        queryParams[first] = result;
    }
}
function smoothScrollBy(top = 0, left = 0) {
    if (detectIE() === 8) {
        window.scrollBy(left, top);
    }
    else {
        window.scrollBy({
            top: top,
            left: left,
            behavior: "smooth",
        });
    }
}
function smoothScroll(top = 0, left = 0) {
    window.scroll({
        top: top,
        left: left,
        behavior: "smooth",
    });
}
function scrollElementInCenter(element) {
    const { top, bottom } = element.getBoundingClientRect();
    if (bottom > document.documentElement.clientHeight) {
        smoothScrollBy(bottom - document.documentElement.clientHeight / 2);
    }
    if (top < 0) {
        smoothScrollBy(top - document.documentElement.clientHeight / 2);
    }
}
