import { fetch } from "whatwg-fetch";
import { AllIconNames } from "react-ui";
import { ErrorFirstCallback } from "../../../../Common/TypingUtils";
import type { IForm } from "../../Engine/Form/IForm";
import type { IPublicForm } from "../PublicForm/IPublicForm";
import type { IKeWebTemplatesApiClient } from "../Types/IKeWebTemplatesApiClient";
import { AUOOptions } from "../Types/AUOOptions";
import type { LayoutEventsStatic } from "../../Controls/Internals/LayoutEvents/LayoutEvents";
export interface ExtraPageActionAttachment {
    tid?: string;
    description: string;
    handler: (instancePath?: string) => void;
    disabled: boolean;
}
type FormContainer = string | HTMLElement;
type EngineEntryPoint = () => void;
export interface ActionDescription {
    icon: AllIconNames;
    text?: string | (() => string);
    onClick: () => void;
    hint?: undefined | string;
    id: string;
    tid?: string;
    weight: number;
}
interface RenderFormSettings {
    gfv: string;
    engineVersion?: undefined | null | string;
    theme?: undefined | null | string;
    resourceZone?: string;
    renderForm: undefined | EngineEntryPoint;
    templatesApi?: IKeWebTemplatesApiClient;
    panelActions?: ActionDescription[];
}
type StartFormFunc = (pageContent: FormContainer, extendedSettings: RenderFormSettings) => Promise<IForm>;
interface CandyGlobalInjectedVar {
    start: (pageContent: FormContainer, options: StartFormOptions, callback?: ErrorFirstCallback<IPublicForm>) => void;
    keformsUrl?: string;
    staffServiceUrl: string;
    picklistUrl: string;
    nidusUrl: string;
    fsPrinterUrl: string;
    partialRenderingForIE8: boolean;
    entryPoint: {
        [key: string]: StartFormFunc;
    };
    loadError?: unknown;
}
declare global {
    interface Window {
        CF: CandyGlobalInjectedVar;
    }
    namespace NodeJS {
        interface Global {
            CF: CandyGlobalInjectedVar;
        }
    }
}
export declare function isEngineVersionGreaterThanOrEqual(left: string, right: string): boolean;
export interface StartFormOptions extends Partial<AUOOptions> {
    ns: string;
    draftId: string;
    gfv?: string;
    resourceZone?: string;
    templatesApi?: IKeWebTemplatesApiClient;
    feedbackEnabled?: boolean;
    isDisabledCloseButton?: boolean;
    isDisabledCloseAndSaveButton?: boolean;
    isDisabledCleanForm?: boolean;
    theme?: undefined | null | string;
    authentication?: "browser-credentials-include" | {
        authSid: string;
    };
    fetch?: typeof fetch;
    model?: {
        [path: string]: unknown;
    };
    panelActions?: ActionDescription[];
    eventEmitter?: LayoutEventsStatic;
    employerId?: string;
    showNavigationSidebar?: boolean;
    onRemoveAttachment?: () => void;
    onClearAttachment?: () => void;
    attachmentId?: string;
    attachmentPageActions?: ExtraPageActionAttachment[];
}
declare const CandyClientExports: CandyGlobalInjectedVar;
export { CandyClientExports };
