"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.modelChangeEventPath = modelChangeEventPath;
exports.createLegacyPublicForm = createLegacyPublicForm;
const Errors_1 = require("../../../../../Common/Errors");
function modelChangeEventPath(path, fieldName) {
    if (fieldName != undefined) {
        if (path == undefined) {
            throw new Error("Path must be defined");
        }
        return ["model", "change", path, fieldName];
    }
    if (path != undefined) {
        return ["model", "change", path];
    }
    return ["model", "change"];
}
function mapChunkValues(chunk) {
    return Object.keys(chunk)
        .filter(path => chunk[path] && Object.prototype.hasOwnProperty.call(chunk[path], "value"))
        .map(path => {
        const result = chunk[path];
        if (result == undefined) {
            return {};
        }
        return { [path]: result.value };
    });
}
function reduceChunkValues(chunks) {
    return chunks.reduce((result, chunk) => ({
        ...result,
        ...chunk,
    }), {});
}
class LegacyPublicForm {
    form;
    constructor(form) {
        this.form = form;
    }
    navigateByHighlanderPaths(paths) {
        throw new Error("Method not implemented.");
    }
    cancelSelectRelative() {
        throw new Error("Method not implemented.");
    }
    getNavigationPathMapper() {
        throw new Error("Method not implemented.");
    }
    batchUpdate(updateProc) {
        throw new Error("Method not implemented.");
    }
    runCheck() {
        throw new Error("Method not implemented.");
    }
    setErrorLog(errorLog) {
        throw new Error("Method not implemented.");
    }
    async validateChildrenByPath(path) {
        throw new Error("Method not implemented.");
    }
    __INTERNAL__importFromEmployeeService() {
        throw new Error("Method not implemented.");
    }
    async __INTERNAL__getSelectFunctionForDisableEmployeeInSS(pathSpec) {
        throw new Error("Method not implemented.");
    }
    getModalUnitOpenedInstance(pageId) {
        throw new Error("Method not implemented.");
    }
    getTabsState() {
        throw new Error("Method not implemented.");
    }
    navigate(navigationPath) {
        throw new Error("Method not implemented.");
    }
    importEmployerDocuments(options) {
        throw new Error("Method not implemented.");
    }
    importEmployees(employees, options) {
        throw new Error("Method not implemented.");
    }
    print(printType, body) {
        throw new Error("Method not implemented.");
    }
    runMassAction(actionId, parameters) {
        throw new Error("Method not implemented.");
    }
    getByGId(gId, filter) {
        return this.form.picklistsProvider.getDigestApiPicklist(gId).query(filter);
    }
    on(eventName, listener) {
        this.form.eventManager.on(["form", eventName], listener);
    }
    off(eventName, listener) {
        this.form.eventManager.off(["form", eventName], listener);
    }
    onModelChange(path, listener) {
        throw Error("form.onModelChange() is not supported yet");
    }
    save() {
        return this.form.changeSet.send();
    }
    close() {
        this.form.changeSet.stopAutosend();
        if (this.form.unmount) {
            this.form.unmount();
        }
        return this.form.changeSet.send();
    }
    getTourSteps() {
        // For old keweb
        return [];
    }
    validate() {
        const keys = Object.keys(this.form.model.getChunk(this.form.settings.formRoot));
        // @ts-ignore Это в старых движках тут такой вызов
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return this.form.validator.validateAll(keys);
    }
    getAdditionalEmployeeInfo(filters) {
        throw new Error("not implemented");
    }
    getEmployees() {
        throw new Error("not implemented");
    }
    mergeRelative() {
        throw new Error("not implemented");
    }
    patchRelative() {
        throw new Error("not implemented");
    }
    sendMetrics(action, payload) {
        return undefined;
    }
    matchEmployee(query) {
        throw new Error("not implemented");
    }
    async reloadInner() {
        throw new Error("not implemented");
    }
    getWorkingEventsForEmployee(filters) {
        throw new Error("not implemented");
    }
    hasErrors() {
        const errorCount = this.form.model.get(this.form.settings.formRoot, "errorsCount");
        return errorCount != undefined && errorCount > 0;
    }
    onErrorsCountChanged(listener) {
        this.form.eventManager.on(modelChangeEventPath(this.form.settings.formRoot, "errorsCount"), listener);
    }
    offErrorsCountChanged(listener) {
        this.form.eventManager.off(modelChangeEventPath(this.form.settings.formRoot, "errorsCount"), listener);
    }
    set(pathOrModelChunk, value) {
        if (pathOrModelChunk !== null &&
            typeof pathOrModelChunk !== "string" &&
            typeof pathOrModelChunk === "object" &&
            !Array.isArray(pathOrModelChunk)) {
            const innerChunk = Object.keys(pathOrModelChunk).reduce((result, path) => ({ [path]: { value: pathOrModelChunk[path] }, ...result }), {});
            this.form.model.fill(innerChunk);
            return;
        }
        if (typeof pathOrModelChunk === "string") {
            this.form.model.set(pathOrModelChunk, "value", value);
            return;
        }
        throw new Error("form.set() parameter should be string path or object with path-value pairs");
    }
    get(pathOrArray) {
        if (Array.isArray(pathOrArray)) {
            return reduceChunkValues(pathOrArray
                .map(path => mapChunkValues(this.form.model.getChunk(path)))
                .reduce((result, array) => result.concat(array), []));
        }
        if (typeof pathOrArray === "string") {
            return reduceChunkValues(mapChunkValues(this.form.model.getChunk(pathOrArray)));
        }
        if (pathOrArray === undefined) {
            return reduceChunkValues(mapChunkValues(this.form.model.getChunk(this.form.settings.formRoot)));
        }
        throw new Error("form.get() parameter should be string path or array of paths");
    }
    getValue(path) {
        return this.form.model.get(path, "value");
    }
    copySubtree(sourceBindingPath, targetBindingPath, doNotCopyPaths) {
        throw new Errors_1.NotImplementedError();
    }
    addInstance(path) {
        throw new Errors_1.NotImplementedError();
    }
    getChildren(path) {
        throw new Errors_1.NotImplementedError();
    }
    uuidGenerate() {
        throw new Errors_1.NotImplementedError();
    }
    showModal(options) {
        throw new Errors_1.NotImplementedError();
    }
    addSection(options) {
        throw new Errors_1.NotImplementedError();
    }
    callInitSection(options) {
        throw new Errors_1.NotImplementedError();
    }
    preferredInterdocumentTransfer(options) {
        throw new Errors_1.NotImplementedError();
    }
    createFiasApiClient(options) {
        throw new Errors_1.NotImplementedError();
    }
    openEmployeeEditor(employeeId) {
        throw new Errors_1.NotImplementedError();
    }
    removeInstance(path, instance) {
        throw new Errors_1.NotImplementedError();
    }
    removeInstances(path, instances) {
        throw new Errors_1.NotImplementedError();
    }
    focusOnControlIfExist(path) {
        throw new Errors_1.NotImplementedError();
    }
    setAutoFieldsToAutoValue(pathPrefixes) {
        return Promise.resolve(undefined);
    }
    getInnerForm() {
        throw new Errors_1.NotImplementedError();
    }
    getAttachmentPublicFormByPath(path) {
        throw new Errors_1.NotImplementedError();
    }
    getGfv() {
        throw new Errors_1.NotImplementedError();
    }
    ensureInstanceFullyLoaded(bindingPath, ...instances) {
        throw new Error("Method not implemented.");
    }
    customNormalize(normalizerId, pathPrefixes) {
        throw new Errors_1.NotImplementedError();
    }
    removeInstancesByRules(rules) {
        throw new Errors_1.NotImplementedError();
    }
    createDnp() {
        throw new Errors_1.NotImplementedError();
    }
    getSelectedInstances(multiplicityPath, tableUniqId, pathToChecked) {
        return Promise.resolve([]);
    }
    async clearFormWithReload() {
        throw new Errors_1.NotImplementedError();
    }
}
function createLegacyPublicForm(form) {
    // @ts-ignore
    return new LegacyPublicForm(form);
}
