import type { Store } from "redux";
import type { ICandyApiClient } from "../../../../../Common/CandyApiClient/ICandyApiClient";
import type { ITemplatesApiClient } from "../../../Engine/Api/TemplateApiClient/ITemplatesApiClient";
import type { CalculationFunctions, CalculationHooks } from "../../../Engine/Core/AutoCalculations/CalculationResources";
import type { ICalculator } from "../../../Engine/Core/AutoCalculations/Calculator";
import type { IChangeSetManager } from "../../../Engine/Core/ChangeSet/IChangeSetManager";
import type { DataDeclaration } from "../../../Engine/Core/DataDeclaration/DataDeclaration";
import type { DataDeclarationHelper } from "../../../Engine/Core/DataDeclaration/DataDeclarationHelper";
import type { IErrorManager } from "../../../Engine/Core/ErrorManager/ErrorManager";
import type { Localization, LocalizationDictionary } from "../../../Engine/Core/Localization/Localization";
import type { Sections } from "../../../Engine/Core/ModelStoring/Sections";
import type { ModelNormalizer } from "../../../Engine/Core/Normalization/ModelNormalizer";
import type { MaskResolver } from "../../../Engine/Core/PathUtils/MaskResolver";
import type { NavigationPathMapper } from "../../../Engine/Core/PathUtils/NavigationPathMapper";
import { PathUtils } from "../../../Engine/Core/PathUtils/PathUtils";
import type { FullFormState } from "../../../Engine/Core/Store/FullFormState";
import type { IValidator } from "../../../Engine/Core/Validator/Validator";
import type { ValidatorResources } from "../../../Engine/Core/Validator/ValidatorResources";
import type { ActiveLogHelper } from "../../../Engine/Externals/ActiveLog/ActiveLogHelper";
import type { IAttachmentsManager } from "../../../Engine/Externals/Attachments/IAttachmentsManager";
import type { IPicklistsProvider } from "../../../Engine/Core/Picklists/IPicklistsProvider";
import type { IFocusProvider } from "../../../Engine/Externals/FocusProvider/IFocusProvider";
import type { IPublicForm } from "../../PublicForm/IPublicForm";
import type { FormSettings } from "../../Types/FormSettings";
import { ModelEvent, ModelEventsListener } from "../../../Engine/Core/EventManagement/ModelEvents/ModelEvent";
import { AbsoluteModelPath } from "../../../../../Common/ModelPath/ModelPath";
import { BindingPath, ModelValue, ViewModel, ViewModelFieldName } from "../../../Engine/Core/CoreTypes";
import { ModelChunk, ReadonlyModelChunk } from "../../../Engine/Core/ModelStoring/ModelChunk";
import { IReadonlyModelHunk } from "../../../Engine/Core/ModelStoring/IReadonlyModelHunk";
import { SectionName } from "../../../Controls/FormParts/Unit/UnitId/UnitId";
export interface RequisitesInfo {
    [key: string]: {
        value: string;
    };
}
export interface FormHooks {
    beforeInit(form: ILegacyForm): void;
    afterInit(form: ILegacyForm): void;
    beforeUnload(form: ILegacyForm): void;
}
export interface ILegacyEventListener {
    off(pathArr: string[], x2: ModelEventsListener): void;
    on(pathArr: string[], x2: ModelEventsListener): void;
    once(pathArr: string[], x2: ModelEventsListener): ModelEventsListener;
}
export interface ILegacyEventEmitter {
    triggerEvent(event: [string[], unknown]): void;
    triggerAll(events: Array<[string[], unknown]>): void;
}
export interface ILegacyEventManager extends ILegacyEventListener, ILegacyEventEmitter {
}
export interface ILegacyModel {
    get(path: BindingPath | AbsoluteModelPath): ViewModel;
    get(path: BindingPath | AbsoluteModelPath, name: "errorsCount" | "warningsCount"): undefined | number;
    get(path: BindingPath | AbsoluteModelPath, name: string): ModelValue;
    remove(path: AbsoluteModelPath): ModelEvent[];
    getChunk(nodePathPrefix: BindingPath): ModelChunk;
    set(path: BindingPath | AbsoluteModelPath, name: ViewModelFieldName, value: ModelValue): ModelEvent[];
    setChildrenWithoutNormalization(path: BindingPath | AbsoluteModelPath, instances: string[]): ModelEvent[];
    fill(modelChunk: ReadonlyModelChunk | IReadonlyModelHunk, init?: boolean): ModelEvent[];
    fillByPrefix(rootPath: BindingPath | AbsoluteModelPath, modelChunk: ReadonlyModelChunk): ModelEvent[];
    removeSection(sectionName: SectionName): ModelEvent[];
    updateAutoFlags(): ModelEvent[];
    updateErrorCount(formRoot: AbsoluteModelPath): ModelEvent[];
    dropEvents(): void;
    flush(): ModelEvent[];
}
export interface ILegacyForm {
    requisites: RequisitesInfo;
    calculationFunctions: CalculationFunctions;
    calculationHooks: CalculationHooks;
    validationFunctions: ValidatorResources;
    dataDeclaration: DataDeclaration;
    calculator: ICalculator;
    changeSet: IChangeSetManager;
    publicForm: IPublicForm;
    activeLogHelper: ActiveLogHelper;
    controlsDispatcher: unknown;
    dataDeclarationHelper: DataDeclarationHelper;
    errorManager: IErrorManager;
    eventManager: ILegacyEventManager;
    focusProvider: IFocusProvider;
    attachments: IAttachmentsManager;
    hooks: FormHooks;
    model: ILegacyModel;
    maskResolver: MaskResolver;
    modelNormalizer: ModelNormalizer;
    navigationPathMapper: NavigationPathMapper;
    pathUtils: typeof PathUtils;
    picklistsProvider: IPicklistsProvider;
    sections: Sections;
    settings: FormSettings;
    store: Store<FullFormState>;
    validator: IValidator;
    localization: Localization;
    localizationDictionary?: LocalizationDictionary;
    unmount?: () => void;
    candyApi: ICandyApiClient;
    templatesApi: ITemplatesApiClient;
}
