import { AbsoluteModelPath, PathToken, PathTokens } from "../../../Common/ModelPath/ModelPath";
import { CalculationFunctions, CalculationHooks } from "../Engine/Core/AutoCalculations/CalculationResources";
import { CalculationsBuilder } from "../Engine/Core/Calculator2/AutoCalculations/CalculationsBuilder";
import { CalculationFunction } from "../Engine/Core/Calculator2/AutoCalculations/CalculationFunction";
import { Expressions } from "../Engine/Core/Calculator2/AutoCalculations/Expressions";
import { KCCondition } from "../Engine/Core/Calculator2/Validations/KCCondition";
import { ValidationFunction } from "../Engine/Core/Calculator2/Validations/ValidationFunction";
import { DataDeclaration } from "../Engine/Core/DataDeclaration/DataDeclaration";
import { LocalizationDictionary } from "../Engine/Core/Localization/Localization";
import { ValidatorResources } from "../Engine/Core/Validator/ValidatorResources";
import { FormHooks, IForm } from "../Engine/Form/IForm";
import { DependenciesRequiredAreEmpty } from "../Engine/Core/Calculator2/Validations/DependenciesRequiredAreEmpty";
import { KCCalculation } from "../Engine/Core/Calculator2/AutoCalculations/KCCalculation";
import { KCSumCalculation } from "../Engine/Core/Calculator2/AutoCalculations/KCSumCalculation";
import { KCCountCalculation } from "../Engine/Core/Calculator2/AutoCalculations/KCCountCalculation";
import { LazyLoadingDeclaration } from "../Engine/Core/LazyLoading/LazyLoadingDeclaration";
import { KCLangRuntimeUtils } from "../../../Common/KCLangRuntimeUtils";
import { FormSettings } from "./Types/FormSettings";
interface AutocalculationInjectableModules {
    form: IForm;
    Expressions: typeof Expressions;
    CalculationFunction: typeof CalculationFunction;
    ValidationFunction: typeof ValidationFunction;
    PathTokens: typeof PathTokens;
    builder: CalculationsBuilder;
    KCCondition: typeof KCCondition;
    KCCalculation: typeof KCCalculation;
    KCSumCalculation: typeof KCSumCalculation;
    KCCountCalculation: typeof KCCountCalculation;
    DependenciesRequiredAreEmpty: typeof DependenciesRequiredAreEmpty;
    kcLangUtils: KCLangRuntimeUtils;
    ap: (path: PathToken[]) => AbsoluteModelPath;
}
export type AutocalculationConfigurator = (modules: AutocalculationInjectableModules) => void;
export interface FormResources {
    requisiteList: string[];
    resourcesHash: string;
    settings: FormSettings;
    hooks?: FormHooks;
    dataDeclaration: DataDeclaration;
    lazyLoadDeclaration: LazyLoadingDeclaration;
    calculationHooks: CalculationHooks;
    calculationFunctions: CalculationFunctions;
    validationFunctions: ValidatorResources;
    localizationDictionary?: LocalizationDictionary;
    autocalculationConfigurators: undefined | AutocalculationConfigurator[];
}
export interface ReloadableFormResources {
    requisiteList?: string[];
    resourcesHash?: string;
    dataDeclaration?: DataDeclaration;
    calculationHooks?: CalculationHooks;
    calculationFunctions?: CalculationFunctions;
    validationFunctions?: ValidatorResources;
    autocalculationConfigurators?: AutocalculationConfigurator[];
}
export {};
