import { Employee, EmployeeList, GetEmployeesWithActivePeriodQuery, GetWorkingEventListRequest, JsonPatchOperation, MatchEmployeeRequest, MatchEmployeeResponse, MergeRelativeRequest, MergeRelativeResponse, PatchRelativeResponse, WorkingEventList } from "@skbkontur/staff-selector-widget";
import { Subscription } from "../../Engine/Form/ModelChangeEventListener";
import { Step } from "../../Controls/FormParts/Tour/TourTypes";
import { ServerErrorLog } from "../../../../Common/CandyApiClient/Types/ErrorLog/ServerErrorLog";
import { BindingPath, ModelValue } from "../../Engine/Core/CoreTypes";
import { ExternalHookFunction } from "../../Engine/Core/EventManagement/ExternalEvents/ExternalHookFunction";
import { PicklistQueryResponse } from "../../Engine/Core/Picklists/PicklistTypes";
import { ModelEventsListener } from "../../Engine/Core/EventManagement/ModelEvents/ModelEvent";
import { DigestQueryFilter } from "../../../../Common/CandyApiClient/DigestApiClient/Types/DigestQueryFilter";
import { IUpdateFormModelAccessor } from "../../Engine/Form/IFormModelAccessor";
import { ModalOptions } from "../../Engine/Core/ModalManager/IModalState";
import { MassActionParameters } from "../../../../Common/CandyApiClient/Types/MassActions/MassActionParameters";
import { AbsoluteResolvedModelPath } from "../../../../Common/ModelPath/ModelPath";
import { IModalFormsRegistry } from "../../Controls/Modal/ModalForm/ModalFormsContext";
import { PrintType } from "../../../../Common/CandyApiClient/Types/Print/PrintType";
import { ImportEmployeesOptions } from "../../../../Common/CandyApiClient/Types/Staff/ImportEmployeesOptions";
import { InstancesDeletingRule } from "../../../../Common/CandyApiClient/Types/InstancesDeleting/InstancesDeletingRule";
import { FiasApi, FiasClientInitOptions } from "../../Controls/ValueEditors/Fias/FiasInput/FiasApi";
import { IFSPrinterClient } from "../../../../Common/FSPrinterClient/IFSPrinterClient";
import { CombineFufResult } from "../../../../Common/CandyApiClient/Types/DraftActions/CombineFufResult";
import { ICandyComponentHandle } from "../../Engine/Core/ComponentHandleRegistry/ICandyComponentHandle";
import { NavigationPathMapper } from "../InjectableModules/InjectableEngineModules";
import { EmployerNormalizedTag, ImportEmployerDocumentsOptions, ImportFromEmployeeServiceOptions } from "../../StaffServiceIntegration/EmployeeServiceTypes";
import { TabInfosState } from "../../Controls/MultiControls/Tabs/Store/TabInfosState";
import { MetricsPayload } from "../../Controls/Internals/Metrics/MetricsProvider";
import { CallInitSectionLightboxProps, InitSectionFormLightboxOptions } from "./InitSectionLightbox";
export interface IPublicForm {
    modalFormsRegistry?: IModalFormsRegistry;
    fsPrinterClient?: IFSPrinterClient;
    on(eventName: string, listener: ExternalHookFunction): void;
    off(eventName: string, listener: ExternalHookFunction): void;
    onModelChange(path: string | string[], listener: unknown): Subscription;
    save(): Promise<void>;
    runCheck(): Promise<void>;
    close(): Promise<void>;
    getTourSteps(): Step[];
    validate(): Promise<void>;
    validateChildrenByPath(path: string): Promise<void>;
    hasErrors(): boolean;
    onErrorsCountChanged(listener: ModelEventsListener): void;
    offErrorsCountChanged(listener: ModelEventsListener): void;
    getByGId(gId: string, filter?: DigestQueryFilter): Promise<PicklistQueryResponse>;
    set(pathOrModelChunk: {
        [path: string]: string | undefined;
    }): void | Promise<void>;
    set(pathOrModelChunk: string, value: ModelValue): void | Promise<void>;
    set(pathOrModelChunk: string | {
        [path: string]: string | undefined;
    }, value?: ModelValue): void | Promise<void>;
    addInstance(path: BindingPath): Promise<string>;
    removeInstance(path: BindingPath, instance: string): Promise<void>;
    removeInstances(path: BindingPath, instances: string[], serverSide?: boolean): Promise<void>;
    batchUpdate(updateProc: (updateAccessor: IUpdateFormModelAccessor) => void): Promise<void>;
    get(pathOrArray?: string | string[]): ModelValue;
    getValue(pathOrArray?: string | string[]): ModelValue;
    getChildren(path: BindingPath): string[];
    setErrorLog(errorLog: Partial<ServerErrorLog>): void;
    copySubtree(sourceBindingPath: BindingPath | AbsoluteResolvedModelPath, targetBindingPath: BindingPath | AbsoluteResolvedModelPath, doNotCopyPaths?: string[]): void;
    reloadInner(): Promise<void>;
    showModal(options: ModalOptions): void;
    uuidGenerate(): string;
    callInitSection(options: CallInitSectionLightboxProps): void;
    addSection(options: InitSectionFormLightboxOptions): Promise<void>;
    openEmployeeEditor(employeeId: string): void;
    setAutoFieldsToAutoValue(pathPrefixes: string[]): Promise<void>;
    focusOnControlIfExist(path: BindingPath): void;
    navigateByHighlanderPaths(paths: BindingPath[]): Promise<undefined | ICandyComponentHandle>;
    getNavigationPathMapper(): NavigationPathMapper;
    importEmployees(employees: Employee[], options?: ImportEmployeesOptions): Promise<void>;
    __INTERNAL__importFromEmployeeService(options?: ImportFromEmployeeServiceOptions): Promise<Employee[] | undefined>;
    __INTERNAL__getSelectFunctionForDisableEmployeeInSS(pathSpec: string): Promise<(employee: Employee) => boolean>;
    importEmployerDocuments(options: ImportEmployerDocumentsOptions): Promise<EmployerNormalizedTag | undefined | null>;
    runMassAction(actionId: string, parameters?: MassActionParameters): Promise<void>;
    customNormalize(normalizerId: string, pathPrefixes: string[]): Promise<void>;
    print(printType: PrintType, body?: string[]): Promise<void>;
    navigate(navigationPath: string): Promise<undefined | ICandyComponentHandle>;
    getAdditionalEmployeeInfo(filters: GetEmployeesWithActivePeriodQuery): Promise<EmployeeList | undefined>;
    getEmployees(query: GetEmployeesWithActivePeriodQuery): Promise<EmployeeList>;
    getWorkingEventsForEmployee(filters: GetWorkingEventListRequest): Promise<WorkingEventList | undefined>;
    removeInstancesByRules(rules: InstancesDeletingRule[]): Promise<void>;
    mergeRelative(bindingPath: BindingPath, employeeId: string, query: MergeRelativeRequest): Promise<MergeRelativeResponse | undefined>;
    patchRelative(employeeId: string, relativeId: string, operations: JsonPatchOperation[]): Promise<PatchRelativeResponse>;
    matchEmployee(query: MatchEmployeeRequest): Promise<MatchEmployeeResponse>;
    cancelSelectRelative(bindingPath: BindingPath): void;
    getTabsState(): TabInfosState;
    getModalUnitOpenedInstance(pageId: string): string | undefined | null;
    createFiasApiClient(options: FiasClientInitOptions): FiasApi;
    ensureInstanceFullyLoaded(bindingPath: BindingPath | AbsoluteResolvedModelPath, ...instances: string[]): Promise<void>;
    createDnp(): Promise<CombineFufResult | undefined>;
    sendMetrics(action: string, payload?: MetricsPayload): void;
    getSelectedInstances(multiplicityPath: BindingPath | AbsoluteResolvedModelPath, tableUniqId: string, pathToChecked: string, checkedValue?: string): Promise<string[]>;
    getAttachmentPublicFormByPath(path: string): IPublicForm;
    getGfv(): string;
    clearFormWithReload(): Promise<void>;
}
