"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAsyncReturnedResultHandler = void 0;
exports.getOperands = getOperands;
exports.getAsyncResultHandler = getAsyncResultHandler;
exports.getAsyncTextResultHandler = getAsyncTextResultHandler;
exports.getAsyncBoolResultHandler = getAsyncBoolResultHandler;
exports.getAsyncErrorResultHandler = getAsyncErrorResultHandler;
exports.wrapDecimalOperand = wrapDecimalOperand;
exports.wrapDecimalOrUndefined = wrapDecimalOrUndefined;
exports.getSum = getSum;
exports.checkValuesArguments = checkValuesArguments;
const big_js_1 = __importDefault(require("big.js"));
const IterableUtils_1 = require("../../../Common/IterableUtils");
function getOperands(argsMapping, operandsGetter) {
    const args = {};
    for (const [argumentName, argumentMapping] of (0, IterableUtils_1.iterateEntries)(argsMapping)) {
        if (argumentMapping.length === 1) {
            args[argumentName] = operandsGetter.get(argumentMapping[0]);
        }
        else if (argumentMapping.length === 2) {
            args[argumentName] = operandsGetter.get(argumentMapping[0], argumentMapping[1]);
        }
        else if (argumentMapping.length === 3) {
            args[argumentName] = operandsGetter.get(argumentMapping[0], argumentMapping[1], argumentMapping[2]);
        }
        else {
            throw new Error(`Invalid argument request: ${JSON.stringify(argumentMapping)}`);
        }
    }
    return args;
}
function getAsyncResultHandler(argsMapping, calcFn) {
    return function resultHandler(operandsGetter) {
        const settings = this.settings;
        const requisites = this.requisites;
        const args = getOperands(argsMapping, operandsGetter);
        const result = calcFn(args, operandsGetter, settings, requisites);
        for (const calculationResult of result) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
            if (isNaN(calculationResult[2])) {
                calculationResult[2] = "";
            }
            else if (calculationResult[2] instanceof big_js_1.default) {
                calculationResult[2] = calculationResult[2].toFixed();
            }
            else {
                calculationResult[2] = String(calculationResult[2]);
            }
        }
        return result;
    };
}
function getAsyncTextResultHandler(argsMapping, calcFn) {
    return function textResultHandler(operandsGetter) {
        const settings = this.settings;
        const requisites = this.requisites;
        const args = getOperands(argsMapping, operandsGetter);
        const result = calcFn(args, settings, requisites, operandsGetter);
        for (const calculationResult of result) {
            if (!calculationResult[2]) {
                calculationResult[2] = "";
            }
            else if (calculationResult[2] instanceof big_js_1.default) {
                calculationResult[2] = calculationResult[2].toFixed();
            }
            else {
                if (typeof calculationResult[2] === "object") {
                    continue;
                }
                calculationResult[2] = String(calculationResult[2]);
            }
        }
        return result;
    };
}
function getAsyncBoolResultHandler(argsMapping, calcFn) {
    return function boolResultHandler(operandsGetter) {
        const settings = this.settings;
        const args = getOperands(argsMapping, operandsGetter);
        return calcFn(args, settings, operandsGetter, this);
    };
}
function getAsyncErrorResultHandler(argsMapping, calcFn) {
    return function errorResultHandler(operandsGetter) {
        const settings = this.settings;
        const requisites = this.requisites;
        const args = getOperands(argsMapping, operandsGetter);
        return calcFn(args, settings, requisites, operandsGetter);
    };
}
function wrapDecimalOperand(value) {
    if (typeof value === "number" && isFinite(value)) {
        return new big_js_1.default(value);
    }
    if (typeof value === "string" && /^-?\d+[.]?\d*$/.test(value)) {
        return new big_js_1.default(value);
    }
    if (value instanceof big_js_1.default) {
        return value;
    }
    return new big_js_1.default(0);
}
function wrapDecimalOrUndefined(value) {
    if (typeof value === "number" && isFinite(value)) {
        return new big_js_1.default(value);
    }
    if (typeof value === "string" && /^-?\d+[.]?\d*$/.test(value)) {
        return new big_js_1.default(value);
    }
    if (value instanceof big_js_1.default) {
        return value;
    }
    return undefined;
}
function getSum(values) {
    let allValuesIsEmpty = true;
    if (values.length === 0) {
        return "";
    }
    let sum = new big_js_1.default(0);
    for (const val of values) {
        if (val) {
            allValuesIsEmpty = false;
            sum = sum.plus(wrapDecimalOperand(val));
        }
    }
    if (allValuesIsEmpty) {
        return "";
    }
    return sum;
}
function checkValuesArguments(args) {
    for (const argName in args) {
        if (!args.hasOwnProperty(argName)) {
            continue;
        }
        // @ts-ignore
        if (args[argName] && args[argName].values && args[argName].values.find(val => val !== "")) {
            return true;
        }
        // @ts-ignore
        if (args[argName] && !args[argName].values && args[argName] !== "") {
            return true;
        }
    }
    return false;
}
exports.getAsyncReturnedResultHandler = getAsyncErrorResultHandler;
