"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilterExpressionToQueryCollectionExpressionVisitor = void 0;
const Errors_1 = require("../../../../Common/Errors");
const AndExpression_1 = require("../../../../Common/CandyApiClient/Types/QueryCollecton/Expressions/AndExpression");
const OrExpression_1 = require("../../../../Common/CandyApiClient/Types/QueryCollecton/Expressions/OrExpression");
const NotExpression_1 = require("../../../../Common/CandyApiClient/Types/QueryCollecton/Expressions/NotExpression");
const StringMatchExpression_1 = require("../../../../Common/CandyApiClient/Types/QueryCollecton/Expressions/StringMatchExpression");
const ModelPath_1 = require("../../../../Common/ModelPath/ModelPath");
const AbsoluteModelFieldPath_1 = require("../../../../Common/ModelPath/AbsoluteModelFieldPath");
const CompileFilterExpression_1 = require("./CompileFilterExpression");
const notSupported = () => {
    throw new Errors_1.NotSupportedError();
};
class FilterExpressionToQueryCollectionExpressionVisitor extends CompileFilterExpression_1.FilterExpressionTransduceVisitor {
    multilinePath;
    constructor(multilinePath) {
        super();
        this.multilinePath = multilinePath;
    }
    transformAndExpression(expression) {
        const expressions = expression.conditions.map(x => this.transformExpression(x));
        return new AndExpression_1.AndExpression(expressions);
    }
    transformOrExpression(expression) {
        const expressions = expression.conditions.map(x => this.transformExpression(x));
        return new OrExpression_1.OrExpression(expressions);
    }
    transformUnaryNotExpression(expression) {
        const expr = this.transformExpression(expression.condition);
        return new NotExpression_1.NotExpression(expr);
    }
    transformEqualExpression(expression) {
        const fullFieldName = this.getFullFieldNameFromDataPath(AbsoluteModelFieldPath_1.AbsoluteModelFieldPath.createFromMask(expression.condition.dataPath, ModelPath_1.PathTokens.each));
        return new StringMatchExpression_1.StringMatchExpression(fullFieldName, expression.condition.value, "Exact", false);
    }
    transformSubstringExpression(expression) {
        const fullFieldName = this.getFullFieldNameFromDataPath(AbsoluteModelFieldPath_1.AbsoluteModelFieldPath.createFromMask(expression.condition.dataPath, ModelPath_1.PathTokens.each));
        return new StringMatchExpression_1.StringMatchExpression(fullFieldName, expression.condition.value, "Substring", false);
    }
    getFullFieldNameFromDataPath(modelFieldPath) {
        const tailPath = this.multilinePath.getSuffixPathFrom(modelFieldPath.path).trimFirstStarIfLastToken();
        const fieldName = modelFieldPath.field;
        const fullFieldName = `${tailPath.toString()}.${fieldName}`;
        return fullFieldName;
    }
    createDefaultResult() {
        notSupported();
    }
    transformByInstancesExpression(expression) {
        notSupported();
    }
    transformEndDateExpression(expression) {
        notSupported();
    }
    transformStartDateExpression(expression) {
        notSupported();
    }
}
exports.FilterExpressionToQueryCollectionExpressionVisitor = FilterExpressionToQueryCollectionExpressionVisitor;
