"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.filterInstancesBatchedAsync = filterInstancesBatchedAsync;
exports.isNotEmptyFilters = isNotEmptyFilters;
exports.getFilterKey = getFilterKey;
exports.resolveFilterDataPath = resolveFilterDataPath;
const ModelPath_1 = require("../../../Common/ModelPath/ModelPath");
const ModelPathBuilder_1 = require("../../../Common/ModelPath/ModelPathBuilder");
const PathMatcher_1 = require("../../../Common/ModelPath/PathMatcher");
const AbsoluteModelFieldPath_1 = require("../../../Common/ModelPath/AbsoluteModelFieldPath");
const IterableUtils_1 = require("../../../Common/IterableUtils");
const AsyncUtils_1 = require("../../../Common/AsyncUtils");
const CompileFilterExpression_1 = require("./FilterExpressions/CompileFilterExpression");
const MultilineHelpers_1 = require("./StateAccessors/MultilineHelpers");
async function filterInstancesBatchedAsync(viewModel, viewModelSelect, filterExpressions, multilinePath, batchSize = 100) {
    const instances = viewModelSelect.children(viewModel, multilinePath);
    const filterConditionFunctions = filterExpressions.map(CompileFilterExpression_1.compileFilterExpression);
    const filteredInstances = [];
    for (const [index, instance] of IterableUtils_1.IterUtils.iterateEntries(instances)) {
        const getInstanceValueFn = (dataPath) => {
            const resolvedFilterPathToMultilinePath = resolveFilterDataPath(dataPath, (0, ModelPath_1.createAbsoluteFromMask)(multilinePath, ModelPath_1.PathTokens.each));
            const resolvedPath = (0, MultilineHelpers_1.resolveMask)(resolvedFilterPathToMultilinePath.toString(), instance);
            const modelFieldPath = AbsoluteModelFieldPath_1.AbsoluteModelFieldPath.createFromMask(resolvedPath, ModelPath_1.PathTokens.each);
            return (0, MultilineHelpers_1.getMultipleValues)(viewModel, modelFieldPath.path, modelFieldPath.field);
        };
        const getInstanceFn = (dataPath) => instance;
        const isMatch = filterConditionFunctions.every(fn => fn(getInstanceValueFn, getInstanceFn));
        if (isMatch) {
            filteredInstances.push(instance);
        }
        if (index % batchSize === 0) {
            await (0, AsyncUtils_1.nextTick)();
        }
    }
    return filteredInstances;
}
function isNotEmptyFilters(filters) {
    return (filters?.length ?? 0) > 0;
}
function getFilterKey(dataPaths) {
    return dataPaths.map(x => x.toLegacyPath()).join("_");
}
function resolveFilterDataPath(filterPath, multiplicityPath) {
    const resultPath = new ModelPathBuilder_1.ModelPathBuilder(false);
    const modelFieldPathFromFilterPath = AbsoluteModelFieldPath_1.AbsoluteModelFieldPath.createFromMask(filterPath, ModelPath_1.PathTokens.each);
    const path = modelFieldPathFromFilterPath.path;
    const matchedPath = path.matchFor(multiplicityPath);
    for (const match of matchedPath) {
        if (match.kind === PathMatcher_1.MatchKind.DifferentSuffix) {
            resultPath.append(match.source ?? []);
        }
        else if (match.kind === PathMatcher_1.MatchKind.MatchedPath) {
            resultPath.append(match.prefix);
        }
        else if (match.kind === PathMatcher_1.MatchKind.MatchedMultiplicity) {
            resultPath.append(match.target);
        }
    }
    return new AbsoluteModelFieldPath_1.AbsoluteModelFieldPath(resultPath.toAbsolutePath(), modelFieldPathFromFilterPath.field);
}
