"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoadIntoDocument = void 0;
class LoadIntoDocument {
    static async diagnoseNetworkError(url) {
        try {
            const response = await fetch(url, {
                method: "GET",
                mode: "cors",
                credentials: "include",
            });
            return {
                status: response.status,
                statusText: response.statusText,
                ok: response.ok,
                headers: Object.fromEntries(response.headers.entries()),
            };
        }
        catch (err) {
            const fetchError = err;
            return {
                error: fetchError.message,
                type: fetchError.name,
                stack: fetchError.stack,
            };
        }
    }
    static script(uri, callback) {
        let scriptLoaded = false;
        const head = document.getElementsByTagName("head")[0] || document.documentElement;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const script = document.createElement("script");
        function loadHandler() {
            if (scriptLoaded) {
                return;
            }
            if (!script.readyState || script.readyState === "loaded" || script.readyState === "complete") {
                scriptLoaded = true;
                callback();
            }
        }
        // Hang the user callback to be called when the script completes to load.
        // NOTE(user): This callback will be called in IE even upon error. In any
        // case it is the client's responsibility to verify that the script ran
        // successfully.
        script.onload = loadHandler;
        script.onreadystatechange = loadHandler;
        // Add an error callback.
        // NOTE(user): Not supported in IE.
        script.onerror = async () => {
            if (script.parentNode) {
                script.parentNode.removeChild(script);
            }
            // используем доп запрос для получения инфы об ошибке
            const diagnosticResult = await LoadIntoDocument.diagnoseNetworkError(uri);
            callback(new Error(`Error while loading script \n ${JSON.stringify(diagnosticResult)}`));
        };
        // Add the script element to the document.
        // NOTE(user): Safari never loads the script if we don't set
        // the src attribute before appending.
        script.src = uri;
        script.setAttribute("crossorigin", "use-credentials");
        head.appendChild(script);
    }
    static styleSheet(uri, ignoreError, callback) {
        const resolve = () => callback(undefined);
        const reject = () => {
            if (ignoreError) {
                callback(undefined);
            }
            else {
                callback(new Error("Error while loading stylesheet."));
            }
        };
        const head = document.getElementsByTagName("head")[0] ?? document.documentElement;
        const link = document.createElement("link");
        link.setAttribute("rel", "stylesheet");
        link.setAttribute("type", "text/css");
        link.addEventListener("load", resolve);
        link.addEventListener("error", reject);
        head.appendChild(link);
        link.setAttribute("href", uri);
    }
}
exports.LoadIntoDocument = LoadIntoDocument;
