"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sortInstances = sortInstances;
const MultiLineTypes_1 = require("../Controls/MultiControls/MultiLine/MultiLineTypes");
const DateHelpers_1 = require("./DateHelpers");
const ReduxStateHelpers_1 = require("./ReduxStateHelpers");
function sortInstances(instances, viewModel, multilineSort) {
    if (multilineSort == undefined) {
        return [...instances];
    }
    const { dataType, value: sortType } = multilineSort;
    const newInstances = [...instances];
    const [, path] = multilineSort.dataPath.split("/*/");
    if (path === undefined) {
        throw new Error("To sort instances path must be as mask");
    }
    newInstances.sort((a, b) => {
        const aValue = (0, ReduxStateHelpers_1.getIn)(viewModel, [a, ...path.split("/"), ".", "value"], "");
        const bValue = (0, ReduxStateHelpers_1.getIn)(viewModel, [b, ...path.split("/"), ".", "value"], "");
        if (dataType === MultiLineTypes_1.DataType.Date) {
            return compareDate(bValue, aValue, sortType);
        }
        else if (dataType === MultiLineTypes_1.DataType.CaseInsensitiveString) {
            return compareCaseInsensitiveString(bValue, aValue, sortType);
        }
        else {
            return compareNumber(aValue, bValue, sortType);
        }
    });
    return newInstances;
}
function compareNumber(a, b, type) {
    if (!a && !b) {
        return 0;
    }
    if (!a) {
        return type === MultiLineTypes_1.SortEnum.Desc ? 1 : -1;
    }
    if (!b) {
        return type === MultiLineTypes_1.SortEnum.Desc ? -1 : 1;
    }
    return type === MultiLineTypes_1.SortEnum.Desc ? Number(b) - Number(a) : Number(a) - Number(b);
}
function compareDate(a, b, type) {
    const aDate = getDate(a);
    const bDate = getDate(b);
    if (!aDate && !bDate) {
        return 0;
    }
    if (!aDate) {
        return type === MultiLineTypes_1.SortEnum.Desc ? -1 : 1;
    }
    if (!bDate) {
        return type === MultiLineTypes_1.SortEnum.Desc ? 1 : -1;
    }
    return type === MultiLineTypes_1.SortEnum.Desc ? aDate.getTime() - bDate.getTime() : bDate.getTime() - aDate.getTime();
}
function compareCaseInsensitiveString(left, right, type) {
    if ((left == undefined || left === "") && (right == undefined || right === "")) {
        return 0;
    }
    if (left == undefined || left === "") {
        return type === MultiLineTypes_1.SortEnum.Desc ? -1 : 1;
    }
    if (right == undefined || right === "") {
        return type === MultiLineTypes_1.SortEnum.Desc ? 1 : -1;
    }
    const result = left.localeCompare(right);
    if (type === MultiLineTypes_1.SortEnum.Asc) {
        return result * -1;
    }
    return result;
}
function getDate(dateString) {
    const dateObj = (0, DateHelpers_1.tryGetValidDateShape)((0, DateHelpers_1.parseDateString)(dateString));
    if (!dateObj) {
        return undefined;
    }
    const { date, month, year } = dateObj;
    return new Date(Date.UTC(year, month, date));
}
