"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getExternalPropertyValue = getExternalPropertyValue;
exports.getValueByPath = getValueByPath;
function isPrimitiveValue(value) {
    return typeof value === "string" || typeof value === "number" || typeof value === "boolean";
}
function getFeatureFlagByPath(state, path) {
    let value = undefined;
    const featureFlagValue = state?.featureFlags?.[path.replace("@featureFlags/", "")];
    if (featureFlagValue == undefined) {
        value = undefined;
    }
    else if (isPrimitiveValue(featureFlagValue)) {
        value = featureFlagValue;
    }
    else {
        // eslint-disable-next-line no-console
        console.warn(`FeatureFlags of type "${typeof featureFlagValue}" is not supported`);
    }
    return value;
}
function getSettingByPath(state, path) {
    let value = undefined;
    const settingValue = state.settings?.[path.replace("@settings/", "")];
    if (settingValue == undefined) {
        value = undefined;
    }
    else if (isPrimitiveValue(settingValue)) {
        value = settingValue;
    }
    else {
        // eslint-disable-next-line no-console
        console.warn(`Setting of type "${typeof settingValue}" is not supported`);
    }
    return value;
}
function getExternalPropertyValue(state, path) {
    if (path.startsWith("@mergedPropertiesForBackendCompatibility/")) {
        // Что это за история: на бекенде getProperty не принимает тип внешней переменной -- просто имя, они какбы
        // сливаются в одну кучу (ну или так было), поэтому для обраной совместимости может приходит такой префикс
        // когда мы потеряли контекс.
        const name = path.replace("@mergedPropertiesForBackendCompatibility/", "");
        return (getFeatureFlagByPath(state, "@featureFlags/" + name) ??
            getSettingByPath(state, "@settings/" + name) ??
            state?.requisites?.["@" + name]?.value ??
            "");
    }
    else if (path.startsWith("@featureFlags/")) {
        return getFeatureFlagByPath(state, path) ?? "";
    }
    else if (path.startsWith("@settings/")) {
        return getSettingByPath(state, path) ?? "";
    }
    else if (path.startsWith("@")) {
        return state?.requisites?.[path]?.value ?? "";
    }
    return "";
}
function getValueByPath(state, selector, path, name = "value") {
    let value;
    if (typeof path === "string") {
        value = getExternalPropertyValue(state, path);
    }
    else {
        value = selector(state, path, name);
    }
    return value ?? "";
}
