"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstancesAfterAdd = getInstancesAfterAdd;
exports.getDisplayedInstances = getDisplayedInstances;
exports.getDisplayedInstancesCount = getDisplayedInstancesCount;
exports.getInstancesAfterRemove = getInstancesAfterRemove;
exports.resolveMask = resolveMask;
exports.getMultipleValues = getMultipleValues;
exports.findFreeInstanceNumber = findFreeInstanceNumber;
const multiLineReducer_1 = require("../../Controls/MultiControls/MultiLine/Store/multiLineReducer");
const PathUtils_1 = require("../../Engine/Core/PathUtils/PathUtils");
const TypingUtils_1 = require("../../../../Common/TypingUtils");
const ModelPath_1 = require("../../../../Common/ModelPath/ModelPath");
const MultilineStateSelect_1 = require("./MultilineStateSelect");
const ViewModelSelect_1 = require("./ViewModelSelect");
function getInstancesAfterAdd(instances, newInstance, instanceIndex) {
    if (!newInstance) {
        return [...instances];
    }
    return [...instances.slice(0, instanceIndex), newInstance, ...instances.slice(instanceIndex, instances.length)];
}
function getDisplayedInstances(state, bindingPath, filteredUniqKey, defaultState = multiLineReducer_1.defaultMultiLinePathState) {
    const multilineState = (0, MultilineStateSelect_1.getMultilineState)(state.multiLineInfos, bindingPath, filteredUniqKey, defaultState);
    const { instances = [], filters = [], sort } = multilineState;
    const children = ViewModelSelect_1.currentViewModelSelect.children(state, bindingPath, []);
    return Object.keys(filters).length || sort || multilineState.useServerFilter ? instances : children;
}
function getDisplayedInstancesCount(state, bindingPath, filteredUniqKey) {
    const instances = getDisplayedInstances(state, bindingPath, filteredUniqKey);
    return instances.length;
}
function getInstancesAfterRemove(instances, instancesToRemove) {
    const instancesToRemoveSet = new Set(instancesToRemove);
    return instances.filter(i => !instancesToRemoveSet.has(i));
}
function resolveMask(mask, instance) {
    const pathTokens = PathUtils_1.PathUtils.split(mask);
    const tokenWithWildcardIndex = pathTokens.findIndex(ch => ch === "*" || ch.startsWith("*."));
    if (tokenWithWildcardIndex === -1) {
        return mask;
    }
    const parentPathTokens = pathTokens.slice(0, tokenWithWildcardIndex);
    const childPathTokens = pathTokens.slice(tokenWithWildcardIndex + 1);
    const tokenWithWildcard = pathTokens[tokenWithWildcardIndex] ?? (0, TypingUtils_1.reject)();
    return [...parentPathTokens, tokenWithWildcard.replace("*", instance), ...childPathTokens].join("/");
}
function getMultipleValues(viewModel, dataPath, fieldName = "value") {
    const splittedPath = dataPath.getPathPartsAsArray();
    const multipleSymbolIndex = splittedPath.findIndex(ch => ModelPath_1.PathTokens.isMultiToken(ch));
    if (multipleSymbolIndex !== -1) {
        const splittedParentPath = splittedPath.slice(0, multipleSymbolIndex);
        const children = ViewModelSelect_1.currentViewModelSelect.children(viewModel, splittedParentPath.join("/"));
        const splittedChildPath = splittedPath.slice(multipleSymbolIndex + 1);
        const resolvedPaths = children.map(child => (0, ModelPath_1.createAbsoluteFromTokens)([...splittedParentPath, child, ...splittedChildPath]));
        const multipleData = resolvedPaths.map(resolvedPath => getMultipleValues(viewModel, resolvedPath, fieldName));
        return multipleData.flat();
    }
    const modelValue = ViewModelSelect_1.currentViewModelSelect.customFieldValueOrUndefined(viewModel, dataPath, fieldName) ?? "";
    return [modelValue];
}
function findFreeInstanceNumber(instances) {
    const sortedInstances = new Set(instances);
    for (let instanceIndex = 0; instanceIndex < sortedInstances.size; instanceIndex++) {
        if (!sortedInstances.has(instanceIndex.toString())) {
            return instanceIndex.toString();
        }
    }
    return sortedInstances.size.toString();
}
