"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPathsViewModels = exports.parseStringViewModel = void 0;
const IterableUtils_1 = require("../../../Common/IterableUtils");
const StringHelpers_1 = require("./StringHelpers");
const parseStringValue = (value, field) => {
    const fieldsToParse = [
        "children",
        "error",
        "warning",
        "disabled",
        "visible",
        "disabledSending",
        "autoFlag",
        "errorsCount",
        "warningsCount",
        "normalized",
        "emptyUnitsVisible",
        "pencilOpened",
        "ignoreAutoFlag",
    ];
    if (fieldsToParse.includes(field)) {
        return JSON.parse(value);
    }
    return value;
};
const parseStringViewModel = (stringViewModel) => {
    const viewModel = {};
    for (const [viewModelField, viewModelValue] of (0, IterableUtils_1.iterateEntries)(stringViewModel)) {
        // @ts-ignore
        viewModel[viewModelField] = parseStringValue(viewModelValue, viewModelField);
    }
    return viewModel;
};
exports.parseStringViewModel = parseStringViewModel;
const getPathsViewModels = (model) => {
    const pathsViewModels = {};
    for (const [fieldName, fieldValue] of (0, IterableUtils_1.iterateEntries)(model)) {
        const [path, viewModelFieldName] = (0, StringHelpers_1.splitLastOccurrence)(fieldName, ".");
        if (!viewModelFieldName) {
            continue;
        }
        if (pathsViewModels[path] == undefined) {
            pathsViewModels[path] = {};
        }
        // @ts-ignore
        pathsViewModels[path][viewModelFieldName] = fieldValue;
    }
    return pathsViewModels;
};
exports.getPathsViewModels = getPathsViewModels;
