"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.selectUserFiles = selectUserFiles;
exports.uploadToDrive = uploadToDrive;
const Testing_1 = require("../Controls/Testing");
const LoadIntoDocument_1 = require("../Helpers/LoadIntoDocument");
let fileInput;
function selectUserFiles(multiple) {
    return new Promise(resolve => {
        const input = getFileInput();
        const savedOnFocus = document.body.onfocus;
        // no other way to detect closure of open file window
        document.body.onfocus = function handleFocus(...args) {
            const result = savedOnFocus ? savedOnFocus.call(this, ...args) : null;
            setTimeout(() => {
                if (input.files == undefined || input.files.length === 0) {
                    document.body.onfocus = savedOnFocus;
                    resolve([]);
                }
            }, 1000); // другого способа я не нашел
            return result;
        };
        input.multiple = multiple;
        input.onchange = () => onFileInputChange(input, files => {
            document.body.onfocus = savedOnFocus;
            resolve(Array.prototype.slice.call(files, 0));
        });
        if ((0, Testing_1.isTestingMode)()) {
            window.lastInputElement = input;
        }
        else {
            input.click();
        }
    });
}
function uploadToDrive(file, onProgress) {
    let initDrive = Promise.resolve();
    if (!window.kontur || !window.kontur.drive || !window.kontur.drive.upload) {
        initDrive = new Promise((resolve, reject) => {
            // expected host: any.testkontur.ru or any.kontur.ru
            const host = window.location.hostname.split(".");
            host[0] = "api";
            const driveClientJsUri = `https://${host.join(".")}/drive/v1/js/client/js`;
            LoadIntoDocument_1.LoadIntoDocument.script(driveClientJsUri, error => {
                if (window.kontur == undefined || window.kontur.drive == undefined) {
                    reject(new Error("Drive client is not loaded"));
                    return;
                }
                if (error) {
                    reject(error);
                }
                else {
                    window.kontur.drive.init(err => {
                        if (err) {
                            reject(err);
                        }
                        else {
                            resolve();
                        }
                    });
                }
            });
        });
    }
    return {
        initDrive: initDrive,
        getDriveTask: () => {
            if (window.kontur == undefined || window.kontur.drive == undefined) {
                throw new Error("Drive client is not loaded");
            }
            const driveTask = window.kontur.drive.upload(file, { daysToLive: 30, srv: "candyFront" });
            if (driveTask.progress) {
                driveTask.progress(onProgress);
            }
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return driveTask;
        },
    };
}
function getFileInput() {
    if (fileInput == undefined) {
        const input = document.createElement("input");
        input.type = "file";
        input.style.display = "none";
        // IE doesn't show open dialog if the input isn't in document.
        document.body.appendChild(input);
        fileInput = input;
    }
    return fileInput;
}
function onFileInputChange(input, callback) {
    if (fileInput != undefined && fileInput.parentNode != undefined) {
        fileInput.parentNode.removeChild(fileInput);
        fileInput.onchange = null;
        fileInput = undefined;
    }
    callback(input.files);
}
