import { CreateDocumentRequest, CreateDocumentResponse, MergeRelativeResponse, Employee, EmployeeList, GetDocumentsQuery, GetEmployeesWithActivePeriodQuery, GetRelativesQuery, GetWorkingEventListRequest, MergeRelativeRequest, ObjectDocumentList, relativeDef, RelativeListDef, StaffClient, WorkingEventList, MatchEmployeeRequest, MatchEmployeeResponse, JsonPatchOperation, PatchRelativeResponse } from "@skbkontur/staff-selector-widget";
import { FormSettings } from "../Forms/Types/FormSettings";
export interface IFormStaffClient {
    getWorkingEventsForEmployee(query: GetWorkingEventListRequest): Promise<WorkingEventList | undefined>;
    getAdditionalEmployeeInfo(query: GetEmployeesWithActivePeriodQuery): Promise<EmployeeList | undefined>;
    getDocuments(query: GetDocumentsQuery, signal?: AbortSignal): Promise<ObjectDocumentList>;
    createDocument(request: CreateDocumentRequest, signal?: AbortSignal): Promise<CreateDocumentResponse>;
    getEmployee(employeeId: string, signal?: AbortSignal): Promise<Employee>;
    getEmployees(query: GetEmployeesWithActivePeriodQuery, signal?: AbortSignal): Promise<EmployeeList>;
    getRelative(employeeId: string, relativeId: string, signal?: AbortSignal): Promise<relativeDef>;
    getRelatives(employeeId: string, query?: GetRelativesQuery, signal?: AbortSignal): Promise<RelativeListDef>;
    mergeRelative(employeeId: string, query: MergeRelativeRequest, signal?: AbortSignal): Promise<MergeRelativeResponse>;
    patchRelative(employeeId: string, relativeId: string, operations: JsonPatchOperation[], signal?: AbortSignal): Promise<PatchRelativeResponse>;
    matchEmployee(query: MatchEmployeeRequest, signal?: AbortSignal): Promise<MatchEmployeeResponse>;
}
type FormStaffClientOptions = Pick<FormSettings, "staffServiceUrl" | "employerId">;
export declare class FormStaffClient implements IFormStaffClient {
    protected readonly staffClient: StaffClient;
    private readonly employerId;
    constructor(settings: FormStaffClientOptions);
    getWorkingEventsForEmployee(query: GetWorkingEventListRequest): Promise<WorkingEventList | undefined>;
    getAdditionalEmployeeInfo(query: GetEmployeesWithActivePeriodQuery): Promise<EmployeeList | undefined>;
    getDocuments(query: GetDocumentsQuery, signal?: AbortSignal): Promise<ObjectDocumentList>;
    createDocument(request: CreateDocumentRequest, signal?: AbortSignal): Promise<CreateDocumentResponse>;
    getEmployee(employeeId: string, signal?: AbortSignal): Promise<Employee>;
    getEmployees(query: GetEmployeesWithActivePeriodQuery, signal?: AbortSignal): Promise<EmployeeList>;
    getRelative(employeeId: string, relativeId: string, signal?: AbortSignal): Promise<relativeDef>;
    getRelatives(employeeId: string, query?: GetRelativesQuery, signal?: AbortSignal): Promise<RelativeListDef>;
    mergeRelative(employeeId: string, query: MergeRelativeRequest, signal?: AbortSignal): Promise<MergeRelativeResponse>;
    patchRelative(employeeId: string, relativeId: string, operations: JsonPatchOperation[], signal?: AbortSignal): Promise<PatchRelativeResponse>;
    matchEmployee(query: MatchEmployeeRequest, signal?: AbortSignal): Promise<MatchEmployeeResponse>;
}
export {};
