"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropsBuilder = void 0;
exports.isExpressionPropValue = isExpressionPropValue;
const IterableUtils_1 = require("../../../../Common/IterableUtils");
const ConverterResult_1 = require("../../generators/markupGenerator/ConverterResults/ConverterResult");
const XmlNode_1 = require("../XmlParser/XmlNode");
const ModelPath_1 = require("../../../../Common/ModelPath/ModelPath");
function isExpressionPropValue(value) {
    return typeof value === "object" && value != undefined && "type" in value && value.type === "Expression";
}
class PropsBuilder {
    props = {};
    constructor(initialProps) {
        if (initialProps != undefined) {
            for (const [key, value] of (0, IterableUtils_1.iterateEntries)(initialProps)) {
                this.set(key, value);
            }
        }
    }
    set(name, value) {
        if (value === undefined) {
            return;
        }
        this.props[name] = value;
    }
    build() {
        return Object.entries(this.props).map(([name, value]) => {
            let serializedValue;
            if (isExpressionPropValue(value)) {
                serializedValue = `{${value.expression}}`;
            }
            else if (typeof value === "string") {
                serializedValue = `"${value.replace(/"/g, "&quot;")}"`;
            }
            else if (value instanceof XmlNode_1.ASTNode) {
                serializedValue = `{${value}}`;
            }
            else if (Array.isArray(value) || (0, ModelPath_1.isModelPath)(value) || typeof value === "object") {
                serializedValue = `{${this.objectToString(value)}}`;
            }
            else {
                serializedValue = `{${value}}`;
            }
            return `${name}=${serializedValue}`;
        });
    }
    objectToString(object) {
        if (object === undefined) {
            return "";
        }
        if (typeof object === "string") {
            return JSON.stringify(object);
        }
        if ((0, ModelPath_1.isModelPath)(object)) {
            if (object.isAbsolute()) {
                return `ap([${object
                    .getPathPartsAsArray()
                    .map(x => x === ModelPath_1.PathTokens.each ? "each" : x === ModelPath_1.PathTokens.current ? "current" : JSON.stringify(x))
                    .join(", ")}])`;
            }
            else {
                return `mp([${object
                    .getPathPartsAsArray()
                    .map(x => x === ModelPath_1.PathTokens.each ? "each" : x === ModelPath_1.PathTokens.current ? "current" : JSON.stringify(x))
                    .join(", ")}])`;
            }
        }
        if (typeof object === "number" || typeof object === "boolean") {
            return String(object);
        }
        if (object instanceof ConverterResult_1.ConverterResult) {
            return object.getRenderBody();
        }
        if (Array.isArray(object)) {
            return `[${object.map(x => this.objectToString(x)).join(", ")}]`;
        }
        if (typeof object === "object") {
            if (object === null) {
                return "";
            }
            if (isExpressionPropValue(object)) {
                return `${object.expression}`;
            }
            let result = "";
            for (const [key, value] of Object.entries(object)) {
                if (value instanceof XmlNode_1.ASTNode) {
                    result += `${key}: ${value},`;
                }
                else if (typeof value === "number" || typeof value === "boolean") {
                    result += `${key}: ${value},`;
                }
                else if (typeof value === "string") {
                    result += `${key}: ${JSON.stringify(value)},`;
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                }
                else if (isExpressionPropValue(value)) {
                    result += `${key}: ${value.expression},`;
                }
                else if (typeof value === "object") {
                    result += `${key}: ${this.objectToString(value)},`;
                }
                else if (value === null) {
                    result += `${key}: ${value},`;
                }
            }
            return `{${result}}`;
        }
        // eslint-disable-next-line @typescript-eslint/no-base-to-string
        throw new Error(`${object} должен быть объектом`);
    }
    toString() {
        return this.build().join(" ");
    }
    toStatementsArray() {
        return this.build();
    }
}
exports.PropsBuilder = PropsBuilder;
