import * as t from "@babel/types";
import { SugarNodeBase } from "../../generators/markupGenerator/Serializer/SugarSerializer";
import { ExpressionPropValue } from "../CodeGeneration/PropsBuilder";
import { CatchedError, ExtendedError } from "../../../../Common/Errors/ExtendedError";
export type AttributeValue = boolean | string | number | AttributeValueObject | AttributeValue[] | Function | ASTNode;
export interface AttributeValueObject {
    [x: string]: AttributeValue | undefined;
}
export interface XmlNodeParameters {
    attributes?: AttributeCollection;
    children?: readonly XmlNode[];
    name: string;
    parent?: XmlNode;
    source?: XmlNodeSourceWithDir;
    attributeSources?: AttributeSourceCollection;
}
export type AttributeCollection = Record<string, undefined | AttributeValue>;
export interface AttributeSourceCollection {
    [key: string]: undefined | XmlNodeSource;
}
export interface XmlNodeSourcePosition {
    line: number;
    column: number;
}
export interface XmlNodeSourceLocation {
    start: XmlNodeSourcePosition;
    end: XmlNodeSourcePosition;
}
export interface XmlNodeSource {
    sourceText: string;
    location: XmlNodeSourceLocation;
}
export interface XmlNodeSourceWithDir extends XmlNodeSource {
    currentDir: string;
}
export declare function getNodeSource(node: XmlNode): string;
export declare class XmlNode {
    readonly attributes: AttributeCollection;
    children: XmlNode[];
    name: string;
    parent: undefined | XmlNode;
    readonly source?: XmlNodeSourceWithDir;
    readonly attributeSources?: AttributeSourceCollection;
    constructor(createParams: XmlNodeParameters);
    clone(): XmlNode;
    getAllAttributeNames(): Iterable<string>;
    getChildren(childrenName?: string): readonly XmlNode[];
    getFirstChild(childName?: string): undefined | XmlNode;
    getRoot(): XmlNode;
    getParent(): undefined | XmlNode;
    getParentStrict(): XmlNode;
    getParentPageStrict(): XmlNode;
    getParentPageOrUndefined(): undefined | XmlNode;
    private findAncestor;
    setChildren(inputChildren: XmlNode[]): void;
    replaceChildren(inputChildren: XmlNode[]): void;
    setParent(parentNode: undefined | XmlNode): void;
    hasAttribute(attrName: string): boolean;
    attr(attrName: string, defaultValue?: string): string;
    getAttributeAsNumber(attrName: string): undefined | number;
    getAttributeAsNumber(attrName: string, defaultValue: number): number;
    getAttributeAsNumberStrict(attrName: string): number;
    getAttributeAsArray(attrName: string, type?: "number"): AttributeValue[];
    attrAsLengthUnit(attrName: string): undefined | number | string;
    attrAsBoolean(attrName: string): undefined | boolean;
    attrAsString(attrName: string): undefined | string;
    attrAsString(attrName: string, defaultValue: string): string;
    attrAsStringStrict(attrName: string): string;
    boolAttr(attrName: string, defaultValue?: boolean): boolean;
    setAttribute(attrName: undefined | string, value: undefined | AttributeValue): void;
    removeAttribute(attrName: string): void;
    attrAsStringLocalized(attrName: string): undefined | string | ExpressionPropValue;
}
export declare class ASTNode {
    private readonly node;
    constructor(node: t.Node);
    toString(): string;
}
export declare class SugarReadingError extends ExtendedError {
}
export declare class SugarNodeReadingError extends SugarReadingError {
    readonly sourceXmlNode: XmlNode;
    constructor(message: string, sourceXmlNode: XmlNode | SugarNodeBase, innerError?: CatchedError);
}
export declare class SugarAttributeReadError extends SugarReadingError {
    readonly sourceXmlNode: XmlNode;
    readonly attributeName: string;
    constructor(message: string, sourceXmlNode: XmlNode | SugarNodeBase, attributeName: string, innerError?: CatchedError);
}
