"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathTokensCache = void 0;
exports.extractPath = extractPath;
exports.extractTokens = extractTokens;
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const classicFunctionRegEx = /^\s*function\s*\(\s*([A-Za-z0-9_]+)\s*\)\s*\{\s*(?:(?:\"use strict\"|\'use strict\');?)?\s*return\s+\1\s*([\.\[].*?)?\s*;?\s*\}\s*$/;
const arrowFunctionRegEx = /^\s*\(?\s*([A-Za-z0-9_]+)\s*\)?\s*=>\s*\1\s*([\.\[].*?)?\s*$/;
function extractPath(lambdaSource) {
    const lambda = typeof lambdaSource === "string" ? lambdaSource : lambdaSource.toString();
    const match = classicFunctionRegEx.exec(lambda) || arrowFunctionRegEx.exec(lambda);
    if (match && match.length === 3) {
        const result = (match[2] || "").replace(/(^\.\s*)/g, "");
        const doubleQuotesMatch = /^\[\"([^\[\]\"]+)\"\]$/.exec(result);
        if (doubleQuotesMatch) {
            return doubleQuotesMatch[1] ?? (0, TypingUtils_1.reject)();
        }
        const singleQuotesMatch = /^\[\'([^\[\]\']+)\'\]$/.exec(result);
        if (singleQuotesMatch) {
            return singleQuotesMatch[1] ?? (0, TypingUtils_1.reject)();
        }
        return result;
    }
    throw new Error(`Not supported or invalid path: <${lambda}>`);
}
function extractTokens(path) {
    return path.split(/[\s\.\[\]]+/g).filter(x => x);
}
class PathTokensCache {
    cache = new Map();
    getOrAdd(lambdaPath) {
        const lambdaString = lambdaPath.toString();
        if (!this.cache.has(lambdaString)) {
            const path = extractPath(lambdaString);
            const tokens = extractTokens(path);
            this.cache.set(lambdaString, tokens);
        }
        return this.cache.get(lambdaString);
    }
    has(lambdaPath) {
        const lambdaString = lambdaPath.toString();
        return this.cache.has(lambdaString);
    }
    get size() {
        return this.cache.size;
    }
}
exports.PathTokensCache = PathTokensCache;
