"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LengthUnitValueParser = void 0;
const SugarValueConstraintError_1 = require("../../Errors/SugarValueConstraintError");
const DefaultWrapperValueParser_1 = require("./DefaultWrapperValueParser");
const RequiredWrapperValueParser_1 = require("./RequiredWrapperValueParser");
class LengthUnitValueParser {
    required = new RequiredWrapperValueParser_1.RequiredWrapperValueParser(this);
    default = (defaultValue) => new DefaultWrapperValueParser_1.DefaultWrapperValueParser(this, defaultValue);
    parseValue(value) {
        if (value == undefined) {
            return undefined;
        }
        if (typeof value === "number") {
            return value;
        }
        if (typeof value === "string") {
            if (/^[+-]?[0-9]+$/.test(value)) {
                return parseInt(value, 10);
            }
            if (/^[+-]?([0-9]+)?\.?[0-9]+(px|em|ex|%|in|cm|mm|pt|pc)$/.test(value)) {
                return value;
            }
        }
        throw new SugarValueConstraintError_1.SugarValueConstraintError(`Value cannot be conveterd to unit of length\n  Value: ${JSON.stringify(value)}`);
    }
}
exports.LengthUnitValueParser = LengthUnitValueParser;
