"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttributeValueSource = void 0;
const XmlNode_1 = require("../../../../../common/XmlParser/XmlNode");
const SugarSerializer_1 = require("../../SugarSerializer");
const ValueSourceUtils_1 = require("./ValueSourceUtils");
class AttributeValueSource {
    attrName;
    valueParser;
    constructor(attrName, valueParser) {
        this.attrName = attrName;
        this.valueParser = valueParser;
    }
    extractValue(sugarXmlNode, serializer, tracer) {
        try {
            const attributeNodes = sugarXmlNode
                .getChildren()
                .filter(x => SugarSerializer_1.SugarSerializer.isAttributeValueNode(x, sugarXmlNode))
                .filter(x => SugarSerializer_1.SugarSerializer.getAttributeNameFromValueNode(x, sugarXmlNode) === this.attrName.toLowerCase());
            const getAttributeValueFromSingleChildNodeWithoutMarkup = (childNode) => {
                if (!(0, ValueSourceUtils_1.isAllXmlNodesATextNodes)(childNode.getChildren())) {
                    throw new Error("Declaring this attribute via child element is not allowed!");
                }
                const result = (0, ValueSourceUtils_1.joinTextNodesViaSpaceAndDeserializeSugarNodes)(childNode.getChildren(), null);
                if (result.length > 1 || typeof result[0] !== "string") {
                    throw new Error("Unknown error: invalid reduced nodes count!");
                }
                return result[0];
            };
            const getAttributeValueAndTraceChildNode = () => {
                switch (attributeNodes.length) {
                    case 0:
                        return sugarXmlNode.attributes[this.attrName];
                    case 1: {
                        const childNode = attributeNodes[0];
                        if (childNode == undefined) {
                            throw new Error("Invalid data in markup!");
                        }
                        tracer(childNode);
                        return getAttributeValueFromSingleChildNodeWithoutMarkup(childNode);
                    }
                    default:
                        throw new Error("Declaring attribute via multiple child elements is not allowed!");
                }
            };
            tracer(this.attrName);
            return this.valueParser.parseValue(getAttributeValueAndTraceChildNode(), Object.prototype.hasOwnProperty.call(sugarXmlNode.attributes, this.attrName));
        }
        catch (e) {
            throw new XmlNode_1.SugarAttributeReadError(`Unable to read value of attribute '${this.attrName}'.`, sugarXmlNode, this.attrName, e);
        }
    }
}
exports.AttributeValueSource = AttributeValueSource;
