"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkupOrAttributeValueSource = void 0;
const XmlNode_1 = require("../../../../../common/XmlParser/XmlNode");
const SugarSerializer_1 = require("../../SugarSerializer");
const ValueSourceUtils_1 = require("./ValueSourceUtils");
class MarkupOrAttributeValueSource {
    attrName;
    valueParser;
    constructor(attrName, valueParser) {
        this.attrName = attrName;
        this.valueParser = valueParser;
    }
    extractValue(sugarXmlNode, serializer, tracer) {
        try {
            const attributeNodes = sugarXmlNode
                .getChildren()
                .filter(x => SugarSerializer_1.SugarSerializer.isAttributeValueNode(x, sugarXmlNode))
                .filter(x => SugarSerializer_1.SugarSerializer.getAttributeNameFromValueNode(x, sugarXmlNode) === this.attrName.toLowerCase());
            tracer(this.attrName);
            switch (attributeNodes.length) {
                case 0: {
                    return this.valueParser.parseValue(sugarXmlNode.attributes[this.attrName], Object.prototype.hasOwnProperty.call(sugarXmlNode.attributes, this.attrName));
                }
                case 1: {
                    const childNode = attributeNodes[0];
                    if (childNode == undefined) {
                        throw new Error("Invalid data in markup!");
                    }
                    if (childNode?.children == undefined) {
                        return undefined;
                    }
                    const reducedNodeCollection = (0, ValueSourceUtils_1.joinTextNodesViaSpaceAndDeserializeSugarNodes)(childNode.children, serializer).map(x => (typeof x === "string" ? this.valueParser.parseValue(x, true) : x));
                    if (reducedNodeCollection.length === 0) {
                        throw new Error("Empty child attribute tag blocks not allowed!");
                    }
                    tracer(childNode);
                    return reducedNodeCollection.length > 0 ? reducedNodeCollection : undefined;
                }
                default:
                    throw new Error("Declaring attribute via multiple child elements is not allowed!");
            }
        }
        catch (e) {
            throw new XmlNode_1.SugarAttributeReadError(`Unable to read value of attribute '${this.attrName}'.`, sugarXmlNode, this.attrName, e);
        }
    }
}
exports.MarkupOrAttributeValueSource = MarkupOrAttributeValueSource;
