"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SkipAttributeValueSource = void 0;
const DeprecationReason_1 = require("../../DeprecationReason");
class SkipAttributeValueSource {
    attrName;
    reason;
    constructor(attrName, reason) {
        this.reason = reason;
        this.attrName = attrName;
    }
    extractValue(sugarXmlNode, serializer, tracer) {
        tracer(this.attrName);
        if (!sugarXmlNode.attributes.hasOwnProperty(this.attrName)) {
            return undefined;
        }
        switch (this.reason) {
            case DeprecationReason_1.DeprecationReason.Removed:
                serializer.warnOnce(`${sugarXmlNode.name}.${this.attrName}`, `[WARN] Property ${sugarXmlNode.name}.${this.attrName} has been removed. Please remove usage of this property.`);
                break;
            case DeprecationReason_1.DeprecationReason.InvalidUsage:
                serializer.warnOnce(`${sugarXmlNode.name}.${this.attrName}`, `[WARN] Property ${sugarXmlNode.name}.${this.attrName} does not exists. Please remove usage of this property.`);
                break;
            case DeprecationReason_1.DeprecationReason.Typo:
                serializer.warnOnce(`${sugarXmlNode.name}.${this.attrName}`, `[WARN] Property ${sugarXmlNode.name}.${this.attrName} does not exists. Please remove usage of this property.`);
                break;
            case DeprecationReason_1.DeprecationReason.UntypedUsage:
                break;
            case DeprecationReason_1.DeprecationReason.InjectedUntypedByPreprocessor:
                break;
            default:
                break;
        }
        return undefined;
    }
}
exports.SkipAttributeValueSource = SkipAttributeValueSource;
