"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAllXmlNodesATextNodes = isAllXmlNodesATextNodes;
exports.joinTextNodesViaSpaceAndDeserializeSugarNodes = joinTextNodesViaSpaceAndDeserializeSugarNodes;
function isAllXmlNodesATextNodes(nodes) {
    return nodes.every(innerNode => innerNode.name === "#innertext" && typeof innerNode.attributes["#innertext"] === "string");
}
function joinTextNodesViaSpaceAndDeserializeSugarNodes(nodes, serializer) {
    const pushTextNodeToResultListIfNeeded = (context) => {
        if (context.intermediateText != undefined) {
            context.result.push(context.intermediateText);
            context.intermediateText = undefined;
        }
    };
    const reduceContext = nodes.reduce((context, xmlNode) => {
        if (xmlNode.name == "#innertext" && typeof xmlNode.attributes["#innertext"] === "string") {
            const elementValue = xmlNode.attributes["#innertext"];
            const joinedElementValues = (elementValue?.split(/\r?\n/) ?? [])
                .map(item => item.trim())
                .filter(item => item.length > 0)
                .join(" ");
            const requiredDelimiter = context.intermediateText == undefined ? "" : " ";
            context.intermediateText = (context.intermediateText ?? "") + requiredDelimiter + joinedElementValues;
        }
        else {
            if (serializer == undefined) {
                throw new Error("Sugar serializer is required for this operation!");
            }
            pushTextNodeToResultListIfNeeded(context);
            const sugarNode = serializer.deserializeFromSugarXmlNode(xmlNode);
            context.result.push(sugarNode);
        }
        return context;
    }, {
        result: [],
        intermediateText: undefined,
    });
    pushTextNodeToResultListIfNeeded(reduceContext);
    return reduceContext.result;
}
