"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SugarNodeMetadata = exports.SugarAttributesMixinMetadata = exports.PropertiesContainerMetadataBase = void 0;
const XmlNode_1 = require("../../../common/XmlParser/XmlNode");
class PropertiesContainerMetadataBase {
    propertyDeserializers = [];
    addPropertyDeserializer(propertyDeserializer) {
        this.propertyDeserializers.push(propertyDeserializer);
    }
    getPropertyDeserializers() {
        return this.propertyDeserializers;
    }
    fillFromSugarXmlNode(targetNode, sourceXmlNode, serializer, tracer) {
        for (const propertyDeserializer of this.propertyDeserializers) {
            propertyDeserializer.fillFromSugarXmlNode(targetNode, sourceXmlNode, serializer, tracer);
        }
    }
}
exports.PropertiesContainerMetadataBase = PropertiesContainerMetadataBase;
class SugarAttributesMixinMetadata extends PropertiesContainerMetadataBase {
}
exports.SugarAttributesMixinMetadata = SugarAttributesMixinMetadata;
class SugarNodeMetadata extends PropertiesContainerMetadataBase {
    get sugarNodeName() {
        return this.nodeName || "";
    }
    canDeserializeFunc;
    nodeName;
    documentationFilesRoot;
    documentationFilesContext;
    description;
    constructor(canDeserialize) {
        super();
        this.canDeserializeFunc = canDeserialize;
    }
    getNodeName() {
        return this.nodeName;
    }
    setNodeName(nodeName) {
        this.nodeName = nodeName;
    }
    canDeserialize(sugarXmlNode) {
        if (this.nodeName == undefined) {
            throw new Error("Node defintion does not contains sugarNode attribute");
        }
        if (sugarXmlNode.name === this.nodeName) {
            if (this.canDeserializeFunc != undefined) {
                return this.canDeserializeFunc(sugarXmlNode);
            }
            return true;
        }
        return false;
    }
    fillFromSugarXmlNode(targetNode, sourceXmlNode, serializer, tracer) {
        const usedAttributes = new Set();
        const usedChildren = new Set();
        const usedChildElementsTracer = (childElement) => {
            if (typeof childElement === "string") {
                usedAttributes.add(childElement);
            }
            else {
                usedChildren.add(childElement);
            }
        };
        super.fillFromSugarXmlNode(targetNode, sourceXmlNode, serializer, usedChildElementsTracer);
        for (const childElement of sourceXmlNode.getChildren()) {
            if (!usedChildren.has(childElement)) {
                if (serializer.options.unknownProperties === "error") {
                    throw new XmlNode_1.SugarAttributeReadError(`Unknown child element ${sourceXmlNode.name}.${childElement.name} (class ${targetNode.constructor.name})`, childElement, "");
                }
                else if (serializer.options.unknownProperties === "warn") {
                    // eslint-disable-next-line no-console
                    console.warn(`Unknown child element ${sourceXmlNode.name}.${childElement.name} (class ${targetNode.constructor.name})`);
                }
            }
        }
        for (const attributeName of sourceXmlNode.getAllAttributeNames()) {
            if (!usedAttributes.has(attributeName)) {
                if (serializer.options.unknownProperties === "error") {
                    throw new XmlNode_1.SugarAttributeReadError(`Unknown attribute ${sourceXmlNode.name}.${attributeName} (class ${targetNode.constructor.name})`, sourceXmlNode, attributeName);
                }
                else if (serializer.options.unknownProperties === "warn") {
                    // eslint-disable-next-line no-console
                    console.warn(`Unknown attribute ${sourceXmlNode.name}.${attributeName} (class ${targetNode.constructor.name})`);
                }
            }
        }
    }
}
exports.SugarNodeMetadata = SugarNodeMetadata;
