import { ExpressionPropValue } from "../../../common/CodeGeneration/PropsBuilder";
import { IRequireContext } from "../../../common/RequireContext/IRequireContext";
import { XmlNode } from "../../../common/XmlParser/XmlNode";
import { AbsoluteModelPath, ModelPath } from "../../../../../Common/ModelPath/ModelPath";
import { IDataBindingAttributes } from "../ElementProcessors/CommonNodeProperties/DataBindingMixinNode";
import { DeprecationReason } from "./DeprecationReason";
import { ArrayValueParser } from "./Properties/ValuesParsers/ArrayValueParser";
import { BooleanValueParser } from "./Properties/ValuesParsers/BooleanValueParser";
import { DelimitedStringArrayValueParser } from "./Properties/ValuesParsers/DelimitedStringArrayValueParser";
import { EnumValueParser } from "./Properties/ValuesParsers/EnumValueParser";
import { InlineStyleValueParser } from "./Properties/ValuesParsers/InlineStyleValueParser";
import { ISugarAttributeValueParser } from "./Properties/ValuesParsers/ISugarAttributeValueParser";
import { LengthUnitValueParser } from "./Properties/ValuesParsers/LengthUnitValueParser";
import { LocalizedStringValueParser } from "./Properties/ValuesParsers/LocalizedStringValueParser";
import { AnyExpressionValueParser, NumberValueParser } from "./Properties/ValuesParsers/NumberValueParser";
import { ObjectValueParser } from "./Properties/ValuesParsers/ObjectValueParser";
import { StringArrayValueParser, StringValueParser } from "./Properties/ValuesParsers/StringValueParser";
export declare function sugarNode(nodeName: string, description: string, docFilesContext?: IRequireContext, canDeserialize?: (node: XmlNode) => boolean): ClassDecorator;
export declare function attrMixinNode(description?: string, docFilesLocation?: string): ClassDecorator;
export declare function children(onlyTagName?: string | string[], nodeClasses?: NodeClass[], skipNodeNames?: string[]): PropertyDecorator;
export declare function singleChild(tagName?: string, nodeClasses?: NodeClass[], dec?: Exclude<string, "">): PropertyDecorator;
export declare function attr(attrName: string, valueParser: ISugarAttributeValueParser, dec: Exclude<string, "">): PropertyDecorator;
export declare function markupAttr(attrName: string, valueParser: ISugarAttributeValueParser, dec: Exclude<string, "">): PropertyDecorator;
export declare function deprecatedAttr(attrName: string, reason: DeprecationReason): PropertyDecorator;
export declare function attrMixin(nodeClass: NodeClass): PropertyDecorator;
export declare const attrType: {
    array: (innerValue: ISugarAttributeValueParser) => ArrayValueParser;
    stringArray: StringArrayValueParser;
    delimitedStringArray: DelimitedStringArrayValueParser;
    string: StringValueParser;
    typeName: StringValueParser;
    bindingPath: StringValueParser;
    customControl: StringValueParser;
    javascriptExpression: StringValueParser;
    pathToFile: StringValueParser;
    helperFunctionName: StringValueParser;
    picklistId: StringValueParser;
    modelPath: StringValueParser;
    inlineStyle: InlineStyleValueParser;
    localizedString: LocalizedStringValueParser;
    lengthUnit: LengthUnitValueParser;
    boolean: BooleanValueParser;
    number: NumberValueParser;
    objectLiteral: <T>() => AnyExpressionValueParser<T>;
    arrayLiteral: <T>() => AnyExpressionValueParser<T[]>;
    color: StringValueParser;
    enum: (...values: string[]) => EnumValueParser;
    object: ObjectValueParser;
};
export type LocalizedString = string | ExpressionPropValue;
export type MarkupNodeCollection = Array<string | SugarNodeBase>;
export declare function omitLocalization(param: string | LocalizedString): string;
export declare function omitLocalization(param: string | LocalizedString | undefined): string | undefined;
export declare function findSugarNodeBases(markupAttrValue: undefined | LocalizedString | MarkupNodeCollection): SugarNodeBase[];
export declare function localizedStringToJsx(str: LocalizedString): string;
export type HtmlLengthUnit = string | number;
export declare class SugarNodeBase {
    sourceXmlNode: XmlNode;
    parent: undefined | SugarNodeBase;
    getOwnPathForKCLang(): ModelPath | undefined;
    protected getOwnPath(): undefined | ModelPath;
    findParentOfTypeStrict<T extends SugarNodeBase>(nodeClass: NodeClassG<T>): T;
    findParentOfType<T extends SugarNodeBase>(nodeClass: NodeClassG<T>): undefined | T;
    getFullPath(): AbsoluteModelPath;
    getFullPathForChildren(): AbsoluteModelPath;
    protected isMultiple(): boolean;
    get xmlChildren(): readonly XmlNode[];
    getParentsRecursive(): Iterable<SugarNodeBase>;
}
export declare class UnknownSugarNode extends SugarNodeBase implements IDataBindingAttributes {
    readonly name: string;
    constructor(name: string);
    get path(): string | undefined;
    get optional(): boolean | undefined;
    get fetchfn(): undefined | string;
    get defaultValue(): undefined | string;
    get requisite(): undefined | boolean;
    get settings(): undefined | boolean;
    get disabled(): undefined | boolean;
    protected getOwnPath(): undefined | ModelPath;
}
export type NodeClass = typeof SugarNodeBase;
export type NodeClassG<T extends SugarNodeBase> = new () => T;
export interface AllSugarSerializerOptions {
    unknownProperties: "allowed" | "warn" | "error";
}
export type SugarSerializerOptions = Partial<AllSugarSerializerOptions>;
export declare class SugarSerializer {
    private readonly nodeClasses;
    readonly options: Readonly<AllSugarSerializerOptions>;
    constructor(options?: SugarSerializerOptions);
    static isAttributeValueNode(node: XmlNode, parentNode: XmlNode): boolean;
    static getAttributeNameFromValueNode(node: XmlNode, parentNode: XmlNode): string;
    registerNode(nodeClass: NodeClass): void;
    deserializeFromSugarXmlNode(xmlNode: XmlNode, customNodeClasses?: NodeClass[]): SugarNodeBase;
    warnOnce(key: string, message: string): void;
    private getNodeClassMetadata;
}
export declare function getNodeClass(node: SugarNodeBase): NodeClass;
