"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SugarSerializer = exports.UnknownSugarNode = exports.SugarNodeBase = exports.attrType = void 0;
exports.sugarNode = sugarNode;
exports.attrMixinNode = attrMixinNode;
exports.children = children;
exports.singleChild = singleChild;
exports.attr = attr;
exports.markupAttr = markupAttr;
exports.deprecatedAttr = deprecatedAttr;
exports.attrMixin = attrMixin;
exports.omitLocalization = omitLocalization;
exports.findSugarNodeBases = findSugarNodeBases;
exports.localizedStringToJsx = localizedStringToJsx;
exports.getNodeClass = getNodeClass;
const chalk_1 = __importDefault(require("chalk"));
const PropsBuilder_1 = require("../../../common/CodeGeneration/PropsBuilder");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const ClassMetadataAccess_1 = require("./ClassMetadataAccess");
const DeserializeTracer_1 = require("./Properties/DeserializeTracer");
const MixinPropertyDeserializer_1 = require("./Properties/MixinPropertyDeserializer");
const SinglePropertyDeserializer_1 = require("./Properties/SinglePropertyDeserializer");
const ArrayValueParser_1 = require("./Properties/ValuesParsers/ArrayValueParser");
const BooleanValueParser_1 = require("./Properties/ValuesParsers/BooleanValueParser");
const DelimitedStringArrayValueParser_1 = require("./Properties/ValuesParsers/DelimitedStringArrayValueParser");
const EnumValueParser_1 = require("./Properties/ValuesParsers/EnumValueParser");
const InlineStyleValueParser_1 = require("./Properties/ValuesParsers/InlineStyleValueParser");
const LengthUnitValueParser_1 = require("./Properties/ValuesParsers/LengthUnitValueParser");
const LocalizedStringValueParser_1 = require("./Properties/ValuesParsers/LocalizedStringValueParser");
const NumberValueParser_1 = require("./Properties/ValuesParsers/NumberValueParser");
const ObjectValueParser_1 = require("./Properties/ValuesParsers/ObjectValueParser");
const StringValueParser_1 = require("./Properties/ValuesParsers/StringValueParser");
const AttributeValueSource_1 = require("./Properties/ValuesSources/AttributeValueSource");
const ChildrenNodesValueSource_1 = require("./Properties/ValuesSources/ChildrenNodesValueSource");
const SingleChildValueSource_1 = require("./Properties/ValuesSources/SingleChildValueSource");
const SkipAttributeValueSource_1 = require("./Properties/ValuesSources/SkipAttributeValueSource");
const SugarNodeMetadata_1 = require("./SugarNodeMetadata");
const MarkupOrAttributeValueSource_1 = require("./Properties/ValuesSources/MarkupOrAttributeValueSource");
function sugarNode(nodeName, description, docFilesContext, canDeserialize) {
    return target => {
        const sugarNodeMetadata = new SugarNodeMetadata_1.SugarNodeMetadata(canDeserialize);
        sugarNodeMetadata.setNodeName(nodeName);
        if (docFilesContext != undefined && typeof docFilesContext !== "string") {
            sugarNodeMetadata.documentationFilesContext = docFilesContext;
        }
        sugarNodeMetadata.description = description;
        for (const propertyDeserializer of (0, ClassMetadataAccess_1.getPropertyDeserializers)(target)) {
            sugarNodeMetadata.addPropertyDeserializer(propertyDeserializer);
        }
        (0, ClassMetadataAccess_1.setSugarNodeMetadata)(target, sugarNodeMetadata);
    };
}
function attrMixinNode(description, docFilesLocation) {
    return target => {
        const sugarNodeMetadata = new SugarNodeMetadata_1.SugarAttributesMixinMetadata();
        for (const propertyDeserializer of (0, ClassMetadataAccess_1.getPropertyDeserializers)(target)) {
            sugarNodeMetadata.addPropertyDeserializer(propertyDeserializer);
        }
        (0, ClassMetadataAccess_1.setPropertiesContainerMetadata)(target, sugarNodeMetadata);
    };
}
function children(onlyTagName, nodeClasses = [], skipNodeNames = []) {
    return (targetClass, propName) => {
        const onlyTagNames = onlyTagName == undefined || Array.isArray(onlyTagName) ? onlyTagName : [onlyTagName];
        (0, ClassMetadataAccess_1.setPropertyDeserializer)(targetClass, propName, new SinglePropertyDeserializer_1.SinglePropertyDeserializer(propName.toString(), new ChildrenNodesValueSource_1.ChildrenNodesValueSource(onlyTagNames != undefined
            ? node => onlyTagNames.includes(node.name) && !skipNodeNames.includes(node.name)
            : node => !skipNodeNames.includes(node.name), nodeClasses)));
    };
}
function singleChild(tagName, nodeClasses = [], dec) {
    return (targetClass, propName) => {
        (0, ClassMetadataAccess_1.setPropertyDeserializer)(targetClass, propName, new SinglePropertyDeserializer_1.SinglePropertyDeserializer(propName.toString(), new SingleChildValueSource_1.SingleChildValueSource(tagName, nodeClasses), dec));
    };
}
function attr(attrName, valueParser, dec) {
    return (targetClass, propName) => {
        (0, ClassMetadataAccess_1.setPropertyDeserializer)(targetClass, propName, new SinglePropertyDeserializer_1.SinglePropertyDeserializer(propName.toString(), new AttributeValueSource_1.AttributeValueSource(attrName, valueParser), dec));
    };
}
function markupAttr(attrName, valueParser, dec) {
    return (targetClass, propName) => {
        (0, ClassMetadataAccess_1.setPropertyDeserializer)(targetClass, propName, new SinglePropertyDeserializer_1.SinglePropertyDeserializer(propName.toString(), new MarkupOrAttributeValueSource_1.MarkupOrAttributeValueSource(attrName, valueParser), dec));
    };
}
function deprecatedAttr(attrName, reason) {
    return (targetClass, propName) => {
        (0, ClassMetadataAccess_1.setPropertyDeserializer)(targetClass, propName, new SinglePropertyDeserializer_1.SinglePropertyDeserializer(propName.toString(), new SkipAttributeValueSource_1.SkipAttributeValueSource(attrName, reason), "DEPRECATED"));
    };
}
function attrMixin(nodeClass) {
    return (targetClass, propName) => {
        (0, ClassMetadataAccess_1.setPropertyDeserializer)(targetClass, propName, new MixinPropertyDeserializer_1.MixinPropertyDeserializer(propName.toString(), nodeClass));
    };
}
exports.attrType = {
    array: (innerValue) => new ArrayValueParser_1.ArrayValueParser(innerValue),
    stringArray: new StringValueParser_1.StringArrayValueParser(),
    delimitedStringArray: new DelimitedStringArrayValueParser_1.DelimitedStringArrayValueParser(),
    string: new StringValueParser_1.StringValueParser(),
    typeName: new StringValueParser_1.StringValueParser(),
    bindingPath: new StringValueParser_1.StringValueParser(),
    customControl: new StringValueParser_1.StringValueParser(),
    javascriptExpression: new StringValueParser_1.StringValueParser(),
    pathToFile: new StringValueParser_1.StringValueParser(),
    helperFunctionName: new StringValueParser_1.StringValueParser(),
    picklistId: new StringValueParser_1.StringValueParser(),
    modelPath: new StringValueParser_1.StringValueParser(),
    inlineStyle: new InlineStyleValueParser_1.InlineStyleValueParser(),
    localizedString: new LocalizedStringValueParser_1.LocalizedStringValueParser(),
    lengthUnit: new LengthUnitValueParser_1.LengthUnitValueParser(),
    boolean: new BooleanValueParser_1.BooleanValueParser(),
    number: new NumberValueParser_1.NumberValueParser(),
    objectLiteral: () => new NumberValueParser_1.AnyExpressionValueParser(),
    arrayLiteral: () => new NumberValueParser_1.AnyExpressionValueParser(),
    color: new StringValueParser_1.StringValueParser(),
    enum: (...values) => new EnumValueParser_1.EnumValueParser(values),
    object: new ObjectValueParser_1.ObjectValueParser(),
};
function omitLocalization(param) {
    if (param == undefined) {
        return undefined;
    }
    if (typeof param === "string") {
        return param;
    }
    const result = eval(`(t => {{ return ${param?.expression} }})(x => x)`);
    if (typeof result == "string") {
        return result;
    }
    // NOTE
    throw new Error("Unexpected omitLocalization result");
}
function findSugarNodeBases(markupAttrValue) {
    if (typeof markupAttrValue === "string" || markupAttrValue == undefined || (0, PropsBuilder_1.isExpressionPropValue)(markupAttrValue)) {
        return [];
    }
    return markupAttrValue.map(x => (typeof x === "string" ? undefined : x)).filter(TypingUtils_1.isNotNullOrUndefined);
}
function localizedStringToJsx(str) {
    if (typeof str === "string") {
        return str;
    }
    return `\{${str.expression}\}`;
}
class SugarNodeBase {
    sourceXmlNode;
    parent;
    getOwnPathForKCLang() {
        return this.getOwnPath();
    }
    getOwnPath() {
        return undefined;
    }
    findParentOfTypeStrict(nodeClass) {
        if (this.parent == undefined) {
            throw new Error(`Unable to find parent of class ${nodeClass.name}`);
        }
        if (this.parent instanceof nodeClass) {
            return this.parent;
        }
        return this.parent.findParentOfTypeStrict(nodeClass);
    }
    findParentOfType(nodeClass) {
        if (this.parent instanceof nodeClass) {
            return this.parent;
        }
        return this.parent?.findParentOfType(nodeClass);
    }
    getFullPath() {
        if (this.parent == undefined) {
            const ownPath = this.getOwnPath();
            if (ownPath != undefined) {
                return (0, ModelPath_1.emptyAbsoluteModelPath)().joinWith(ownPath).normalize();
            }
            return (0, ModelPath_1.emptyAbsoluteModelPath)();
        }
        else {
            const ownPath = this.getOwnPath();
            if (ownPath != undefined) {
                return this.parent.getFullPathForChildren().joinWith(ownPath).normalize();
            }
            return this.parent.getFullPathForChildren();
        }
    }
    getFullPathForChildren() {
        if (this.isMultiple()) {
            return this.getFullPath().joinWith(ModelPath_1.PathTokens.current).normalize();
        }
        else {
            return this.getFullPath();
        }
    }
    isMultiple() {
        return false;
    }
    get xmlChildren() {
        return this.sourceXmlNode.getChildren();
    }
    *getParentsRecursive() {
        if (this.parent != undefined) {
            yield this.parent;
            yield* this.parent.getParentsRecursive();
        }
    }
}
exports.SugarNodeBase = SugarNodeBase;
class UnknownSugarNode extends SugarNodeBase {
    name;
    constructor(name) {
        super();
        this.name = name;
    }
    get path() {
        return this.sourceXmlNode.attrAsString("path");
    }
    get optional() {
        return this.sourceXmlNode.attrAsBoolean("optional");
    }
    get fetchfn() {
        return this.sourceXmlNode.attrAsString("fetchfn");
    }
    get defaultValue() {
        return this.sourceXmlNode.attrAsString("defaultValue");
    }
    get requisite() {
        return this.sourceXmlNode.attrAsBoolean("requisite");
    }
    get settings() {
        return this.sourceXmlNode.attrAsBoolean("settings");
    }
    get disabled() {
        return this.sourceXmlNode.attrAsBoolean("disabled");
    }
    getOwnPath() {
        return this.path != undefined ? (0, ModelPath_1.createFromMask)(this.path, "auto", ModelPath_1.PathTokens.each) : undefined;
    }
}
exports.UnknownSugarNode = UnknownSugarNode;
const defaultOptions = {
    unknownProperties: "allowed",
};
const warnOnceKeys = new Set();
class SugarSerializer {
    nodeClasses = [];
    options;
    constructor(options) {
        this.options = {
            ...defaultOptions,
            ...(options ?? {}),
        };
    }
    static isAttributeValueNode(node, parentNode) {
        return node.name.startsWith(parentNode.name + ".");
    }
    static getAttributeNameFromValueNode(node, parentNode) {
        if (!this.isAttributeValueNode(node, parentNode)) {
            throw new Error(`${node.name} is not attribute value node`);
        }
        return node.name.split(".")[1] ?? (0, TypingUtils_1.reject)();
    }
    registerNode(nodeClass) {
        this.nodeClasses.push(nodeClass);
    }
    deserializeFromSugarXmlNode(xmlNode, customNodeClasses = []) {
        const nodeClasses = [...customNodeClasses, ...this.nodeClasses];
        for (const nodeClass of nodeClasses) {
            const metadata = this.getNodeClassMetadata(nodeClass);
            if (metadata.canDeserialize(xmlNode)) {
                const result = new nodeClass();
                result.sourceXmlNode = xmlNode;
                metadata.fillFromSugarXmlNode(result, xmlNode, this, DeserializeTracer_1.nullDeserializeTracer);
                return result;
            }
        }
        const unknownResult = new UnknownSugarNode(xmlNode.name);
        unknownResult.sourceXmlNode = xmlNode;
        return unknownResult;
    }
    warnOnce(key, message) {
        if (warnOnceKeys.has(key)) {
            return;
        }
        warnOnceKeys.add(key);
        // eslint-disable-next-line no-console
        console.warn(chalk_1.default.yellow(message));
    }
    getNodeClassMetadata(nodeClass) {
        return (0, ClassMetadataAccess_1.getSugarNodeMetadata)(nodeClass);
    }
}
exports.SugarSerializer = SugarSerializer;
function getNodeClass(node) {
    // @ts-ignore
    return node.constructor;
}
