import { AutoValidationLevels } from "../../../../../Common/AutoValidationUtils";
import { LocalizedString, SugarNodeBase } from "../../markupGenerator/Serializer/SugarSerializer";
export declare class TypeCheckNodeBase extends SugarNodeBase {
    description?: LocalizedString;
    checkType?: "error" | "warning";
    force?: boolean;
}
export type TypeCheckNodeCLass<T extends TypeCheckNodeBase> = new () => T;
export declare class PatternTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    requiredDescription?: never;
    value: string;
}
export declare class EnumerationTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    nameProp?: string;
    value: string;
}
export declare class AttachmentsTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    extentions?: string[];
    maxSize?: number;
    fnsConvertFileExtentions?: string[];
    count?: number;
    totalSize?: number;
    countExceededDescription?: LocalizedString;
}
export declare class LengthTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    value: number;
}
export declare class MinlengthTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    value: number;
}
export declare class MaxlengthTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    value: number;
}
export declare class TotaldigitsTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    value: number;
}
export declare class IntegerdigitsTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    value: number;
}
export declare class ValueEqlAutoValueTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    level: AutoValidationLevels;
    condition?: string;
}
export declare class FractiondigitsTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    value: number;
}
export declare class DigestcheckTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    gId?: string;
    keyColumnName?: string;
}
export declare class MininclusiveTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    value: number;
}
export declare class MaxinclusiveTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    value: number;
}
export declare class CustomValidationTypeCheckNode extends TypeCheckNodeBase {
    name: string;
    value: string;
}
export declare class ForcedTypeChecksNode extends SugarNodeBase {
    name: string;
    children: Array<CustomValidationTypeCheckNode | TypeCheckNode>;
}
export type TypeBase = "string" | "pattern" | "integer" | "decimal" | "boolean" | "date" | "gYear" | "gMonth" | "valueEqlAutoValue";
export declare class TypeNode extends SugarNodeBase {
    base?: TypeBase;
    name?: string;
    requireDescription?: never;
    requiredDescription?: LocalizedString;
    description?: LocalizedString;
    isCaseIndependentValid?: boolean;
    checkType?: "error" | "warning";
    children: Array<CustomValidationTypeCheckNode | TypeCheckNode | ForcedTypeChecksNode>;
    getValidations<T extends TypeCheckNode>(typeCheckClass: TypeCheckNodeCLass<T>): T[];
    flattenTypeCheckNodes(): TypeCheckNode[];
    isNodeForced(node: TypeCheckNode): boolean;
}
export declare function ofType<T>(cl: new (...args: unknown[]) => T): (x: unknown) => x is T;
export type TypeCheckNode = PatternTypeCheckNode | EnumerationTypeCheckNode | AttachmentsTypeCheckNode | LengthTypeCheckNode | MinlengthTypeCheckNode | MaxlengthTypeCheckNode | TotaldigitsTypeCheckNode | IntegerdigitsTypeCheckNode | FractiondigitsTypeCheckNode | DigestcheckTypeCheckNode | MininclusiveTypeCheckNode | MaxinclusiveTypeCheckNode;
