import { FileChooser, Page } from "playwright-core";
import { CandyEditorUrlOptions } from "../../CandyApi/CandyEditorUrlOptions";
import { CandyBrowserSideObjectHandle } from "./CandyBrowserSideObjectHandle";
import { CandyWebElementHandle } from "./CandyWebElementHandle";
export interface DownloadedFilesInfo {
    content: Buffer;
    contentType: undefined | string;
    filename: string;
}
export declare abstract class CandyBrowserPage {
    abstract readonly page: Page;
    pressCtrlA(): Promise<void>;
    pressHome(): Promise<void>;
    pressEnd(): Promise<void>;
    pressManyTimes(key: string, iterations: number, timeout?: number): Promise<void>;
    pressCtrlV(): Promise<void>;
    pressCtrlC(): Promise<void>;
    pressCtrlZ(): Promise<void>;
    abstract down(key: string): Promise<void>;
    abstract press(key: string): Promise<void>;
    abstract up(key: string): Promise<void>;
    abstract goto(url: string): Promise<void>;
    abstract refresh(): Promise<void>;
    abstract refreshAndSkipNavigationError(): Promise<void>;
    abstract scrollTo(x: number, y: number): Promise<void>;
    abstract changeZoom(zoomValue: string): Promise<void>;
    abstract release(): Promise<void>;
    abstract waitForDownloadEvent(): Promise<FileChooser>;
    abstract expectDownloadedFile(downloadCallback: () => Promise<void>): Promise<{
        content: Buffer;
        filename: string;
    }>;
    abstract evaluateScript(script: string): Promise<void>;
    abstract setClipBoardData(data: string): Promise<void>;
    abstract setUrlOptions(opts: CandyEditorUrlOptions): Promise<void>;
    abstract evaluatePromisedExpression(script: string): Promise<void>;
    abstract evaluateExpressionAsHandle(script: string): Promise<CandyBrowserSideObjectHandle>;
    abstract evaluateFunctionAsHandle<TR, TA extends Array<ExecuteScriptArgs<any>>>(script: (args: ExtractHandleTypes<TA>) => TR, ...args: TA): Promise<CandyBrowserSideObjectHandle1<TR>>;
    abstract url(): Promise<string>;
    abstract takePageScreenshot(): Promise<Buffer>;
    abstract acceptNextDialog(): Promise<void>;
    abstract moveMouse(point: {
        x: number;
        y: number;
    }): Promise<void>;
    abstract getAllConsoleMessage(): string[];
}
export interface CandyBrowserSideObjectHandle1<T> extends CandyBrowserSideObjectHandle {
    jsonValue(): Promise<T>;
}
export type ExecuteScriptArgs<T> = CandyBrowserSideObjectHandle1<T> | CandyWebElementHandle | T;
export type ExtractHandleTypes<T> = {
    [P in keyof T]: T[P] extends CandyBrowserSideObjectHandle1<infer U> ? U : T[P] extends CandyWebElementHandle ? HTMLElement : T[P];
};
export interface CandyCookie {
    name: string;
    value: string;
    domain?: string;
}
