"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlaywrightCandyBrowserPage = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = require("fs-extra");
const CandyBrowserPage_1 = require("../Candy/CandyBrowserPage");
const AfterPageLoadActions_1 = require("../AfterPageLoadActions");
const CommonUtils_1 = require("../../PageObjects/CommonUtils");
const PlaywrightCandyBrowserSideObjectHandle_1 = require("./PlaywrightCandyBrowserSideObjectHandle");
class PlaywrightCandyBrowserPage extends CandyBrowserPage_1.CandyBrowserPage {
    page;
    acceptNextDialogFlag = false;
    constructor(page) {
        super();
        this.page = page;
        this.page.on("dialog", async (dialog) => {
            if (this.acceptNextDialogFlag) {
                await dialog.accept();
            }
            else {
                await dialog.dismiss();
            }
        });
    }
    async setUrlOptions(opts) {
        const url = new URL(this.page.url());
        Object.entries(opts).forEach(([key, value]) => {
            url.searchParams.append(key, value);
        });
        /** переодичсески стреляет попап на сохранение данных при перезагрузке */
        await this.page.evaluate(`__CANDY_FORMS_TESTING_MODE__FormObject().publicForm.save()`);
        await this.page.goto(url.toString());
        await (0, AfterPageLoadActions_1.onAfterPageLoad)(this.page);
    }
    async acceptNextDialog() {
        this.acceptNextDialogFlag = true;
    }
    async discardNextDialog() {
        this.acceptNextDialogFlag = false;
    }
    getAllConsoleMessage() {
        return [];
    }
    async moveMouse({ x, y }) {
        await this.page.mouse.move(x, y);
    }
    async goto(url) {
        await this.page.goto(url);
    }
    async refresh() {
        this.acceptNextDialogFlag = true;
        await this.page.reload();
        await (0, AfterPageLoadActions_1.onAfterPageLoad)(this.page);
    }
    async refreshAndSkipNavigationError() {
        try {
            await this.page.reload({ timeout: 10000 });
        }
        catch {
            // just skip errors
        }
    }
    async release() {
        await this.page.close();
    }
    async waitForDownloadEvent() {
        const fileChooserPromise = this.page.waitForEvent("filechooser");
        return fileChooserPromise;
    }
    async expectDownloadedFile(downloadCallback) {
        const downloadPromise = this.page.waitForEvent("download");
        await downloadCallback();
        const download = await downloadPromise;
        // Wait for the download process to complete and save the downloaded file somewhere.
        const fileName = download.suggestedFilename();
        const filePath = path_1.default.resolve(__dirname, "..", "..", "..", "download", fileName);
        await download.saveAs(filePath);
        const fileContent = (0, fs_extra_1.readFileSync)(filePath);
        try {
            (0, fs_extra_1.statSync)(filePath);
            (0, fs_extra_1.unlinkSync)(filePath);
        }
        catch (e) {
            //nope
        }
        return { content: fileContent, filename: fileName };
    }
    async evaluateScript(script) {
        await this.page.evaluate(script);
    }
    async setClipBoardData(data) {
        // @ts-ignore
        await this.page.evaluate((data1) => {
            const input = document.createElement("textarea");
            input.innerHTML = data1;
            document.body.appendChild(input);
            input.select();
            const result = document.execCommand("copy");
            document.body.removeChild(input);
            return result;
        }, data);
    }
    async evaluatePromisedExpression(script) {
        await this.page.evaluate(script);
    }
    async evaluateExpressionAsHandle(script) {
        const jsHandle = await this.page.evaluateHandle(script);
        return new PlaywrightCandyBrowserSideObjectHandle_1.PlaywrightCandyBrowserSideObjectHandle(jsHandle);
    }
    // @ts-ignore
    async evaluateFunctionAsHandle(script, ...args) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        const arrayArgs = [args].flat().map(x => x.internalHandle ?? x.elementHandle ?? x);
        const handle = await this.page.evaluateHandle(
        // @ts-ignore
        script, arrayArgs);
        return new PlaywrightCandyBrowserSideObjectHandle_1.PlaywrightCandyBrowserSideObjectHandle(handle);
    }
    takePageScreenshot() {
        // @ts-ignore
        return this.page.screenshot({ type: "png", encoding: "binary" });
    }
    down(key) {
        return this.page.keyboard.down(key);
    }
    press(key) {
        return this.page.keyboard.press(key);
    }
    up(key) {
        return this.page.keyboard.up(key);
    }
    async url() {
        return this.page.url();
    }
    async changeZoom(_zoomValue) {
        const zoomStyle = `document.body.style.zoom=${_zoomValue}`;
        await this.evaluateScript(zoomStyle);
    }
    async scrollTo(x, y) {
        await this.page.mouse.wheel(x, y);
        await (0, CommonUtils_1.delay)(20);
    }
}
exports.PlaywrightCandyBrowserPage = PlaywrightCandyBrowserPage;
