import { AuthInfo, IKonturApi } from "../Core/Api/KonturApi";
import { ICandyApiClient } from "../../../../Common/CandyApiClient/ICandyApiClient";
import { DraftFullId } from "../../../../Common/CandyApiClient/Types/DraftFullId";
import { ServerInnerResponse } from "../../../../Common/CandyApiClient/Types/GetInner/ServerInnerResponse";
import { CreateDraftResponse } from "../../../../Common/CandyApiClient/Types/CreateDraft/CreateDraftResponse";
import { ServerChangeSet } from "../../../../Common/CandyApiClient/Types/ChangeSet/ServerChangeSet";
import { ClientRequisites } from "../Core/RequisitesBuilding/ClientRequisites";
import { ClientInfo } from "../Core/RequisitesBuilding/ClientInfo";
export interface Outer {
    [key: string]: number | string | boolean;
}
export interface Inner {
    [key: string]: number | string | boolean;
}
export interface Presentations {
    outer?: Outer;
    inner?: Inner;
    clientRequisites?: ClientRequisites;
    formsClientInfo?: ClientInfo;
    fuf?: Buffer;
}
export interface DraftInfo {
    ns: string;
    draftId: string;
}
export interface CandyServiceTestContextOptions {
    domain: string;
    resourceZone?: string;
}
export declare class CandyServiceTestContext {
    readonly konturApi: IKonturApi;
    readonly candyApi: ICandyApiClient;
    readonly testUserSid: string;
    readonly externSession?: string;
    constructor(konturApi: IKonturApi, candyApi: ICandyApiClient, testUserAuthInfo: AuthInfo);
    static initialize(options?: CandyServiceTestContextOptions): Promise<CandyServiceTestContext>;
    static ensureAliveApi(candyApi: ICandyApiClient): Promise<void>;
    downloadFile(url: string): Promise<Buffer>;
    getInner(draftFullId: DraftFullId): Promise<ServerInnerResponse>;
    createDefaultDraft(gfv: string): Promise<CreateDraftResponse>;
    createDraftFromFuf(fufContent: Buffer): Promise<CreateDraftResponse>;
    setPresentations(draftInfo: DraftInfo, presentations: Presentations): Promise<void>;
    sendChangeSet(draftInfo: DraftInfo, changeSet: ServerChangeSet): Promise<void>;
    private getCookie;
}
