"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.forEachTestFixture = void 0;
exports.stablePagePaddings = stablePagePaddings;
const test_1 = require("@playwright/test");
const AfterPageLoadActions_1 = require("../BrowserWrapper/AfterPageLoadActions");
const TypingUtils_1 = require("../../../../Common/TypingUtils");
const draftInfoFixture_1 = require("./draftInfoFixture");
const pageOptionsFixture_1 = require("./pageOptionsFixture");
exports.forEachTestFixture = (0, test_1.mergeTests)(draftInfoFixture_1.draftInfoFixture, pageOptionsFixture_1.pageOptionsFixture).extend({
    forEachTest: [
        // @ts-ignore мини хак для получения pageObject
        async ({ page, draftInfo, pageOptions, pageObject }, use) => {
            await gotoEditorPage(page, draftInfo.editorUrl ?? (0, TypingUtils_1.reject)("gfv in pageOptions is not defined"), pageOptions);
            await setFakeRequisites(page, draftInfo, pageOptions);
            await (0, AfterPageLoadActions_1.onAfterPageLoad)(page);
            pageOptions.withNavigationPanel || (await hideNavigationPanel(pageObject));
            await use();
        },
        { auto: true, timeout: 3 * 60 * 1000 },
    ],
});
async function setFakeRequisites(page, draftInfo, pageOptions) {
    if (pageOptions.fakeRequisites) {
        await page.evaluateHandle(({ draftId, requisites }) => {
            sessionStorage.setItem("FakeRequisites." + draftId, requisites);
        }, { draftId: draftInfo.draftId, requisites: JSON.stringify(pageOptions.fakeRequisites) });
    }
    if (pageOptions.fakeErrorLog) {
        const fullErrorLog = { errors: [], warnings: [], ...pageOptions.fakeErrorLog };
        await page.evaluateHandle(({ draftId, errorLog }) => {
            sessionStorage.setItem("FakeErrorLog." + draftId, errorLog);
        }, { draftId: draftInfo.draftId, errorLog: JSON.stringify(fullErrorLog) });
    }
}
async function stablePagePaddings(page) {
    const size = page.viewportSize();
    await page.setViewportSize({ width: 400, height: 400 });
    await page.mouse.wheel(400, 400);
    await page.mouse.wheel(-400, -400);
    await page.setViewportSize({ width: size?.width ?? 1280, height: size?.height ?? 720 });
    await page.mouse.wheel(400, 400);
    await page.mouse.wheel(-400, -400);
}
async function hideNavigationPanel(pageObject) {
    await pageObject.navigationPanel.hide();
    await pageObject.showMenuButton.hide();
}
async function gotoEditorPage(page, editorUrl, pageOptions) {
    const pageUrl = new URL(editorUrl);
    pageUrl.searchParams.append("debug", "true");
    pageOptions?.urlOptions &&
        Object.entries(pageOptions.urlOptions).forEach(([key, value]) => {
            const boolToString = value === true ? "true" : value === false ? "false" : value;
            pageUrl.searchParams.append(key, boolToString);
        });
    pageOptions.withStaffService && pageUrl.searchParams.append("employerId", "0f184d52-ab45-4792-870e-25cb3a91a97c");
    await page.goto(pageUrl.toString());
}
