"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CandyPageWithNavigationBase = void 0;
const uuid_1 = require("uuid");
const FluentControlFactory_1 = require("../PageObjects/Bases/ControlDeclaration/FluentControlFactory");
const NavigationPanel_1 = require("../PageObjects/Controls/NavigationPanel");
const ControlsBase_1 = require("../PageObjects/Bases/ControlsBase");
const CommonUtils_1 = require("../PageObjects/CommonUtils");
const TypingUtils_1 = require("../../../../Common/TypingUtils");
const Fixtures_1 = require("../Fixtures");
const DeleteUnitModal_1 = require("../PageObjects/Controls/DeleteUnitModal");
const ClearUnitModal_1 = require("../PageObjects/Controls/ClearUnitModal");
const ModelPath_1 = require("../../../../Common/ModelPath/ModelPath");
const AfterPageLoadActions_1 = require("../BrowserWrapper/AfterPageLoadActions");
const PageBase_1 = require("./PageBase");
class CandyPageWithNavigationBase extends PageBase_1.PageBase {
    showMenuButton = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("ButtonMenu").in(this);
    pinMenuButton = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("PinContainer").in(this);
    navigationPanel = (0, FluentControlFactory_1.control)(NavigationPanel_1.NavigationPanel).asIs().in(this);
    longOperationModal = ControlsBase_1.ControlBase.as("LongOperationModal").in(this);
    tabsWrapper = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("PanelTabs").in(this);
    tabs = ControlsBase_1.ControlBase.as("Tab").listIn(this.tabsWrapper);
    // private readonly fufChecker = new FufChecker(this.rootPageWrapper);
    deleteModal = (0, FluentControlFactory_1.control)(DeleteUnitModal_1.DeleteUnitModal).as("UnitRemoveConfirmModal").in(this);
    clearModal = (0, FluentControlFactory_1.control)(ClearUnitModal_1.ClearUnitModal).as("UnitClearConfirmModal").in(this);
    async suspendNextUpload() {
        await this.rootPage.evaluateScript("__CANDY_FORMS_TESTING_MODE__SuspendNextUpload()");
    }
    async resumeLastSuspendedUpload() {
        await this.rootPage.evaluateScript("__CANDY_FORMS_TESTING_MODE__ResumeLastSuspendedUpload()");
    }
    async setFakeErrorLog(errorLog, draftId) {
        if ((0, TypingUtils_1.isNullOrEmpty)(draftId)) {
            throw new Error("Cannot set error log for page without draftInfo");
        }
        await this.rootPage.evaluateFunctionAsHandle(([id, errorLogValue]) => {
            window.__CANDY_FORMS_TESTING_MODE__SetFakeErrorLogForDraft?.(id, errorLogValue);
        }, draftId, errorLog);
    }
    //
    async togglePinNavigationPanel(isVisible) {
        if (isVisible != undefined) {
            if (isVisible === false && (await this.pinMenuButton.isInViewport)) {
                await this.pinMenuButton.hover();
                await this.pinMenuButton.click();
                try {
                    await Fixtures_1.expect.poll(() => this.navigationPanel.isInViewport, { timeout: 3000 }).toBeFalsy();
                }
                catch (e) {
                    if (await this.pinMenuButton.isInViewport) {
                        await this.pinMenuButton.click();
                    }
                    await Fixtures_1.expect.poll(() => this.navigationPanel.isInViewport, { timeout: 3000 }).toBeFalsy();
                }
            }
            else if (isVisible && !(await this.pinMenuButton.isInViewport)) {
                await this.showMenuButton.hover();
                await this.pinMenuButton.click();
                await (0, CommonUtils_1.delay)(500);
            }
            return;
        }
        await this.showMenuButton.hover();
        await this.pinMenuButton.click();
        await (0, CommonUtils_1.delay)(500);
    }
    async expandNavigationPanel() {
        if (!(await this.showMenuButton.isPresent())) {
            await this.showMenuButton.show();
            await this.showMenuButton.hover();
        }
    }
    async openNavigationPanel() {
        await this.showMenuButton.show();
        await this.navigationPanel.show();
        await Fixtures_1.expect.poll(() => this.navigationPanel.isPresent(), { timeout: 10000 }).toBeTruthy();
    }
    async closeNavigationPanel() {
        await this.hover();
        await this.navigationPanel.hide();
        await this.showMenuButton.hide();
        // await delay(20);
        //await expect.poll(() => this.navigationPanel.formSaveStatus.isPresent(), { timeout: 10000 }).toBeFalsy();
    }
    async collapseNavigationPanel() {
        await this.hover();
        await Fixtures_1.expect.poll(() => this.showMenuButton.text, { timeout: 10000 }).toContain("Меню");
    }
    async clearForm() {
        try {
            await this.rootPage.acceptNextDialog();
            await this.navigationPanel.actionList.clickClearFormButton();
        }
        catch (e) {
            await this.navigationPanel.actionList.clickClearFormButton();
            // await this.rootPageWrapper.acceptAnyAlertOrConfirm();
        }
        await (0, CommonUtils_1.delay)(2000);
        await (0, AfterPageLoadActions_1.onAfterPageLoad)(this.rootPage.page);
    }
    async runCheck(isFake) {
        await this.navigationPanel.checkButton.click();
        if (!isFake) {
            await Fixtures_1.expect
                .poll(() => this.navigationPanel.embeddedLog.errorLogHeader.isPresent(), {
                timeout: 60000,
                intervals: [25],
            })
                .toBeTruthy();
            await (0, CommonUtils_1.delay)(1500);
        }
        return this.navigationPanel.embeddedLog;
    }
    async prepareBrowserForDownload() {
        await this.expandNavigationPanel();
        if (await this.tabsWrapper.isPresent()) {
            await this.tabs.itemAt(0).click();
        }
        await this.navigationPanel.ensureAllDataSaved();
        return async () => {
            await this.navigationPanel.actionList.clickDownloadButton();
            await this.collapseNavigationPanel();
        };
    }
    async getImageFromFuf() {
        await this.expandNavigationPanel();
        await this.navigationPanel.ensureAllDataSaved();
        return async () => {
            await this.navigationPanel.actionList.clickDownloadButton();
            await this.collapseNavigationPanel();
        };
    }
    async ensureAllDataSaved() {
        await this.expandNavigationPanel();
        await this.navigationPanel.ensureAllDataSaved();
        await this.collapseNavigationPanel();
    }
    async ensureCalculations() {
        const pendingTask = this.rootPage.evaluateFunctionAsHandle(async () => {
            // @ts-ignore
            const form = window.__CANDY_FORMS_TESTING_MODE__FormObject();
            if (form != undefined) {
                await form.waitForPendingCalculations();
            }
        });
        await Promise.race([pendingTask, (0, CommonUtils_1.delay)(30000)]);
    }
    async ensureAllDataSavedWithPublicForm() {
        await this.rootPage.evaluatePromisedExpression(`__CANDY_FORMS_TESTING_MODE__FormObject().publicForm.save()`);
        await Fixtures_1.expect
            .poll(() => this.navigationPanel.formSaveStatus.text, { timeout: 10000 })
            .toContain("Все изменения сохранены");
    }
    async refreshWithNormalization() {
        await this.setViewModelField((0, ModelPath_1.createAbsolute)("/Root"), "resourcesHash", (0, uuid_1.v4)());
        await this.rootPage.acceptNextDialog();
        await this.ensureAllDataSaved();
        await this.rootPage.refresh();
    }
}
exports.CandyPageWithNavigationBase = CandyPageWithNavigationBase;
