"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FufChecker = void 0;
const adm_zip_1 = __importDefault(require("adm-zip"));
const iconv_lite_1 = __importDefault(require("iconv-lite"));
const xml2js_1 = require("xml2js");
const Fixtures_1 = require("../Fixtures");
class FufChecker {
    rootPageWrapper;
    constructor(rootPageWrapper) {
        this.rootPageWrapper = rootPageWrapper;
    }
    async checkFuf(candyPage, expectedFufContentPart, forceEncoding) {
        const downloadCallback = await candyPage.prepareBrowserForDownload();
        const { content, filename } = await this.rootPageWrapper.expectDownloadedFile(downloadCallback);
        let fufContent = "";
        if (filename.endsWith(".zip")) {
            const zip = new adm_zip_1.default(content);
            zip.getEntries().forEach(entry => {
                if (entry.entryName.endsWith(".xml")) {
                    const file = zip.readFile(entry);
                    if (file) {
                        fufContent = this.bufferWithXmlToString(file, forceEncoding);
                    }
                }
            });
        }
        else {
            fufContent = this.bufferWithXmlToString(content, forceEncoding);
        }
        const fufAsJson = await this.fufToJson(fufContent);
        if (typeof expectedFufContentPart === "object") {
            (0, Fixtures_1.expect)(fufAsJson).toMatchObject(expectedFufContentPart);
        }
        else {
            // @ts-ignore
            await expectedFufContentPart(fufAsJson);
        }
    }
    // public async getImageFromFuf(expectedFufContentPart: (image: Buffer) => Promise<void> | void): Promise<void> {
    //     const { content } = await this.rootPageWrapper.expectDownloadedFile(x => x.filename.endsWith(".zip"));
    //     const zip = new AdmZip(content);
    //
    //     for (const entry of zip.getEntries()) {
    //         if (entry.entryName.endsWith(".png") || entry.entryName.endsWith(".jpg")) {
    //             const image = zip.readFile(entry);
    //             if (image == undefined) {
    //                 throw new Error("Не удалось прочитать файл из архива");
    //             }
    //             await expectedFufContentPart(image);
    //             return;
    //         }
    //     }
    // }
    async getFufImagesCount(candyPage) {
        const downloadCallback = await candyPage.getImageFromFuf();
        const { content } = await this.rootPageWrapper.expectDownloadedFile(downloadCallback);
        const zip = new adm_zip_1.default(content);
        return zip.getEntries().reduce((count, entry) => (this.isImage(entry.entryName) ? count + 1 : count), 0);
    }
    bufferWithXmlToString(file, forceEncoding) {
        if (forceEncoding == undefined) {
            const contentAsUtf8 = iconv_lite_1.default.decode(file, "utf8");
            if (contentAsUtf8.includes('encoding="windows-1251"')) {
                return iconv_lite_1.default.decode(file, "win1251");
            }
            return contentAsUtf8;
        }
        return iconv_lite_1.default.decode(file, forceEncoding ?? "utf8");
    }
    isImage(filename) {
        return filename.endsWith(".png") || filename.endsWith(".jpg");
    }
    async fufToJson(fufContent) {
        return new Promise((resolve, reject) => {
            (0, xml2js_1.parseString)(fufContent, {
                mergeAttrs: true,
            }, (err, result) => {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(result);
                }
            });
        });
    }
}
exports.FufChecker = FufChecker;
