"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PageBase = void 0;
const IForm_1 = require("../../../../Engine/src/Engine/Form/IForm");
const CompoundControl_1 = require("../PageObjects/Bases/CompoundControl");
const ControlsBase_1 = require("../PageObjects/Bases/ControlsBase");
const Selector_1 = require("../PageObjects/Bases/Selectors/Selector");
const ExtendedPromise_1 = require("../ExtenedPromises/ExtendedPromise");
const AfterPageLoadActions_1 = require("../BrowserWrapper/AfterPageLoadActions");
const AsyncUtils_1 = require("../../../../Common/AsyncUtils");
class PageBase extends CompoundControl_1.CompoundControl {
    formComponent;
    constructor(context, selector) {
        super(context, selector);
        this.formComponent = new ControlsBase_1.ControlBase(this, new Selector_1.ReactSelector("Form", { "data-tid": "LoadedFormComponent" }, true));
    }
    async setValueToModel(path, value) {
        await this.rootPage.evaluateFunctionAsHandle(([inPathString, inValue]) => window.publicForm.set(inPathString, inValue), path.toLegacyPath(), value);
    }
    getViewModelField(bindingPath, fieldName) {
        const waitOptions = IForm_1.CalculationAwaitingOption.CalculationsAndValidations;
        return (0, ExtendedPromise_1.extendPromise)(async () => 
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        await (await this.rootPage.evaluateFunctionAsHandle(async ([inBindingPath, inFieldName, waitOption]) => {
            const form = window.__CANDY_FORMS_TESTING_MODE__FormObject?.();
            await form?.waitForPendingCalculations(waitOption);
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return form?.getViewModelField(inBindingPath, inFieldName);
        }, bindingPath.toLegacyPath(), fieldName, waitOptions)).jsonValue());
    }
    async setViewModelField(bindingPath, fieldName, value) {
        await (await this.rootPage.evaluateFunctionAsHandle(([inBindingPath, inFieldName, inValue]) => window
            .__CANDY_FORMS_TESTING_MODE__FormObject?.()
            ?.setViewModelField(inBindingPath, inFieldName, inValue), bindingPath.toLegacyPath(), fieldName, value)).jsonValue();
    }
    async reloadInner() {
        await this.rootPage.evaluateScript(`window.__CANDY_FORMS_TESTING_MODE__FormObject().publicForm.reloadInner()`);
    }
    async refresh() {
        await this.rootPage.refresh();
        await (0, AfterPageLoadActions_1.onAfterPageLoad)(this.rootPage.page);
    }
    async nidusCleanup(nidusClient, nidusName, nidusReferenceKey = "00000000-0000-0000-0000-000000000000") {
        const result = await nidusClient.search(nidusName, nidusReferenceKey, "");
        const entries = result.items;
        entries.forEach(async (entry) => {
            try {
                await nidusClient.delete(nidusName, nidusReferenceKey, entry.id ?? "");
            }
            catch (err) {
                //nope
            }
        }, this);
    }
    async waitToCloseCheckSumModalAsync(ms = 60000, interval = 200) {
        await this.checkOpenCheckSumModal(ms, interval);
    }
    async checkOpenCheckSumModal(ms, interval) {
        let ticks = ms;
        try {
            while (ticks > 0) {
                if (await this.checkUiUtilsModal()) {
                    await this.skipUiUtilsModal();
                }
                ticks -= interval;
                await (0, AsyncUtils_1.delay)(interval);
            }
        }
        catch {
            //noop
        }
    }
    async skipUiUtilsModal() {
        await this.rootPage.evaluateScript("__CANDY_FORMS_TESTING_MODE__SkipUIUtilsModal()");
    }
    async checkUiUtilsModal() {
        const res = await this.rootPage.evaluateFunctionAsHandle(() => typeof window.__CANDY_FORMS_TESTING_MODE__SkipUIUtilsModal === "function");
        return res.jsonValue();
    }
}
exports.PageBase = PageBase;
