"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UITestFormPageWrapper = void 0;
const IForm_1 = require("../../../../../Engine/src/Engine/Form/IForm");
const AsyncUtils_1 = require("../../../../../Common/AsyncUtils");
const DraftPresentationType_1 = require("../../../../../Common/CandyApiClient/Types/Presentations/DraftPresentationType");
const ErrorsItemAdapterProxy_1 = require("./AdapterProxies/ErrorsItemAdapterProxy");
const GenericControlAdapterProxy_1 = require("./AdapterProxies/GenericControlAdapterProxy");
const PanelAdapterProxy_1 = require("./AdapterProxies/PanelAdapterProxy");
const UnitAdapterProxy_1 = require("./AdapterProxies/UnitAdapterProxy");
//потом
var ClearPageMethod;
(function (ClearPageMethod) {
    ClearPageMethod["OnlyParent"] = "OnlyParent";
    ClearPageMethod["ParentAndChildren"] = "ParentAndChildren";
})(ClearPageMethod || (ClearPageMethod = {}));
class UITestFormPageWrapper {
    page;
    candyApi;
    draftInfo;
    pageObject;
    panel;
    constructor(page, candyApi, draftInfo, pageObject) {
        this.page = page;
        this.candyApi = candyApi;
        this.draftInfo = draftInfo;
        this.pageObject = pageObject;
        this.panel = new PanelAdapterProxy_1.PanelAdapterProxy(this);
    }
    async wait(tm) {
        await (0, AsyncUtils_1.delay)(tm);
    }
    errorLogItem(name) {
        return new ErrorsItemAdapterProxy_1.ErrorsItemAdapterProxy(this, name);
    }
    unit(id) {
        return new UnitAdapterProxy_1.UnitAdapterProxy(this, id);
    }
    control(name) {
        return new GenericControlAdapterProxy_1.GenericControlAdapterProxy(this, name);
    }
    async restartForm() {
        await this.pageObject.refresh();
    }
    async controlExists(nodePath) {
        const result = await (await this.page.evaluateHandle((controlPath) => Boolean(window.__CANDY__pageObjectRegistry?.getPageObject(controlPath)), nodePath)).jsonValue();
        if (result == undefined) {
            return false;
        }
        if (typeof result === "boolean") {
            return result;
        }
        throw new Error("InvalidProgramState");
    }
    async saveForm() {
        await this.page.evaluateHandle(() => window.publicForm.form.changeSet.send());
    }
    async setErrorLogFromFile(errorLog) {
        await this.page.evaluateHandle(
        // @ts-ignore
        (formErrorLog) => window.publicForm.setErrorLog(formErrorLog), errorLog);
    }
    async getValueFromControl(inNodePath, inStatePath) {
        const variableHandle = await this.page.evaluateHandle(this.executeAction, {
            nodePath: inNodePath,
            statePath: inStatePath,
            waitOption: IForm_1.CalculationAwaitingOption.CalculationsAndValidations,
        });
        const result = await variableHandle.jsonValue();
        // eslint-disable-next-line
        return result;
    }
    async setValueToControl(inNodePath, inStatePath, inValue) {
        await this.page.evaluateHandle(this.executeAction, {
            nodePath: inNodePath,
            statePath: inStatePath,
            value: inValue,
            waitOption: IForm_1.CalculationAwaitingOption.CalculationsAndValidations,
        });
    }
    async setValueToModel(inNodePath, inFieldName, inValue) {
        await this.page.evaluateHandle(({ nodePath, fieldName, value }) => {
            window.publicForm.form.setViewModelField(nodePath, fieldName, value);
        }, { nodePath: inNodePath, fieldName: inFieldName, value: inValue });
    }
    async getValueFromModel(inNodePath, inFieldName) {
        const result = await (await this.page.evaluateHandle(({ nodePath, fieldName, waitOption }) => {
            const formObject = window.publicForm.form;
            const waitForPendingCalculations = formObject?.waitForPendingCalculations?.(waitOption) ?? Promise.resolve();
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return waitForPendingCalculations.then(() => formObject?.getViewModelField(nodePath, fieldName));
        }, {
            nodePath: inNodePath,
            fieldName: inFieldName,
            waitOption: IForm_1.CalculationAwaitingOption.CalculationsAndValidations,
        })).jsonValue();
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return result;
    }
    async close() {
        await this.page.close();
    }
    executeAction = ({ nodePath, statePath, value, waitOption, }) => {
        const pageObject = window.__CANDY__pageObjectRegistry?.getPageObject(nodePath) ??
            window.__CANDY__pageObjectRegistry?.getPageObject(nodePath + "/default");
        const formObject = window.__CANDY_FORMS_TESTING_MODE__FormObject?.();
        if (pageObject != undefined) {
            // @ts-ignore
            if (pageObject[statePath] != undefined) {
                // @ts-ignore
                // eslint-disable-next-line @typescript-eslint/no-unsafe-return
                const getResult = () => pageObject[statePath](value);
                const waitForPendingCalculations = formObject?.waitForPendingCalculations?.(waitOption) ?? Promise.resolve();
                // eslint-disable-next-line @typescript-eslint/no-unsafe-return
                return waitForPendingCalculations.then(() => getResult());
            }
            else {
                throw new Error(`Component with ${nodePath} don't have adapter or adapter with ${statePath}`);
            }
        }
        else {
            throw new Error(`Couldn't find component with '${nodePath}'`);
        }
    };
    async setFuf(fuf) {
        await this.candyApi.setPresentations(this.draftInfo, { fuf: fuf });
        await this.refreshPage();
    }
    async setInner(inner) {
        await this.candyApi.setPresentations(this.draftInfo, { inner: inner });
        await this.refreshPage();
    }
    async setOuter(outer) {
        await this.candyApi.candyApi.forDraft(this.draftInfo).patchPresentation(DraftPresentationType_1.DraftPresentationType.outer, outer);
        await this.refreshPage();
    }
    async setRequisites(fakeRequisites) {
        await this.page.evaluateHandle(({ draftId, requisites }) => {
            sessionStorage.setItem("FakeRequisites." + draftId, requisites);
        }, { draftId: this.draftInfo.draftId, requisites: JSON.stringify(fakeRequisites) });
        await this.refreshPage();
    }
    async confirmRemoveUnit() {
        await this.page.locator(`[data-tid="UnitRemoveConfirmModalSubmitButton"]`).click();
    }
    async confirmClearUnit(method = ClearPageMethod.OnlyParent) {
        await this.page.locator(`[data-tid="${method}"]`).click();
        await this.page.locator('[data-tid="UnitClearConfirmModalSubmitButton"]').click();
    }
    async refreshPage() {
        const isServerNormalization = await this.page
            .evaluateHandle(() => window.publicForm.form.settings.useServerSideFLangNormalization)
            .then(d => d.jsonValue());
        if (isServerNormalization) {
            await this.pageObject.refreshWithNormalization();
        }
        else {
            await this.pageObject.refresh();
        }
    }
}
exports.UITestFormPageWrapper = UITestFormPageWrapper;
