"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.delay = delay;
exports.nextTick = nextTick;
exports.ensureAtLeastOneNodeExists = ensureAtLeastOneNodeExists;
exports.ensureNodeNotExists = ensureNodeNotExists;
const ReactSelectorTypingBridge_1 = require("./Selectors/ReactSelectorTypingBridge");
function delay(timeout) {
    return new Promise(resolve => {
        setTimeout(resolve, timeout);
    });
}
function nextTick() {
    return new Promise(resolve => {
        setTimeout(resolve, 0);
    });
}
async function ensureAtLeastOneNodeExists(page, fn, getFailMessage) {
    for (let i = 0; i < 20; i++) {
        const result = await fn();
        let hasNodes = false;
        try {
            hasNodes = await (await (0, ReactSelectorTypingBridge_1.evaluateHandler)(page, ([x]) => x?.hasNodes(), result)).jsonValue();
        }
        catch (error) {
            // eslint-disable-next-line no-console
            console.error(`Fail to execute node search. ${getFailMessage()}`);
            hasNodes = false;
        }
        if (hasNodes) {
            return result;
        }
        else {
            await delay(Math.pow(2, Math.max(0, i - 10)) * 4 + 100);
            continue;
        }
    }
    throw new Error(getFailMessage());
}
async function ensureNodeNotExists(page, fn, getFailMessage) {
    for (let i = 0; i < 10; i++) {
        const result = await fn();
        let hasNodes = false;
        try {
            hasNodes = await (await (0, ReactSelectorTypingBridge_1.evaluateHandler)(page, ([x]) => x?.hasNodes(), result)).jsonValue();
        }
        catch (error) {
            // eslint-disable-next-line no-console
            console.error(`Fail to execute node search. ${getFailMessage()}`);
            hasNodes = false;
        }
        if (hasNodes) {
            return result;
        }
        else {
            await delay(100);
            continue;
        }
    }
    throw new Error(getFailMessage());
}
