"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FluentControlFactory = void 0;
exports.control = control;
const JustParentSelector_1 = require("../Selectors/JustParentSelector");
const Selector_1 = require("../Selectors/Selector");
const ControlList_1 = require("../ControlList");
const ReactNodeAtIndexSelector_1 = require("../Selectors/ReactNodeAtIndexSelector");
const IsSelector_1 = require("../Selectors/IsSelector");
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
const PathHelper_1 = require("../../../../../../Generator/src/generators/markupGenerator/ComponentMarkupBuilder/PathHelper");
class FluentControlFactory {
    selector;
    controlConstructor;
    constructor(controlConstructor) {
        this.controlConstructor = controlConstructor;
        this.selector = (0, Selector_1.toSelector)(this.controlConstructor.defaultElementSelector) ?? new JustParentSelector_1.JustParentSelector();
    }
    in(searchContext) {
        return new this.controlConstructor(searchContext, this.selector);
    }
    listIn(searchContext) {
        return new ControlList_1.ControlList(searchContext, this.selector, this.controlConstructor);
    }
    tid(tidString) {
        return this.prop("data-tid", tidString);
    }
    first() {
        if (this.selector instanceof Selector_1.ReactSelector) {
            this.selector = this.selector.first();
            return this;
        }
        else {
            throw new Error("'first' can be applied to ReactSelector only");
        }
    }
    second() {
        if (this.selector instanceof Selector_1.ReactSelector) {
            this.selector = this.selector.second();
            return this;
        }
        else {
            throw new Error("'second' can be applied to ReactSelector only");
        }
    }
    prop(nameOrPicker, value) {
        const name = typeof nameOrPicker === "string" ? nameOrPicker : (0, PathHelper_1.extractPath)(nameOrPicker.toString());
        if (this.selector instanceof Selector_1.ReactSelector) {
            this.selector = this.selector.internalSetPropsFilter({
                ...(this.selector.internalGetPropsFilter() ?? {}),
                [name]: value,
            });
            return this;
        }
        else {
            throw new Error("prop setter can be applied to ReactSelector or DomSelector only");
        }
    }
    path(bindingPath) {
        return this.prop("bindingPath", bindingPath);
    }
    asIs() {
        this.selector =
            (0, Selector_1.toSelector)(this.controlConstructor.defaultElementSelector) ??
                (0, TypingUtils_1.reject)(`defaultElementSelector must be specified`);
        return this;
    }
    unitId(idValue) {
        return this.tid(idValue);
    }
    itemAt(index) {
        this.selector = new ReactNodeAtIndexSelector_1.ReactNodeAtIndexSelector(this.selector, index);
        return this;
    }
    as(selector) {
        if (typeof selector === "string") {
            this.selector = (0, Selector_1.toSelector)(selector);
        }
        else if ((0, IsSelector_1.isTestBoundingReactComponent)(selector)) {
            this.selector = (0, Selector_1.toSelector)(Selector_1.ReactSelector.from(selector));
        }
        else {
            // @ts-ignore Щас мне лень
            this.selector = selector;
        }
        return this;
    }
}
exports.FluentControlFactory = FluentControlFactory;
function control(controlConstructor) {
    return new FluentControlFactory(controlConstructor);
}
