"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ControlBase = void 0;
const ExtendedPromise_1 = require("../../ExtenedPromises/ExtendedPromise");
const IForm_1 = require("../../../../../Engine/src/Engine/Form/IForm");
const CommonUtils_1 = require("./CommonUtils");
const ReactSelectorTypingBridge_1 = require("./Selectors/ReactSelectorTypingBridge");
const ControlFactoryStatics_1 = require("./ControlFactoryStatics");
class ControlBase extends ControlFactoryStatics_1.ControlFactoryStatics {
    static defaultElementSelector;
    context;
    selector;
    constructor(context, selector) {
        super();
        this.context = context;
        this.selector = selector;
    }
    get rootPage() {
        return this.context.rootPage;
    }
    get rootPageWrapper() {
        return this.context.rootPageWrapper;
    }
    async getRootElement() {
        const selectorResult = await this.getSelectorResult();
        const result = await (await (0, ReactSelectorTypingBridge_1.evaluateHandler)(this.rootPage, ([x]) => x.getDomElement(), selectorResult)).asElement();
        if (result == undefined) {
            throw new Error("Element not found");
        }
        return result;
    }
    async hover() {
        const root = await this.getRootElement();
        await root.hover();
    }
    getSelectorResultForChildSearch() {
        return this.getSelectorResult();
    }
    getSelectorResult() {
        return (0, CommonUtils_1.ensureAtLeastOneNodeExists)(this.rootPage, () => this.selector.execute(this.context), () => `Не найден элемент по селектору\n    ${this.selector.getDescription(this.context)}`);
    }
    getProp(propName) {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const result = await (0, ReactSelectorTypingBridge_1.evaluateHandler)(this.rootPage, ([x, propName1, waitOption]) => {
                const formObject = window.publicForm.form;
                const waitForPendingCalculations = formObject?.waitForPendingCalculations?.(waitOption) ?? Promise.resolve();
                return waitForPendingCalculations.then(() => x.getProps()[propName1]);
            }, await this.getSelectorResult(), propName, IForm_1.CalculationAwaitingOption.CalculationsAndValidations);
            if (result == undefined) {
                throw new Error("InvalidProgramState");
            }
            return result.jsonValue();
        });
    }
    isActiveElement() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const result = await (0, ReactSelectorTypingBridge_1.evaluateHandler)(this.rootPage, ([selectorResult]) => {
                const domElement = selectorResult.getDomElement();
                return domElement != undefined && domElement === document.activeElement;
            }, await this.getSelectorResult());
            return result.jsonValue();
        });
    }
    isHoveredElement() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const result = await (0, ReactSelectorTypingBridge_1.evaluateHandler)(this.rootPage, ([selectorResult]) => {
                const domElement = selectorResult.getDomElement();
                return domElement != undefined && domElement.matches(":hover");
            }, await this.getSelectorResult());
            return result.jsonValue();
        });
    }
    isPresent() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            try {
                await this.getSelectorResult();
                return true;
            }
            catch (e) {
                return false;
            }
        });
    }
    isNotPresent() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            try {
                await (0, CommonUtils_1.ensureNodeNotExists)(this.rootPage, () => this.selector.execute(this.context), () => `Не найден элемент по селектору\n    ${this.selector.getDescription(this.context)}`);
                return false;
            }
            catch (e) {
                return true;
            }
        });
    }
    getSelectorDescription() {
        return this.selector.getDescription(this.context);
    }
    async takeScreenshot() {
        return (await this.getRootElement()).takeScreenshot();
    }
    async hide() {
        await this.rootPage.evaluateFunctionAsHandle(([x]) => {
            x.style.visibility = "hidden";
        }, await this.getRootElement());
    }
    async show() {
        await this.rootPage.evaluateFunctionAsHandle(([x]) => {
            x.style.visibility = "visible";
        }, await this.getRootElement());
    }
    get isInViewport() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.isPresent() &&
            (await this.rootPage.evaluateFunctionAsHandle(([x]) => {
                const bounding = x.getBoundingClientRect();
                return (bounding.top >= 0 &&
                    bounding.left >= 0 &&
                    bounding.bottom <= (window.innerHeight || document.documentElement.clientHeight) &&
                    bounding.right <= (window.innerWidth || document.documentElement.clientWidth));
            }, await this.getRootElement())).jsonValue());
    }
    async scrollIntoView() {
        await this.rootPage.evaluateFunctionAsHandle(([x]) => {
            x.scrollIntoView({ block: "center", inline: "center" });
        }, await this.getRootElement());
    }
    get text() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const element = await this.getRootElement();
            return element.textContent();
        });
    }
    async click() {
        const rootElement = await this.getRootElement();
        await rootElement.click();
    }
    async setAttribute(name, value) {
        const rootElement = await this.getRootElement();
        await rootElement.setAttribute(name, value);
    }
    pick(pickier, ...args) {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const result = await (0, ReactSelectorTypingBridge_1.evaluateHandler)(this.rootPage, ([x, pickierAsStringForEval, ...inArgs]) => {
                const inPicker = eval(pickierAsStringForEval);
                // eslint-disable-next-line @typescript-eslint/no-unsafe-return
                return inPicker(x.getProps(), ...inArgs);
            }, await this.getSelectorResult(), `(${pickier.toString()})`, ...args);
            if (result == undefined) {
                throw new Error("InvalidProgramState");
            }
            return result.jsonValue();
        });
    }
    async focus() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.focus();
    }
    async blur() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await this.rootPage.press("Tab");
    }
}
exports.ControlBase = ControlBase;
