"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactSelector = void 0;
exports.toSelector = toSelector;
const IterableUtils_1 = require("../../../../../../Common/IterableUtils");
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
const ReactSelectorTypingBridge_1 = require("./ReactSelectorTypingBridge");
const ReactNodeAtIndexSelector_1 = require("./ReactNodeAtIndexSelector");
class ReactSelector {
    selectorString;
    props;
    ignoreContext;
    constructor(selectorString, props, ignoreContext = false) {
        this.selectorString = selectorString;
        this.props = props;
        this.ignoreContext = ignoreContext;
    }
    first() {
        return new ReactNodeAtIndexSelector_1.ReactNodeAtIndexSelector(this, 0);
    }
    second() {
        return new ReactNodeAtIndexSelector_1.ReactNodeAtIndexSelector(this, 1);
    }
    at(index) {
        return new ReactNodeAtIndexSelector_1.ReactNodeAtIndexSelector(this, index);
    }
    internalSetPropsFilter(props) {
        return new ReactSelector(this.selectorString, props, this.ignoreContext);
    }
    internalGetPropsFilter() {
        return this.props;
    }
    async execute(context) {
        const attributesSelector = Object.entries(this.props ?? {})
            .map(([name, value]) => {
            if (name === "bindingPath") {
                if (typeof value === "string") {
                    if (value.startsWith("~~")) {
                        return `[${name}$=${JSON.stringify(value.replace("~~", ""))}]`;
                    }
                    if (!value.startsWith("/")) {
                        return `[${name}=${JSON.stringify("/" + value)}]`;
                    }
                }
            }
            return `[${name}=${JSON.stringify(value)}]`;
        })
            .reduce((x, y) => x + y, "");
        const selector = this.selectorString + attributesSelector;
        if (this.ignoreContext) {
            return (0, ReactSelectorTypingBridge_1.evaluateHandler)(context.rootPage, ([selectorString]) => window.rts.querySelector(selectorString), selector);
        }
        else {
            return (0, ReactSelectorTypingBridge_1.evaluateHandler)(context.rootPage, ([parentReactSelector, selectorString]) => parentReactSelector.querySelector(selectorString), await context.getSelectorResultForChildSearch(), selector);
        }
    }
    getDescription(context) {
        return `${context.getSelectorDescription()} ${this.getCurrentSelectorDescription()}`;
    }
    getCurrentSelectorDescription() {
        return (this.selectorString +
            Iterator.from((0, IterableUtils_1.iterateEntries)(this.props ?? {}))
                .map(([name, value]) => `[${name}=${value}]`)
                .toArray()
                .join(""));
    }
    static from(componentType, propFilter) {
        const componentName = extractNameFromComponent(componentType) ?? (0, TypingUtils_1.reject)("Unable to extract component name");
        if (propFilter != undefined && typeof propFilter === "string") {
            return new ReactSelector(componentName, { "data-tid": propFilter });
        }
        if (propFilter != undefined && typeof propFilter === "object") {
            return new ReactSelector(componentName, propFilter);
        }
        return new ReactSelector(componentName);
    }
}
exports.ReactSelector = ReactSelector;
function toSelector(input) {
    return typeof input === "string" ? new ReactSelector(input) : input;
}
function extractNameFromComponent(comp) {
    // @ts-ignore
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return comp.displayName ?? comp.render?.name ?? comp.name;
}
