"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KebabPopupContent = exports.KebabControl = exports.KebabActionControl = void 0;
const ControlsBase_1 = require("../../../Bases/ControlsBase");
const FluentControlFactory_1 = require("../../../Bases/ControlDeclaration/FluentControlFactory");
const ExtendedPromise_1 = require("../../../../ExtenedPromises/ExtendedPromise");
const ControlList_1 = require("../../../Bases/ControlList");
const Selector_1 = require("../../../Bases/Selectors/Selector");
const JustParentSelector_1 = require("../../../Bases/Selectors/JustParentSelector");
class KebabActionControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "KebabAction";
    get disabled() {
        return this.getProp("disabled");
    }
    get path() {
        return this.getProp("path");
    }
}
exports.KebabActionControl = KebabActionControl;
class KebabControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "Kebab";
    menuItems = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(KebabActionControl.as("MenuItem"))).in(this);
    get disabled() {
        return (0, ExtendedPromise_1.extendPromise)(async () => Boolean(await this.getProp("disabled")));
    }
    blur() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const element = await this.getRootElement();
            if (element == undefined) {
                throw new Error("Element not found");
            }
            await this.rootPage.press("Escape");
        });
    }
    selectItemByIndex(index) {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const popupContent = await this.getItems(KebabPopupContent);
            await popupContent.content.items.itemAt(index).click();
        });
    }
    async getItems(contentClass) {
        await this.click();
        const popup = new KebabPopupControl(this, new Selector_1.ReactSelector("Popup PopupMenu ScrollContainer"), contentClass);
        return popup;
    }
}
exports.KebabControl = KebabControl;
class KebabPopupContent extends ControlsBase_1.ControlBase {
    items = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(KebabActionControl.as("MenuItem"))).in(this);
}
exports.KebabPopupContent = KebabPopupContent;
class KebabPopupControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "Popup";
    content;
    constructor(context, selector, contentControl) {
        super(context, selector);
        this.content = new contentControl(this, new JustParentSelector_1.JustParentSelector());
    }
}
