"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComboboxControl = void 0;
const ExtendedPromise_1 = require("../../../../ExtenedPromises/ExtendedPromise");
const ControlsBase_1 = require("../../../Bases/ControlsBase");
const FluentControlFactory_1 = require("../../../Bases/ControlDeclaration/FluentControlFactory");
const Selector_1 = require("../../../Bases/Selectors/Selector");
const ControlList_1 = require("../../../Bases/ControlList");
const DropdownControl_1 = require("../../DropdownControl");
const LabelControl_1 = require("../../LabelControl");
const MenuItemControl_1 = require("../../MenuItemControl");
const ButtonControl_1 = require("../Action/ButtonControl");
const ComboboxDropdownContent_1 = require("../../ComboboxDropdownContent");
const Fixtures_1 = require("../../../../Fixtures");
const InputControl_1 = require("./InputControl");
const FxInputControl_1 = require("./FxInputControl");
class ComboboxControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "withResolvePaths(**(ComboBox))";
    static emptyValue = null;
    internalInput = (0, FluentControlFactory_1.control)(InputControl_1.InputControl).as("Input").in(this);
    internalComboBox = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("ComboBox[bindingPath]").in(this);
    internalComboBoxView = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("ComboBoxView").in(this);
    menu = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(MenuItemControl_1.MenuItemControl.as("MenuItem")))
        .as("Menu")
        .in(this);
    wrapperInput = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("InputLikeText").in(this);
    formulaTooltip = (0, FluentControlFactory_1.control)(FxInputControl_1.FormulaTooltipControl).as("Tooltip[formula]").in(this);
    switchToAutoButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("Button").in(this);
    hint = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("PopupPin").in(this);
    get value() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const value = await this.internalComboBoxView.getProp("textValue");
            return value ?? "";
        });
    }
    async switchToAuto() {
        await this.switchToAutoButton.click();
    }
    get placeholderText() {
        return this.wrapperInput.text;
    }
    get isFocused() {
        return this.internalInput.isFocused;
    }
    async pressEnter() {
        await this.internalInput.pressEnter();
    }
    get errorMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalComboBox.getProp("error"));
    }
    get warningMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalComboBox.getProp("warning"));
    }
    get isValid() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const error = await this.internalComboBox.getProp("error");
            const warning = await this.internalComboBox.getProp("warning");
            return !error && !warning;
        });
    }
    valueOfProp(prop) {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalComboBox.getProp(prop));
    }
    selectValueByIndex(index) {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const disablePortal = await this.getProp("disablePortal");
            if (disablePortal) {
                await this.wrapperInput.click();
                await this.menu.itemAt(index).click();
            }
            else {
                const dropDown = await this.getDropdown(ComboboxDropdownContent_1.ComboboxDropdownContent);
                if (index === 0) {
                    await Fixtures_1.expect.poll(() => dropDown.spinner.isNotPresent()).toBeTruthy();
                }
                await dropDown.content.items.itemAt(index).click();
            }
            await this.blur();
        });
    }
    typeValueAndSelectSingle(searchValue) {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const dropDown = await this.getDropdown(ComboboxDropdownContent_1.ComboboxDropdownContent);
            await this.internalInput.clearAndType(searchValue);
            await Fixtures_1.expect.poll(() => dropDown.spinner.isNotPresent()).toBeTruthy();
            await dropDown.content.items.itemAt(0).click();
        });
    }
    typeValueAndPressTab(value) {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            await this.clearAndType(value);
            await this.rootPage.press("Tab");
        });
    }
    async clearAndType(text) {
        await this.click();
        await this.internalInput.clearAndType(text);
    }
    dropDownItemValue(itemNumber) {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const dropDown = await this.getDropdown(ComboboxDropdownContent_1.ComboboxDropdownContent);
            return dropDown.content.items.itemAt(itemNumber).text;
        });
    }
    async hover() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.hover();
    }
    blur() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            await this.internalInput.blur();
        });
    }
    async clear() {
        await this.click();
        await this.rootPage.pressCtrlA();
        await this.rootPage.press("Backspace");
    }
    async getDropdown(contentClass) {
        await this.click();
        const dropDownMenu = new DropdownControl_1.DropdownControl(this, new Selector_1.ReactSelector("CustomComboBox ComboBoxView DropdownContainer"), contentClass);
        return dropDownMenu;
    }
}
exports.ComboboxControl = ComboboxControl;
