"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DatePickerControl = void 0;
const ExtendedPromise_1 = require("../../../../ExtenedPromises/ExtendedPromise");
const ControlsBase_1 = require("../../../Bases/ControlsBase");
const CommonUtils_1 = require("../../../Bases/CommonUtils");
const FluentControlFactory_1 = require("../../../Bases/ControlDeclaration/FluentControlFactory");
class DatePickerControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "withResolvePaths(**(DatePicker))";
    internalCandyDatePicker = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("DatePicker[bindingPath]").in(this);
    internalDateInput = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("DateInput").in(this);
    get value() {
        return this.internalDateInput.getProp("value");
    }
    get highlighted() {
        return this.internalCandyDatePicker.pick(x => x.highlighted ?? false);
    }
    get isValid() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const error = await this.internalCandyDatePicker.getProp("error");
            return !Boolean(error);
        });
    }
    get error() {
        return this.internalCandyDatePicker.pick(x => Boolean(x.error));
    }
    get errorMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalCandyDatePicker.getProp("error"));
    }
    async clear() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        // BT-HACK это на случай если промахнулись
        await (0, CommonUtils_1.delay)(200);
        await element.click();
        await this.rootPage.pressCtrlA();
        await this.rootPage.press("Backspace");
    }
    async blur() {
        await this.getRootElement();
        await this.rootPage.press("Tab");
    }
    async clearAndType(text) {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await this.clear();
        const childElement = await element.findByCssSelector("label span");
        if (childElement == undefined) {
            throw new Error("Element not found");
        }
        await this.rootPage.press("Backspace");
        // Дата в новом DatePicker заполнялась некорректно с точками https://github.com/skbkontur/retail-ui/issues/1059
        const textWithoutDot = text.replace(/\./g, "");
        await childElement.deprecatedType(textWithoutDot);
    }
    async focus() {
        const element = await this.getRootElement();
        await element.focus();
    }
    async click() {
        const element = await this.getRootElement();
        await element.click();
    }
    get isFocused() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            await this.rootPage.press("1");
            await this.rootPage.press("1");
            const value = (await this.value) === "11";
            await this.rootPage.pressCtrlA();
            await this.rootPage.press("Backspace");
            return value;
        });
    }
}
exports.DatePickerControl = DatePickerControl;
