"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileLoaderControl = exports.FileLoaderControlButton = void 0;
const ExtendedPromise_1 = require("../../../../../ExtenedPromises/ExtendedPromise");
const ControlsBase_1 = require("../../../../Bases/ControlsBase");
const CompoundControl_1 = require("../../../../Bases/CompoundControl");
const FluentControlFactory_1 = require("../../../../Bases/ControlDeclaration/FluentControlFactory");
const ButtonControl_1 = require("../../Action/ButtonControl");
const ClickableAreaControl_1 = require("../../../ClickableAreaControl");
const LabelControl_1 = require("../../../LabelControl");
const TooltipContentBaseControl_1 = require("../../../TooltipContentBaseControl");
const HtmlButtonControl_1 = require("../../../HtmlControls/HtmlButtonControl");
const CommonUtils_1 = require("../../../../CommonUtils");
const Fixtures_1 = require("../../../../../Fixtures");
// const writeFileAsync = promisify(writeFile);
class ViewFileRemoveTooltip extends TooltipContentBaseControl_1.TooltipContentBaseControl {
    removeButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).tid("RemoveButton").in(this);
}
class ViewFileControl extends CompoundControl_1.CompoundControl {
    progressBar = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("UploadFileProgress").in(this);
    fileName = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("FileName").tid("FileName").in(this);
    removeArea = (0, FluentControlFactory_1.control)(ClickableAreaControl_1.ClickableAreaControl).as("RemoveButtonContainer").tid("RemoveArea").in(this);
    removeTooltip = (0, FluentControlFactory_1.control)(ViewFileRemoveTooltip).as("Tooltip").tid("RemoveTooltip").in(this);
    viewFileError = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("ViewFileError").in(this);
    viewFileWarning = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("ViewFileWarning").in(this);
    async remove() {
        await this.removeArea.show();
        await this.removeArea.click();
        await this.removeTooltip.removeButton.click();
    }
}
class FileLoaderControlButton extends ControlsBase_1.ControlBase {
    internalButtonHtmlElement = (0, FluentControlFactory_1.control)(HtmlButtonControl_1.HtmlButtonControl).as("button").in(this);
    async click() {
        await this.internalButtonHtmlElement.click();
    }
    get disabled() {
        return (0, ExtendedPromise_1.extendPromise)(async () => Boolean(await this.getProp("disabled")));
    }
}
exports.FileLoaderControlButton = FileLoaderControlButton;
class FileLoaderControl extends CompoundControl_1.CompoundControl {
    static defaultElementSelector = "withResolvePaths(**(FileLoader))";
    addButton = (0, FluentControlFactory_1.control)(FileLoaderControlButton).as("AddButton").in(this);
    replaceButton = (0, FluentControlFactory_1.control)(FileLoaderControlButton).as("ReplaceButton").in(this);
    overflowDocuments = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("div").tid("fileloader-overflow-documents").in(this);
    files = (0, FluentControlFactory_1.control)(ViewFileControl).as("ViewFile").listIn(this);
    async addFiles(filesPath, wait = false) {
        const prevCount = await this.files.count;
        const fileChooserPromise = this.rootPage.waitForDownloadEvent();
        await this.addButton.click();
        const fileChooser = await fileChooserPromise;
        await fileChooser.setFiles(filesPath);
        if (wait) {
            await Fixtures_1.expect.poll(() => this.files.count).toBe(prevCount + filesPath.length);
            const count = await this.files.count;
            await Fixtures_1.expect
                .poll(() => this.files.itemAt(count - 1).progressBar.isNotPresent(), { timeout: 60 * 1000 })
                .toBeTruthy();
        }
        else {
            await (0, CommonUtils_1.delay)(1500);
        }
    }
    async replaceFile(filesPath, wait = false) {
        const fileChooserPromise = this.rootPage.waitForDownloadEvent();
        await this.replaceButton.click();
        const fileChooser = await fileChooserPromise;
        await fileChooser.setFiles(filesPath);
        if (wait) {
            await Fixtures_1.expect.poll(() => this.files.count).toBe(filesPath.length);
            const count = await this.files.count;
            await Fixtures_1.expect.poll(() => this.files.itemAt(count).progressBar.isNotPresent()).toBeTruthy();
        }
        else {
            await (0, CommonUtils_1.delay)(1500);
        }
    }
}
exports.FileLoaderControl = FileLoaderControl;
