"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FxState = exports.FxInputControl = exports.FormulaTooltipControl = void 0;
const ControlsBase_1 = require("../../../Bases/ControlsBase");
const ExtendedPromise_1 = require("../../../../ExtenedPromises/ExtendedPromise");
const FluentControlFactory_1 = require("../../../Bases/ControlDeclaration/FluentControlFactory");
const ButtonControl_1 = require("../Action/ButtonControl");
const HtmlInputControl_1 = require("../../HtmlControls/HtmlInputControl");
const InputControl_1 = require("./InputControl");
class FormulaTooltipControl extends ControlsBase_1.ControlBase {
    get opened() {
        return (0, ExtendedPromise_1.extendPromise)(async () => (await this.getProp("trigger")) === "opened");
    }
    get formulaText() {
        return this.getProp("formula");
    }
}
exports.FormulaTooltipControl = FormulaTooltipControl;
class FxInputControl extends InputControl_1.InputControl {
    static defaultElementSelector = "withResolvePaths(**(FxInput))";
    highlightedFxInputElement = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase)
        .as(`span[class^="FxInput--highlighted"]`)
        .in(this);
    internalInputHtmlElement = (0, FluentControlFactory_1.control)(HtmlInputControl_1.HtmlInputControl).as("input").in(this);
    internalCandyFxInput = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("FxInput[bindingPath]").in(this);
    Icon = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("InternalIcon").in(this);
    switchToAutoButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("Button").in(this);
    formulaTooltip = (0, FluentControlFactory_1.control)(FormulaTooltipControl).as("Tooltip[formula]").in(this);
    get isHighlighted() {
        return this.highlightedFxInputElement.isPresent();
    }
    get isValid() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const error = await this.internalCandyFxInput.getProp("error");
            const warning = await this.internalCandyFxInput.getProp("warning");
            return !error && !warning;
        });
    }
    async switchToAuto() {
        await this.switchToAutoButton.click();
    }
    get errorMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalCandyFxInput.getProp("error"));
    }
    get warningMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalCandyFxInput.getProp("warning"));
    }
    get autoValue() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalCandyFxInput.getProp("autoValue"));
    }
    get fxState() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const autoFlag = await this.internalCandyFxInput.getProp("autoFlag");
            if (autoFlag) {
                return FxState.Auto;
            }
            return FxState.Manual;
        });
    }
    async focus() {
        await this.internalInputHtmlElement.focus();
    }
    async blur() {
        await this.internalInputHtmlElement.blur();
    }
}
exports.FxInputControl = FxInputControl;
var FxState;
(function (FxState) {
    FxState["Auto"] = "Auto";
    FxState["Manual"] = "Manual";
})(FxState || (exports.FxState = FxState = {}));
