"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InputControl = void 0;
const ControlsBase_1 = require("../../../Bases/ControlsBase");
const ExtendedPromise_1 = require("../../../../ExtenedPromises/ExtendedPromise");
const FluentControlFactory_1 = require("../../../Bases/ControlDeclaration/FluentControlFactory");
const HtmlInputControl_1 = require("../../HtmlControls/HtmlInputControl");
const PicklistControl_1 = require("./Picklist/PicklistControl");
class InternalCandyInputControl extends ControlsBase_1.ControlBase {
    get highlighted() {
        return this.getProp("highlighted");
    }
}
class InputControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "withResolvePaths(**(Input))";
    internalInputHtmlElement = (0, FluentControlFactory_1.control)(HtmlInputControl_1.HtmlInputControl).as("input").in(this);
    internalCandyInput = (0, FluentControlFactory_1.control)(InternalCandyInputControl).as("Input[bindingPath]").in(this);
    tooltipLink = (0, FluentControlFactory_1.control)(PicklistControl_1.TooltipControl).as("ReactMarkdown[linkTarget]").in(this);
    getInternalInputHtmlElement() {
        return this.internalInputHtmlElement;
    }
    async type(text) {
        await this.getInternalInputHtmlElement().type(text);
    }
    get value() {
        return this.getInternalInputHtmlElement().value;
    }
    get isHighlighted() {
        return this.internalCandyInput.highlighted;
    }
    get isDisabled() {
        return this.internalInputHtmlElement.getProp("disabled");
        // return this.getProp<boolean>("disabled");
    }
    get isValid() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const error = await this.internalCandyInput.getProp("error");
            return !error;
        });
    }
    get isFocused() {
        return this.internalInputHtmlElement.isActiveElement();
    }
    get errorMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalCandyInput.getProp("error"));
    }
    get warningMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalCandyInput.getProp("warning"));
    }
    async pressEnter() {
        await this.getInternalInputHtmlElement().pressEnter();
    }
    async pressDelete() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        await this.rootPage.press("Delete");
    }
    async clearAndTypeFromKeyboard(text) {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        await this.rootPage.pressCtrlA();
        await this.rootPage.press("Backspace");
        await this.rootPage.page.keyboard.type(text);
    }
    async typeFromKeyboard(text) {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        // await this.rootPage.pressCtrlA();
        // await this.rootPage.press("ArrowLeft");
        await this.rootPage.page.keyboard.type(text);
    }
    async clearAndType(text) {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        await this.rootPage.pressCtrlA();
        await this.rootPage.press("Backspace");
        await this.getInternalInputHtmlElement().type(text);
    }
    async clickAndType(text) {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        await this.getInternalInputHtmlElement().type(text);
    }
    async clickAndPressBackspace() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        await this.rootPage.press("Backspace");
    }
    //NOTE иногда в скриншотных тестах хром не принимает хак из clearAndType,
    //оставил старую логику в этом методе.
    async clearAndTypeByKeyboard(text) {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        // BT-HACK это на случай если промахнулись
        await element.click();
        await this.rootPage.pressCtrlA();
        await this.rootPage.press("Backspace");
        await this.getInternalInputHtmlElement().type(text);
    }
    async focus() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.focus();
    }
    async blur() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await this.rootPage.press("Tab");
    }
    async clear() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        // BT-HACK это на случай если промахнулись
        await element.click();
        await this.rootPage.pressCtrlA();
        await this.rootPage.press("Backspace");
        await this.rootPage.press("Tab");
    }
    get placeholder() {
        return this.getProp("placeholder");
    }
}
exports.InputControl = InputControl;
