"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PicklistControl = exports.TooltipControl = exports.ModalControl = void 0;
const test_1 = require("@playwright/test");
const ExtendedPromise_1 = require("../../../../../ExtenedPromises/ExtendedPromise");
const ControlsBase_1 = require("../../../../Bases/ControlsBase");
const FluentControlFactory_1 = require("../../../../Bases/ControlDeclaration/FluentControlFactory");
const Selector_1 = require("../../../../Bases/Selectors/Selector");
const ButtonControl_1 = require("../../Action/ButtonControl");
const JustParentSelector_1 = require("../../../../Bases/Selectors/JustParentSelector");
const ReactUIInputControl_1 = require("../../../ReactUIControls/ReactUIInputControl");
const InputControl_1 = require("../InputControl");
const CommonUtils_1 = require("../../../../CommonUtils");
const PicklistModalContent_1 = require("./PicklistModalContent");
class ModalControl extends ControlsBase_1.ControlBase {
    constructor(context, selector, contentControl) {
        super(context, selector);
        this.content = new contentControl(this, new JustParentSelector_1.JustParentSelector());
    }
    content;
}
exports.ModalControl = ModalControl;
class TooltipControl extends ControlsBase_1.ControlBase {
    get active() {
        return (0, ExtendedPromise_1.extendPromise)(async () => (await this.getProp("trigger")) !== "closed");
    }
}
exports.TooltipControl = TooltipControl;
class PicklistControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "withResolvePaths(**(Picklist))";
    tooltip = (0, FluentControlFactory_1.control)(TooltipControl).as("Tooltip").unitId("tooltip").in(this);
    internalPicklist = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("Picklist").in(this);
    internalBasePicklist = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("BasePicklist").in(this);
    picklistModal = (0, FluentControlFactory_1.control)(PicklistModalContent_1.PicklistModalContent).as("PicklistModal").in(this);
    input = (0, FluentControlFactory_1.control)(ReactUIInputControl_1.ReactUIInputControl).asIs().in(this);
    openButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).asIs().in(this);
    internalInput = (0, FluentControlFactory_1.control)(InputControl_1.InputControl).as("Input").in(this);
    get isDisabled() {
        const typePropName = "type";
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalPicklist.getProp(typePropName).then(res => {
            if (res === "button") {
                return this.isButtonDisabled;
            }
            return Promise.all([this.isButtonDisabled, this.internalInput.isDisabled]).then(all => all.every(x => x));
        }));
    }
    get isButtonDisabled() {
        return this.openButton.disabled;
    }
    get errorMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalPicklist.getProp("error"));
    }
    get warningMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.internalPicklist.getProp("warning"));
    }
    async open() {
        await this.openButton.click();
    }
    async pressDown() {
        await this.picklistModal.moveFocusToNextItem();
    }
    async openModalPicklist() {
        await this.openButton.click();
        const modalContent = await this.getModal(PicklistModalContent_1.PicklistModalContent);
        await test_1.expect.poll(() => modalContent.content.isPresent()).toEqual(true);
        await (0, CommonUtils_1.delay)(1000);
        return modalContent.content;
    }
    async setValuesViaInput(values) {
        await this.input.clearAndType(values);
        await this.input.blur();
        // Эта штука показывает, что контрол обработал значения через справочник
        await test_1.expect.poll(() => this.internalBasePicklist.getProp("value")).toBe(values);
    }
    async clearAndType(text) {
        await this.input.clearAndType(text);
    }
    async blur() {
        await this.input.blur();
    }
    async getModal(contentClass) {
        const modal = new ModalControl(this, new Selector_1.ReactSelector("PicklistModal Modal"), contentClass);
        return modal;
    }
    isTooltipActive() {
        return this.tooltip.active;
    }
    get isFocused() {
        return this.internalInput.isFocused;
    }
}
exports.PicklistControl = PicklistControl;
