"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ToggleControl = exports.TooltipControl = exports.TooltipContent = void 0;
const ControlsBase_1 = require("../../../Bases/ControlsBase");
const ExtendedPromise_1 = require("../../../../ExtenedPromises/ExtendedPromise");
const FluentControlFactory_1 = require("../../../Bases/ControlDeclaration/FluentControlFactory");
const ClickableAreaControl_1 = require("../../ClickableAreaControl");
class TooltipContent extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "TooltipContent";
}
exports.TooltipContent = TooltipContent;
class TooltipControl extends ControlsBase_1.ControlBase {
    tooltipContent = (0, FluentControlFactory_1.control)(TooltipContent).asIs().in(this);
    get active() {
        return (0, ExtendedPromise_1.extendPromise)(async () => (await this.getProp("trigger")) !== "closed");
    }
    get isOpened() {
        return (0, ExtendedPromise_1.extendPromise)(async () => (await this.tooltipContent.isPresent()) == true);
    }
    get value() {
        return this.getProp("value");
    }
}
exports.TooltipControl = TooltipControl;
class ToggleControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "Toggle";
    internalLabel = (0, FluentControlFactory_1.control)(ClickableAreaControl_1.ClickableAreaControl).as("label").in(this);
    internalReactUIToggle = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("Toggle[checked]").in(this);
    tooltip = (0, FluentControlFactory_1.control)(TooltipControl).as("Tooltip[allowedPositions]").in(this);
    get error() {
        return this.internalReactUIToggle.pick(x => Boolean(x.error));
    }
    get warning() {
        return this.internalReactUIToggle.pick(x => Boolean(x.warning));
    }
    get disabled() {
        return this.internalReactUIToggle.pick(x => Boolean(x.disabled));
    }
    get captionPosition() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.getProp("captionPosition"));
    }
    get errorMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.getProp("error"));
    }
    get warningMessage() {
        return (0, ExtendedPromise_1.extendPromise)(async () => this.getProp("warning"));
    }
    async click(checkState = true) {
        if (checkState) {
            // NOTE Иногда почему-то не кликается, кликнем ещё раз
            const currentState = await this.checked;
            await this.internalLabel.click();
            let newState = await this.checked;
            if (newState === currentState) {
                await this.internalLabel.click();
                newState = await this.checked;
                if (newState === currentState) {
                    await this.internalLabel.click();
                    newState = await this.checked;
                    if (newState === currentState) {
                        await this.internalLabel.click();
                    }
                }
            }
        }
        else {
            const rootElement = await this.getRootElement();
            await rootElement.click();
        }
    }
    get checked() {
        return this.internalReactUIToggle.getProp("checked");
    }
    isTooltipActive() {
        return this.tooltip.active;
    }
}
exports.ToggleControl = ToggleControl;
