"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComboboxOldControl = void 0;
const ExtendedPromise_1 = require("../../ExtenedPromises/ExtendedPromise");
const ControlsBase_1 = require("../Bases/ControlsBase");
const FluentControlFactory_1 = require("../Bases/ControlDeclaration/FluentControlFactory");
const Selector_1 = require("../Bases/Selectors/Selector");
const ControlList_1 = require("../Bases/ControlList");
const Fixtures_1 = require("../../Fixtures");
const DropdownControl_1 = require("./DropdownControl");
const InputControl_1 = require("./CandyControls/ValueEditors/InputControl");
const LabelControl_1 = require("./LabelControl");
const MenuItemControl_1 = require("./MenuItemControl");
const ComboboxDropdownContent_1 = require("./ComboboxDropdownContent");
class ComboboxOldControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "withResolvePaths(withViewModelSelect(Connect(ComboBox)))";
    static emptyValue = null;
    internalInput = (0, FluentControlFactory_1.control)(InputControl_1.InputControl).as("Input").in(this);
    internalComboBox = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("ComboBox").in(this);
    menu = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(MenuItemControl_1.MenuItemControl.as("MenuItem")))
        .as("Menu")
        .in(this);
    label = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("InputLikeText").in(this);
    get text() {
        return this.label.text;
    }
    get value() {
        return this.internalComboBox.getProp("value");
    }
    async clearAndType(text) {
        await this.click();
        await this.internalInput.clearAndType(text);
    }
    get errorMessage() {
        return this.internalComboBox.getProp("error");
    }
    get warningMessage() {
        return this.internalComboBox.getProp("warning");
    }
    selectValueByIndex(index) {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const disablePortal = await this.getProp("disablePortal");
            if (disablePortal) {
                await this.label.click();
                await this.menu.itemAt(index).click();
            }
            else {
                const dropDown = await this.getDropdown(ComboboxDropdownContent_1.ComboboxDropdownContent);
                await dropDown.content.items.itemAt(index).click();
            }
        });
    }
    typeValueAndSelectSingle(searchValue) {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const dropDown = await this.getDropdown(ComboboxDropdownContent_1.ComboboxDropdownContent);
            await Fixtures_1.expect.poll(() => dropDown.content.items.count).toBeGreaterThan(0);
            await this.internalInput.clearAndType(searchValue);
            await Fixtures_1.expect.poll(() => dropDown.content.items.count).toEqual(1);
            await dropDown.content.items.itemAt(0).click();
        });
    }
    async getDropdown(contentClass) {
        await this.click();
        const dropDownMenu = new DropdownControl_1.DropdownControl(this, new Selector_1.ReactSelector("ComboBoxRenderer Portal"), contentClass);
        return dropDownMenu;
    }
    async pressEnter() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.pressEnter();
    }
    get isFocused() {
        return this.internalInput.isFocused;
    }
}
exports.ComboboxOldControl = ComboboxOldControl;
