"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CrossFitTableControl = void 0;
const ExtendedPromise_1 = require("../../../ExtenedPromises/ExtendedPromise");
const CommonUtils_1 = require("../../Bases/CommonUtils");
const CompoundControl_1 = require("../../Bases/CompoundControl");
const ControlContructorBinding_1 = require("../../Bases/ControlDeclaration/ControlContructorBinding");
const FluentControlFactory_1 = require("../../Bases/ControlDeclaration/FluentControlFactory");
const ButtonControl_1 = require("../CandyControls/Action/ButtonControl");
const ClickableAreaControl_1 = require("../ClickableAreaControl");
const ControlList_1 = require("../../Bases/ControlList");
const LabelControl_1 = require("../LabelControl");
const LinkControl_1 = require("../CandyControls/Action/LinkControl");
const PagerControl_1 = require("../PagerControl");
const PopupMenu_1 = require("../PopupMenu");
const Selector_1 = require("../../Bases/Selectors/Selector");
const ControlsBase_1 = require("../../Bases/ControlsBase");
const TableRemoveRowsUtils_1 = require("../TableRemoveRowsUtils");
const HeaderMenu_1 = require("./HeaderMenu");
class CrossFitTableGroupedTableHeaderCell extends CompoundControl_1.CompoundControl {
}
class CrossFitTableColgroupButton extends CompoundControl_1.CompoundControl {
    button = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("Button").in(this);
    tableCells = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(CrossFitTableGroupedTableHeaderCell.as("TableCell"))).in(this);
    click() {
        return this.button.click();
    }
    get collapsed() {
        return (0, ExtendedPromise_1.extendPromise)(async () => (await this.tableCells.count) === 0);
    }
}
class RowMenuContent extends CompoundControl_1.CompoundControl {
    copyButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-copy").in(this);
    copyToEndButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-copy-to-end").in(this);
    removeButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-remove").in(this);
}
class RowMenu extends CompoundControl_1.CompoundControl {
    menu = (0, FluentControlFactory_1.control)(PopupMenu_1.PopupMenuControl.withContent(RowMenuContent)).as("PopupMenu").in(this);
    async removeViaRowMenu() {
        await this.menu.open();
        if (!(await this.menu.content.removeButton.isPresent())) {
            await this.menu.open();
        }
        await this.menu.content.removeButton.click();
    }
}
class CrossFitTableSideBar extends CompoundControl_1.CompoundControl {
    rowsMenu = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(RowMenu.as("RowMenu"))).in(this);
    removeRowButtons = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(ControlsBase_1.ControlBase.as("RemoveRowButton"))).in(this);
}
class CrossFitTableFilterInfo extends CompoundControl_1.CompoundControl {
    resetFilters = (0, FluentControlFactory_1.control)(LinkControl_1.LinkControl).tid("FilterReset").in(this);
    foundInstancesText = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("span[data-tid='FoundInstancesText']").in(this);
    isPresent() {
        return this.foundInstancesText.isPresent();
    }
}
class CrossFitTableControl extends CompoundControl_1.CompoundControl {
    static defaultElementSelector = "CrossfitTable";
    rows;
    addButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("AddRowButton").in(this);
    pager = (0, FluentControlFactory_1.control)(PagerControl_1.PagerControl).as("Pager").in(this);
    hideColumnButtons = CrossFitTableColgroupButton.as("CrossfitTableHeader ColgroupButton").listIn(this);
    sideBar = (0, FluentControlFactory_1.control)(CrossFitTableSideBar).as("MultiLineSidebar").in(this);
    filterInfo = (0, FluentControlFactory_1.control)(CrossFitTableFilterInfo).as("FilterInfo").in(this);
    scrollToRightArrow = (0, FluentControlFactory_1.control)(ClickableAreaControl_1.ClickableAreaControl).as("div").tid("ScrollableRightArrow").in(this);
    scrollToLeftArrow = (0, FluentControlFactory_1.control)(ClickableAreaControl_1.ClickableAreaControl).as("div").tid("ScrollableLeftArrow").in(this);
    constructor(context, selector, childFactory) {
        super(context, selector);
        this.rows = new ControlList_1.ControlList(this, new Selector_1.ReactSelector("MultiLine Row"), childFactory);
    }
    static withRow(childFactory) {
        return (0, ControlContructorBinding_1.controlOf)(CrossFitTableControl, childFactory);
    }
    async addRow() {
        await this.scrollIntoView();
        await this.addButton.click();
        // BT-HACK защита от анимированного скроллинга
        await (0, CommonUtils_1.delay)(50);
    }
    async addRows(n) {
        await this.scrollIntoView();
        for (let i = 0; i < n; i++) {
            await this.addButton.click();
            // BT-HACK защита от анимированного скроллинга
            await (0, CommonUtils_1.delay)(50);
        }
    }
    findHeaderMenuByText(text, headerMenuContent) {
        return new HeaderMenu_1.HeaderMenu(this, new Selector_1.ReactSelector(`HeaderMenu[header=${text}]`), headerMenuContent);
    }
    async removeAllRows() {
        return (0, TableRemoveRowsUtils_1.removeRowByIndexСyclical)(this, 0);
    }
    removeRowsExceptFirst() {
        return (0, TableRemoveRowsUtils_1.removeRowByIndexСyclical)(this, 1);
    }
}
exports.CrossFitTableControl = CrossFitTableControl;
