"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmbeddedLogControl = exports.ErrorsGroupControl = exports.ErrorListControl = void 0;
const pdf_parse_1 = __importDefault(require("pdf-parse"));
const ExtendedPromise_1 = require("../../ExtenedPromises/ExtendedPromise");
const CompoundControl_1 = require("../Bases/CompoundControl");
const FluentControlFactory_1 = require("../Bases/ControlDeclaration/FluentControlFactory");
const ControlList_1 = require("../Bases/ControlList");
const ControlsBase_1 = require("../Bases/ControlsBase");
const ButtonControl_1 = require("./CandyControls/Action/ButtonControl");
const LabelControl_1 = require("./LabelControl");
const LoaderControl_1 = require("./LoaderControl");
const TooltipContentBaseControl_1 = require("./TooltipContentBaseControl");
class ErrorItemControl extends CompoundControl_1.CompoundControl {
    sourceLinkControl = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("span").tid("SourceItemText").in(this);
    description = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("span").tid("DescriptionItemText").in(this);
    descriptionItemHiddenText = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("HiddenText").in(this);
    async touched() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const error = await this.getProp("error");
            return Boolean(error.touched);
        });
    }
}
class InterdocumentCheckIssueControl extends CompoundControl_1.CompoundControl {
    description = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("div").tid("Description").in(this);
}
class ErrorListControl extends CompoundControl_1.CompoundControl {
    attachmentGroup = (0, FluentControlFactory_1.control)(ErrorsGroupControl).as("section").tid("protocol-attachment").in(this);
    warrantGroup = (0, FluentControlFactory_1.control)(ErrorsGroupControl).as("section").tid("protocol-warrant").in(this);
    items = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(ErrorItemControl.as("ErrorsItem"))).in(this);
    interdocumentCheckIssues = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(InterdocumentCheckIssueControl.as("InterdocumentCheckIssue"))).in(this);
}
exports.ErrorListControl = ErrorListControl;
class ErrorsGroupControl extends CompoundControl_1.CompoundControl {
    items = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(ErrorItemControl.as("ListItemWrapper"))).in(this);
}
exports.ErrorsGroupControl = ErrorsGroupControl;
class ErrorLogTooltip extends TooltipContentBaseControl_1.TooltipContentBaseControl {
    // Магические числа - это такая иконка в закрытии tooltip
    closeButton = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("svg[viewBox='0 0 20 20']").in(this);
    descriptionItemText = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("span").tid("DescriptionItemText").in(this);
}
class CheckingStatusControl extends CompoundControl_1.CompoundControl {
    textControl = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("p").tid("Text").in(this);
    get text() {
        return this.textControl.text;
    }
}
class EmbeddedLogControl extends CompoundControl_1.CompoundControl {
    header = (0, FluentControlFactory_1.control)(CompoundControl_1.CompoundControl).as("ErrorLogCounters").in(this);
    checkingStatus = (0, FluentControlFactory_1.control)(CheckingStatusControl).as("CheckingStatus").in(this);
    checkButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("ActionButton").prop("type", "check").in(this);
    errorList = (0, FluentControlFactory_1.control)(ErrorListControl).as("ErrorsList").in(this);
    loader = (0, FluentControlFactory_1.control)(LoaderControl_1.LoaderControl).as("Loader").in(this);
    initialLoader = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("Spinner").in(this);
    tooltip = (0, FluentControlFactory_1.control)(ErrorLogTooltip).as("ErrorLogTooltip Tooltip").in(this);
    printErrorLog = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuAction").tid("PrintErrorLog").in(this);
    errorLogHeader = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("ErrorGroupHeader").in(this);
    get hasErrors() {
        return this.errorList.isPresent();
    }
    async downloadErrorLogAsPdf(candyServiceContext) {
        const fileUrl = await this.printErrorLog.getProp("href");
        if (fileUrl == undefined) {
            throw new Error("InvalidProgramState");
        }
        return candyServiceContext.downloadFile(fileUrl);
    }
    async downloadErrorLogAsPdfConvertedToText(candyServiceContext) {
        const file = await this.downloadErrorLogAsPdf(candyServiceContext);
        const fileParsed = await (0, pdf_parse_1.default)(file);
        return fileParsed.text.replace(/\n/gi, " ");
    }
    get loading() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            if (await this.initialLoader.isPresent()) {
                return true;
            }
            if (!(await this.loader.isPresent())) {
                return false;
            }
            return this.loader.isActive;
        });
    }
    async clickErrorLinkIfNotTouched(errorIndex) {
        // await expect(this.errorList.items.itemAt(errorIndex).touched()).to.eventually.be.false();
        await this.errorList.items.itemAt(errorIndex).sourceLinkControl.click();
    }
    async checkTooltip(errorIndex, errorText) {
        // await expect(this.tooltip.isPresent()).to.eventually.eql(true);
        // await expect(this.tooltip.descriptionItemText.text).to.eventually.contain(errorText);
        await this.tooltip.closeButton.click();
    }
}
exports.EmbeddedLogControl = EmbeddedLogControl;
