"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NavigationPanel = exports.NavigationPanelActionList = exports.FufUploadModalControl = exports.InquiryUploadModalControl = void 0;
const test_1 = require("playwright/test");
const ExtendedPromise_1 = require("../../ExtenedPromises/ExtendedPromise");
const ControlsBase_1 = require("../Bases/ControlsBase");
const CommonUtils_1 = require("../Bases/CommonUtils");
const CompoundControl_1 = require("../Bases/CompoundControl");
const FluentControlFactory_1 = require("../Bases/ControlDeclaration/FluentControlFactory");
const Selector_1 = require("../Bases/Selectors/Selector");
const ControlList_1 = require("../Bases/ControlList");
const AfterPageLoadActions_1 = require("../../BrowserWrapper/AfterPageLoadActions");
const ButtonControl_1 = require("./CandyControls/Action/ButtonControl");
const ClickableAreaControl_1 = require("./ClickableAreaControl");
const LinkControl_1 = require("./CandyControls/Action/LinkControl");
const DropdownControl_1 = require("./DropdownControl");
const LabelControl_1 = require("./LabelControl");
const DropdownMenuControl_1 = require("./DropdownMenuControl");
const UnitItemControl_1 = require("./UnitItemControl");
const EmbeddedLogControl_1 = require("./EmbeddedLogControl");
class MoreActionsMenuContent extends CompoundControl_1.CompoundControl {
    downloadButton = (0, FluentControlFactory_1.control)(LinkControl_1.LinkControl).as("MenuItem").tid("panel-action-fuf").in(this);
    clearFormButton = (0, FluentControlFactory_1.control)(LinkControl_1.LinkControl).as("MenuItem").tid("panel-action-remove").in(this);
    uploadFufButton = (0, FluentControlFactory_1.control)(LinkControl_1.LinkControl).as("MenuItem").tid("panel-action-fuf-upload").in(this);
}
class MoreActionsMenu extends CompoundControl_1.CompoundControl {
    menu = new DropdownMenuControl_1.DropdownMenuControl(this, new Selector_1.ReactSelector("DropdownMenu"), MoreActionsMenuContent);
}
class HideUnitButtonControl extends ControlsBase_1.ControlBase {
    get text() {
        return (0, ExtendedPromise_1.extendPromise)(async () => (await super.text)?.trim());
    }
    isPresent() {
        return this.getProp("hasEmptyUnit");
    }
}
class InquiryUploadModalControl extends ControlsBase_1.ControlBase {
    uploadButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).tid("UploadInquiryButton").in(this);
    reloadFormButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).tid("ReloadFormButton").in(this);
    fufUploadError = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("FufUploadError").in(this);
    inquiryCount = LabelControl_1.LabelControl.as("div[data-tid=InquiryCount]").in(this);
    wrongGFVError = LabelControl_1.LabelControl.as("div[data-tid=WrongGFVError]").in(this);
    addedInquiryCount = LabelControl_1.LabelControl.as("div[data-tid=AddedInquiryCount]").in(this);
    updatedInquiryCount = LabelControl_1.LabelControl.as("div[data-tid=UpdatedInquiryCount]").in(this);
}
exports.InquiryUploadModalControl = InquiryUploadModalControl;
class FufUploadModalControl extends ControlsBase_1.ControlBase {
    uploadButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("Button").in(this);
    fufFileProperties = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("FufFileProperties").in(this);
    fufUploadError = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("FufUploadError").in(this);
    fufFileUploader = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("FileUploader").in(this);
    get fileProperties() {
        return this.fufFileProperties.getProp("fileProperties");
    }
    get success() {
        return (0, ExtendedPromise_1.extendPromise)(async () => !(await this.fufUploadError.isPresent()));
    }
    get error() {
        return this.fufUploadError.isPresent();
    }
    get text() {
        return this.fufUploadError.text;
    }
}
exports.FufUploadModalControl = FufUploadModalControl;
class NavigationPanelActionList extends CompoundControl_1.CompoundControl {
    downloadButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("ActionsBarButton").tid("ActionDownload").in(this);
    historyEditingButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl)
        .as("ActionsBarButton")
        .tid("panel-action-history-editing")
        .in(this);
    clearFormButton = (0, FluentControlFactory_1.control)(LinkControl_1.LinkControl).as("ActionsBarButton").tid("panel-action-remove").in(this);
    uploadFufButton = (0, FluentControlFactory_1.control)(LinkControl_1.LinkControl)
        .as("ActionsBarButton")
        .tid("panel-action-fuf-upload")
        .in(this);
    moreActionsMenu = (0, FluentControlFactory_1.control)(MoreActionsMenu).as("ActionsBarButton").tid("MoreActionsList").in(this);
    async clickClearFormButton() {
        await this.clearFormButton.click();
        await (0, CommonUtils_1.delay)(3000);
        await (0, AfterPageLoadActions_1.onAfterPageLoad)(this.rootPage.page);
    }
    async clickUploadFufButton(filesPath, contentClass) {
        const fileChooserPromise = this.rootPage.waitForDownloadEvent();
        if (await this.uploadFufButton.isPresent()) {
            await this.uploadFufButton.click();
        }
        else {
            await this.moreActionsMenu.click();
            await this.moreActionsMenu.menu.content.uploadFufButton.click();
        }
        const dropDownMenu = new DropdownControl_1.DropdownControl(this, new Selector_1.ReactSelector("FufUploadModal", undefined, true), contentClass);
        if (await dropDownMenu.content.fufFileUploader.isPresent()) {
            await dropDownMenu.content.fufFileUploader.click();
        }
        const fileChooser = await fileChooserPromise;
        await fileChooser.setFiles(filesPath);
        await (0, CommonUtils_1.delay)(1500);
        return dropDownMenu;
    }
    async clickDownloadButton() {
        if (await this.downloadButton.isPresent()) {
            await this.downloadButton.click();
        }
        else {
            await this.moreActionsMenu.click();
            await (0, CommonUtils_1.delay)(500);
            await this.moreActionsMenu.menu.content.downloadButton.click();
        }
    }
    async openHistoryEditingWidget() {
        await this.historyEditingButton.click();
        await (0, CommonUtils_1.delay)(200);
    }
}
exports.NavigationPanelActionList = NavigationPanelActionList;
class NavigationPanel extends CompoundControl_1.CompoundControl {
    static defaultElementSelector = "withResolvePaths(**(Panel))";
    actionList = (0, FluentControlFactory_1.control)(NavigationPanelActionList).as("ActionsBar").in(this);
    runTourButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("ActionsBarButton").tid("panel-action-tour").in(this);
    checkButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("Connect(ActionButton)[type=check]").in(this);
    sendActionButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("SendActionButton").in(this);
    zeroReportingButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl)
        .as("ActionsBarButton")
        .tid("panel-action-zero-report")
        .in(this);
    formSaveStatus = (0, FluentControlFactory_1.control)(ClickableAreaControl_1.ClickableAreaControl).as("FormSaveStatus").in(this);
    pin = (0, FluentControlFactory_1.control)(ClickableAreaControl_1.ClickableAreaControl).as("Pin").in(this);
    embeddedLog = (0, FluentControlFactory_1.control)(EmbeddedLogControl_1.EmbeddedLogControl).as("EmbeddedLog").in(this);
    unitList = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(UnitItemControl_1.UnitItemSingleControl.as("UnitItem"))).in(this);
    unit2NavigationItem = new UnitItemControl_1.UnitItemMultipleFlatControl(this, new Selector_1.ReactSelector("UnitItem", { "data-tid": "unit2" }), ControlsBase_1.ControlBase);
    hideUnitButton = (0, FluentControlFactory_1.control)(HideUnitButtonControl).as("HideUnitButton").in(this);
    recalculateDirtyMasksButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("RecalculateButton").in(this);
    async ensureAllDataSaved() {
        await this.formSaveStatus.click();
        await test_1.expect.poll(() => this.formSaveStatus.text, { timeout: 20000 }).toContain("Все изменения сохранены");
    }
}
exports.NavigationPanel = NavigationPanel;
