"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactUIInputControl = void 0;
const ControlsBase_1 = require("../../Bases/ControlsBase");
const HtmlInputControl_1 = require("../HtmlControls/HtmlInputControl");
const ExtendedPromise_1 = require("../../../ExtenedPromises/ExtendedPromise");
const FluentControlFactory_1 = require("../../Bases/ControlDeclaration/FluentControlFactory");
class ReactUIInputControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "Input";
    internalInputHtmlElement = (0, FluentControlFactory_1.control)(HtmlInputControl_1.HtmlInputControl).as("input").in(this);
    getInternalInputHtmlElement() {
        return this.internalInputHtmlElement;
    }
    async type(text) {
        await this.getInternalInputHtmlElement().type(text);
    }
    get value() {
        return this.getInternalInputHtmlElement().value;
    }
    get isDisabled() {
        return this.getProp("disabled");
    }
    get isValid() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const error = await this.getProp("error");
            return !error;
        });
    }
    get isFocused() {
        return this.internalInputHtmlElement.isActiveElement();
    }
    get focused() {
        return this.internalInputHtmlElement.isActiveElement();
    }
    async pressEnter() {
        await this.getInternalInputHtmlElement().pressEnter();
    }
    async clearAndType(text) {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        // await element.click();
        // // BT-HACK это на случай если промахнулись
        // await delay(1);
        await element.click();
        // await this.rootPage.pressCtrlA();
        // await this.rootPage.press("Backspace");
        await this.getInternalInputHtmlElement().clearAndType(text);
    }
    async clickAndType(text) {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        await this.getInternalInputHtmlElement().type(text);
    }
    async focus() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.focus();
    }
    async clear() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await element.click();
        await this.rootPage.pressCtrlA();
        await this.rootPage.press("Backspace");
    }
    async blur() {
        const element = await this.getRootElement();
        if (element == undefined) {
            throw new Error("Element not found");
        }
        await this.rootPage.press("Tab");
    }
    get placeholder() {
        return this.getProp("placeholder");
    }
    get error() {
        return (0, ExtendedPromise_1.extendPromise)(async () => Boolean(await this.getProp("error")));
    }
    get warning() {
        return (0, ExtendedPromise_1.extendPromise)(async () => Boolean(await this.getProp("warning")));
    }
}
exports.ReactUIInputControl = ReactUIInputControl;
