"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StickyTableControl = exports.StickyTableSideBarWithRows = void 0;
const CommonUtils_1 = require("../Bases/CommonUtils");
const CompoundControl_1 = require("../Bases/CompoundControl");
const ControlContructorBinding_1 = require("../Bases/ControlDeclaration/ControlContructorBinding");
const FluentControlFactory_1 = require("../Bases/ControlDeclaration/FluentControlFactory");
const Selector_1 = require("../Bases/Selectors/Selector");
const ControlList_1 = require("../Bases/ControlList");
const ReactNodeAtIndexSelector_1 = require("../Bases/Selectors/ReactNodeAtIndexSelector");
const ButtonControl_1 = require("./CandyControls/Action/ButtonControl");
const ClickableAreaControl_1 = require("./ClickableAreaControl");
const MultilineControl_1 = require("./MultilineControl");
const MultilineFooterControl_1 = require("./MultilineFooterControl");
const PagerControl_1 = require("./PagerControl");
const PopupMenu_1 = require("./PopupMenu");
const HeaderMenu_1 = require("./CrossFitTable/HeaderMenu");
const TableRemoveRowsUtils_1 = require("./TableRemoveRowsUtils");
class RemoveButton extends CompoundControl_1.CompoundControl {
}
class RowMenuContent extends CompoundControl_1.CompoundControl {
    addPreviousButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-add-previous").in(this);
    addNextButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-add-next").in(this);
    copyButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-copy").in(this);
    copyToEndButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-copy-to-end").in(this);
    removeButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-remove").in(this);
}
class RowMenu extends CompoundControl_1.CompoundControl {
    menu = (0, FluentControlFactory_1.control)(PopupMenu_1.PopupMenuControl.withContent(RowMenuContent)).as("PopupMenu").in(this);
    async copyAfterViaRowMenu() {
        await this.openMenu();
        await this.menu.content.copyButton.click();
    }
    async removeViaRowMenu() {
        await this.openMenu();
        if (!(await this.menu.content.removeButton.isPresent())) {
            await this.openMenu();
        }
        await this.menu.content.removeButton.click();
    }
    async copyToEndViaRowMenu() {
        await this.openMenu();
        await this.menu.content.copyToEndButton.click();
    }
    async addNextViaRowMenu() {
        await this.openMenu();
        await this.menu.content.addNextButton.click();
    }
    async addPreviousViaRowMenu() {
        await this.openMenu();
        await this.menu.content.addPreviousButton.click();
    }
    async openMenu() {
        await this.menu.open();
        if (!(await this.menu.content.isPresent())) {
            await this.menu.open();
        }
    }
}
class StickyTableSideBar extends CompoundControl_1.CompoundControl {
    rowsMenu = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(RowMenu.as("RowMenu"))).in(this);
    removeRowButtons = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(RemoveButton.as("RemoveRowButton"))).in(this);
}
class StickyTableSideBarWithRows extends CompoundControl_1.CompoundControl {
    rows;
    rowsMenu = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(RowMenu.as("RowMenu"))).in(this);
    removeRowButtons = (0, FluentControlFactory_1.control)(ControlList_1.ControlList.of(RemoveButton.as("RemoveRowButton"))).in(this);
    constructor(context, selector, childFactory) {
        super(context, selector);
        this.rows = new ControlList_1.ControlList(this, new Selector_1.ReactSelector("div[data-tid='SidebarRow']"), childFactory);
    }
    static of(rowFactory) {
        return (0, ControlContructorBinding_1.controlOf)(StickyTableSideBarWithRows, rowFactory);
    }
}
exports.StickyTableSideBarWithRows = StickyTableSideBarWithRows;
class StickyTableControl extends CompoundControl_1.CompoundControl {
    static defaultElementSelector = "StickyTable";
    rows;
    addButtons;
    addButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("AddRowButton").in(this);
    pager = (0, FluentControlFactory_1.control)(PagerControl_1.PagerControl).as("Pager").in(this);
    sideBar = (0, FluentControlFactory_1.control)(StickyTableSideBar).as("MultiLineSidebar").in(this);
    scrollToRightArrow = (0, FluentControlFactory_1.control)(ClickableAreaControl_1.ClickableAreaControl).as("div").tid("ScrollableRightArrow").in(this);
    scrollToLeftArrow = (0, FluentControlFactory_1.control)(ClickableAreaControl_1.ClickableAreaControl).as("div").tid("ScrollableLeftArrow").in(this);
    constructor(context, selector, childFactory) {
        super(context, selector);
        this.rows = new ControlList_1.ControlList(this, new Selector_1.ReactSelector("MultiLine Row"), childFactory);
        this.addButtons = new ControlList_1.ControlList(this, new Selector_1.ReactSelector("AddRowButton"), childFactory);
    }
    static withRow(childFactory) {
        return (0, ControlContructorBinding_1.controlOf)(StickyTableControl, childFactory);
    }
    async addRow() {
        await this.scrollIntoView();
        await this.addButton.click();
        // BT-HACK защита от анимированного скроллинга
        await (0, CommonUtils_1.delay)(250);
    }
    async addRows(n) {
        await this.scrollIntoView();
        for (let i = 0; i < n; i++) {
            await this.addButton.click();
            // BT-HACK защита от анимированного скроллинга
            await (0, CommonUtils_1.delay)(250);
        }
    }
    getMultilineByPath(bindingPath, row, instance) {
        return new MultilineControl_1.MultilineControl(this, instance !== undefined
            ? new ReactNodeAtIndexSelector_1.ReactNodeAtIndexSelector(new Selector_1.ReactSelector("MultiLine", { bindingPath: bindingPath }), instance)
            : new Selector_1.ReactSelector("MultiLine", { bindingPath: bindingPath }), row);
    }
    getSidebarByPath(bindingPath, rowFactory) {
        return new StickyTableSideBarWithRows(this, new Selector_1.ReactSelector("MultiLineSidebar", { bindingPath: bindingPath }), rowFactory);
    }
    getPagerByPath(bindingPath) {
        return new PagerControl_1.PagerControl(this, new Selector_1.ReactSelector("Pager", { bindingPath: bindingPath }));
    }
    getMultilineFooterByPath(bindingPath) {
        return new MultilineFooterControl_1.MultilineFooterControl(this, new Selector_1.ReactSelector("Footer", { bindingPath: bindingPath }));
    }
    findHeaderMenuByText(text, headerMenuContent) {
        return new HeaderMenu_1.HeaderMenu(this, new Selector_1.ReactSelector(`HeaderMenu[header=${text}]`), headerMenuContent);
    }
    async removeAllRows() {
        return (0, TableRemoveRowsUtils_1.removeRowByIndexСyclical)(this, 0);
    }
    removeRowsExceptFirst() {
        return (0, TableRemoveRowsUtils_1.removeRowByIndexСyclical)(this, 1);
    }
}
exports.StickyTableControl = StickyTableControl;
