import { ExtendedPromise } from "../../ExtenedPromises/ExtendedPromise";
import { ControlBase } from "../Bases/ControlsBase";
import { ISearchContext } from "../Bases/Selectors/ISearchContext";
import { ISelector } from "../Bases/Selectors/ISelector";
import { ControlList } from "../Bases/ControlList";
import { UnitId } from "../../../../../Engine/src/Controls/FormParts/Unit/UnitId/UnitId";
import { ControlConstructor } from "../../BrowserWrapper/BrowserPageWrapper";
import { ClickableAreaControl } from "./ClickableAreaControl";
import { TooltipContentBaseControl } from "./TooltipContentBaseControl";
import { LinkControl } from "./CandyControls/Action/LinkControl";
import { LabelControl } from "./LabelControl";
import { InputControl } from "./CandyControls/ValueEditors/InputControl";
import { ButtonControl } from "./CandyControls/Action/ButtonControl";
import { KebabControl } from "./CandyControls/Action/KebabControl";
declare class UnitItemTooltip extends TooltipContentBaseControl {
    readonly closeButton: LabelControl;
    readonly descriptionItemText: LabelControl;
    isPresent(): ExtendedPromise<boolean>;
}
declare class IconButtonControl extends ControlBase {
    static defaultElementSelector: string;
    get disabled(): ExtendedPromise<boolean>;
}
declare class UnitItemControl extends ControlBase {
    static defaultElementSelector: string;
    protected readonly menuItem: ControlBase;
    protected readonly content: ControlBase;
    protected readonly itemName: ControlBase;
    click(): Promise<void>;
    get text(): ExtendedPromise<undefined | string>;
    get unitId(): ExtendedPromise<UnitId | undefined>;
    get filled(): ExtendedPromise<boolean>;
    get active(): ExtendedPromise<boolean>;
    get multiple(): Promise<boolean>;
    get current(): Promise<boolean>;
    isPresent(): ExtendedPromise<boolean>;
}
declare class MultipleUnitPagingControl extends ControlBase {
    readonly nextLink: LinkControl;
    readonly previousLink: LinkControl;
    readonly input: InputControl;
    goToPage(instanceNumber: string): Promise<void>;
}
declare class MenuItemControl extends ControlBase {
    get current(): ExtendedPromise<boolean>;
}
declare class ItemMultipleInstanceNestedControl extends MenuItemControl {
    readonly kebab: KebabControl;
    readonly deleteIcon: IconButtonControl;
    readonly addButton: ButtonControl;
    readonly copyButton: ButtonControl;
    readonly clearButton: ButtonControl;
    readonly deleteButton: ButtonControl;
    addInstance(): Promise<void>;
    copyInstance(): Promise<void>;
    clearInstance(): Promise<void>;
    deleteInstance(): Promise<void>;
}
declare class ItemMultipleFlatInstanceControl<TNestedItems extends ControlBase> extends UnitItemControl {
    private readonly nestedItemsControlType;
    readonly deleteIcon: IconButtonControl;
    readonly copyIcon: IconButtonControl;
    readonly kebab: KebabControl;
    readonly addButton: ButtonControl;
    readonly copyButton: ButtonControl;
    readonly clearButton: ButtonControl;
    readonly deleteButton: ButtonControl;
    constructor(context: ISearchContext, selector: ISelector, controlType: ControlConstructor<TNestedItems>);
    deleteInstance(): Promise<void>;
    copyInstance(): Promise<void>;
    clearInstance(): Promise<void>;
    get nested(): TNestedItems;
    click(): Promise<void>;
}
declare abstract class UnitItemControlBase extends UnitItemControl {
}
export declare class UnitItemSingleControl extends UnitItemControlBase {
    readonly nestedList: ControlBase;
    readonly kebab: KebabControl;
    readonly deleteButton: ButtonControl;
    readonly clearButton: ButtonControl;
    readonly plusButton: ButtonControl;
    nestedItems: ControlList<UnitItemControl>;
    clearInstance(): Promise<void>;
    deleteInstance(): Promise<void>;
}
declare class MultipleFlatRootItem extends LabelControl {
    readonly addButton: ClickableAreaControl;
}
export declare class UnitItemMultipleFlatControl<TNestedItems extends ControlBase> extends UnitItemControl {
    readonly tooltip: UnitItemTooltip;
    readonly nestedItemsControlType: ControlConstructor<TNestedItems>;
    readonly pager: MultipleUnitPagingControl;
    readonly kebab: KebabControl;
    readonly addButton: ButtonControl;
    readonly copyButton: ButtonControl;
    readonly clearButton: ButtonControl;
    readonly deleteButton: ButtonControl;
    constructor(context: ISearchContext, selector: ISelector, nestedItemsControlType: ControlConstructor<TNestedItems>);
    get instancesFlat(): ExtendedPromise<ControlList<ItemMultipleFlatInstanceControl<TNestedItems>>>;
    get instancesNested(): ExtendedPromise<ControlList<ItemMultipleInstanceNestedControl>>;
    get disableAppend(): ExtendedPromise<unknown>;
    get rootItem(): ExtendedPromise<MultipleFlatRootItem>;
    get addIcon(): ExtendedPromise<ControlBase>;
    addInstanceByKebab(): Promise<void>;
    addInstanceByPlus(): Promise<void>;
    addNInstanceByPlus(n: number): Promise<void>;
    addNInstanceByKebab(n: number): Promise<void>;
    addInstance(): Promise<void>;
    copyInstance(): Promise<void>;
    clearInstance(): Promise<void>;
    deleteInstance(): Promise<void>;
}
export {};
