"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UnitItemMultipleFlatControl = exports.UnitItemSingleControl = void 0;
const ExtendedPromise_1 = require("../../ExtenedPromises/ExtendedPromise");
const ControlsBase_1 = require("../Bases/ControlsBase");
const FluentControlFactory_1 = require("../Bases/ControlDeclaration/FluentControlFactory");
const Selector_1 = require("../Bases/Selectors/Selector");
const ControlList_1 = require("../Bases/ControlList");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const ClickableAreaControl_1 = require("./ClickableAreaControl");
const TooltipContentBaseControl_1 = require("./TooltipContentBaseControl");
const LinkControl_1 = require("./CandyControls/Action/LinkControl");
const LabelControl_1 = require("./LabelControl");
const InputControl_1 = require("./CandyControls/ValueEditors/InputControl");
const ButtonControl_1 = require("./CandyControls/Action/ButtonControl");
const KebabControl_1 = require("./CandyControls/Action/KebabControl");
class UnitItemTooltip extends TooltipContentBaseControl_1.TooltipContentBaseControl {
    closeButton = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("svg[viewBox='0 0 20 20']").in(this);
    descriptionItemText = (0, FluentControlFactory_1.control)(LabelControl_1.LabelControl).as("div").tid("DescriptionItemText").in(this);
    isPresent() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const trigger = await this.getProp("trigger");
            return trigger !== "closed";
        });
    }
}
class IconButtonControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "IconButton";
    get disabled() {
        return this.pick(x => x.disabled ?? false);
    }
}
class UnitItemControl extends ControlsBase_1.ControlBase {
    static defaultElementSelector = "UnitItem";
    menuItem = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("MenuItem").first().in(this);
    content = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("Content").first().in(this);
    itemName = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("UnitItemName").first().in(this.content);
    async click() {
        await this.itemName.click();
    }
    get text() {
        return this.itemName.text;
    }
    get unitId() {
        return this.pick(x => x.id);
    }
    get filled() {
        return this.menuItem.pick(x => !x.empty);
    }
    get active() {
        return this.menuItem.pick(x => !!x.active);
    }
    get multiple() {
        return this.pick(x => x.multiple);
    }
    get current() {
        return this.pick(x => x.isCurrent);
    }
    isPresent() {
        return this.pick(x => !x.hidden);
    }
}
class MultipleUnitPagingControl extends ControlsBase_1.ControlBase {
    nextLink = (0, FluentControlFactory_1.control)(LinkControl_1.LinkControl).as("a").tid("NextInstanceLink").in(this);
    previousLink = (0, FluentControlFactory_1.control)(LinkControl_1.LinkControl).as("a").tid("PreviousInstanceLink").in(this);
    input = (0, FluentControlFactory_1.control)(InputControl_1.InputControl).as("Input").tid("InstanceNumber").in(this);
    async goToPage(instanceNumber) {
        await this.input.clearAndType(instanceNumber);
        await this.input.rootPage.press("Enter");
    }
}
class MenuItemControl extends ControlsBase_1.ControlBase {
    get current() {
        return this.pick(x => x.active ?? false);
    }
}
class ItemMultipleInstanceNestedControl extends MenuItemControl {
    kebab = (0, FluentControlFactory_1.control)(KebabControl_1.KebabControl).tid("KebabMenu").in(this);
    deleteIcon = (0, FluentControlFactory_1.control)(IconButtonControl).prop("icon", "XIcon").in(this);
    addButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-add-previous").in(this);
    copyButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-copy").in(this);
    clearButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-clear").in(this);
    deleteButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-remove").in(this);
    async addInstance() {
        await this.kebab.click();
        await this.addButton.click();
    }
    async copyInstance() {
        await this.kebab.click();
        await this.copyButton.click();
    }
    async clearInstance() {
        await this.kebab.click();
        await this.clearButton.click();
    }
    async deleteInstance() {
        await this.kebab.click();
        await this.deleteButton.click();
    }
}
class ItemMultipleFlatInstanceControl extends UnitItemControl {
    nestedItemsControlType;
    deleteIcon = (0, FluentControlFactory_1.control)(IconButtonControl).prop("icon", "XIcon").in(this);
    copyIcon = (0, FluentControlFactory_1.control)(IconButtonControl).prop("icon", "CopyIcon").in(this);
    kebab = (0, FluentControlFactory_1.control)(KebabControl_1.KebabControl).tid("KebabMenu").in(this);
    addButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-add-previous").in(this);
    copyButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-copy").in(this);
    clearButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-clear").in(this);
    deleteButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-remove").in(this);
    constructor(context, selector, controlType) {
        super(context, selector);
        this.nestedItemsControlType = controlType;
    }
    async deleteInstance() {
        await this.kebab.click();
        await this.deleteButton.click();
    }
    async copyInstance() {
        await this.kebab.click();
        await this.copyButton.click();
    }
    async clearInstance() {
        await this.kebab.click();
        await this.clearButton.click();
    }
    get nested() {
        return new this.nestedItemsControlType(this, new Selector_1.ReactSelector("NestedItems"));
    }
    async click() {
        await this.content.click();
    }
}
class UnitItemControlBase extends UnitItemControl {
}
class UnitItemSingleControl extends UnitItemControlBase {
    nestedList = (0, FluentControlFactory_1.control)(ControlsBase_1.ControlBase).as("NestedItems").in(this);
    kebab = (0, FluentControlFactory_1.control)(KebabControl_1.KebabControl).tid("KebabMenu").in(this);
    deleteButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-remove").in(this);
    clearButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-clear").in(this);
    plusButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("IconButton").tid("Add").in(this);
    nestedItems = UnitItemControl.as("UnitItem").listIn(this.nestedList);
    async clearInstance() {
        await this.kebab.click();
        await this.clearButton.click();
    }
    async deleteInstance() {
        await this.kebab.click();
        await this.deleteButton.click();
    }
}
exports.UnitItemSingleControl = UnitItemSingleControl;
class MultipleFlatRootItem extends LabelControl_1.LabelControl {
    addButton = (0, FluentControlFactory_1.control)(ClickableAreaControl_1.ClickableAreaControl).as("IconButton").first().in(this);
}
class UnitItemMultipleFlatControl extends UnitItemControl {
    tooltip = (0, FluentControlFactory_1.control)(UnitItemTooltip).as("Tooltip").in(this);
    nestedItemsControlType;
    pager = (0, FluentControlFactory_1.control)(MultipleUnitPagingControl).as("MultipleUnitPaging").in(this);
    kebab = (0, FluentControlFactory_1.control)(KebabControl_1.KebabControl).tid("KebabMenu").in(this);
    addButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-add-previous").in(this);
    copyButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-copy").in(this);
    clearButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-clear").in(this);
    deleteButton = (0, FluentControlFactory_1.control)(ButtonControl_1.ButtonControl).as("MenuItem").tid("row-menu-remove").in(this);
    constructor(context, selector, nestedItemsControlType) {
        super(context, selector);
        this.nestedItemsControlType = nestedItemsControlType;
    }
    get instancesFlat() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const unitId = (await this.unitId) ?? (0, TypingUtils_1.reject)("UnitId must be defined");
            const nestedItemsControlType = this.nestedItemsControlType;
            const itemConstructor = class extends ItemMultipleFlatInstanceControl {
                constructor(context, selector) {
                    super(context, selector, nestedItemsControlType);
                }
            };
            return new ControlList_1.ControlList(this, new Selector_1.ReactSelector("MenuItem", { "data-tid": "Instance", "data-page-id": unitId.pageId }), itemConstructor);
        });
    }
    get instancesNested() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const unitId = (await this.unitId) ?? (0, TypingUtils_1.reject)("UnitId must be defined");
            return ItemMultipleInstanceNestedControl.as("MenuItem")
                .prop("data-tid", "Instance")
                .prop("data-page-id", unitId.pageId)
                .listIn(this);
        });
    }
    get disableAppend() {
        return this.rootItem.addButton.getProp("disabled");
    }
    get rootItem() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const unitId = (await this.unitId) ?? (0, TypingUtils_1.reject)("UnitId must be defined");
            return new MultipleFlatRootItem(this, new Selector_1.ReactSelector(`MenuItem[data-tid=Root][data-page-id=${unitId.pageId}]`));
        });
    }
    get addIcon() {
        return (0, ExtendedPromise_1.extendPromise)(async () => {
            const unitId = (await this.unitId) ?? (0, TypingUtils_1.reject)("UnitId must be defined");
            return (0, FluentControlFactory_1.control)(IconButtonControl).prop("data-tid", "Add").prop("data-page-id", unitId.pageId).in(this);
        });
    }
    async addInstanceByKebab() {
        await this.kebab.click();
        await this.addButton.click();
    }
    async addInstanceByPlus() {
        await this.rootItem.hover();
        await this.rootItem.addButton.click();
    }
    async addNInstanceByPlus(n) {
        for (let i = 0; i < n; i++) {
            await this.rootItem.hover();
            await this.rootItem.addButton.click();
        }
    }
    async addNInstanceByKebab(n) {
        for (let i = 0; i < n; i++) {
            await this.kebab.click();
            await this.addButton.click();
        }
    }
    async addInstance() {
        await this.kebab.click();
        await this.addButton.click();
    }
    async copyInstance() {
        await this.kebab.click();
        await this.copyButton.click();
    }
    async clearInstance() {
        await this.kebab.click();
        await this.clearButton.click();
    }
    async deleteInstance() {
        await this.kebab.click();
        await this.deleteButton.click();
    }
}
exports.UnitItemMultipleFlatControl = UnitItemMultipleFlatControl;
