"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCpuLimits = getCpuLimits;
const fs = __importStar(require("fs"));
const os = __importStar(require("os"));
function readNumberFromFile(file) {
    return Number(fs.readFileSync(file, "utf-8"));
}
function readAllNumbersFromFile(file) {
    return fs
        .readFileSync(file, "utf-8")
        .split(" ")
        .map(i => Number(i));
}
function getCpuLimits() {
    if (process.platform !== "linux") {
        return getCpuLimitFromOs();
    }
    return getCpuLimitFromEnv() ?? getCpuLimitFromCgroup() ?? getCpuLimitFromOs();
}
function getCpuLimitFromOs() {
    // eslint-disable-next-line no-console
    console.log(`Use cpu limit from os.cpus`);
    return os.cpus().length;
}
function getCpuLimitFromEnv() {
    try {
        const concurrencyLimit = Number(process.env.BUILD_CANDY_FORM_CONCURRENCY_LIMIT);
        if (!isNaN(concurrencyLimit) && concurrencyLimit > 0) {
            // eslint-disable-next-line no-console
            console.log(`Use cpu limit from environment BUILD_CANDY_FORM_CONCURRENCY_LIMIT`);
            return concurrencyLimit;
        }
        return null;
    }
    catch {
        return null;
    }
}
function getCpuLimitFromCgroup() {
    try {
        const cpuLimits = readAllNumbersFromFile("/sys/fs/cgroup/cpu.max");
        const cpuCfsQuotaUs = cpuLimits[0];
        const cpuCfsPeriodUs = cpuLimits[1];
        return calculateCpuLimit(cpuCfsQuotaUs, cpuCfsPeriodUs, "cgroup v2");
    }
    catch (e1) {
        // eslint-disable-next-line no-console
        console.log(`Don't use limit from cgroup v2`, e1);
        try {
            const cpuCfsQuotaUs = readNumberFromFile("/sys/fs/cgroup/cpu/cpu.cfs_quota_us");
            const cpuCfsPeriodUs = readNumberFromFile("/sys/fs/cgroup/cpu/cpu.cfs_period_us");
            return calculateCpuLimit(cpuCfsQuotaUs, cpuCfsPeriodUs, "cgroup");
        }
        catch (e2) {
            // eslint-disable-next-line no-console
            console.log(`Don't use limit from cgroup`, e2);
            return null;
        }
    }
}
function calculateCpuLimit(cpuCfsQuotaUs, cpuCfsPeriodUs, type) {
    if (cpuCfsQuotaUs && cpuCfsPeriodUs) {
        const cpuLimit = cpuCfsQuotaUs / cpuCfsPeriodUs;
        if (cpuLimit > 0) {
            // eslint-disable-next-line no-console
            console.log(`Use cpu limit from ${type}`);
            return cpuLimit;
        }
    }
    throw new Error(`${type} is not use`);
}
